/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/


package gov.lbl.srm.db;

import gov.lbl.srm.util.*;
import java.sql.*;
import java.util.*;

public abstract class TDBProtocol {
  static String _gH2Protocol = "jdbc:h2";
  static String _gDerbyProtocol = "jdbc:derby";

  protected String _value = null;

  protected TDBProtocol(String val) {
    _value = val;
    _value = _value+":";
  }

  public static final TDBProtocol H2_PROTOCOL = new TDBProtocolH2(_gH2Protocol);
  public static final TDBProtocol DERBY_PROTOCOL = new TDBProtocolDerby(_gDerbyProtocol);

  public String getValue() {
    return _value;
  }

  public abstract void init(String dbPath);
  public abstract Connection getConnection(TSRMFootprintDB fp);
}

class TDBProtocolH2 extends TDBProtocol {
  protected TDBProtocolH2(String val) {
    super(val);
  }

  public void init(String dbPath) {    
    if((dbPath != null) && (dbPath.length() > 0)) {     
      if(dbPath.endsWith("/")) {
	dbPath = dbPath.substring(0,dbPath.length()-1);
      }
    } else {
      dbPath = ".";
    }

    _value = _value+dbPath;
  }

  public Connection getConnection(TSRMFootprintDB fp) {
    TSRMLog.info(this.getClass(), null, "event=connect", "protocol="+getValue());
       
    try {
      return DriverManager.getConnection(getValue()+"/"+fp._dbName+
					 ";AUTO_SERVER=TRUE;DB_CLOSE_ON_EXIT=FALSE", 
					 fp._dbUser, fp._password); 
    } catch (Exception e) {
      e.printStackTrace();
      TSRMLog.exception(this.getClass(), "getConnection", e);
      return null;
    }
  }
}

class TDBProtocolDerby extends TDBProtocol {
  protected TDBProtocolDerby(String val) {
    super(val);
  }

  public void init(String dbPath) {}

  public Connection getConnection(TSRMFootprintDB fp) {    
    TSRMLog.info(this.getClass(), null, "event=connect", "protocol="+getValue());
	
    Properties props = new Properties(); // connection properties
    // providing a user name and _password is optional in the embedded
    // and derbyclient frameworks
    props.put("user", fp._dbUser);
    props.put("password", fp._password);
	
    /* By default, the schema APP will be used when no username is
     * provided.
     * Otherwise, the schema name is the same as the user name (in this
     * case "user1" or USER1.)
     *
     * Note that user authentication is off by default, meaning that any
     * user can connect to your database using any _password. To enable
     * authentication, see the Derby Developer's Guide.
     */
    
    /*
     * This connection specifies create=true in the connection URL to
     * cause the database to be created when connecting for the first
     * time. To remove the database, remove the directory derbyDB (the
     * same as the database name) and its contents.
     *
     * The directory derbyDB will be created under the directory that
     * the system property derby.system.home points to, or the current
     * directory (user.dir) if derby.system.home is not set.
     */
    
    try {
      return DriverManager.getConnection(getValue() + fp._dbName+ ";create=true", props);
    } catch (Exception e) {
      e.printStackTrace();
      TSRMLog.exception(this.getClass(), "getConnection", e);
      return null;
    } 
  }
}