/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/


package gov.lbl.srm.util;

import  java.util.*;
import  java.io.*;
import  java.sql.*;

import  gov.lbl.srm.db.*;
import  gov.lbl.srm.storage.*;
import  gov.lbl.srm.server.*;
import  gov.lbl.srm.StorageResourceManager.*;
 
public class TSRMCacheLogFootprint implements ISRMLogCache {
    TSRMFootprintDB _db = TSRMServer.getFootprintDB();
    HashMap _deviceLookup = new HashMap();
    private boolean _isReading = false;
    int _deviceCounter = 0;
    
    public TSRMCacheLogFootprint() 
    {
	TSRMLog.info(this.getClass(), null, "event=cacheloginit", null);
	System.out.println("......................snap shot");
	_db._aux.listFileAttrs(-1);

    }

    public void clean() {
	TSRMLog.info(this.getClass(), null, "event=CleaningUpDB.",  null);
	System.out.println("......................snap shot");
	_db._aux.listFileAttrs(-1);
    }


    public void accountCreated(TAccount user) {
	if (user == null) {
	    return;
	}
	try {
	    boolean result=_db.addUser(user.getID());

	    result=_db.mkDir(user.getTopDir().getCanonicalPath());
	    //_db._aux.listUsers();
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "addUserFailed,uid="+user.getID(), e);
	}
    }
    
    public void deviceCreated(TBasicDevice device) {
	if (device == null) {
	    return;
	}
	_deviceLookup.put(device.getTag(), device);
	try {
	    boolean result = _db.addComponent(device.getTag(), device.getAccessLatency().getValue(), 
					      device.getTotalSizeInMB(), null, device.getDeviceDisk());
	    if (!result) {
		TSRMUtil.startUpInfo("Unable to add device:"+device.getDeviceDisk()+" Check eventlog for details. May need to cleanup the database");
		System.exit(1);
	    }
	    _deviceCounter++;
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "addDeviceFailed,device="+device.getTag(), e);
	}	
    }

    public void tokenCreated(TSRMStorage.iSRMSpaceToken token) {
	if (_isReading) {
	    return;
	}
	if (token == null) {
	    return;
	}
	try {
	    _db._aux.listToken(null);
	    _db.addSpaceToken(token.getHostDevice().getTag(), token.getOwner().getID(), token.getID(), null, 
			      token.getTokenAttributes().getTotalSpaceBytes(), 
			      token.getTokenAttributes().getGuaranteedSpaceBytes(), token.getTokenAttributes().getLifeTimeInSeconds());
	    //_db._aux.listToken(token.getID());
	    //_db.updateSpaceToken(token.getID(), -1, -1, 3333);
	    //_db._aux.listToken(null);
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "addTokenFailed,id="+token.getID(), e);
	}
    }

    public void fileIsAdded(TSRMLocalFile f, long size, TSRMStorage.iSRMSpaceToken token) 
    {
	System.out.println(".................................. file is added, is reading?="+_isReading+" token?"+ f.getToken());
	if (_isReading) {
	    return;
	}

	if (f.getToken() != null) {
	    addFile(f);
	}
    }

    public void tokenRemoved(TSRMStorage.iSRMSpaceToken token)
    {
	if (_isReading) {
	    return;
	}

	if (token == null) {
	    return;
	}
	try {
	    _db.deleteSpaceToken(token.getID());
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "deleteTokenFailed,id="+token.getID(), e);
	}
    }

    public void tokenExpired(String tid) {
	try {
	    _db.deleteSpaceToken(tid);
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "deleteTokenFailed,id="+tid, e);
	}	
    }

    public void removeEntry(TSRMLocalFile f) {
	if (_isReading) {
	    return;
	}
	if (f == null) {
	    return;
	}
	try {
	    if (f.getToken() != null) {
		_db.fileWithSpace(f.getCanonicalPath(), f.getToken().getID(), false);
	    }
	    _db.rmFile(f.getCanonicalPath());
	} catch (Exception e) {
	    TSRMLog.info(this.getClass(), null, "event=failedRmFile", "path="+f.getCanonicalPath());
	    e.printStackTrace();
	    return;
	}
    }

    public boolean removeFile(TSRMLocalFile f) {
	removeEntry(f);
	TSRMNameSpace.removeFile(f);
	return true;
    }

    public void addFile(TSRMLocalFile f) {
	if (_isReading) {
	    return;
	}
	try {
	    _db.createFile(f.getCanonicalPath(), f.isSurl()); 

	    if (f.getToken() == null) {
		return;
	    }
	    _db.fileWithSpace(f.getCanonicalPath(), f.getToken().getID(), true);

	    Hashtable ht = new Hashtable();
	    ht.put("size", f.getCurrentSize());
	    if (f.getAttr().getCreatedAt() > 0) {
		ht.put("createdAtTime", new java.sql.Timestamp(f.getAttr().getCreatedAt()));	  
	    }
	    //ht.put("lastModificationTime", new java.sql.Timestamp(678910));
		
	    ht.put("lifetimeassigned", new Integer((int)(f.getLifeTimeRemainingInSeconds()+f.getLifeTimeUsedInSeconds())));
	    ht.put("lifetimeEndsAt", new Long(f.getExpirationTimeInMilliSeconds()));
	    ///// the attributes below shall be fixed and no need to override??
	    ht.put("filelocality", f.getLocality());
	    ht.put("fileStorageType", f.getFileStorageType());
	    if (f.getToken() != null) {
		ht.put("retentionpolicy", f.getToken().getType().getRetentionPolicy());
	    }
	    ht.put("accesslatency", f.getLatency());
	    ht.put("isReady", new Boolean(f.isReady()));
	    if (f.getPhysicalPath() != null) {
		ht.put("physicalPath", f.getPhysicalPath().toString());
	    }
	    //ht.put("type", ); //type is taken care when createFile() is called
	    //ht.put("checksumtype", );
	    //ht.put("checksumvalue", );

	    _db.updateFileAttributes(f.getCanonicalPath(), ht);
	} catch (Exception e) {
	    TSRMLog.info(this.getClass(), null, "event=failedAddFile", "path="+f.getCanonicalPath());
	    e.printStackTrace();
	    return;
	}
    }

    public boolean readLogEnds() { 	
	try {
	    int dbDeviceCounter = _db.getDeviceCounter();
	    System.out.println(".....................db devices:"+dbDeviceCounter+"  here:"+_deviceCounter);
	    if (dbDeviceCounter > _deviceCounter) {	   
		TSRMUtil.startUpInfo("devices in db is more than what is in the config file. Exit");
		System.exit(1);
	    }
	    _isReading = true;
	    TSRMCacheLogFootprintReader r = new TSRMCacheLogFootprintReader(_deviceLookup);
	    r.reengineer();
	    
	    _isReading = false;
	} catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}
	return true;
    }    
}


//
// read back from db
//
class TSRMCacheLogFootprintReader {
    Statement _statement = TSRMServer.getFootprintDB().getStatement();
    HashMap _deviceLookup = null;
    HashMap _accountLookup = new HashMap();

    TSRMCacheLogFootprintReader(HashMap devices) 
    {
	_deviceLookup = devices;
    }	

    public boolean reengineer() {
	reengineerAccounts();	

        reengineerTokens();

	boolean result = reengineerFiles();

	return result;
    }

    void reengineerAccounts() {
	ResultSet rs = null;
	Vector accts = new Vector();
	Vector acctIds = new Vector();
	
	try {
	    rs = _statement.executeQuery("SELECT uid, u_id from User");
	    while(rs.next()) {
		String curr = rs.getString(1);
		accts.add(curr);
		acctIds.add(rs.getString(2));
	    }
	    rs.close();

	    for (int i=0; i<accts.size(); i++) {
		String curr = (String)(accts.get(i));
		System.out.println("...............??"+curr+"    "+acctIds.get(i));
		if (!curr.equals(TAccountManager._SUPER_USER.getID())) {		    
		    _accountLookup.put(acctIds.get(i), TAccountManager.createAccount(curr));
		} else {
		    _accountLookup.put(acctIds.get(i), TAccountManager._SUPER_USER);
		}
	    }
	} catch (java.sql.SQLException e) {
	    TSRMLog.exception(this.getClass(), "listUsers", e);
	    throw new TSRMException("failed to call db for listing users", false);
	}	
    }

    public boolean reengineerTokens() {
	ResultSet rs = null;
	Vector v = new Vector();

	try {
	    rs = _statement.executeQuery("SELECT spaceToken from SpaceAssignment");
	    while(rs.next()) {
		String curr = rs.getString(1);
		v.add(curr);
	    }
	    rs.close();

	    for (int i=0; i<v.size(); i++) {
		String curr = (String)(v.get(i));
		establishToken(curr);
	    }
	} catch (java.sql.SQLException e) {
	    TSRMLog.exception(this.getClass(), "listTokens", e);
	    throw new TSRMException("failed to call db for listing tokens", false);
	} catch (TSRMException e) {
	    e.printStackTrace();
	    System.exit(1);
	}

	
	//
	// double check the consistency of tids here and max in the system.
	// i.e. donnt want cache log has token (v0, v2), and max in the system is 2. which 
	// will generate next token as v2 and causes conflict.
	//
	if (v.size() > 0) {
	    Object[] tidSet = v.toArray();	
	    TSRMUtil.sync(tidSet);
	}
	return true;
    }
    
    private ResultSet getDBTokenInfo(String tid) {
	ResultSet rs = null;
	try {
	    String[] attrList = {"SpaceAssignment.componentId", "SpaceAssignment.userId",
				 "SpaceAssignment.totalSize", "SpaceAssignment.lifeTimeAssigned", 
				 "SpaceAssignment.lifeTimeStart", "SpaceAssignment.userSpaceTokenDesc", 
				 "Component.ComponentName"};
	    String where = "SpaceAssignment.spaceToken = "+tid;
	    String sql = TSRMFootprintUtil.generateSqlSelect("SpaceAssignment, Component", attrList, "SpaceAssignment.spaceToken", tid);
	    rs = _statement.executeQuery(sql);
	    if (!rs.next()) {
		// not supposed to be here!
		return null;
	    }
	    return rs;
	} catch (java.sql.SQLException e) {
	    e.printStackTrace();
	    return null;
	}
    }

    private void establishToken(String tid) {
	TSRMLog.info(this.getClass(), null, "event=establishToken", "value="+tid);

	ResultSet rs = getDBTokenInfo(tid);
	if (rs == null) {
	    return;
	}
	try {	    
	    String deviceTag = rs.getString(7);
	    if (deviceTag == null) {
		rs.close();
		throw new TSRMException("Unexpected inconsistency in token. Device info is not avaialble for "+tid, false);
	    }

	    TBasicDevice device = (TBasicDevice)(_deviceLookup.get(deviceTag));
	    if (device == null) {
		rs.close();
		throw new TSRMException("Unexpected inconsistency in token. Device is not recognized for "+tid, false);
	    }
 
	    TSRMSpaceType spaceType = TSRMSpaceType.getSpaceType(tid);		
	    if (spaceType == null) {	       
		rs.close();
		throw new TSRMException("Unexpected inconsistency in token. Space type is not supported for "+tid, false);
	    }

	    String accountOid = rs.getString(2);
	    long size = rs.getLong(3);	    
	    long durSeconds = rs.getLong(4);
	    
	    long creationTimeMilliSeconds = rs.getTimestamp(5).getTime();
	    String desc = rs.getString(6);
	    rs.close();	    

	    TAccount user = (TAccount)(_accountLookup.get(accountOid)); //TAccountManager.getAccount(accountName, true);
	    if (user == null) {
		throw new TSRMException("Unknown user in db with u_id="+accountOid, false);
	    }
	    System.out.println(".........user="+user.getID());

	    if (TSRMCacheLog.isExpired(creationTimeMilliSeconds, durSeconds)) {
		TSRMUtil.startUpInfo("... <<<<<< >>>>>>>detected expired token: "+tid);
		System.out.println(" time to remove the token and its files?");
		// removed token
		TSRMServer.getFootprintDB().deleteTokenForce(tid);
	    }

	    recreateToken(spaceType, size, desc, tid, user, durSeconds, creationTimeMilliSeconds, device);

	} catch (java.sql.SQLException e) {
	    e.printStackTrace();
	} 
    }

    private void recreateToken(TSRMSpaceType spaceType, long size, String desc, String tid,
			       TAccount user, long durSeconds, long creationTimeMilliSeconds,
			       TBasicDevice device) 
    {
	TTokenAttributes tokenAttr = new TTokenAttributes(spaceType, size);
	tokenAttr.setLifeTimeInSeconds(durSeconds);
	tokenAttr.setTokenID(tid);		    
	
	if ((desc != null) && (desc.length() > 0)) {
	    tokenAttr.setUserDesc(desc);		
	}
	TSRMStorage.iSRMSpaceToken token = device.getToken(tid);		    
	if (token == null) {
	    token = device.accomodate(tokenAttr);
	    if (token != null) {
		token.setOwner(user);	
		if (durSeconds > 0) {
		    token.setCreationTimeInMilliSeconds(creationTimeMilliSeconds);
		}
	    } else {
		TSRMUtil.startUpInfo("!!!Cannt accomodate token: "+tid+" in: "+device.getTag());
	    }
	}

	TSRMServer.getFootprintDB().cacheToken(tid, token);	
    }

    public boolean reengineerFiles() {
	
	boolean result = true;
	/*
	Set tids = _fileLookup.keySet();
	Iterator iter = tids.iterator();

	while (iter.hasNext()) {
	    String tid = (String)(iter.next());
	    HashMap files = (HashMap)(_fileLookup.get(tid));

	    TSRMUtil.startUpInfo("token id="+tid+"  => "+_tokenLookup.get(tid));

	    TSRMStorage.iSRMSpaceToken token = (TSRMStorage.iSRMSpaceToken)(_tokenLookup.get(tid));
	    if (token == null) {
		// something is wrong, this token doesnt exist. cannt do anything;
		removeRelatedFiles(files);
		continue;
	    }

	    //
	    // we dont bother whether the token is expired or not..user can extend the lifetime
	    //
	    Object[] strArray = files.values().toArray();
	    for (int i=0; i<files.size(); i++) {
		String desc = (String)(strArray[i]);

		TSRMLocalFile file = recreateFile(token, desc.split(TSRMCacheLog._seperator));
		
		if (file == null) {
		    TSRMUtil.startUpInfo("==> "+desc+"==> null");
		} else {
		    TSRMUtil.startUpInfo("==> "+desc+"==> "+file.getName());
		    if (!file.isSurl()) {
			TSRMNameSpace.addEntry(file.getSourceURI().toString());
			TSRMNameSpace.addEntry(file.getSourceURI().toString(), file, null);
		    } else {
			TSRMNameSpace.addEntry(file.getSiteURL().getSURL().toString());
			TSRMNameSpace.addEntry(file.getSiteURL().getSURL().toString(), file, null);
		    }
		    TSRMLog.getCacheLog().addFile(file);
		}
	    }
	}
	*/
	return result;
    }

  
    /*
    public void removeRelatedFiles(HashMap files) {
	Object[] strArray = files.values().toArray();
	for (int i=0; i<files.size(); i++) {
	    String desc = (String)(strArray[i]);
	    String[] fileDesc = desc.split(TSRMCacheLog._seperator);
	    if (fileDesc.length >= 9) {
		String location = fileDesc[8];
		TSRMUtil.cleanFilePath(location);
	    }
	}
    }     
    */
    public  TSRMLocalFile recreateFile(TSRMStorage.iSRMSpaceToken token,
				       String[] fileDesc) 
    {	
	long bytes = Long.parseLong(fileDesc[4]);
		
	TSRMLog.debug(this.getClass(), null, "event=recreateFile", "name="+fileDesc[2]+" bytes="+bytes);
		     	
	TFileStorageType fileType = null;
	if (fileDesc.length >= 8) {
	    fileType = TFileStorageType.fromValue(fileDesc[7]);
	}

	TSRMLocalFile result = recreateFilePath(fileDesc[2], fileType, fileDesc[6]);	  	

	try {
	    if ((result.getToken() != null) && (result.getToken() !=token)) {
		removeTrace(result);
		return null;
	    }		   	    	    	    
	    
	    if (!TSRMCacheLog.readSourceURI(result, fileDesc[3])) {
		removeTrace(result);
		return null;
	    }

	    //result.setSourceURI(src.getURI());	    
	    token.findSpaceForFile(result, bytes);
	    /*	    
	    boolean sfnAssigned = Boolean.valueOf(fileDesc[5]).booleanValue();
	    
	    if (!sfnAssigned) {
		result.setPhysicalLocation(src, null);
	    } else {
		TSupportedURL sfn = TSupportedURL.create(result.getSiteURL());
		result.setPhysicalLocation(sfn, null);
		result.setSFNAppointedByUser();
	    }
	    */
	    if (fileDesc.length >= 9) {
		String location = fileDesc[8];		
		result.setPhysicalLocation(location, null);
	    }
	    
	    if (fileDesc.length >= 10) {
		long expTime = Long.parseLong(fileDesc[9]);		
		if ((expTime > 0) && (System.currentTimeMillis() > expTime)) {
		    TSRMUtil.startUpInfo("File is expired"+result.getCanonicalPath());
		    removeTrace(result);
		} else {
		    result.setInheritedLifeTime(expTime);
		}
	    }

	    long actualBytes = result.getCurrentSize();

	    if (actualBytes <= 0) {
		removeTrace(result);
		return null;
	    }
	    
	    if (actualBytes == bytes) { // ok, everything matches
		//result.setSourceURI(src.getSURL());	    
		//token.findSpaceForFile(result, bytes);	    
	    } else {
		TSRMLog.info(getClass(), null, "event=conflictedSizeFound action=removingFileFromDisk", "path="+result.getCanonicalPath());
		removeTrace(result);
		return null;
	    }
	
	    result.updateToken();
	    return result;
	} catch (Exception e) {
	    e.printStackTrace();
	    removeTrace(result);
	    return null;
	}
    }

    private void removeTrace(TSRMLocalFile f) {
	//f.deleteMe();
	f.unsetToken(false);

	TSRMUtil.wipe(f);
	/*
	TSRMLocalDir p = f.getParent();

	p.rmFileName(f.getName());

	while (p != null) {
	    if (p.isEmpty()) {		
		TSRMLocalDir p1 = p.getParent();
		if (p1 != null) {
		    p1.rmDirName(p.getName());
		}
		p = p1;
	    } else {
		break;
	    }
	}
	*/
    }

    private TSRMLocalFile recreateFilePath(String path, TFileStorageType fileType, String surlTag) {
	String slash = path.substring(0,1);
	String[] pathTokens = path.substring(1).split(slash);

	String uid = pathTokens[0];
	TAccount user = TAccountManager.getAccount(uid, true);

	if (user == null) {
	    TSRMLog.debug(this.getClass(), null, "error=recreatingFailed", "uid="+uid);
	    return null;
	}

	boolean isSurl = false;
	if (surlTag.equals("S")) {
	    isSurl = true;
	}

	TSRMLocalDir dir = user.getTopDir();

	for (int i=1; i<pathTokens.length-1; i++) {
	    String curr = pathTokens[i];
	    dir = dir.createDir(curr, isSurl);
	}

	return dir.createSubFile(pathTokens[pathTokens.length-1], fileType, isSurl);
    }

    private TBasicDevice getDevice(String tag, int sizeMB, TSRMSpaceType type) {
	TSRMStorage.iSRMSpace v = TSRMStorage.getSpaceManager(type);
	TBasicDevice result = null;
	if (v != null) {
	    result = v.getDevice(tag);
	}

	if (result == null) {
	    return null;	    
	}

	if (result.getTotalSizeInMB() <= sizeMB) {
	    return result;
	}
	return null;   
    }


    public  TBasicDevice getDevice(String tag, int sizeMB) {
	TSRMSpaceType type = TSRMSpaceType.Volatile;
	TBasicDevice result = getDevice(tag, sizeMB, type);
	if (result != null) {
	    return result;
	}

	type = TSRMSpaceType.Durable;
	result =getDevice(tag, sizeMB, type);
	if (result != null) {
	    return result;
	}

	type = TSRMSpaceType.Permanent;
	result = getDevice(tag, sizeMB, type);
	return result;
    }

    void printContents() {
	
	// now put the tokens to the devices
	TSRMUtil.startUpInfo("--------------------------- starts");
        /*TSRMUtil.startUpInfo("# of tokens: "+_tokenLookup.size());
	for (int i=0; i<_tokenLookup.size(); i++) {
	    TSRMUtil.startUpInfo((_tokenLookup.values().toArray())[i]);
	}
	// now put the files to the tokens
	TSRMUtil.startUpInfo("# of tokens that have files: "+_fileLookup.size());
	for (int i=0; i < _fileLookup.size(); i++) {
	    HashMap v = (HashMap)((_fileLookup.values().toArray())[i]);
	    TSRMUtil.startUpInfo("==========");
	    for (int j=0; j < v.size(); j++) {
		 TSRMUtil.startUpInfo("\t"+(v.values().toArray())[j]);
	    }
	    } */
	TSRMUtil.startUpInfo("--------------------------- ends");	
    }
}

