/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/


package gov.lbl.srm.db;

import java.sql.*;

import java.util.*;

import gov.lbl.srm.StorageResourceManager.*;
import gov.lbl.srm.StorNet.TBandwidthResponseParameters;
import gov.lbl.srm.StorNet.TResponse;
import org.apache.axis.types.URI;
import gov.lbl.srm.util.*;
import gov.lbl.srm.storage.*;
import gov.lbl.srm.server.*;
//import gov.lbl.srm.server.TSRMLocalDir;
//import gov.lbl.srm.server.StaticToken;
import gov.lbl.srm.server.Config;

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//TSRMFootprintDB
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public class TSRMFootprintDB
{
  private static String _DefDBMode = "embedded";

  private Connection _conn = null;
  private Statement _statement = null; 

  private boolean _debug;
  private TDBDriverCode _driver = TDBDriverCode.H2_DRIVER;

  private TDBProtocol _protocol = TDBProtocol.H2_PROTOCOL;

  protected String _dbName="bestman3";
  protected String _dbUser = "bestman";
  protected String _password = "bestman#1";
  
  private boolean _isConnectionOpen=false;

    public TSRMFootprintDBDoubleChecker _aux= null;
    private HashMap _cachedTokens = new HashMap();

    public static final String _DefTableBandwidth = "BandwidthCommited";    
    public static final String _DefTableComp = "Component";
    public static final String _DefTableDir = "Directory";    
    public static final String _DefTableOpDir = "DirOption";        
    public static final String _DefTableFileAttr = "FileAttributes";
    public static final String _DefTableFileSP = "FileAttributesSP";    
    public static final String _DefTableFile = "Files";    
    public static final String _DefTableRequest = "Request";    
    public static final String _DefTableFileRequest = "RequestExecution";    
    public static final String _DefTableLsInput = "RequestLevelInputForLs";        
    public static final String _DefTableSpace = "SpaceAssignment";            
    public static final String _DefTableTxfParameter = "TransferParameter";
    public static final String _DefTableUser = "User";
    
    public static final String _DefTableReqInputCopy = "RequestLevelInputForCopy";
    public static final String _DefTableReqInputGet  = "RequestLevelInputForGet";
    public static final String _DefTableReqInputPut  = "RequestLevelInputForPut";

  GeneralHashFunctionLibrary _hash = new GeneralHashFunctionLibrary();

  private long getHashValue(String v) {
    return _hash.RSHash(v);
  }

    public void cacheToken(String tid, TSRMStorage.iSRMSpaceToken token) {
	if (token != null) {
	    _cachedTokens.put(tid, token);
	}
    }

    public TSRMStorage.iSRMSpaceToken getCachedToken(String tid) {
	return (TSRMStorage.iSRMSpaceToken)(_cachedTokens.get(tid));
    }
  //
  // 
  //
  public TSRMFootprintDB(String dbName, String dbUser, String pwd, boolean debug) {
    _debug = debug;
        
    if((dbName != null) && (dbName.length()>0)) {
	_dbName = dbName;
    } 
    
    if((dbUser != null) && (dbUser.length()>0)) {
	_dbUser = dbUser;
    }
    
    if((pwd != null) && (pwd.length()>0)) {
	_password = pwd;
    }  
  }
    
    private void addToHashMap(HashMap container, String v1, String v2) {
	if (v2 != null) {
	    container.put(v1, v2);
	}
    }

    private void addToHashMap(HashMap container, String v1, Object v2) {
	if (v2 != null) {
	    container.put(v1, v2.toString());
	}
    }

    private void addToHashMap(HashMap container, TRetentionPolicyInfo rpi) {		
	if (rpi == null) {
	    return;
	}
	TRetentionPolicy retentionPolicy = rpi.getRetentionPolicy(); 	
	TAccessLatency accessLatency = rpi.getAccessLatency();

	addToHashMap(container, "targetFileRetentionPolicy", retentionPolicy);	
	addToHashMap(container, "targetFileAccessLatency", accessLatency);
    }

    
  public TSRMFootprintDB(TDBDriverCode driver, boolean debug) {
    _debug = debug;    
    if (driver != null) {
      _driver = driver;
    } else {
      TSRMLog.debug(this.getClass(), null, "event=defaultDriverApplied", "reason=nullDriverInput");
    }
  }

  public String getDesc() {
    String[] desc = {"driver=", _driver.getName(), " protocol=", _protocol.getValue(),
		     " _dbName=", _dbName, " _dbUser=", _dbUser, " debug="+_debug};
    return TSRMFootprintUtil.appendString(desc);
  }

  private int assertCompOid(String name, String evt) {
      int result = findCompOid(name, evt);

      if (result == -1) {
	  throw new TSRMException("Comp does not exist in the user table. Value= " + name, false);
      }
      return result;
  }

  private int findCompOid(String name, String evt) {
    TSRMLog.info(this.getClass(), null, evt, "action=findCompOid name="+name);
    int result = -1;

    try {
	ResultSet rs = _statement.executeQuery(TSRMFootprintUtil.generateSqlSelect(_DefTableComp, "componentId", "componentName", name));

	if(rs.next()) {
	   result = rs.getInt(1);
	   rs.close();
	   TSRMLog.info(this.getClass(), null, evt, "oidFound="+result);
	   return result;
	} else {
	   rs.close();	    
	   TSRMLog.error(this.getClass(), null, evt, "error=NoComp:"+name);
	   return -1;	   
      }
    } catch (java.sql.SQLException e) {
       TSRMLog.exception(this.getClass(), evt, e);
       throw new TSRMException("failed to find comp:"+name, false);
    }       
  } 
  
  private int assertUserOid(String uid, String evt) {
      int result = findUserOid(uid, evt);

      if (result == -1) {
	  throw new TSRMException("User does not exist in the user table: " + uid, false);
      }
      return result;
  }

  private int findUserOid(String uid, String evt) {
    TSRMLog.info(this.getClass(), null, evt, "action=findUserOid uid="+uid);
    int result = -1;

    try {
	ResultSet rs = _statement.executeQuery(TSRMFootprintUtil.generateSqlSelect(_DefTableUser, "u_id", "uid", uid));
      
      if(rs.next()) {
	result = rs.getInt(1);
	rs.close();
	TSRMLog.info(this.getClass(), null, evt, "oidFound="+result);
	return result;
      } else {
	rs.close();
	TSRMLog.error(this.getClass(), null, evt, "error=NoUid:"+uid);
	return -1;
      }
    } catch (java.sql.SQLException e) {
       TSRMLog.exception(this.getClass(), evt, e);
       throw new TSRMException("failed to find user:"+uid, false);
    }       
  } 

  private boolean checkExistenceSpaceToken(String tid) {
    String evt = "checkExistenceSpaceToken";
    TSRMLog.info(this.getClass(), null, evt, "action=checkSpaceToken token="+tid);
    boolean  result = false;

    try {
	ResultSet rs = _statement.executeQuery(TSRMFootprintUtil.generateSqlSelect(_DefTableSpace, "spaceToken", "spaceToken", tid));
	if (rs.next()) {
	    result = true;
	}
	rs.close();
    } catch (java.sql.SQLException e) {
	TSRMLog.exception(this.getClass(), evt, e);       
    }
    TSRMLog.info(this.getClass(), null, evt, "result="+result);
    return result;
  }

  public Statement getStatement() {
      return _statement;
  }

  public void initiate (TDBProtocol protocol, String dbPath)			
  {   
      if (protocol != null) {
	  _protocol = protocol;
      }
      
      TSRMLog.info(TSRMFootprintDB.class, null,"event=init", "mode="+_DefDBMode);
      TSRMLog.info(this.getClass(), null, "event=DBConf", getDesc()); 
      
      if (!loadDriver()) {
	  System.exit(1);
      }
      
      _protocol.init(dbPath);
      _conn = _protocol.getConnection(this); 
      if (_conn == null) {
	  System.err.println("Exiting BeStMan. Reason: Unable to connect to:"+_dbName);
	  System.exit(1);
      }
      
      try {
	  TSRMLog.info(this.getClass(), null, "event=connected", "db=_dbName");
	  
	  // We want to control transactions manually. Autocommit is on by
	  // default in JDBC.
	  _conn.setAutoCommit(false);
	  
	  //Creating a statement object that we can use for running various
	  //SQL statements commands against the database.
	  //
	  _statement = _conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
					     ResultSet.CONCUR_READ_ONLY);            
	  
	  if (!checkSchemaExistence()) {
	      createTables();
	  }
	  _aux = new TSRMFootprintDBDoubleChecker(_conn.createStatement());
      } catch (Exception e) {     
	  TSRMLog.exception(TSRMFootprintDB.class,"initiate",e);
	  //throw new TSRMException(sqle.getMessage(), false);
	  closeConnection();
	  System.exit(1);
      } 
      _isConnectionOpen=true;
  
      TSRMLog.info(TSRMFootprintDB.class, null, "event=init", "result=success");    
      
      _aux.listDevices();
  }
    
    private void storeInput(SrmLsRequest lsRequest, String rid, String uid) {
	String evtName = "storeLsInput";
	String authID = lsRequest.getAuthorizationID();
	
	String ssiStr = TSRMFootprintUtil.getStorageSystemInfoString(lsRequest.getStorageSystemInfo());
	boolean fullDetailedList = TSRMFootprintUtil.getBoolean(lsRequest.getFullDetailedList(), false);
	boolean allLevelRecursive = TSRMFootprintUtil.getBoolean(lsRequest.getAllLevelRecursive(), false);
	
	int numOfLevels = TSRMFootprintUtil.getInt(lsRequest.getNumOfLevels(), 0);
	int offset = TSRMFootprintUtil.getInt(lsRequest.getOffset(), 0);
	int count = TSRMFootprintUtil.getInt(lsRequest.getCount(), 0);
	
	ResultSet rs = null;  
	
	//insert into RequestLevelInputTable     
	String[] attrs = {"rid", "storageSystemInfo", "fullDetailedList", "allLevelRecursive", "numOfLevels", "offset", "count"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableLsInput, attrs);
	
	String[] strV = {rid, ssiStr};
	boolean[] booleanV = {fullDetailedList, allLevelRecursive};
	int[] intV ={numOfLevels, offset, count};
	
	TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, booleanV, intV, null, null);
    }
    
    private void storeRequest(SrmLsRequest lsRequest, String rid, int u_id) {
	String evtName="storeLsRequest";
	
	//insert into RequestTable 
	ResultSet rs = null;  
	ArrayOfAnyURI arrayOfAnyURI = lsRequest.getArrayOfSURLs();
	URI[] uris = arrayOfAnyURI.getUrlArray();
	int totalNumFilesInRequest = uris.length;    
	String authID = lsRequest.getAuthorizationID();
	String fileType = null;
	if (lsRequest.getFileStorageType() != null) {
	    fileType = lsRequest.getFileStorageType().getValue();
	}
	
	long ttime = System.currentTimeMillis();
	java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(ttime);
	
	String[] attrs = {"rid", "authID", "requestLevelInputId", "fileType", "requestType", "status", 
			  "explanation", "bandwidthStatus", "userId", "totalNumFilesInRequest", 
			  "createdTime", "statusTime"};
	
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableRequest, attrs);
	
	String[] strV = {rid, authID, rid, fileType, "LS", "SRM_REQUEST_QUEUED", null, "SRM_REQUEST_QUEUED"};
	boolean[] booleanV = null;
	int[] intV ={u_id, totalNumFilesInRequest};
	Timestamp[] tsV={queuedTimeStamp, queuedTimeStamp};
	
	TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, booleanV, intV, tsV, null);
    }
    
    private void storeExec(SrmLsRequest lsRequest, String rid) throws Exception /*called from the debug part*/ 
    {
	String evtName="storeLsExec";
	ResultSet rs = null;  
	ArrayOfAnyURI arrayOfAnyURI = lsRequest.getArrayOfSURLs();
	URI[] uris = arrayOfAnyURI.getUrlArray();
        
	for(int i = 0; i < uris.length; i++) {	    
	    long tt = System.currentTimeMillis();
	    java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(tt);
	    
	    String[] attrs = {"rid", "fromSURL", "status",  "pos", "statusTime"};
	    
	    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableFileRequest, attrs);
	    
	    String[] strV = {rid, uris[i].toString(),  "SRM_REQUEST_QUEUED"};
	    int[] intV={i};
	    Timestamp[] tsV={queuedTimeStamp};
	    
	    TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, tsV, null);      
	}  
    }
    //::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    //Directory related Api's
    //::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    
    public void store(SrmLsRequest lsRequest, String rid, String uid) throws Exception       
    {
	String evtName="event=storeLs";
	TSRMLog.info(TSRMFootprintDB.class,null, evtName+"Starts", "uid="+uid+" rid="+rid);
	
	TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
	TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
	
	int u_id = assertUserOid(uid, evtName);
	
	storeInput(lsRequest, rid, uid);
	storeRequest(lsRequest, rid, u_id);

	ArrayOfAnyURI arrayOfAnyURI = lsRequest.getArrayOfSURLs();
	URI[] uris = arrayOfAnyURI.getUrlArray();
        
	for(int i = 0; i < uris.length; i++) {
	    storeFileReqCommon(rid, i, uris[i], null, null, evtName, null);
	}
	//storeExec(lsRequest, rid);
	
	TSRMLog.info(TSRMFootprintDB.class,null,evtName+"Ends", null);
    }
    
    private Vector getSpaceTokens(int fileAttributesId) throws Exception {
	_aux.listFiles();
	_aux.listFileAttrs(-1);
	_aux.listTokenAssignment();
	
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "spaceToken", "fileAttributesId="+fileAttributesId);
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	try {
	    Vector vec = new Vector ();
	    while(rs.next()) {
		vec.addElement(rs.getString(1));
	    }
	    return vec;
	} finally {
	    rs.close();
	}
    }


  private int getFileAttrID(String path) throws Exception 
  {
      long hashValue = getHashValue(path);
      return getFileAttrID(hashValue);
  }

  private int getFileAttrID(long hashValue) throws Exception 
  {
      ResultSet rs = getFileRecord(hashValue);
      try {
	  if (rs.next()) {
	      return rs.getInt(1);
	  }
	  return -1;
      } finally {
	  rs.close();
      }
  }

  private ResultSet getFileRecord(String name) throws Exception 
  {
    long hashValue = getHashValue(name);
    return getFileRecord(hashValue);
  }

  private ResultSet getFileRecord(long hashValue) throws Exception 
  {
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileAttr, "fileAttributesId", "hashValue="+hashValue);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    return rs;
  }
  
  private ResultSet getDirRecord(String name) throws Exception 
  {    
    //GeneralHashFunctionLibrary ghl = new GeneralHashFunctionLibrary();
    long hashValue = getHashValue(name);

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirId", "hashValue="+hashValue);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
					   
    return rs;
  }


    private int getFileId(int dirID, String filename) throws Exception 
    {    
	//GeneralHashFunctionLibrary ghl = new GeneralHashFunctionLibrary();
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFile, "fileAttributesId", "fileName=\'"+filename+"\' and "+"dirId="+dirID);
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	if (rs.next()) {
	    int id = rs.getInt(1);
	    rs.close();	
	    return id;
	} 
	return -1;      
    }

  private int getDirId(String name) throws Exception 
  {    
    //GeneralHashFunctionLibrary ghl = new GeneralHashFunctionLibrary();
    long hashValue = getHashValue(name);

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirId", "hashValue="+hashValue);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
    //ResultSet rs = _statement.executeQuery("SELECT dirId,fileAttributesId from Directory where hashValue="+hashValue);
	
    if (rs.next()) {
	int id = rs.getInt(1);
	rs.close();	
	return id;
    } 
    return -1;      
  }

  public boolean hasSubDir(String curr, String subdir) {
      try {
	  String name = null;
	  if (curr.endsWith("/")) {
	      name = curr + subdir;
	  } else {
	      name = curr + "/" + subdir;
	  }
	  if (!name.endsWith("/")) {
	      name +="/";
	  }
	  int result = getDirId(name);
	  TSRMLog.info(this.getClass(), null, "event=hasSubDir", "value="+name+" result="+result);
	  if (result > -1) {
	      return true;
	  }
      } catch (Exception e) {
	  TSRMLog.exception(this.getClass(), "details:", e);
      }
      return false;
  }

  public boolean hasSubFile(String curr, String subfile) {
      String evtName = "event=hasSubFile";
      try {
	  TSRMLog.info(this.getClass(), null, evtName, "value="+curr+subfile);
	  int result = getDirId(curr);
	  if (result < 0) {
	      return false;
	  }
	  result = getFileId(result, subfile);
	  TSRMLog.info(this.getClass(), null, evtName, "result="+result);
	  if (result > -1) {
	      return true;
	  }
      } catch (Exception e) {
	  TSRMLog.exception(this.getClass(), "details:", e);
      }
      return false;
  }

  public TSRMLocalFile getSubFile(TSRMLocalDir dir, String subfile) {
      String curr = dir.getCanonicalPath();
      String evtName = "event=getSubFile";
      try {
	  TSRMLog.info(this.getClass(), null, evtName, "value="+curr+subfile);
	  int result = getDirId(curr);
	  if (result < 0) {
	      return null;
	  }
	  result = getFileId(result, subfile);

	  TSRMLog.info(this.getClass(), null, evtName, "result="+result);
	  if (result > -1) {
	      TSRMLocalFile f = new TSRMLocalFile(dir, subfile, null, true);
	      Vector v = getSpaceTokens(result);
	      for (int i=0; i<v.size(); i++) {
		  System.out.println("...listing token: "+i+"/"+v.size()+" => "+v.get(i));
	      }
	      f.setToken(getCachedToken((String)(v.get(0))));	      
	      //boolean isReady = isFileReady(result);

	      boolean isReady = false;
	      long size = -1;
	      String loc = null;
	      long expTime = -1;

	      String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileAttr, 
							       "isReady, size, physicalPath, lifeTimeEndsAt", 
							       "fileAttributesId = "+result);
	      ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	      try {
		  if (rs.next()) {
		      isReady = rs.getBoolean(1);
		      size = rs.getLong(2);
		      loc = rs.getString(3);
		      expTime = rs.getLong(4);
		  }
	      } finally {
		  rs.close();
	      }	
	      
	      if (isReady) {
		  f.setPhysicalLocation(loc, null);
		  f.setInheritedLifeTime(expTime);
		  System.out.println("........................is file expired?"+(System.currentTimeMillis() - expTime));
		  f.getToken().findSpaceForFile(f, size);
		  f.updateToken();
	      }
	      ///////////////////f.setReady();
	      return f;
	  }
      } catch (Exception e) {
	  TSRMLog.exception(this.getClass(), "details:", e);
      }
      return null;
  }


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//createMetaDataPathDetail
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private TMetaDataPathDetail createMetaDataPathDetail (ResultSet rs, String path, int fileAttributesId) 
  throws Exception 
{	
   String[] strArray = {"path=", path, " fileAttributeId="+fileAttributesId};
   String evtName = "event=createMetaDataPathDetail";
   TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));

   TMetaDataPathDetail metaDataPathDetail = new TMetaDataPathDetail();     	  
   metaDataPathDetail.setPath(path);
   metaDataPathDetail.setSize(new org.apache.axis.types.UnsignedLong(rs.getLong(1)));
 	 
   metaDataPathDetail.setCreatedAtTime (TSRMFootprintUtil.createCalendar(rs.getTimestamp(2)));   
   metaDataPathDetail.setLastModificationTime (TSRMFootprintUtil.createCalendar(rs.getTimestamp(3)));   

   String fSType = rs.getString(4);
   if(fSType != null) {
     metaDataPathDetail.setFileStorageType(TFileStorageType.fromValue(fSType));       
   }

   TRetentionPolicyInfo retentionPolicyInfo = TSRMFootprintUtil.getRetentionPolicyInfo(rs.getString(5), rs.getString(6));
   metaDataPathDetail.setRetentionPolicyInfo(retentionPolicyInfo);

   String fileLocality = rs.getString(7);
   if(fileLocality != null) {
     metaDataPathDetail.setFileLocality(TFileLocality.fromValue(fileLocality));       
   }

   String fileType = rs.getString(8);
   if(fileType != null) {
     metaDataPathDetail.setType(TFileType.fromValue(fileType));
   }
   metaDataPathDetail.setLifetimeAssigned(new Integer(rs.getInt(9)));
   long lifetimeLeft = rs.getLong(10)-System.currentTimeMillis();
   if (lifetimeLeft > 0) {
       metaDataPathDetail.setLifetimeLeft((int)(lifetimeLeft/1000));
   } else {
       metaDataPathDetail.setLifetimeLeft(-1);
   }

   metaDataPathDetail.setOwnerPermission(TSRMFootprintUtil.getOwnerPermission(rs.getString(11), rs.getString(12)));   
   metaDataPathDetail.setOwnerPermission(TSRMFootprintUtil.getOwnerPermission(rs.getString(13), rs.getString(14)));   
   
   String omode = rs.getString(15);
   if(omode != null) {
     metaDataPathDetail.setOtherPermission(TPermissionMode.fromValue(omode));
   }

   String checkSumType = rs.getString(16);
   if(checkSumType != null) {
     metaDataPathDetail.setCheckSumType(checkSumType);
   }

   String checkSumValue = rs.getString(17);
   if(checkSumValue != null) {
     metaDataPathDetail.setCheckSumValue(checkSumValue);
   }

   Vector vec = getSpaceTokens(fileAttributesId);   
   if (vec.size() > 0) {   
       String[] sTokens = new String[vec.size()];
       for(int i = 0; i < vec.size(); i++) {
	   sTokens[i] = (String)vec.elementAt(i);
       }
       metaDataPathDetail.setArrayOfSpaceTokens(TSRMUtil.convertToArray(sTokens));
   } else {       
       metaDataPathDetail.setArrayOfSpaceTokens(null);
   }

   TSRMLog.info(TSRMFootprintDB.class,null, evtName+"Ends",TSRMFootprintUtil.appendString(strArray));
	
   metaDataPathDetail.setStatus(TSRMUtil.createReturnStatus(TStatusCode.SRM_SUCCESS, "read back from db"));
   return metaDataPathDetail;
}

//:::::::::: :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateSubDirs
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void updateFileAttrTbl(String p, long hashValue, int fileAttributesId) throws Exception 
{
    String[] attrs1 = {"path", "hashValue"};	
    String stmt1 = TSRMFootprintUtil.generateSqlUpdate(_DefTableFileAttr, attrs1, "fileAttributesId"); 
    PreparedStatement psUpdate = _conn.prepareStatement(stmt1);
    
    psUpdate.setString(1,p);
    psUpdate.setLong(2,hashValue);
    psUpdate.setInt(3,fileAttributesId);
    psUpdate.executeUpdate();
    _conn.commit();
}
	
private void updateSubDirs (String source, String target, String match) throws Exception 
{
    String evtName = "updateSubDirs";
    String strArray[] = {"source=", source, " target=", target, " match=", match};
    TSRMLog.info(TSRMFootprintDB.class,null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    long hashValue = getHashValue(source);
    ResultSet rs = _statement.executeQuery("SELECT dirId,fileAttributesId from Directory where hashValue = "  + hashValue);
  
    if (rs.next()) {
	int sourceDirId = rs.getInt(1);
	int fileAttributesId = rs.getInt(2);
	String temp2 = source.substring(match.length());
	String tt = target+temp2; 
	
	hashValue = getHashValue(tt);
	
	String[] attrs0 = {"dirEntry", "hashValue"};
	String stmt0 = TSRMFootprintUtil.generateSqlUpdate(_DefTableDir, attrs0, "dirId");
	PreparedStatement psUpdate = _conn.prepareStatement(stmt0);
		
	psUpdate.setString(1,tt);
	psUpdate.setLong(2,hashValue);
	psUpdate.setInt(3,sourceDirId);
	psUpdate.executeUpdate();
	_conn.commit();
		
	updateFileAttrTbl(tt, hashValue, fileAttributesId);
	
	rs.close();
	
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirEntry", "parentId="+sourceDirId);         
	Statement rrsstt = _conn.createStatement();
	ResultSet rrss = rrsstt.executeQuery(sql);
	
	while(rrss.next()) {
	    String dirEntry = rrss.getString(1);
	    updateSubDirs(dirEntry,target,match);
	}
	
	rrss.close();
	rrsstt.close();
    }
    rs.close();

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mv
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void alertDirExistence(String path, String evtName, String err) throws Exception {
    ResultSet rs = getDirRecord(path);
    try {
	if (rs.next()) {
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    throw new TSRMFootprintDBException(err + path);
	}
    } finally {
	rs.close();
    }
}


private ResultSet alertDirNonExistence(String path, String evtName, String err) throws Exception {
    ResultSet rs = getDirRecord(path);
    if (!rs.next()) {
      TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
      throw new TSRMFootprintDBException(err + path);
    }
    return rs;
}

private boolean mvDir(ResultSet rs, String source, String targetParentDir, String targetName) throws Exception{
     int sourceDirId = rs.getInt(1);
     int sourceFileAttributesId = rs.getInt(2);
     rs.close();

     String target = targetParentDir+"/"+targetName;
     alertDirExistence(target, "event=mvDir", "target already exists:");
     rs = alertDirNonExistence(targetParentDir, "event=mvDir", "target parent dir does not exist:");

     int targetParentDirId = rs.getInt(1);
     int targetParentFileAttributesId = rs.getInt(2);
     //check if target already exists 
     rs.close();

     String[] attrs0 = {"dirEntry", "hashValue", "parentId"};
     String stmt0 = TSRMFootprintUtil.generateSqlUpdate(_DefTableDir, attrs0, "dirId");

     PreparedStatement psUpdate = _conn.prepareStatement(stmt0);
     
     psUpdate.setString(1,target);
     long hashValue2 = getHashValue(target);
     psUpdate.setLong(2,hashValue2);
     psUpdate.setInt(3,targetParentDirId);
     psUpdate.setInt(4,sourceDirId);

     psUpdate.executeUpdate();
     _conn.commit();
     
     updateFileAttrTbl(target, hashValue2, sourceFileAttributesId);     

     Statement rrsstt = _conn.createStatement();
     String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirEntry, dirId", "parentId="+sourceDirId);
     ResultSet rrss = rrsstt.executeQuery(sql);
     
     while(rrss.next()) {
       String dirEntry = rrss.getString(1);
       int ddirId = rrss.getInt(2);
       updateSubDirs(dirEntry,target,source);
     }
     rrss.close();
     rrsstt.close();
     
     TSRMLog.info(TSRMFootprintDB.class, null, "event=mvCommitted", null);
     
     rs.close();
     return true;
  }           

public boolean mv(String source, String target) throws Exception {
    String evtName = "event=mv";
    String[] strArray = {"source=", source, " target=", target};
    TSRMLog.info(TSRMFootprintDB.class, null,"event=mvStarts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("source", source, evtName);
    TSRMFootprintUtil.noTrivialString("target", target, evtName);
    
    //check the target's parent in the directory table
    int idxT = target.lastIndexOf("/");
    if(idxT == -1) {
	TSRMLog.error(this.getClass(), null, evtName, "error=invalidTgtDir:"+target);
	return false;
    }
    
    String targetParentDir = target.substring(0,idxT);
    String targetName = target.substring(idxT+1);

    ResultSet rs = getDirRecord(source);
    
    if(rs.next()) {
	return mvDir(rs, source, targetParentDir, targetName);
    } else {
	//check in the files table
	ResultSet rst = getFileRecord(target);
	try {
	    if(rst.next()) {
		TSRMLog.error(TSRMFootprintDB.class,null,"event=mv", "msg=targetAlreadyExists");
		throw new TSRMFootprintDBException("Target already exists " + target); 
	    }
	} finally {
	    rst.close();
	}
	
	int idx = source.lastIndexOf("/");
	if(idx != -1) {
	    String path = source.substring(0,idx);
	    String fileName = source.substring(idx+1);
	    
	    rs = getDirRecord(path);
	    
	    if(rs.next()) {
		int dirId = rs.getInt(1);
		Statement stt = _conn.createStatement();
		ResultSet rs2 = stt.executeQuery("SELECT fileId,fileAttributesId from Files where dirId=" + dirId + 
						 " and fileName=" + "\'"+fileName+"\'");
		if(rs2.next()) { 
		    idx = target.lastIndexOf("/");
		    if(idx != -1) {
			int fileId = rs2.getInt(1);
			int fileAttributesId = rs2.getInt(2);
			String targetDir = target.substring(0,idx);
			String targetFileName = target.substring(idx+1);
			
			String[] stmt0 = {"update Files set fileName=? where ", " fileId=?;"};			      
			
			PreparedStatement psUpdate = _conn.prepareStatement(TSRMFootprintUtil.appendString(stmt0));
			
			psUpdate.setString(1,targetFileName);
			psUpdate.setInt(2,fileId);
			psUpdate.executeUpdate();
			_conn.commit();
			
			long targetHashValue = getHashValue(targetDir);
			updateFileAttrTbl(targetFileName, targetHashValue, fileAttributesId);
						
			TSRMLog.info(TSRMFootprintDB.class,"committed","event=mv", TSRMFootprintUtil.appendString(strArray));
			
			rs2.close();
			stt.close();
			rs.close();
			return true;
		    } else {		    
			TSRMLog.info(TSRMFootprintDB.class,"end","event=mv", TSRMFootprintUtil.appendString(strArray));
			return false;
		    }
		} else {
		    rs2.close();
		    stt.close();
		    rs.close();
		    String[] err ={"Given path does not exists in the directory table.", path, " ", fileName};
		    String[] errInfo = {" source=", source, " target=", target, " parent=", path, " filename=", fileName, TSRMFootprintUtil.appendString(err)};
		    TSRMLog.info(TSRMFootprintDB.class,"error","event=mv",TSRMFootprintUtil.appendString(err));
		    throw new TSRMFootprintDBException(TSRMFootprintUtil.appendString(err)); 
		}
	    } else {      
		rs.close();
		String[] err ={"Given path does not exists in the directory table.", path, " ", fileName};
		String[] errInfo = {" source=", source, " target=", target, " parent=", path, " filename=", fileName, TSRMFootprintUtil.appendString(err)};
		TSRMLog.info(TSRMFootprintDB.class,"error","event=mv",TSRMFootprintUtil.appendString(err));
		throw new TSRMFootprintDBException(TSRMFootprintUtil.appendString(err)); 
	    }
	} else {    
	    TSRMLog.info(TSRMFootprintDB.class,null,"event=mvEnds", TSRMFootprintUtil.appendString(strArray));
	    return false;
	}
	//return false;
    }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//list
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    private TMetaDataPathDetail getMetadataResult(int fid, String path) throws Exception {
	String[] attrList = {"size","createdAtTime","lastModificationTime","fileStorageType",
			     "retentionPolicy", "accessLatency", "fileLocality", "type", "lifeTimeAssigned",
			     "lifeTimeEndsAt", "ownerPermission", "userID", "groupPermission", "groupID",
			     "otherPermission", "checkSumType", "checkSumValue"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileAttr, attrList, "fileAttributesId="+fid);
	ResultSet rs= _statement.executeQuery(sql);
	
	try {
	    if (rs.next()) {
		TMetaDataPathDetail metaDataPathDetail = createMetaDataPathDetail(rs,path,fid);
		return metaDataPathDetail;
	    }
	    return null;
	} finally {
	    rs.close();
	}
    }
    				    				    				    				  
private TMetaDataPathDetail listDir(int dirID, int fileID, String dirName, int recursiveLevel) throws Exception {
    String evtName = "listDir";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "dirName="+dirName+"  dirId="+dirID);

    TMetaDataPathDetail detail = getMetadataResult(fileID, dirName);
    if (detail == null) {
	return TSRMUtil.createTMetaDataPathDetail(dirName, TSRMUtil.createReturnStatus(TStatusCode.SRM_INTERNAL_ERROR, "No info in db"));
    }

    Vector vec = new Vector ();
	
    String[] attrList = {"dirEntry", "fileAttributesId", "dirId"};
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, attrList, "parentID="+dirID);

    ResultSet rs = null;
    Statement currStmt = null;
    
    try {	
	currStmt =_conn.createStatement();
	rs = TSRMFootprintUtil.execSql(currStmt, sql);
	
	while(rs.next()) {
	    String subDirName = rs.getString(1);
	    int currFid = rs.getInt(2);

	    System.out.println(dirName+" has subdir:"+subDirName);
	    
	    if(recursiveLevel > 0) {
		int currDid = rs.getInt(3);
		TMetaDataPathDetail recDir = listDir(currDid, currFid, subDirName, recursiveLevel-1);

		if (recDir != null) {
		    vec.addElement(recDir);
		}
	    } else {	    
		TMetaDataPathDetail curr = getMetadataResult(currFid, subDirName);
		if (curr != null) {
		    vec.addElement(curr); 
		}
	    } 	
	}

	TSRMLog.info(TSRMFootprintDB.class, null, evtName+" status=subdirScanned", "dirName="+dirName+" vecSize="+vec.size());

	rs.close();
	//get the files from the file table
	//Statement st = _conn.createStatement();	
 _aux.listFiles();
 System.out.println(".... does the above has dirID="+dirID);
        rs= currStmt.executeQuery("SELECT fileName, fileAttributesId from Files where dirId="+dirID);
	
	while(rs.next()) {
	    String fileName = rs.getString(1);
	    int currFid = rs.getInt(2); 

	    System.out.println(dirName+" has subfile:"+fileName);
	    TMetaDataPathDetail currFile = getMetadataResult(currFid, dirName+fileName);
	    
	    if (currFile != null) {
		vec.addElement(currFile); 
	    }
	}
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+" status=subfileScanned", "dirName="+dirName+" vecSize="+vec.size());

	if (vec.size() > 0) {
	    TMetaDataPathDetail[] m = new TMetaDataPathDetail[vec.size()];
	    vec.toArray(m);       
	    detail.setArrayOfSubPaths(TSRMUtil.convertToArray(m));
	}
	return detail;
    } finally {
	rs.close();
	currStmt.close();
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "dirName="+dirName);
    }
}

public TMetaDataPathDetail listFile(String path, boolean recursive) throws Exception 
{
    String[] strArray = {"path=", path, " recursive="+recursive};
    String evtName = "listFile";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    try {
	TSRMFootprintUtil.noTrivialString("path", path, evtName);
	
	long hashValue = getHashValue(path);
	String[] attrList = {"dirId", "dirEntry", "fileAttributesId"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, attrList, "hashValue="+hashValue);
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	if (rs.next()) {	      
	    rs.close();
	    return TSRMUtil.createTMetaDataPathDetail(path, TSRMUtil.createReturnStatusInvalidPath(path+" is not a file, but a dir!"));
	}
	int fid = getFileAttrID(path);
	if (fid > -1) {
	    return getMetadataResult(fid, path);
	}
	// path does not exists
	return TSRMUtil.createTMetaDataPathDetail(path, TSRMUtil.createReturnStatusInvalidPath(path+" is not an existing path"));
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", TSRMFootprintUtil.appendString(strArray));
    }
}

public TMetaDataPathDetail listDir(String path, int recursiveLevel) throws Exception 
{
_aux.listDir();
    String[] strArray = {"path=", path, " recursiveLevel="+recursiveLevel};
    String evtName = "listDir";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    try {
	TSRMFootprintUtil.noTrivialString("path", path, evtName);
	
	long hashValue = getHashValue(path);
	String[] attrList = {"dirId", "dirEntry", "fileAttributesId"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, attrList, "hashValue="+hashValue);
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	if (rs.next()) {	      
	    int  dirID = rs.getInt(1);
	    int  fileID = rs.getInt(3);
	    String dirName = rs.getString(2);
	    rs.close();
	    return listDir(dirID, fileID, dirName, recursiveLevel);
	} 

	// path does not exists
	return TSRMUtil.createTMetaDataPathDetail(path, TSRMUtil.createReturnStatusInvalidPath(path+" is not an existing path"));
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", TSRMFootprintUtil.appendString(strArray));
    }
}

public boolean isDirEmpty(String name) throws Exception{
    int dirId = getDirId(name);
    return isDirEmpty(dirId);
}

private boolean isDirEmpty(int dirId) throws Exception {
    Statement stmt = _conn.createStatement();
    ResultSet rs = stmt.executeQuery("SELECT dirId from Directory where parentId=" + dirId);

    try {
	if (rs.next()) {
	    TSRMLog.info(this.getClass(), null, "dir=name", "result=subDirExists");
	    return false;
	}
	rs = stmt.executeQuery("SELECT fileName from Files where dirId="+ dirId);
	if (rs.next()) {
	    TSRMLog.info(this.getClass(), null, "dir=name", "result=subfileExists");
	    return false;
	}
	return true;
    } finally {
	rs.close();
    }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//rmDir
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean rmDir(String path, boolean recursive) throws Exception {
    String[] strArray = {"path=", path, " recursive="+recursive};
    
    String evtName = "event=rmDir";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("path", path, evtName);

    long hashValue = getHashValue(path);  
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirId, fileAttributesId", "hashValue="+hashValue);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    try {
	while (rs.next()) {
	    int dirID = rs.getInt(1);
	    int fileAttributesID = rs.getInt(2);
	    if (!recursive) {
		if (!isDirEmpty(dirID)) {
		    TSRMLog.error(TSRMFootprintDB.class, null, evtName, "status=NotDeleted msg=dirIsNotEmpty");
		    return false; // err str?
		}
		deleteDirEntry(dirID, fileAttributesID);
	    } else {
		deleteDirRecursively(dirID, fileAttributesID);
	    }
	    return true;
	}
	String err ="NoSuchPath="+path;
	TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	return false;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
	rs.close();
    }
}

// this does not mix with deleting contents
public void deleteDirEntry(String path) {
    String evtName = "event=deleteDirEntry";
    TSRMLog.info(this.getClass(), null, evtName, "path="+path);

    try {
	long hashValue = getHashValue(path);  
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableDir, "dirId, fileAttributesId", "hashValue="+hashValue);
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	while (rs.next()) {
	    int dirId = rs.getInt(1);
	    int fileAttributesId = rs.getInt(2);
	    
	    rs.close();
	    deleteDirEntry(dirId, fileAttributesId);
	    return;
	}
	TSRMLog.info(this.getClass(), null, evtName, "status=NoDirFound");
	rs.close();
	return;	
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), "failed_deleting_dir", e);
    }
}

private void deleteDirEntry(int dirId, int fileAttributesId) throws Exception
{
    PreparedStatement psDelete = _conn.prepareStatement("DELETE from Directory where dirId=?");    
    psDelete.setInt(1,dirId);    
    psDelete.executeUpdate();
    _conn.commit();
	
    String sql = TSRMFootprintUtil.generateSqlDelete(_DefTableFileAttr, "fileAttributesId");
    psDelete = _conn.prepareStatement(sql);
    psDelete.setInt(1,fileAttributesId);
    psDelete.executeUpdate();
    _conn.commit();
}

private void deleteDirRecursively(int dirID, int fileAttributesID) throws Exception
{
    String evtName = "event=deletRecursively";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "dirID="+dirID+" fileAttrID="+fileAttributesID);     

    Statement stmt = _conn.createStatement(); 
    ResultSet rs = stmt.executeQuery("SELECT fileId,fileAttributesId from Files where " + "dirId=" + dirID);
			
    String fidVecStr = null;
    String fileAttrIdStr = null;

    while(rs.next()) { 
	int subfileId = rs.getInt(1);
	int subfileAttrId = rs.getInt(2);
	if (fidVecStr == null) {
	    fidVecStr = "(" + subfileId;
	    fileAttrIdStr = "(" + subfileId;
	} else {	    
	    fidVecStr += ", " + subfileId;
	    fileAttrIdStr +=", " + subfileAttrId;
	}
    }
    rs.close();
    
    TSRMLog.info(this.getClass(), null, evtName, "fids="+fidVecStr);
    TSRMLog.info(this.getClass(), null, evtName, "fileAttrs="+fileAttrIdStr);
    if (fidVecStr != null) {
	fidVecStr +=") ";
	String sql = "DELETE from Files where fileId IN "+fidVecStr;
	PreparedStatement psDelete = _conn.prepareStatement(sql);
	
	psDelete.executeUpdate();
	TSRMLog.info(this.getClass(), null, evtName, "sql="+sql);
	_conn.commit();
    }		
	    
    deleteDirEntry(dirID, fileAttributesID);
	    
    rs = stmt.executeQuery("SELECT dirId, fileAttributesId from Directory where " + "parentId=" + dirID);
    
    while(rs.next()) {   
	int subdirID = rs.getInt(1);
	int subdirFileAttrID = rs.getInt(2);
	deleteDirRecursively(subdirID, subdirFileAttrID);
    }
    rs.close();
    stmt.close();


    if (fileAttrIdStr != null) {
	fileAttrIdStr +=") ";
	String sql = "DELETE from "+_DefTableFileAttr+" where fileAttributesId IN "+fileAttrIdStr;
	PreparedStatement psDelete = _conn.prepareStatement(sql);
		
	psDelete.executeUpdate();
	TSRMLog.info(this.getClass(), null, evtName, "sql="+sql);
	_conn.commit();
    }

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "dirID="+dirID);     
}


public boolean hasValidFileInDir(String path) throws Exception {
    String evtName = "event=hasValidFileInDir";
    TSRMLog.info(this.getClass(), null, evtName, "path="+path);

    int dirId = getDirId(path);
    if (dirId < 0) {
	TSRMLog.info(this.getClass(), null, evtName, "status=NoDirFound");
	return false;
    }
    
    //String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFile, "fileAttributesId", "dirId="+dirId);
    long now = System.currentTimeMillis();
    String attrFid = _DefTableFile+".fileAttributesId";
    String attrFidRef = _DefTableFileAttr+".fileattributesId";
    String joinCond = attrFid+" = "+attrFidRef+" AND FileAttributes.lifeTimeEndsAt > "+now;
    String attrDid = _DefTableFile + ".dirId";
    String sql = "SELECT "+attrFid+" from "+_DefTableFile +", "+_DefTableFileAttr+" where "+attrDid+" = "+ dirId+" AND "+joinCond;

    System.out.println(evtName+" checking: "+sql);
    
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    boolean result = false;
    if (rs.next()) {
	result = true;
    }
    rs.close();
    TSRMLog.info(this.getClass(), null, evtName, "path="+path+" result="+result);
    return result;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//rmFile
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    public boolean rmFile(String path) throws Exception 
    {	
	String evtName = "event=rmFile";
	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "path="+path);
	TSRMFootprintUtil.noTrivialString("path", path, evtName);

	String dName = null;
	String fName = path;

	int idx = path.lastIndexOf("/");
	if(idx != -1) {
	    dName = path.substring(0,idx+1); 
	    fName = path.substring(idx+1); 
	}

	int dirId = findDirId(dName);
					       
	if(dirId == -1) {
	    String err = "msg=NoSuchDir dir="+dName;
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    return false;

	}

	String sql = "SELECT fileId, fileAttributesId from Files where fileName="+"\'"+fName+"\'" + " and dirId = "+dirId;

	ResultSet rs = _statement.executeQuery(sql);
		
	try {	
	    while (rs.next()) {
		int fid = rs.getInt(1);
		int attrid = rs.getInt(2);
		TSRMLog.info(this.getClass(), null, evtName, "fid="+fid+" attrid="+attrid);
		
		String sqlDeleteInFile = TSRMFootprintUtil.generateSqlDelete(_DefTableFile, "fileId");
		int[] fidV = {fid};
		TSRMFootprintUtil.execSql(_conn, evtName, sqlDeleteInFile, null, null, fidV, null, null);
		
		//PreparedStatement psDelete = _conn.prepareStatement("DELETE from Files where fileId=?");	    
		//psDelete.setInt(1,fileId);
		//psDelete.executeUpdate();
		//_conn.commit();
		
		String sqlDeleteInFileAttr = TSRMFootprintUtil.generateSqlDelete(_DefTableFileAttr, "fileAttributesId");
		int[] attrV = {attrid};
		TSRMFootprintUtil.execSql(_conn, evtName, sqlDeleteInFileAttr, null, null, attrV, null, null);
		
		_aux.listFileAttrs(-1);
		return true;
	    }
	    String err = "msg=NoSuchPath path="+path;
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    return false;	    
	} finally {
	    rs.close();
	    TSRMLog.info(TSRMFootprintDB.class,"end","event=rmFile", "path="+path);
	}		
    }


    private int findDirId(String currNodeName) throws Exception{
	if (currNodeName == null) {
	    return -1;
	}

	long computedHashValue = getHashValue(currNodeName);
	String sql = "SELECT dirId from Directory where hashValue = " + computedHashValue;
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	
	try {
	    if (rs.next()) {
		return rs.getInt(1);
	    }
	    return -1;
	} finally {
	    rs.close();
	}
    }

    //
    // cleaned up version
    //
    public boolean mkDir(String path) throws Exception {
	try {
	    String evtName = "event=mkDir";
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "path="+path);
	    
	    TSRMFootprintUtil.noTrivialString("path", path, evtName);
	    
	    Vector nodeNames = new Vector();
	    StringTokenizer stk = new StringTokenizer(path,"/");
	    int counter = 0;
	    while(stk.hasMoreTokens()) {
		String token = stk.nextToken();
		if (counter == 0) {
		    //nodeNames.add("/"+token);
		    nodeNames.add("/"+token+"/");
		} else {
		    //nodeNames.add(nodeNames.get(counter-1)+"/"+token);
		    nodeNames.add(nodeNames.get(counter-1)+token+"/");
		}
		counter ++;
	    }       
	    
	    int size = nodeNames.size(); 
	    if (size == 0) {
		String err = "error=noSlashInPath value="+path;
		TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
		return false;
	    }
	    
	    for(int i = 0; i < size; i++) {
		String currNodeName = (String)nodeNames.elementAt(i);
		
		if (findDirId(currNodeName) > -1) {
		    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=none value="+currNodeName);       
		    continue;
		}
		int fileAttributesId = insertIntoFileAttributes(currNodeName,"DIRECTORY");
		
		if (fileAttributesId == -1) {
		    String err = "Could not assign FileAttributesId for path:" +path;
		    TSRMLog.info(TSRMFootprintDB.class,"error", evtName, err);
		    return false; 
		}  
		if (fileAttributesId == -2) {
		    String err = " Path already exists in "+_DefTableFileAttr+" table. Path="+path;
		    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
		    return false;
		} 
		
		int dirId = -1;		
		if (i > 0) {
		    String parentDirName = (String)(nodeNames.elementAt(i-1));
		    dirId = findDirId(parentDirName);
		}
		
		String[] attrs = {"dirEntry", "parentId", "fileAttributesId", "hashValue"}; 
		String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableDir, attrs);
		
		String[] strV = {currNodeName};
		int[] intV = {dirId, fileAttributesId};
		long[] longV = {getHashValue(currNodeName)};
		
		TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, null, longV); 
		
	    }//end for
	    return true;
	} finally {
	    TSRMLog.info(TSRMFootprintDB.class, null, "event=mkDir status=end", null);
	}
    }
    
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//insertIntoFileAttributes
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private int insertIntoFileAttributes(String fullPath, 
				     String fileType) throws Exception 
{
  String evtName = "event=insertIntoFileAttributes";
  try {   
      String[] strArray = {"path=", fullPath, " type=", fileType};
      TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
            
      long hashValue = getHashValue(fullPath);      
      int fileAttrID = getFileAttrID(hashValue);
      if (fileAttrID > -1) {
	  TSRMLog.info(TSRMFootprintDB.class, null, evtName+" status=exists", null);
	  //return fileAttrID;
	  return -2; // already exists
      }
      
      long ttime = System.currentTimeMillis();
      java.sql.Timestamp date = new java.sql.Timestamp(ttime);

      String[] attrs = {"path", "type", "createdAtTime", "lastModificationTime", "hashValue"};
      String psInsert = TSRMFootprintUtil.generateSqlInsert(_DefTableFileAttr, attrs);
      String[] strV = {fullPath, fileType};
      long[] longV={hashValue};
      Timestamp[] tV={date, date};
      TSRMFootprintUtil.execSql(_conn, evtName, psInsert, strV, null, null, tV, longV);
      
      return getFileAttrID(hashValue);     
  } catch(Exception e) {
      TSRMLog.exception(TSRMFootprintDB.class,"insertIntoFileAttributes",e);
      throw e;       
  } finally {
      TSRMLog.info(TSRMFootprintDB.class, null, evtName+" status=end", null);
  }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//createFile
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean createFile (String path, boolean isSurl) throws Exception {	
    String evtName="event=createFile";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "path="+path+" isSurl="+isSurl);
    
    try {	    
	TSRMFootprintUtil.noTrivialString("path", path, evtName);
	
	String fName = path;
	String dirName = null;
	    
	int idx = path.lastIndexOf("/");
	if(idx != -1) {
	    fName = path.substring(idx+1);
	    dirName = path.substring(0,idx+1);
	}
	
	int dirId = findDirId(dirName);
	if (dirId == -1) {
	    String err = "error=NoDirId dir="+dirName+" path="+path;
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    return false;
	}
	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "dirName="+dirName);
	
	int fileAttributesId = insertIntoFileAttributes(path,"FILE");
	
	if(fileAttributesId == -1) {
	    String err = "error=NofileAttributesId path=" + path;
	    TSRMLog.info(TSRMFootprintDB.class,"error", evtName, err);
	    return false;
	}
	
	boolean value = true;
	if (fileAttributesId == -2) {
	    String err = "status=AlreadyExists path= " + path;
	    TSRMLog.info(TSRMFootprintDB.class,"skipping.this.insert", evtName, err);
	    return false;
	} 
	
	TSRMLog.info(this.getClass(), null, evtName, "fid="+fileAttributesId+" dirId="+dirId);

	String attrs[] = {"dirId", "fileName", "fileAttributesId", "isSurl"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableFile, attrs); 
	PreparedStatement psInsert = _conn.prepareStatement(sql);
	
	psInsert.setInt(1,dirId);
	psInsert.setString(2,fName);
	psInsert.setInt(3,fileAttributesId);
	psInsert.setBoolean(4, isSurl);
	
	psInsert.executeUpdate();      
	_conn.commit();
	
	TSRMLog.info(TSRMFootprintDB.class,null,  evtName, "action=committed sql="+sql);	    
	return value;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateRequest
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void updateRequest(String rid, Hashtable ht)  throws Exception     
{
    String evtName="event=udpateRequest";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid);

    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);    
    TSRMFootprintUtil.noTrivialHashtable("requestAttrs", ht, evtName);
    
    assertRid(rid, evtName);

    String[] validAttrList = {"userRequestDesc","authID","status", "explanation","bandwidthStatus","requestType",
			      "fileType","spaceToken","targetFileRetentionPolicy","targetFileAccessLatency",
			      "transferParametersAccessPattern","transferParametersConnectionType",
			      "transferParametersClientNetworks","transferParametersTransferProtocols",
			      "desiredTotalRequestTime", "totalNumFilesInRequest", "numOfCompletedFiles", "numOfFailedFiles", "numOfWaitingFiles",			   
			      "createdTime", "statusTime"};

    assertKeysInList(ht, validAttrList, evtName);

    StringBuffer buf = new StringBuffer("update Request set ");
 
    int validAttrValueCounter = 1;
    int[] posInSql = new int[validAttrList.length];

    for (int counter = 0; counter < validAttrList.length; counter++) {
	posInSql[counter] = -1; // initialization
	String key = validAttrList[counter];
	Object obj = ht.get(key);
	if(obj != null) {
	    if (validAttrValueCounter == 1) {
		buf.append(key+"=?");	    
	    } else {
		buf.append(", "+key+"=?");
	    }
	    posInSql[counter] = validAttrValueCounter;
	    validAttrValueCounter ++;
	}
    }

    buf.append(" where rid=?;");

    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "sql="+buf.toString());

    PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());

    for (int i=0; i<posInSql.length; i++) {
	int pos = posInSql[i];
	if (pos == -1) {
	    continue;
	}       

	Object value = ht.get(validAttrList[i]);
	if (i <= 13) {
	    psUpdate.setString(pos, value.toString());	
	} else if (i >= 19) {
	    psUpdate.setTimestamp(pos, (java.sql.Timestamp)(value));
	} else {
	    psUpdate.setInt(pos, ((Integer)value).intValue());
	}
    }
       
    psUpdate.setString(validAttrValueCounter, rid);
    psUpdate.executeUpdate();
 
    _conn.commit();

    TSRMLog.info(TSRMFootprintDB.class, null, evtName="Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updatePinLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean updatePinLifeTime(String rid, String localSURL, int lifeTime) throws Exception 
{				 
  String evtName = "event=updatePinLifeTime";
  
  String[] strArray = {"rid=", rid, " localSURL=", localSURL, " lifetime="+lifeTime};
  TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));

  TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
  TSRMFootprintUtil.noTrivialString("localSurl", localSURL, evtName);

  /*
  if(remoteSURL == null || remoteSURL.equals("")) {
      remoteSURL="NA";
  }
  */

  
  ResultSet rs = _statement.executeQuery("SELECT rid from RequestExecution where " +
					 " rid="+"\'"+rid+"\'" + " and fromSURL="+"\'"+localSURL+"\'");
					 
  if (rs.next()) {
      rs.close();
  } else {  
     rs.close();
     String err="Could not find a matching row to update";
     TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
		
     throw new TSRMFootprintDBException(err+" rid="+rid+" localSURL="+localSURL);
  }


  try {
      String str = "update RequestExecution set pinLifeTime=? where rid=? and fromSURL=?";
	  
      PreparedStatement psUpdate = _conn.prepareStatement(str);
      psUpdate.setInt(1,lifeTime);
      psUpdate.setString(2,rid);
      psUpdate.setString(3,localSURL);
      
      psUpdate.executeUpdate();
      
      _conn.commit();
      
      TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Commited", null);      
  } catch(Exception e) {
      TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
      throw new TSRMFootprintDBException("\nDB:Exception at updatePinLifeTime " + e.getMessage());					 
  }
  
  TSRMLog.info(TSRMFootprintDB.class,"end","event=updatePinLifeTime", " rid="+rid);
  
  return true;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateFileRequest
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
public void updateFileRequest(String rid, int filePos, TSRMRequestPut req) {
    Hashtable ht = new Hashtable();
    ht.put("status", req.getReturnStatus().getStatusCode().toString());
    if (req.getReturnStatus().getExplanation() != null) {
	ht.put("explanation", req.getReturnStatus().getExplanation());
    }
    if (req.getTransferURL() != null) {
	ht.put("TURL", req.getTransferURL());
    }
    // not file level: ht.put("spaceToken", req.getSpaceToken().getID());
    Integer lifetime = req.getRemainingTURLLifeTimeInSeconds();
    if (lifetime != null) {
	ht.put("pinLifeTime", lifetime);
    }
    try {
	updateFileRequest(rid, filePos, ht);
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), "updateFileStatus", e);
	e.printStackTrace();
    }
}

public void updateFileRequest(String rid, int filePos, TSRMRequestCopy req) {
    Hashtable ht = new Hashtable();
    ht.put("status", req.getReturnStatus().getStatusCode().toString());
    if (req.getReturnStatus().getExplanation() != null) {
	ht.put("explanation", req.getReturnStatus().getExplanation());
    }

    ht.put("actualFileSize", req.getFileSize());
    // not file level: ht.put("spaceToken", req.getSpaceToken().getID());
    Integer lifetime = req.getRemainingPinTime();
    if (lifetime != null) {
	ht.put("pinLifeTime", lifetime);
    }
    try {
	updateFileRequest(rid, filePos, ht);
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), "updateFileStatus", e);
	e.printStackTrace();
    }
}

public String getSiteUrl(String rid, int pos) {
    String evtName="event=getSiteUrl";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid+" pos="+pos);

    try {
	TSRMFootprintUtil.noTrivialString("rid", rid, evtName);	
	String[] attrs = {"fromSURL", "toSURL"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, attrs, "rid", rid);
	sql += " and pos="+pos;
	ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	if (rs.next()) {
	    String from = rs.getString(1);
	    String to = rs.getString(2);
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "from="+from+" to="+to);
	    if ((from != null) && (from.length() > 0)) {
		return from;
	    }
	    if ((to != null) && (to.length() > 0)) {
		return to;
	    }
	    return null;
	} else {
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"NoMatch", "rid="+rid+" pos="+pos);
	    return null;
	}
    } catch (Exception e) {
	return null;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "rid="+rid+" pos="+pos);
    }
}

public void updateFileRequest(String rid, int filePos, Hashtable ht) throws Exception 				  
{
    String evtName="event=updateFileRequestExec";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid+" pos="+filePos);

    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    
    if (ht == null || ht.size() == 0) {
	TSRMLog.error(TSRMFootprintDB.class, null, evtName, "hashtable cannot be empty");
	throw new TSRMFootprintDBException("\nDB: Hashtable cannot be null or empty");
    }

    assertFileReq(rid, filePos, evtName);
    
    String[] validAttrList = {"pinLifeTime", "statusTime", "status", "explanation", "TURL", "actualFileSize"};			      
    
    assertKeysInList(ht, validAttrList, evtName);

    StringBuffer buf = new StringBuffer("update "+_DefTableFileRequest+" set ");

    int validAttrValueCounter = 1;
    int[] posInSql = new int[validAttrList.length];

    for (int counter = 0; counter < validAttrList.length; counter++) {
	posInSql[counter] = -1; // initialization
	String key = validAttrList[counter];
	Object obj = ht.get(key);
	if(obj != null) {
	    if (validAttrValueCounter == 1) {
		buf.append(key+"=?");	    
	    } else {
		buf.append(", "+key+"=?");
	    }
	    posInSql[counter] = validAttrValueCounter;
	    validAttrValueCounter ++;
	}
    }
				       
    buf.append(" where rid=? and pos=?;");

    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "sql="+buf.toString());

    PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());

    for (int i=0; i<posInSql.length; i++) {
	int pos = posInSql[i];
	if (pos == -1) {
	    continue;
	}       
	
	Object value = ht.get(validAttrList[i]);
	if (i == 0) {
	    psUpdate.setInt(pos, ((Integer)value).intValue());
	} else if (i == 1) {
	    psUpdate.setTimestamp(pos, (java.sql.Timestamp)(value));
	} else {
	    psUpdate.setString(pos, value.toString());
	}
    }
					
    psUpdate.setString(validAttrValueCounter, rid);
    psUpdate.setInt(validAttrValueCounter+1, filePos);
    psUpdate.executeUpdate();
    					
    _conn.commit();
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateFileLifeTime
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean updateFileLifeTime(String path, int newFileLifeTime) throws Exception     
{
    String evtName="event=updateFileLifeTime";
    String[] strArray = {"path=", path, " newFileLifeTime="+newFileLifeTime};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("path", path, evtName);
    
    assertFilePath(evtName, path);
    
    String str = TSRMFootprintUtil.generateSqlUpdate(_DefTableFileAttr, "lifeTimeAssigned", "hashValue");
    
    TSRMLog.info(TSRMFootprintDB.class,"queryprepared","event=updateFileLifeTime", "query="+str);
    PreparedStatement psUpdate = _conn.prepareStatement(str);
    
    long hashValue = getHashValue(path);
    psUpdate.setInt(1,newFileLifeTime);
    psUpdate.setLong(2,hashValue);
    psUpdate.executeUpdate();
    
    _conn.commit();
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    return true; 
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateFileAttributes
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
private void fileAddToToken(int fileAttributesId, String spaceToken) throws Exception {
    String evt = "event=assocFileToSpace";
    TSRMLog.info(this.getClass(), null, evt, "fid="+fileAttributesId+" token="+spaceToken);

    long hashValue = getHashValue(spaceToken);
    String cond = "hashValue="+hashValue + " and fileAttributesId="+fileAttributesId;

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "spaceToken", cond);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    try {
	if (rs.next()) {
	    TSRMLog.info(this.getClass(), null, evt, "status=alreadyAssociated");
	    return;
	} 
	String[] attrList = {"spaceToken", "fileAttributesId", "hashValue"};
	String insertSql = TSRMFootprintUtil.generateSqlInsert(_DefTableFileSP, attrList);

	String[] strV = {spaceToken};
	int[] intV = {fileAttributesId};
	long[] longV = {hashValue};
	TSRMFootprintUtil.execSql(_conn, evt, insertSql, strV, null, intV, null, longV);	
    } finally {
	rs.close();		    
    }
}

private void fileOutOfToken(int fileAttributesId, String spaceToken) throws Exception {
    String evt = "event=unassocFileToSpace";
    TSRMLog.info(this.getClass(), null, evt, "fid="+fileAttributesId+" token="+spaceToken);
    long hashValue = getHashValue(spaceToken);
    String cond = "hashValue="+hashValue + " and fileAttributesId="+fileAttributesId;

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "spaceToken", cond);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);		    	     		    		   		  

    try {
	if (!rs.next()) {
	    TSRMLog.info(this.getClass(), null, evt, "status=noActionNeeded");
	    return;
	} 
	String[] attrList = {"fileAttributesId", "hashValue"};
	String deleteSql = TSRMFootprintUtil.generateSqlDelete(_DefTableFileSP, attrList);

	int[] intV = {fileAttributesId};
	long[] longV = {hashValue};
	TSRMFootprintUtil.execSql(_conn, evt, deleteSql, null, null, intV, null, longV);	
    } finally {
	rs.close();		    
    }
}


private int assertFilePath(String evtName, String path) throws Exception {   
    //long hashValue = getHashValue(path);
    int fileAttributesId = getFileAttrID(path);
    
    if (fileAttributesId == -1) {
	String err = "error=NoSuchPathInDB. path=" + path;
	TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	throw new TSRMFootprintDBException(err);
    }    
    return fileAttributesId;
}

public void fileWithSpace(String path, String tokenList, boolean fileIsInSpace) throws Exception {
    String evtName = "event=fileWithSpace";
    TSRMLog.info(this.getClass(), null, evtName+"Starts", "path="+path+" tokens="+tokenList+" isFileInSpace="+fileIsInSpace);
    TSRMFootprintUtil.noTrivialString("path", path, evtName);

    int fileAttributesId = assertFilePath(evtName, path);

    StringTokenizer stk = new StringTokenizer(tokenList,",");
    while(stk.hasMoreTokens()) {
	String currToken = stk.nextToken();
	if (fileIsInSpace) {
	    fileAddToToken(fileAttributesId, currToken);
	} else {
	    fileOutOfToken(fileAttributesId, currToken);
	}
    }
}

private void assertKeysInList(Hashtable ht, String[] validAttrList, String evtName) throws TSRMFootprintDBException 
{
    Object[] keys = ht.keySet().toArray();
    
    for(int i = 0; i < keys.length; i++) {
	String key = (String) keys[i];
	boolean found = false;
	for (int k=0; k<validAttrList.length; k++) {
	    if (key.equalsIgnoreCase(validAttrList[k])) {
		found = true;
	    }
	}
	if (!found) {
	    String err = "error=InvalidAttr name="+key;
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    throw new TSRMFootprintDBException(err);
	}
    }
}

public void updateFileAttributes(String path, Hashtable ht) throws Exception 
{
    String evtName = "event=updateFileAttributes";    

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "path="+path);
    TSRMFootprintUtil.noTrivialString("path", path, evtName);
    
    TSRMFootprintUtil.noTrivialHashtable("fileAttrs", ht, evtName);

    assertFilePath(evtName, path);

    String[] validAttrList = {"size", "lifetimeEndsAt", "lifetimeassigned", "createdAtTime", "lastModificationTime",		      
			      "isReady", "physicalPath",
			      "filelocality", "fileStorageType", "retentionpolicy", "accesslatency", 
			      "ownerpermission", "grouppermission", "otherpermission", "type", 
			      "userid", "groupid", "checksumtype", "checksumvalue"};

    assertKeysInList(ht, validAttrList, evtName);

    StringBuffer buf = new StringBuffer("update "+_DefTableFileAttr+" set ");
 
    int validAttrValueCounter = 1;
    int[] posInSql = new int[validAttrList.length];

    for (int counter = 0; counter < validAttrList.length; counter++) {
	posInSql[counter] = -1; // initialization
	String key = validAttrList[counter];
	Object obj = ht.get(key);
	if(obj != null) {
	    if (validAttrValueCounter == 1) {
		buf.append(key+"=?");	    
	    } else {
		buf.append(", "+key+"=?");
	    }
	    posInSql[counter] = validAttrValueCounter;
	    validAttrValueCounter ++;
	}
    }
    buf.append(" where hashValue=?;");

    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "sql="+buf.toString());

    PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());

    for (int i=0; i<posInSql.length; i++) {
	int pos = posInSql[i];
	if (pos == -1) {
	    continue;
	}       

	Object value = ht.get(validAttrList[i]);
	if ((i == 0) || (i==1)) {
	    psUpdate.setLong(pos, ((Long)value).longValue());
	} else if (i == 2) {
	    psUpdate.setInt(pos, ((Integer)value).intValue());
	} else if (i < 5) {
	    psUpdate.setTimestamp(pos, (java.sql.Timestamp)(value));
	} else if (i == 5) {
	    psUpdate.setBoolean(pos, ((Boolean)value).booleanValue());
	} else {
	    psUpdate.setString(pos, value.toString());	
	}
    }
       
    long hashValue = getHashValue(path);
    psUpdate.setLong(validAttrValueCounter, hashValue);
    psUpdate.executeUpdate();
 
    _conn.commit();
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=commit");       

    _aux.listFileAttrs(hashValue);
    _aux.listFileAttrs(-1);
    _aux.listFiles();
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//isConnectionOpen
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean isConnectionOpen() {
  return _isConnectionOpen;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//closeConnection
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void closeConnection() {
    _isConnectionOpen=false;
    try {	
	_conn.close();
	// the shutdown=true attribute shuts down DB
	//DriverManager.getConnection("jdbc:derby:;shutdown=true");
	
	// To shut down a specific database only, but keeep the
	// engine running (for example for connecting to other
	// databases), specify a database in the connection URL:
	//DriverManager.getConnection("jdbc:derby:" + _dbName + ";shutdown=true");
    } catch (SQLException se) {    
	TSRMLog.exception(TSRMFootprintDB.class,"closeConnection",se);
	if (( (se.getErrorCode() == 50000)
	      && ("XJ015".equals(se.getSQLState()) ))) {
	    // we got the expected exception
	    System.out.println("\nDB:TSRMFootprintDB shut down normally");
	    // Note that for single database shutdown, the expected
	    //  SQL state is "08006", and the error code is 45000.
	} else {
	    // if the error code or SQLState is different, we have
	    // an unexpected exception (shutdown failed)
	    System.err.println("TSRMFootprintDB did not shut down normally");
	    printSQLException(se);
	}
    } finally {    
	try {
	    if (_conn != null) {
		_conn.close();
		_statement.close();
		_conn = null;
	    }
	} catch(SQLException sqle) {
	    TSRMLog.exception(TSRMFootprintDB.class,"closeConnection",sqle);
	    printSQLException(sqle);
	}  
    }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//Space Related Api's
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// addSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean addSpaceToken(String name, String uid, String spaceToken,
			     String userDesc, long totalSize, 
			     long guaranteedSize, long lifeTimeAssigned) 
    throws Exception 
{
    String evtName="event=addSpaceToken";
    String[] strArray = {"name=", name, " uid=", uid, " spaceToken=", spaceToken, 
			 " userDesc=", userDesc, " totalSize="+totalSize, 
			 " guaranteedSize="+guaranteedSize,  " lifeTimeAssigned="+lifeTimeAssigned};
    TSRMLog.info(TSRMFootprintDB.class,null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    TSRMFootprintUtil.noTrivialString("name", name, evtName);
    
    int u_id = assertUserOid(uid, evtName);
    int cid = assertCompOid(name, evtName);
    

    if (checkExistenceSpaceToken(spaceToken)) {
	TSRMLog.info(this.getClass(), null, evtName, "spaceTokenAlreadyExists");
	return false;
    }

    ResultSet rs=null;
    long ttime = System.currentTimeMillis();
    java.sql.Timestamp lifetimeStart = new java.sql.Timestamp(ttime);
    
    try { 
	String[] attrs = {"spaceToken", "userSpaceTokenDesc","componentId","userId", "lifeTimeStart",
			  "totalSize","guaranteedSize","lifeTimeAssigned"};			  
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableSpace, attrs);
	
	String[] strV = {spaceToken, userDesc};
	int[] intV = {cid,u_id};
	long[] longV ={totalSize, guaranteedSize, lifeTimeAssigned};
	Timestamp[] tV={lifetimeStart};
	
	TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, tV, longV);
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class,"addSpaceToken",e);
	return false;	
    }

    TSRMLog.info(TSRMFootprintDB.class,null,evtName+"Ends",null);	      
    return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// updateSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean updateSpaceToken(String spaceToken, long nTotalSize, long nGSize, long nLifeTime)
    throws Exception 
{
    String evtName = "event=updateSpaceToken";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "spaceToken="+spaceToken);	      	      
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);

    if ((nTotalSize <=0) && (nGSize <=0) && (nLifeTime <=0)) {
	TSRMLog.info(this.getClass(), null, evtName, "action=emptyInputNoAction");
	return false;
    }

    if (!checkExistenceSpaceToken(spaceToken)) {
	TSRMLog.info(this.getClass(), null, evtName, "noSuchSpaceToken");
	return false;
    }

    StringBuffer buf = new StringBuffer("update "+_DefTableSpace+" set ");

    int[] sizeIdx = {-1, -1, -1}; // total, gsize, lifetime
    long[] valIdx = {nTotalSize, nGSize, nLifeTime};
    String[] nameIdx = {"totalSize", "guaranteedSize", "lifeTimeAssigned"};

    int counter = 0;

    for (int i=0; i<valIdx.length; i++) {
	if (valIdx[i] > 0) {	   
	    counter ++;
	    buf.append(nameIdx[i]);
	    buf.append("=?, ");
	    sizeIdx[i] = counter;
	}
    }
    
    buf.deleteCharAt(buf.length()-2);
    buf.append(" where spaceToken=?;");
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, " sql="+buf.toString());
    
    PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());

    for (int i=0; i<sizeIdx.length; i++) {
	if (sizeIdx[i] > 0) {
	    psUpdate.setLong(sizeIdx[i], valIdx[i]);
	} 
    }

    psUpdate.setString(counter+1, spaceToken);
    psUpdate.executeUpdate();    
    _conn.commit();    
    TSRMLog.info(TSRMFootprintDB.class,"end", evtName, null);

    return true;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//deleteSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean deleteSpaceToken(String spaceToken) throws Exception {
    String evtName="event=deleteSpaceToken";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "spaceToken="+spaceToken);	      
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);
    
    long hashValue = getHashValue(spaceToken);
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "spaceToken", "hashValue="+hashValue);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
    
    try {
	if (rs.next()) {
	    String err = "cannot delete spacetoken some files are still refers to this spacetoken";
	    TSRMLog.info(TSRMFootprintDB.class,"error", evtName, err);
	    throw new TSRMFootprintDBException(err);
	}
    } finally {
	rs.close();
    }

    deleteTokenEntry(evtName, spaceToken);
    return true;
}

private void deleteTokenEntry(String evtName, String spaceToken) {
    String sql = TSRMFootprintUtil.generateSqlDelete(_DefTableSpace, "spaceToken");//"DELETE from SpaceAssignment where spaceToken=?";
    String[] strV = {spaceToken};
    TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, null, null, null);
    TSRMLog.info(TSRMFootprintDB.class,"end", evtName, null);

}

public boolean deleteTokenForce(String spaceToken) {
    String evtName="event=deleteTokenForce";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "spaceToken="+spaceToken);	      
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);

    deleteTokenEntry(evtName, spaceToken);

    // delete file entries in that space   
    String sqlCond = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "fileAttributesId", "spaceToken", spaceToken);
    
    String sql = "DELETE from Files where fileAttributesId IN ("+sqlCond+")";

    try {
	TSRMLog.debug(this.getClass(), null, evtName, "status=deleteFromFile");
	PreparedStatement psDelete = _conn.prepareStatement(sql);
	_conn.commit();
	
	// delete mappings from fileattributessp table
	TSRMLog.debug(this.getClass(), null, evtName, "status=deleteFromFileSP");
	sql = TSRMFootprintUtil.generateSqlDelete(_DefTableFileSP, "spaceToken", spaceToken);
	//sql = "DELETE from FileAttributesSP where spaceToken="+spaceToken;
	psDelete = _conn.prepareStatement(sql);
	_conn.commit();
    } catch (java.sql.SQLException e) {
	TSRMLog.exception(this.getClass(), evtName, e);
	return false;
    }
    return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// addComponent
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public int getDeviceCounter() 
    throws Exception 
{
    String sql=TSRMFootprintUtil.generateSqlSelect(_DefTableComp, "componentName", null, null);
    ResultSet rs = _statement.executeQuery(sql);

    int counter = 0;
    while (rs.next()) {
	counter++;
    }
    rs.close();
    return counter;
}

private ResultSet findDevice(String name) 
    throws Exception
{
    String sql=TSRMFootprintUtil.generateSqlSelect(_DefTableComp, "mountingPoint, componentType, totalSize", "componentName", name);
    ResultSet rs = _statement.executeQuery(sql);

    return rs;
}

private boolean checkInconsistantDevice(ResultSet rs, String name, String type,int totalSize, String mountingPoint) 
    throws Exception 
{
    try {
	String cmp = rs.getString(1);
	String ctype = rs.getString(2);
	int size = rs.getInt(3);
	if (cmp.equals(mountingPoint) && ctype.equals(type)) {
	    if (totalSize < size) {
		TSRMLog.error(TSRMFootprintDB.class, null, "event=DeviceInDBSizeDiffs", 
			      "name="+name+" sizeRecorded="+size+" sizeInput="+totalSize);
		return true;   
	    }
	    TSRMLog.info(TSRMFootprintDB.class, null, "event=DeviceAlreadyInDB", "name="+name);
	} else {
	    TSRMLog.error(TSRMFootprintDB.class, null, "event=inconsistantDeviceFound", "name="+name+" mpExpected="+cmp+" typeExpected="+ctype);
	    return true;
	}
    } finally {
	rs.close();
    }
    return false;
}

public boolean addComponent(String name, String type, int totalSize, String description, String mountingPoint) 			    
    throws Exception 
{
    String evtName = "event=addComponent";
    String[] strArray = {"name=", name, " type=", type, " totalSize="+totalSize, 
			 " desc=", description, " moutingPoint=", mountingPoint};
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, TSRMFootprintUtil.appendString(strArray));
    
    try {
	TSRMFootprintUtil.noTrivialString("componentName", name, evtName);
	TSRMFootprintUtil.noTrivialString("mountingPoint", mountingPoint, evtName);
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), "bad input", e);
	return false;
    }

    ResultSet rs = findDevice(name);
    if (rs.next()) {
	if (checkInconsistantDevice(rs, name, type, totalSize, mountingPoint)) {
	    return false;
	}
	return true; // no need to add to db
    } 
    
    String[] attrs = {"componentName", "componentType", "description", "mountingPoint", "totalSize"};
    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableComp, attrs);

    String[] strV = {name, type, description, mountingPoint};
    int[] intV ={totalSize};

    try {
	TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, null, null);
    } catch (Exception e) {
	return false;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=end"); 
    }

    return true;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//updateComponent
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void updateComponent(String name, Hashtable ht) throws Exception 
{
    String evtName = "event=updateComponent";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "comp="+name);

    TSRMFootprintUtil.noTrivialString("componentName", name, evtName);
    TSRMFootprintUtil.noTrivialHashtable("componentAttrs", ht, evtName);
   
    String[] validAttrList = {"totalsize", "componentType", "mountingPoint", "description"};
    assertKeysInList(ht, validAttrList, evtName);

    StringBuffer buf = new StringBuffer("update Component set ");    

    int validAttrValueCounter = 1;
    int[] posInSql = new int[validAttrList.length];

    for (int counter = 0; counter < validAttrList.length; counter++) {
	posInSql[counter] = -1; // initialization
	String key = validAttrList[counter];
	Object obj = ht.get(key);
	if(obj != null) {
	    if (validAttrValueCounter == 1) {
		buf.append(key+"=?");	    
	    } else {
		buf.append(", "+key+"=?");
	    }
	    posInSql[counter] = validAttrValueCounter;
	    validAttrValueCounter ++;
	}
    }
    buf.append(" where componentName=?;");

    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "sql="+buf.toString());

    PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());
    
    for (int i=0; i<posInSql.length; i++) {
	int pos = posInSql[i];
	if (pos == -1) {
	    continue;
	}       
	Object value = ht.get(validAttrList[i]);
	if (i == 0) {
	    psUpdate.setInt(pos, ((Integer)value).intValue());
	} else {
	    psUpdate.setString(pos, value.toString());	
	}
    }
	
    psUpdate.setString(validAttrValueCounter, name);
    psUpdate.executeUpdate();
 
    _conn.commit();    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "comp="+name);
}


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//deleteComponent
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean deleteComponent(String name) throws Exception 
{    
    String evtName = "event=deleteComponent";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "name="+name);

    TSRMFootprintUtil.noTrivialString("componentName", name, evtName);
    
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableComp, "componentId", "componentName", "name");

    ResultSet rs = _statement.executeQuery(sql);
					   
    try {
	if (rs.next()) {
	    int id = rs.getInt(1);
	    sql = TSRMFootprintUtil.generateSqlDelete(_DefTableSpace, "componentId");
	    PreparedStatement psDelete = _conn.prepareStatement(sql);
	    
	    psDelete.setInt(1,id);
	    psDelete.executeUpdate();
	    _conn.commit();
	    
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "event=DeleteFromSpaceAssignment");
	    
	    rs.close();
	    
	    sql = TSRMFootprintUtil.generateSqlDelete(_DefTableComp, "componentId");
	    psDelete = _conn.prepareStatement(sql);
	    
	    psDelete.setInt(1,id);
	    psDelete.executeUpdate();
	    _conn.commit();
	    
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "event=DeleteFromComponentTable");
	    return true;
	} else {	
	    String err = "NoSuchComponent:" + name;
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	    return false;
	}
    } finally {
	rs.close();
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//srmGetSpaceMetaData
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private TRetentionPolicyInfo getRetentionPolicyInfo(String tid, int compId) throws Exception
{
    Statement s = _conn.createStatement();
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableComp, "componentType", "componentId="+compId);
    ResultSet rs = s.executeQuery(sql);
				     
    if(!rs.next()) {
	return null;
    }
    
    TSRMSpaceType spaceType = TSRMSpaceType.getSpaceType(tid);		
    
    String type = rs.getString(1);
    TRetentionPolicyInfo retentionPolicyInfo = new TRetentionPolicyInfo();
    TAccessLatency accessLatency = TAccessLatency.fromString(type.toUpperCase());    
    TRetentionPolicy retentionPolicy = spaceType.getRetentionPolicy();
    retentionPolicyInfo.setRetentionPolicy(retentionPolicy);
    retentionPolicyInfo.setAccessLatency(accessLatency);

    return retentionPolicyInfo;
}

private TMetaDataSpace getSpaceMetaData(String curr, int u_id) throws Exception 
{
    TMetaDataSpace result = new TMetaDataSpace();
    result.setSpaceToken(curr);

    ResultSet rs = _statement.executeQuery("SELECT spaceToken,componentId,userSpaceTokenDesc,totalSize,guaranteedSize, "+
					   "unUsedSize,lifeTimeAssigned,lifeTimeStart from "+_DefTableSpace+" where "+
					   "spaceToken="+"\'"+curr+"\'" + " and userId="+u_id);
    try {
	while (rs.next()) {
	    String tt = rs.getString(1);
	    int compId = rs.getInt(2);
	    String userDesc = rs.getString(3);
	    long totalSize = rs.getLong(4);
	    long guaranSize = rs.getLong(5);
	    long unUsedSize = rs.getLong(5);
	    int lifeTimeAssigned = rs.getInt(7);
	    java.sql.Timestamp lifeTimeStart = rs.getTimestamp(8);
	    
	    result.setRetentionPolicyInfo(getRetentionPolicyInfo(curr, compId));
	    
	    result.setTotalSize(new org.apache.axis.types.UnsignedLong(totalSize));	
	    result.setGuaranteedSize(new org.apache.axis.types.UnsignedLong(guaranSize));	
	    result.setUnusedSize(new org.apache.axis.types.UnsignedLong(unUsedSize));	
	    result.setLifetimeAssigned(lifeTimeAssigned);
	    
	    return result;
	}
	result.setStatus(TSRMUtil.createReturnStatus(TStatusCode.SRM_INVALID_REQUEST, TSRMUtil._DefNoSuchSpaceToken));
	return result;
    } finally {
	rs.close();
    }
}

public TMetaDataSpace[] srmGetSpaceMetaData (String uid, String[] spaceTokens) throws Exception 
{
    String evtName = "event=srmGetSpaceMetaData";
    
    TSRMLog.info(TSRMFootprintDB.class, null,evtName+"Starts", "uid="+uid);
    
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    TSRMFootprintUtil.noTrivialArray("spaceTokens", spaceTokens, evtName);
    
    TSRMLog.info(TSRMFootprintDB.class,"input", evtName, TSRMFootprintUtil.appendString(spaceTokens, "|" ));
    
    int u_id = assertUserOid(uid, evtName);
    
    TMetaDataSpace[] result = new TMetaDataSpace[spaceTokens.length];
    for(int i = 0; i < spaceTokens.length; i++) { 
	String curr = spaceTokens[i];

	StaticToken t = StaticToken.find(Config._staticTokenList, curr);
	if (t != null) {
	    result[i] = t.getMetadata();
	} else {
	    result[i] = getSpaceMetaData(curr, u_id);
	    result[i].setOwner(uid);
	}
    }

  TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);

  return result;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//srmGetSpaceTokens
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String[] srmGetSpaceTokens(String userDesc, String uid) throws Exception     
{
    String evtName = "event=srmGetSpaceTokens";
    
    String[] strArray = {"userdesc=", userDesc, " uid=", uid};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    //TSRMFootprintUtil.noTrivialString("userDesc", userDesc, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    
    int u_id = assertUserOid(uid, evtName);
    
    Vector vec = new Vector();

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableSpace, "spacetoken", "userId="+u_id);
    if (userDesc != null) {
	sql += " and where userSpaceTokenDesc =" +"\'"+userDesc+"\'";
    }
    ResultSet rs = _statement.executeQuery(sql);

    while(rs.next()) {
	vec.addElement(rs.getString(1));
    }
    rs.close();

    String[] tokens = new String[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
	tokens[i] = (String) vec.elementAt(i);
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return tokens;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getLifeTimeOfSpaceToken
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public int getLifeTimeOfSpaceToken(String uid, String spaceToken) throws Exception 
{
    String evtName = "event=getLifeTimeOfASpaceToken";
    
    String[] strArray={"uid=", uid, " spaceToken=", spaceToken};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    
    int u_id = assertUserOid(uid, evtName);

    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableSpace, "lifeTimeAssigned", "spaceToken", spaceToken);
    sql += " and userId="+u_id;

    ResultSet rs = _statement.executeQuery(sql);
    
    int lifeTimeAssigned = -1;
    if (rs.next()) {
	Integer curr = rs.getInt(1);
	lifeTimeAssigned = curr.intValue();
	rs.close();
    } else {    
	rs.close();	
	//throw new TSRMFootprintDBException("No matching rows found in SpaceAssignment table.");
					   
    }
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return lifeTimeAssigned;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//findSURLSForSpaceToken
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String[] findSURLSForSpaceToken (String spaceToken)  throws Exception 
{
    String evtName="event=findSURLSForSpaceToken";
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "spacetoken="+spaceToken);
    
    TSRMFootprintUtil.noTrivialString("spaceToken", spaceToken, evtName);
    
    long hashValue = getHashValue(spaceToken);
    
    Vector vec = new Vector ();
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileSP, "fileAttributesId", "hashValue="+hashValue);	         
    ResultSet rs = _statement.executeQuery(sql);
    
    while (rs.next()) {
	int fileAttributesId = rs.getInt(1);
	Statement ss2 = _conn.createStatement();
	String sql2 = TSRMFootprintUtil.generateSqlSelect(_DefTableFileAttr, "path", "fileAttributesId="+fileAttributesId);
	ResultSet rs2 = ss2.executeQuery(sql2);
					 
	if (rs2.next()) {
	    vec.addElement(rs2.getString(1));
	}
	rs2.close();
	ss2.close();
	
    }
    rs.close();
    
    String[] surls = new String[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
	surls[i] = (String) vec.elementAt(i);
    }

    if (vec.size() == 0) {
	String err = "No surls matching for this spacetoken " + spaceToken;
	TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
	return null;
    }

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    return surls;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//Data Transfer Api's
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void store(SrmPrepareToGetRequest getRequest, String rid, String uid) throws Exception 		  
{
    String evtName = "event=storeGet";
    
    String[] strArray = {"rid=", rid, " uid=", uid};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    
    ArrayOfTGetFileRequest arrayOfFileRequests = getRequest.getArrayOfFileRequests();	

    if ((arrayOfFileRequests == null) || (arrayOfFileRequests.getRequestArray() == null)) {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=nullInput");
	return;
    }

    TGetFileRequest[] getReqFiles = arrayOfFileRequests.getRequestArray();
    
    int u_id = assertUserOid(uid, evtName);

    storeCommon(rid, getRequest.getAuthorizationID(),
		getRequest.getUserRequestDescription(),
		getRequest.getTargetSpaceToken(), 
		"PREPARE_TO_GET",
		getRequest.getDesiredFileStorageType(),
		getRequest.getTargetFileRetentionPolicyInfo(),
		getRequest.getDesiredTotalRequestTime(),
		getReqFiles.length, u_id);
		  	
    storeReqInput(getRequest.getTransferParameters(), rid);

    ResultSet rs = null;  
    
    //insert into RequestLevelInputTable 
    String ssiStr = TSRMFootprintUtil.getStorageSystemInfoString(getRequest.getStorageSystemInfo());    	
    int desiredPinLifeTime = TSRMFootprintUtil.getInt(getRequest.getDesiredPinLifeTime(), -1);
    
    try {
	String sql = "insert into "+_DefTableReqInputGet+" (rid, desiredPinLifeTime,storageSystemInfo) " + "values (?,?,?)";
	PreparedStatement psInsert = _conn.prepareStatement(sql);
							
	psInsert.setString(1,rid);
	psInsert.setInt(2,desiredPinLifeTime);
	psInsert.setString(3,ssiStr);
	
	psInsert.executeUpdate();      
	_conn.commit();	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");      
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	return;
    }
  
    for(int i = 0; i < getReqFiles.length; i++) {
	TGetFileRequest request = getReqFiles[i];
	storeMe(request, rid, i);
    }
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//store put request
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void store(SrmPrepareToPutRequest putRequest, String rid, String uid) throws Exception 
{
    String evtName = "event=storePut";
    
    String[] strArray = {"rid=", rid, " uid=", uid};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);

    ArrayOfTPutFileRequest arrayOfFileRequests = putRequest.getArrayOfFileRequests();	
    if ((arrayOfFileRequests == null) || (arrayOfFileRequests.getRequestArray() == null)) {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=nullInput");
	return;
    }

    TPutFileRequest[] putReqFiles = arrayOfFileRequests.getRequestArray();    
    int u_id = assertUserOid(uid, evtName);

    _aux.checkFileReqEntries(_statement);
    storeCommon(rid, putRequest.getAuthorizationID(),
		putRequest.getUserRequestDescription(),
		putRequest.getTargetSpaceToken(), 
		"PREPARE_TO_PUT",
		putRequest.getDesiredFileStorageType(),
		putRequest.getTargetFileRetentionPolicyInfo(),
		putRequest.getDesiredTotalRequestTime(),
		putReqFiles.length, u_id);
    _aux.checkFileReqEntries(_statement);		  	
    storeReqInput(putRequest.getTransferParameters(), rid);
    
    String ssiStr = TSRMFootprintUtil.getStorageSystemInfoString(putRequest.getStorageSystemInfo());
    
    int desiredPinLifeTime = TSRMFootprintUtil.getInt(putRequest.getDesiredPinLifeTime(), -1);	
    int desiredFileLifeTime = TSRMFootprintUtil.getInt(putRequest.getDesiredFileLifeTime(), -1);
	
    String overWriteMode = TSRMFootprintUtil.getString(putRequest.getOverwriteOption());
        
    try {
	String sql = "insert into "+_DefTableReqInputPut+" (rid, desiredPinLifeTime, desiredFileLifeTime, storageSystemInfo, overWriteMode) values (?,?,?,?,?)";

	PreparedStatement psInsert = _conn.prepareStatement(sql);

	psInsert.setString(1,rid);
	psInsert.setInt(2,desiredPinLifeTime);
	psInsert.setInt(3,desiredFileLifeTime);
	psInsert.setString(4,ssiStr);
	psInsert.setString(5,overWriteMode);
							
	psInsert.executeUpdate();      
	_conn.commit();	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");      
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	return;
    }
   	    
    for(int i = 0; i < putReqFiles.length; i++) {
	TPutFileRequest request = putReqFiles[i];
	storeMe(request, rid, i);    
    }

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//store copy request
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void store(SrmCopyRequest copyRequest, String rid, String uid, TStatusCode defaultFileLevelStatus) throws Exception 
{		  
    String evtName = "event=storeCopy";

    String[] strArray={"rid=", rid, " uid=", uid};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);

    ArrayOfTCopyFileRequest arrayOfFileRequests = copyRequest.getArrayOfFileRequests();
    if ((arrayOfFileRequests == null) || (arrayOfFileRequests.getRequestArray() == null)) {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=nullInput");
	return;
    }
    
    int u_id = assertUserOid(uid, evtName);    
    TCopyFileRequest[] copyReqFiles = arrayOfFileRequests.getRequestArray();
	
    storeCommon(rid, copyRequest.getAuthorizationID(),
		copyRequest.getUserRequestDescription(),
		copyRequest.getTargetSpaceToken(), 
		"COPY",
		copyRequest.getTargetFileStorageType(),
		copyRequest.getTargetFileRetentionPolicyInfo(),
		copyRequest.getDesiredTotalRequestTime(),
		copyReqFiles.length, u_id);
		  	

    String ssiStrSrc = TSRMFootprintUtil.getStorageSystemInfoString(copyRequest.getSourceStorageSystemInfo());
    String ssiStrTgt = TSRMFootprintUtil.getStorageSystemInfoString(copyRequest.getTargetStorageSystemInfo());
    
    int desiredTargetSURLLifeTime = TSRMFootprintUtil.getInt(copyRequest.getDesiredTargetSURLLifeTime(), -1);
	      
    String overWriteMode = TSRMFootprintUtil.getString(copyRequest.getOverwriteOption());
    
    try {
	//String sql = "insert into "+_DefTableReqInputCopy (rid, desiredSURLLifeTime,sourceStorageSystemInfo,targetStorageSystemInfo,overWriteMode) values (?,?,?,?,?)";
	String[]  attrs = {"rid", "sourceStorageSystemInfo", "targetStorageSystemInfo", "overWriteMode", "desiredSURLLifeTime"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableReqInputCopy, attrs);
	String[] strV = {rid, ssiStrSrc, ssiStrTgt, overWriteMode};
	int[] intV = {desiredTargetSURLLifeTime};
	TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, null, null);	
/*
	PreparedStatement psInsert = _conn.prepareStatement(sql);

	psInsert.setString(1,rid);
	psInsert.setInt(2,desiredTargetSURLLifeTime);
	psInsert.setString(3,ssiStrSrc);
	psInsert.setString(4,ssiStrTgt);
	psInsert.setString(5,overWriteMode);
	
	psInsert.executeUpdate();
	
	_conn.commit();
*/	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");	
    } catch (Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	return;

    }

    for(int i = 0; i < copyReqFiles.length; i++) {
	//insert into RequestExecution table
	TCopyFileRequest request = copyReqFiles[i];
	storeMe(request, rid, i, defaultFileLevelStatus);
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//store BringOnline request
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void storeReqInput(TTransferParameters transferParameters, String rid) {    
    if (transferParameters == null) {
	return;
    }
    String evtName = "event=storeTransferParameter";
    TSRMLog.info(this.getClass(), null, evtName+"Starts", "rid="+rid);

    HashMap attrs = new HashMap<String, String>();
    addToHashMap(attrs, "rid", rid);
    addToHashMap(attrs, "accessPattern", transferParameters.getAccessPattern());
    addToHashMap(attrs, "connectionType", transferParameters.getConnectionType());
    
    ArrayOfString arrayOfClientNetworks = transferParameters.getArrayOfClientNetworks();    
    ArrayOfString arrayOfTransferProtocols = transferParameters.getArrayOfTransferProtocols();      
    
    String strClientNetwork="";
    if ((arrayOfClientNetworks != null) && (arrayOfClientNetworks.getStringArray() != null)) {
	String[] clientNetworks = arrayOfClientNetworks.getStringArray();    
	strClientNetwork = TSRMFootprintUtil.appendString(clientNetworks, ",");
	addToHashMap(attrs, "clientNetworks", strClientNetwork);
    }
    
    String strTransferProtocol="";
    if ((arrayOfTransferProtocols != null) && (arrayOfTransferProtocols.getStringArray() != null)) {
	String[] transferProtocols = arrayOfTransferProtocols.getStringArray();
	strTransferProtocol = TSRMFootprintUtil.appendString(transferProtocols, ",");	
	addToHashMap(attrs, "transferProtocols", strTransferProtocol);
    }
    
    String[] attrList = new String[attrs.size()];
    attrs.keySet().toArray(attrList);
    String[] strVal = new String[attrs.size()];
    attrs.values().toArray(strVal);

    //String[] attrList = {"rid", "accessPattern", "connectionType", "clientNetworks", "transferProtocols"};
    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableTxfParameter, attrList);
    //String[] strVal = {rid, accessPattern.toString(), connectionType.toString(), strClientNetwork, strTransferProtocol};

    TSRMFootprintUtil.execSql(_conn, evtName, sql, strVal, null, null, null, null);
    TSRMLog.info(this.getClass(), null, evtName+"Ends", "rid="+rid);
}


private void storeCommon(String rid, String authId, String desc, String token,
			 String requestType, Object fileStorageType, TRetentionPolicyInfo rpi,
			 Integer totalRequestTime, int totalNumFileReq, int uid) 
{
    String evtName = "event=storeCommon";
    TSRMLog.info(this.getClass(), null, evtName, "type="+requestType+" rid="+rid);

    HashMap strAttrs = new HashMap<String, String>();
    addToHashMap(strAttrs, "rid", rid);
    addToHashMap(strAttrs, "authID", authId);    
    addToHashMap(strAttrs, "userRequestDesc", desc);
    addToHashMap(strAttrs, "spaceToken", token);
    addToHashMap(strAttrs, "requestLevelInputId", rid);
    addToHashMap(strAttrs, "requestType", requestType);
    addToHashMap(strAttrs, "status", "SRM_REQUEST_QUEUED");
    addToHashMap(strAttrs, "fileType", fileStorageType);    
    addToHashMap(strAttrs, rpi);

    String[] strV = new String[strAttrs.size()];
    strAttrs.values().toArray(strV);

    String[] strKeys = new String[strAttrs.size()];
    strAttrs.keySet().toArray(strKeys);

    int trt = 0;
    if ((totalRequestTime != null) && (totalRequestTime.intValue() > 0)) {    
	trt = totalRequestTime.intValue();
    }

    long ttime = System.currentTimeMillis();
    java.sql.Timestamp current = new java.sql.Timestamp(ttime);
      
    int[] intV = {trt, totalNumFileReq, uid};
    Timestamp[] tsV = {current, current};
    
    String[] otherAttrs = {"desiredTotalRequestTime", "totalNumFilesInRequest", "userId", 
                           "createdTime", "statusTime"};    
    ArrayList both = new ArrayList<String>(strKeys.length + otherAttrs.length);
    Collections.addAll(both, strKeys);
    Collections.addAll(both, otherAttrs);

    String[] attrList = new String[both.size()];

    both.toArray(attrList);
    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableRequest, attrList);
    TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, tsV, null);    
}

public void store(SrmBringOnlineRequest bringOnlineRequest, String rid, String uid) throws Exception 
{		      
    String evtName = "event=storeBOL";
    
    String[] strArray={"rid=", rid, " uid=", uid};
    TSRMLog.info(TSRMFootprintDB.class,null,evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);

    ArrayOfTGetFileRequest arrayOfFileRequests = bringOnlineRequest.getArrayOfFileRequests();	
    if ((arrayOfFileRequests == null) || (arrayOfFileRequests.getRequestArray() == null)) {
	TSRMLog.info(TSRMFootprintDB.class,null,evtName, "status=nullInput");
	return;
    }

    TGetFileRequest[] getReqFiles = arrayOfFileRequests.getRequestArray();
    
    int u_id = assertUserOid(uid, evtName);

    storeCommon(rid, bringOnlineRequest.getAuthorizationID(),
		bringOnlineRequest.getUserRequestDescription(),
		bringOnlineRequest.getTargetSpaceToken(), 
		"BRING_ONLINE", 
		bringOnlineRequest.getDesiredFileStorageType(),
		bringOnlineRequest.getTargetFileRetentionPolicyInfo(),
		bringOnlineRequest.getDesiredTotalRequestTime(),
		getReqFiles.length, u_id);
		  	
    storeReqInput(bringOnlineRequest.getTransferParameters(), rid);

    ResultSet rs = null;  
    
    //insert into RequestLevelInputTable 
    String ssiStr = TSRMFootprintUtil.getStorageSystemInfoString(bringOnlineRequest.getStorageSystemInfo());    	
    int desiredLifeTime = TSRMFootprintUtil.getInt(bringOnlineRequest.getDesiredLifeTime(), -1);
    int deferredStartTime = TSRMFootprintUtil.getInt(bringOnlineRequest.getDeferredStartTime(), 0);
    
    try {
	String sql = "insert into RequestLevelInputForBringOnline (rid, desiredPinLifeTime,deferredStartTime, storageSystemInfo) values (?,?,?,?)";
	    
	PreparedStatement psInsert = _conn.prepareStatement(sql);							      
	
	psInsert.setString(1,rid);
	psInsert.setInt(2,desiredLifeTime);
	psInsert.setInt(3,deferredStartTime);
	psInsert.setString(4,ssiStr);
	
	psInsert.executeUpdate();      
	_conn.commit();	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");      
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	return;
    }
  
    for(int i = 0; i < getReqFiles.length; i++) {
	TGetFileRequest request = getReqFiles[i];
	storeMe(request, rid, i);
    }

    TSRMLog.info(TSRMFootprintDB.class, null,evtName+"Ends", null);
}

private void storeDirOption(TDirOption dirOption, String rid, int pos) throws Exception 
{
    String evtName = "event=storeDirOp";
    if (dirOption == null) {
	TSRMLog.info(this.getClass(), null, evtName, "rid="+rid+" pos="+pos+" status=nullDirOpGiven");
	return;
    }
    TSRMLog.info(this.getClass(), null, evtName, "rid="+rid+" pos="+pos);

    boolean isSourceADirectory = dirOption.isIsSourceADirectory();
    boolean allLevelRecursive = false;
    if (dirOption.getAllLevelRecursive() != null) {
	allLevelRecursive = dirOption.getAllLevelRecursive().booleanValue();
    }
    int numOfLevels = 0;
    if (dirOption.getNumOfLevels() != null) {
	numOfLevels = dirOption.getNumOfLevels().intValue();
    }
    
    String[] attrs = {"rid", "pos", "isSourceADirectory", "allLevelRecursive", "numOfLevels"};
    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableOpDir, attrs); 
	
    
    PreparedStatement psInsert = _conn.prepareStatement(sql);
    
    psInsert.setString(1,rid);
    psInsert.setInt(2, pos);
    psInsert.setBoolean(3,isSourceADirectory);
    psInsert.setBoolean(4,allLevelRecursive);
    psInsert.setInt(5,numOfLevels);

    psInsert.executeUpdate();    
    _conn.commit();
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");
}

	       

private void storeMe(TCopyFileRequest request, String rid, int pos, TStatusCode code) {
    String evtName = "event=storeCopyFileReq";
    TSRMLog.info(this.getClass(), null, evtName, "rid="+rid+" pos="+pos);

    try {	
	TDirOption dirOption = request.getDirOption();
	storeDirOption(dirOption, rid, pos);
	   
	storeFileReqCommon(rid, pos, request.getSourceSURL(), request.getTargetSURL(), null, evtName, code);
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");
    } catch(Exception e) {
	TSRMLog.exception(this.getClass(), evtName, e);
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

private void storeMe(TPutFileRequest request, String rid, int pos) {
    String evtName = "event=storePutFileReq";
    TSRMLog.info(this.getClass(), null, evtName, "rid="+rid+" pos="+pos);

    try {
	org.apache.axis.types.UnsignedLong expectedFileSize = request.getExpectedFileSize();	    

	storeFileReqCommon(rid, pos, null, request.getTargetSURL(), expectedFileSize, evtName, null);
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");

	/*	
	String cmd = "insert into RequestExecution " + 
	    "(rid, pos, status, expectedFileSize, statusTime) values (?,?,?,?,?)";	    
		
	long tt = System.currentTimeMillis();
	java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(tt);

	PreparedStatement psInsert = _conn.prepareStatement(cmd);
	
	psInsert.setString(1,rid);
	psInsert.setInt(2,pos);
	psInsert.setString(3,"SRM_REQUEST_QUEUED");
	psInsert.setLong(4,expectedFileSize.longValue());
	psInsert.setTimestamp(5,queuedTimeStamp);

	psInsert.executeUpdate();	
	_conn.commit();
	
	TSRMLog.info(this.getClass(), null, evtName, "action=committed");
	*/
    } catch(Exception e) {
	TSRMLog.exception(this.getClass(), evtName, e);
    } finally {
	TSRMLog.info(this.getClass(), null, evtName+"Ends", null);
    }
}
    
private void storeMe(TGetFileRequest request, String rid, int pos) throws Exception  
{
    String evtName = "event=storeGetFileReq";
    TSRMLog.info(this.getClass(), null, evtName, "rid="+rid+" pos="+pos);

    try {	
	TDirOption dirOption = request.getDirOption();
	storeDirOption(dirOption, rid, pos);

	storeFileReqCommon(rid, pos, request.getSourceSURL(), null, null, evtName, null);
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "action=commited");
    } catch(Exception e) {
	TSRMLog.exception(this.getClass(), evtName, e);
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

private void storeFileReqCommon(String rid, int pos, URI fromSurl, URI toSurl, 
				org.apache.axis.types.UnsignedLong expectedSize, String evtName,
				TStatusCode code) throws Exception 
{
    long tt = System.currentTimeMillis();
    java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(tt);
    
    int[] intV = {pos};	  
    java.sql.Timestamp[] tsV = {queuedTimeStamp};
    
    String[] attrs = {"rid", "status", "fromSURL", "toSURL", "pos", "statusTime", "expectedFileSize"};	
    String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableFileRequest, attrs);	    
    String fromSurlStr = "";
    if (fromSurl != null) {
	fromSurlStr = fromSurl.toString();
    }
    String toSurlStr = "";
    if (toSurl != null) {
	toSurlStr = toSurl.toString();
    }

    long[] longV = new long[1];
    if (expectedSize != null) {
	longV[0] = expectedSize.longValue();
    } else {
	longV[0] = -1;
    }

    String codeStr = "SRM_REQUEST_QUEUED";
    if (code != null) {
	codeStr = code.toString();
    }
    String[] strV = {rid, codeStr, fromSurlStr, toSurlStr};
    TSRMFootprintUtil.execSql(_conn, evtName, sql, strV, null, intV, tsV, longV);	  
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//srmGetRequestTokens
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public TRequestTokenReturn[] srmGetRequestTokens (String userRequestDesc, String userId) throws Exception     
{
  String evtName = "event=srmGetRequestTokens";

  String[] strArray = {"userrequestdesc=", userRequestDesc, " uid=", userId};
  TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));

  TSRMFootprintUtil.noTrivialString("userRequestDesc", userRequestDesc, evtName);
  TSRMFootprintUtil.noTrivialString("uid", userId, evtName);

  int u_id = assertUserOid(userId, evtName);

  Vector vec = new Vector ();
  String[] attrs = {"rid", "createdTime"};
  String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableRequest, attrs, "userRequestDesc", userRequestDesc);
  sql += " and userId="+u_id;
  ResultSet rs = _statement.executeQuery(sql);

  while(rs.next()) {
      TRequestTokenReturn tokenReturn = new TRequestTokenReturn();
      tokenReturn.setRequestToken(rs.getString(1));
      tokenReturn.setCreatedAtTime (TSRMFootprintUtil.createCalendar(rs.getTimestamp(2)));
      vec.addElement(tokenReturn);
  }
  rs.close();
  
  TRequestTokenReturn[] tokens = new TRequestTokenReturn[vec.size()];
  for(int i = 0; i < vec.size(); i++) {
      TRequestTokenReturn curr = (TRequestTokenReturn)vec.elementAt(i);
      tokens[i] = curr;
  }
  
  TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);

  return tokens;
}


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//srmGetRequestSummary
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public TRequestSummary[] srmGetRequestSummary (String[] requestToken,  String userId)  throws Exception
{ 
    String evtName="event=getRequestSummary";
  
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "uid="+userId);
    
    TSRMFootprintUtil.noTrivialArray("requestToken", requestToken, evtName);
    TSRMFootprintUtil.noTrivialString("uid", userId, evtName);
    
    int u_id = assertUserOid(userId, evtName);

    if (requestToken == null) {
	return null;
    }

    TRequestSummary[] summaries = new TRequestSummary[requestToken.length];

    String sqlBase = "SELECT requestType, status, explanation,totalNumFilesInRequest,"+
	" numOfCompletedFiles, numOfWaitingFiles, numOfFailedFiles from Request " +
	" where userId="+u_id;

    for (int i = 0; i < requestToken.length; i++) {
	String rid = requestToken[i];	
	ResultSet rs = _statement.executeQuery(sqlBase + " and rid="+"\'"+rid+"\'");
	
	summaries[i]= new TRequestSummary();
	summaries[i].setRequestToken(rid);
	
	if(rs.next()) {
	    TReturnStatus returnStatus = new TReturnStatus();
	    TStatusCode sCode = TStatusCode.fromValue(rs.getString(2));
	    returnStatus.setStatusCode(sCode);
	    returnStatus.setExplanation(rs.getString(3));
	    
	    summaries[i].setStatus(returnStatus);

	    TRequestType rType = TRequestType.fromValue(rs.getString(1).toUpperCase());
	    summaries[i].setRequestType(rType);

	    summaries[i].setTotalNumFilesInRequest(rs.getInt(4));
	    summaries[i].setNumOfCompletedFiles(rs.getInt(5));
	    summaries[i].setNumOfWaitingFiles(rs.getInt(6));
	    summaries[i].setNumOfFailedFiles(rs.getInt(7));	    
	} else {
	    summaries[i].setStatus(TSRMUtil.createReturnStatus(TStatusCode.SRM_INVALID_REQUEST, "no such rid:"+rid));
	}
	
	rs.close();
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    return summaries;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//srmExtendFileLifeTime
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public TSURLLifetimeReturnStatus[] srmExtendFileLifeTime (String rid, String[] surls, int newPinLifeTime, int newFileLifeTime) 
    throws Exception 
{    
    String evtName="event=extendFileLT";
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid);
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);    
    TSRMFootprintUtil.noTrivialArray("surls", surls, evtName);
    
   TSURLLifetimeReturnStatus[] lifeTimeStatus = new TSURLLifetimeReturnStatus[surls.length];       
   
   //extend the newPinLifeTime for the surls in the RequestExec table
   for(int i = 0; i < surls.length; i++) {
       ResultSet rs = _statement.executeQuery("SELECT status,explanation from "+
					      " RequestExecution where rid="+"\'"+rid+"\'" + " and "+
					      " fromSURL="+"\'"+surls[i]+"\'");
       lifeTimeStatus[i] = new TSURLLifetimeReturnStatus();
       if(rs.next()) {
	   String status = rs.getString(1);
	   String explanation = rs.getString(2);

	   TReturnStatus rStatus = new TReturnStatus();
	   TStatusCode sCode = TStatusCode.fromValue(status);
	   rStatus.setStatusCode(sCode);
	   rStatus.setExplanation(explanation);

	   lifeTimeStatus[i].setStatus(rStatus);	   
	   lifeTimeStatus[i].setSurl(new URI(surls[i]));
	   lifeTimeStatus[i].setFileLifetime(newFileLifeTime);
	   lifeTimeStatus[i].setPinLifetime(newPinLifeTime);
	   
	   StringBuffer buf = new StringBuffer();
	   buf.append("update RequestExecution set pinLifeTime=? where rid =? and fromSURL=?");		      
	   
	   PreparedStatement psUpdate = _conn.prepareStatement(buf.toString());	       
	   
	   psUpdate.setInt(1,newPinLifeTime);
	   psUpdate.setString(2,rid);
	   psUpdate.setString(3,(surls[i]).toString());
	   psUpdate.executeUpdate();
	   _conn.commit();
       } else {       
	   String[] errArray = {"No matching rows found for rid=", rid, "surl=", surls[i]};
	   String err = TSRMFootprintUtil.appendString(errArray);
	   TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
       }
       rs.close();
   }
   
   //extend the newFileLifeTime for the surls in the Files table
   
   for(int i = 0; i < surls.length; i++) {
       long hashValue = getHashValue(surls[i]);
       String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileAttr, "path", "hashValue="+hashValue);
       ResultSet rs = _statement.executeQuery(sql);
					      
       if (rs.next()) {
	   sql = TSRMFootprintUtil.generateSqlUpdate(_DefTableFileAttr, "lifeTimeAssigned", "hashValue");
	   PreparedStatement psUpdate = _conn.prepareStatement(sql);	       
	   
	   psUpdate.setInt(1,newFileLifeTime);
	   psUpdate.setLong(2,hashValue);
	   psUpdate.executeUpdate();
	   _conn.commit();
       } else {       
	   String err="No matching rows found in FileAttributes table for"+ surls[i];
	   TSRMLog.error(TSRMFootprintDB.class, null, evtName, err);
       }
       rs.close();
   }
   
   TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
   
   return lifeTimeStatus;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//deleteRow
//needs to implement more in this
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private String getRequestTableName(String requestType) {
    if(requestType.equalsIgnoreCase("LS")) {
	return "RequestLevelInputForLs";
    } else if (requestType.equalsIgnoreCase("PREPARE_TO_GET")) {
	return _DefTableReqInputGet;
    } else if (requestType.equalsIgnoreCase("PREPARE_TO_PUT")) {
	return _DefTableReqInputPut;
    } else if (requestType.equalsIgnoreCase("COPY")) {
	return _DefTableReqInputCopy;
    } else if (requestType.equalsIgnoreCase("BRING_ONLINE")) {
	return "RequestLevelInputForBringOnline";
    }
    throw new RuntimeException("No such request type!");
}

public boolean deleteRow(String rid) throws Exception {

    String evtName="deleteRow";
    TSRMLog.info(TSRMFootprintDB.class, null,evtName+"Starts", "rid="+rid);
    
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    
    PreparedStatement psDelete = _conn.prepareStatement(TSRMFootprintUtil.generateSqlDelete(_DefTableFileRequest, "rid"));    
    psDelete.setString(1,rid);
    
    psDelete.executeUpdate();
    _conn.commit();
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=deleteRidFromFileReqCommited");
   
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableRequest, "requestType", "rid", rid);
    ResultSet rs = _statement.executeQuery(sql);

    String requestType = null;
    if(rs.next()) {
	requestType = rs.getString(1);	
    } 
    rs.close();
    
    if (requestType == null) {
	return false;
    }
    //
    //
    sql = "DELETE from "+getRequestTableName(requestType)+" where rid=?";
    
    psDelete = _conn.prepareStatement(sql);
    psDelete.setString(1,rid);
    
    psDelete.executeUpdate();
    _conn.commit();
    
    TSRMLog.info(TSRMFootprintDB.class,"committed",evtName, "query="+sql);

    //
    //
    psDelete = _conn.prepareStatement("DELETE from Request where rid=?");
    
    psDelete.setString(1,rid);
    psDelete.executeUpdate();
    _conn.commit();

    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return true;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//Basic Api's
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// deleteUser
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean deleteUser(String uid) throws Exception {
    String evtName = "event=deleteUser";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "uid="+uid);
    
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    
    String sql = TSRMFootprintUtil.generateSqlDelete(_DefTableUser, "uid");
    PreparedStatement psDelete = _conn.prepareStatement(sql);
    psDelete.setString(1,uid);
    
    psDelete.executeUpdate();
    _conn.commit();    
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return true;
}

private int assertRid(String rid, String evtName) throws Exception 
{ 
    ResultSet rs = _statement.executeQuery("SELECT sortId from Request where rid="+"\'"+rid+"\'");		 
    
    try {
	int sortId = -1;
	if (rs.next()) {
	    sortId = rs.getInt(1);
	}
	rs.close();
	if (sortId == -1) {
	    String err = "No request: rid="+rid;
	    TSRMLog.info(TSRMFootprintDB.class, null, evtName, err);
	    throw new TSRMFootprintDBException(err);
	}
	return sortId;
    } finally {
	rs.close();
    }
}

private void assertFileReq(String rid, int pos, String evtName) throws Exception 
{
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, "rid", " rid="+"\'"+rid+"\'" + " and pos="+pos);
    ResultSet rs = _statement.executeQuery(sql);					   

    try {
	if (!rs.next()) {
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName,"error=CouldNotFindMatchingRowToUpdate");
	    throw new TSRMFootprintDBException("Could not find a matching row to update for rid="+rid+" pos="+pos); 
	}
    } finally {
	rs.close();
    }    
}
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//deleteFromBandwidthCommited
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean deleteFromBandwidthCommited (String rid) throws Exception 
    
{
    String evtName = "event=deleteFromBandwidthCommited";
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid);
    
    Vector vec = new Vector ();
    int sortId = assertRid(rid, evtName);
    
    try {
	String sql = TSRMFootprintUtil.generateSqlDelete(_DefTableBandwidth, "sortId");
	PreparedStatement psDelete = _conn.prepareStatement(sql);
	
	psDelete.setInt(1,sortId);
	psDelete.executeUpdate();
	_conn.commit();
	
	return true;
    } catch(Exception e) {    
	TSRMLog.exception(TSRMFootprintDB.class, "rid="+rid, e);
	return false;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getAttributesFromBandwidthCommited
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public TBandwidthResponseParameters[] getAttributesFromBandwidthCommited (String terapathRid) throws Exception 
{    
    String evtName = "event=getAttributesFromBandwidthCommited";   
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "terapathRid="+terapathRid);	       
    
    Vector vec = new Vector ();
    
    String[] attrs = {"timeStart", "timeEnd", "bandwidthReservedkbps", "pos"};
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableBandwidth, attrs, "bandwidthRid", terapathRid);
    sql += " order by pos";

    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    while(rs.next()) {
	TBandwidthResponseParameters curr = new TBandwidthResponseParameters(rs.getLong(3), 
									     rs.getTimestamp(1).getTime(), 
									     rs.getTimestamp(2).getTime(), null, null, null);	
	vec.add(curr);
    }
    rs.close();
    
    TSRMLog.info(TSRMFootprintDB.class,"end", evtName, null);

    TBandwidthResponseParameters[] result = new TBandwidthResponseParameters[vec.size()];
    vec.toArray(result);
    return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//insertIntoBandwidthCommited
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void insertIntoBandwidthCommited(TResponse r) throws Exception     
{
    String evtName = "event=insertIntoBandwidthCommited";
    String[] strArray = {"terapathRid=", r.getRequestId()};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    try { 
	String[] attrs = {"bandwidthRid", "pos", "timeStart", "timeEnd", "bandwidthReservedkbps"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableBandwidth, attrs);
	
	for (int i=0; i<r.getArrayOfReservationData().length; i++) {
	    PreparedStatement psInsert = _conn.prepareStatement(sql);
	    psInsert.setString(1,r.getRequestId());
	    psInsert.setInt(2, i);
	    TBandwidthResponseParameters curr = r.getArrayOfReservationData()[i];
	    psInsert.setTimestamp(3,new java.sql.Timestamp(curr.getBeginTime()));
	    psInsert.setTimestamp(4,new java.sql.Timestamp(curr.getEndTime()));
	    psInsert.setLong(5,curr.getAvailableBandwidth()/(long)1000);
	    psInsert.executeUpdate();
	    _conn.commit();		
	}
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class,"insertIntoBandwidthCommited",e);
	throw new TSRMFootprintDBException(e.getMessage());					
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}							    
/*
public void insertIntoBandwidthCommited(String rid, long ts, long te, long bandwidthwindow) throws Exception     
{
    String evtName = "event=insertIntoBandwidthCommited";
    String[] strArray = {"rid=", rid," ts="+ts, " te="+te, " window="+bandwidthwindow};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    int sortId = assertRid(rid, evtName);
    
    try { 
	String[] attrs = {"sortId", "timestart", "timeend", "bandwidthwindow"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableBandwidth, attrs);
	PreparedStatement psInsert = _conn.prepareStatement(sql);
	
	psInsert.setInt(1,sortId);
	psInsert.setTimestamp(2,new java.sql.Timestamp(ts));
	psInsert.setTimestamp(3,new java.sql.Timestamp(te));
	psInsert.setLong(4,bandwidthwindow);
	psInsert.executeUpdate();
	_conn.commit();		
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class,"insertIntoBandwidthCommited",e);
	throw new TSRMFootprintDBException(e.getMessage());					
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}
*/

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// addUser
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean addUser(String uid) throws Exception     
{
    String evtName="addUser";    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "uid="+uid);
    
    TSRMFootprintUtil.noTrivialString("uid", uid, evtName);
    
    if (findUserOid(uid, evtName) > 0) {
	return false;
    }
    
    try { 
	String[] attrs = {"uid"};
	String sql = TSRMFootprintUtil.generateSqlInsert(_DefTableUser, attrs);
	PreparedStatement psInsert = _conn.prepareStatement(sql); //"insert into User " + "(uid) values (?)");
	
	psInsert.setString(1,uid);
	psInsert.executeUpdate();
	_conn.commit();
	
	TSRMLog.info(TSRMFootprintDB.class,"committed", evtName, null);
	return true;
    } catch(Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	return false;	
    } finally {    
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getAttributesFromFileAttributes
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public Hashtable getAttributesFromFileAttributes(String path) throws Exception 
{   
    String evtName="event=getAttrFromFileAttr";    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "path="+path);
    
    TSRMFootprintUtil.noTrivialString("path", path, evtName);
    
    Hashtable ht = new Hashtable();    
    long hashValue = getHashValue(path);
    
    int fileAttributesId = -1;      
    ResultSet rs = _statement.executeQuery("SELECT path, size, createdAtTime,"+
					   "lastModificationTime, fileStorageType, retentionPolicy, " +
					   "accessLatency,fileLocality,type,lifeTimeAssigned,"+
					   "lifeTimeEndsAt,ownerPermission,userID,groupPermission,groupID,"+
					   "otherPermission,checkSumType,checkSumValue,fileAttributesId "+
					   " From "+_DefTableFileAttr+" where hashValue="+ hashValue);
    
    
    if (rs.next()) {
	ht.put("path",rs.getString(1));
	ht.put("size",new Long(rs.getLong(2)));
	if(rs.getTimestamp(3) != null) {
	    ht.put("createdattime",rs.getTimestamp(3));
	}
	if(rs.getTimestamp(4) != null) {
	    ht.put("lastmodificationtime",rs.getTimestamp(4));
	}
	if(rs.getString(5) != null) {
	    ht.put("filestoragetype",rs.getString(5));
	}
	if(rs.getString(6) != null) {
	    ht.put("retentionpolicy",rs.getString(6));
	}  
	if(rs.getString(7) != null) {
	    ht.put("accesslatency",rs.getString(7));
	}
	if(rs.getString(8) != null) {
	    ht.put("filelocality",rs.getString(8));
	}
	if(rs.getString(9) != null) {
	    ht.put("type",rs.getString(9));
	}
	ht.put("lifetimeassigned",new Integer(rs.getInt(10)));
	ht.put("lifetimeEndsAt",new Integer(rs.getInt(11)));
	if(rs.getString(12) != null) {
	    ht.put("ownerpermission",rs.getString(12));
	}
	if(rs.getString(13) != null) {
	    ht.put("userid",rs.getString(13));
	}
	if(rs.getString(14) != null) {
	    ht.put("grouppermission",rs.getString(14));
	}
	if(rs.getString(15) != null) {
	    ht.put("groupid",rs.getString(15));
	}
	if(rs.getString(16) != null) {
	    ht.put("otherpermission",rs.getString(16));
	} 
	if(rs.getString(17) != null) {
	    ht.put("checksumtype",rs.getString(17));
	}
	if(rs.getString(18) != null) {
	    ht.put("checksumvalue",rs.getString(18));
	}
	fileAttributesId = rs.getInt(19);
    }
    rs.close();
    
    Vector vec = getSpaceTokens(fileAttributesId);
    String str = "";
    for(int i = 0; i < vec.size(); i++) { 
	if (i > 0) {
	    str = str + ",";
	}
	str = str + (String)vec.elementAt(i); 
    }
    
    ht.put("spacetokens",str);
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    return ht;
}

// 
// get status from specified file requests
//

private String getCond(URI[] from, URI[] to) {    
    if ((from == null) && (to == null)) {
	return null;
    }
    
    String prefixFrom = "fromSURL = \'";
    String prefixTo = "toSURL = \'";
    if ((from != null) && (to != null)) {
	TSRMLog.info(TSRMFootprintDB.class, null, "event=getCond()", "fromSize="+from.length+" toSize="+to.length);
	if (from.length != to.length) {
	    TSRMLog.error(TSRMFootprintDB.class, null, "event=getCond()", "error=incompatical to/from urls");
	    return null;
	}
	String cond = "";
	for (int i=0; i<from.length; i++) {
	    TSRMLog.info(TSRMFootprintDB.class, null, "event=getCond()", "from="+from[i]+" to="+to[i]);
	    cond = "("+prefixFrom+from[i].toString()+"\' AND "+prefixTo+to[i].toString()+"\')";
	    if (i<from.length-1) {
		cond +=" OR ";
	    }		
	}
	return cond;
    }

    if (from != null) {
	return generateCond(from, prefixFrom);
    } else {
	return generateCond(to, prefixTo);
    }
}

private String generateCond(URI[] surlArray, String prefix) {
    if ((surlArray == null) || (surlArray.length == 0)) {
	return null;
    }

    String cond = "";
        
    for (int i=0; i<surlArray.length; i++) {
	cond  += prefix+surlArray[i].toString()+"\'"; 	
	if (i<surlArray.length-1) {
	    cond +=" OR ";
	}
    }
    
    return cond;
}

public void activate(String rid) {
    String evtName = "event=activate";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid);
    
    try {
	String str = "update RequestExecution set status=? where rid=? ";
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "query="+str);
	
	PreparedStatement psUpdate = _conn.prepareStatement(str);
	psUpdate.setString(1, "SRM_REQUEST_QUEUED");
	psUpdate.setString(2, rid);

       	psUpdate.executeUpdate();
	_conn.commit();
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), evtName, e);
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"End", "rid="+rid);
    }
}

public Vector getFileStatusesOfRequest(String rid, URI[] fromSurlArray, URI[] toSurlArray) {
    String evtName = "event=getFileStatusesOfRequest";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid);
    
    Vector result = new Vector();
    try {
	Statement s2 = _conn.createStatement();
	String[] attrs = {"status", "explanation", "TURL", "statusTime", "fromSURL", "toSURL", "rid", "actualFileSize"};
	_aux.checkFileReqEntries(s2);
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, attrs, "rid", rid);
	String cond = getCond(fromSurlArray, toSurlArray);
	if (cond != null) {
	    sql += " and (" + cond + ")";
	}
	/*	
	    sql += " and (";
	    String prefix = "fromSURL = \'";
	    if (rid.contains("PUT")) {
		prefix = "toSURL = \'";
	    }
	    for (int i=0; i<surlArray.length; i++) {
		sql += prefix+surlArray[0].toString()+"\'"; 
		if (i<surlArray.length-1) {
		    sql +=" OR ";
		}
	    }
	    sql += " )";
	*/
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "sql="+sql);
	ResultSet rs = s2.executeQuery(sql);
	
	while (rs.next()) {
	    TStatusCode code = TStatusCode.fromValue(rs.getString(1));	    
	    String exp = rs.getString(2);
	    URI turl = TSRMUtil.createTSURL(rs.getString(3));	  	    		
	    URI from = TSRMUtil.createTSURL(rs.getString(5));
	    URI to = TSRMUtil.createTSURL(rs.getString(6));
	    long actualSize = rs.getLong(8);
	    result.add(createFileStatus(rid, code, exp, turl, rs.getTimestamp(4), from, to, actualSize));
	}
	rs.close();
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "resultSize="+result.size());
	return result;
    } catch (Exception e) {
	TSRMLog.exception(this.getClass(), evtName, e);
	return null;
    } finally {
	TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "rid="+rid);
    }
}

private Object createFileStatus(String rid, TStatusCode code, String exp, URI turl, 
				java.sql.Timestamp statusTime, URI from, URI to, 
				long actualSize) 
{
    if (rid.contains("PUT")) {
	TPutRequestFileStatus s = new TPutRequestFileStatus();
	s.setStatus(TSRMUtil.createReturnStatus(code, exp));
	if (code == TStatusCode.SRM_SPACE_AVAILABLE) {
	    s.setTransferURL(turl);	
	}
	s.setSURL(to);
	return s;
    } else if (rid.contains("GET")) {
	TGetRequestFileStatus s = new TGetRequestFileStatus();
	s.setStatus(TSRMUtil.createReturnStatus(code, exp));
	if (code == TStatusCode.SRM_FILE_PINNED) {
	    s.setTransferURL(turl);
	}
	s.setSourceSURL(from);
	return s;
    } else if (rid.contains("COPY")) {
	TCopyRequestFileStatus s = new TCopyRequestFileStatus();
	s.setStatus(TSRMUtil.createReturnStatus(code, exp));
	s.setSourceSURL(from);
	s.setTargetSURL(to);
	if (code == TStatusCode.SRM_SUCCESS) {
	    s.setFileSize(TSRMUtil.createTSizeInBytes(actualSize));
	} else {
	    s.setFileSize(TSRMUtil.createTSizeInBytes(0));
	}
	return s;
    }    
    throw new TSRMException("Unknown type to construct:"+rid, false);
}

private TSRMFileInput getFileInput(String rid, int pos) throws Exception 
{
    int p = rid.indexOf("COPY");
    if (p > 0) {
	SrmCopyRequest req = getReqLevelInputCopy(rid);
	TCopyFileRequest curr = getFileLevelInputCopy(rid, pos);
	return new TSRMFileInputCopy(req, curr, pos);
    }
    p = rid.indexOf("GET");
    if (p > 0) {
	SrmPrepareToGetRequest req = getReqLevelInputGet(rid);
	TGetFileRequest curr = getFileLevelInputGet(rid, pos);
	return new TSRMFileInput(req, curr, pos);	
    }

    p = rid.indexOf("PUT");
    if (p > 0) {
	SrmPrepareToPutRequest req = getReqLevelInputPut(rid);
	TPutFileRequest curr = getFileLevelInputPut(rid, pos);
	return new TSRMFileInput(req, curr, pos);
    }
    TSRMLog.info(this.getClass(), null, "event=noMatchFound", "rid="+rid);
    return null;
}

private TGetFileRequest getFileLevelInputGet(String rid, int pos) throws Exception {
    ResultSet rs = null;
    try {
	TDirOption dirOption = restoreDirOption(rid, pos);

	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, "fromSURL", "rid", rid);
	sql += " and pos="+pos;

	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	
	if (!rs.next()) {
	    return null;
	}

	URI src = TSRMFootprintUtil.uriFromString(rs.getString(1));

	return new TGetFileRequest(src, dirOption);
    } catch (Exception e) {
	e.printStackTrace(); return null;
    } finally {
	rs.close();
    }
}

private TPutFileRequest getFileLevelInputPut(String rid, int pos) throws Exception {
    ResultSet rs = null;
    try {
	String[] attrs = {"toSURL", "expectedFileSize"};	
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, attrs, "rid", rid);
	sql += " and pos="+pos;

	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	
	if (!rs.next()) {
	    return null;
	}

	URI tgt = TSRMFootprintUtil.uriFromString(rs.getString(1));
	org.apache.axis.types.UnsignedLong expectedFileSize = TSRMFootprintUtil.getValue(rs.getLong(2), -1);
	return new TPutFileRequest(tgt, expectedFileSize);
    } catch (Exception e) {
	e.printStackTrace(); return null;
    } finally {
	rs.close();
    }
}

private TCopyFileRequest getFileLevelInputCopy(String rid, int pos) throws Exception {
    ResultSet rs = null;
    try {
	TDirOption dirOption = restoreDirOption(rid, pos);

	String[] attrs = {"fromSURL", "toSURL"};	
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, attrs, "rid", rid);
	sql += " and pos="+pos;

	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	
	if (!rs.next()) {
	    return null;
	}

	URI src = TSRMFootprintUtil.uriFromString(rs.getString(1));
	URI tgt = TSRMFootprintUtil.uriFromString(rs.getString(2));

	return new TCopyFileRequest(src, tgt, dirOption);
    } catch (Exception e) {
	e.printStackTrace(); return null;
    } finally {
	rs.close();
    }
}

private TDirOption restoreDirOption(String rid, int pos) throws Exception {
    ResultSet rs = null;
    
    try {		
	String[] attrs = {"isSourceADirectory", "allLevelRecursive", "numOfLevels"};
	
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableOpDir, attrs, "rid", rid);
	sql += " and pos="+pos;
	System.out.println(sql);

	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	if (!rs.next()) {
	    return null;
	}
	boolean isSourceADirectory = rs.getBoolean(1);
	boolean allLevelRecursive = rs.getBoolean(2);
	int numOfLevels = rs.getInt(3);
	System.out.println("  "+isSourceADirectory+"  "+allLevelRecursive+" "+numOfLevels);
	return new TDirOption(isSourceADirectory, allLevelRecursive, numOfLevels);
    } finally {
	rs.close();
    } 
}

private HashMap getReqLevelInputCommon(String rid) throws Exception {
    ResultSet rs = null;

    try {			
	String[] attrs = {"authID", "userRequestDesc", "spaceToken", "targetFileRetentionPolicy", 
			  "targetFileAccessLatency", "fileType", "desiredTotalRequestTime"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableRequest, attrs, "rid", rid);
	Statement stmt = _conn.createStatement();
	
	rs = stmt.executeQuery(sql);

	if (!rs.next()) {
	    return null;
	}

	HashMap v = new HashMap();
	v.put("authId", rs.getString(1));
	v.put("desc", rs.getString(2));
	v.put("targetSpaceToken", rs.getString(3));
	v.put("targetRPI", TSRMFootprintUtil.rpiFromString(rs.getString(4), rs.getString(5)));
	v.put("targetFST", TSRMFootprintUtil.fstFromString(rs.getString(6)));
	v.put("totalReqTime", TSRMFootprintUtil.getValue(rs.getInt(7), -1));
	
	return v;
    } finally {
	rs.close();
    }
}

private SrmPrepareToPutRequest getReqLevelInputPut(String rid) throws Exception {
    ResultSet rs = null;

    try {	
	HashMap v = getReqLevelInputCommon(rid);
	if (v == null) {
	    return null;
	}
	String attrList[] = {"storageSystemInfo", "desiredPinLifeTime", "desiredFileLifeTime", "overwriteMode"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableReqInputPut, attrList, "rid", rid);
	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	if (!rs.next()) {
	    return null;
	}
	ArrayOfTExtraInfo strSSI = TSRMFootprintUtil.ssiFromString(rs.getString(1));
	Integer targetPinLifetime = TSRMFootprintUtil.getValue(rs.getInt(2), -1);
	Integer targetTransitLifetime = TSRMFootprintUtil.getValue(rs.getInt(3), -1);
	TOverwriteMode overwriteMode = TSRMFootprintUtil.owFromString(rs.getString(4));
	
	return new SrmPrepareToPutRequest((String)(v.get("authId")), null, (String)(v.get("desc")), overwriteMode, 
					  strSSI,(Integer)(v.get("totalReqTime")), 
					  targetPinLifetime, targetTransitLifetime, 
					  (TFileStorageType)(v.get("targetFST")), 
					  (String)(v.get("targetSpaceToken")), 
					  (TRetentionPolicyInfo)(v.get("targetRPI")), null); 

    } finally {	
	rs.close();
    }
}

private SrmPrepareToGetRequest getReqLevelInputGet(String rid) throws Exception {
    ResultSet rs = null;

    try {	
	HashMap v = getReqLevelInputCommon(rid);
	if (v == null) {
	    return null;
	}
	String attrList[] = {"storageSystemInfo", "desiredPinLifeTime"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableReqInputGet, attrList, "rid", rid);
	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	if (!rs.next()) {
	    return null;
	}
	ArrayOfTExtraInfo strSSI = TSRMFootprintUtil.ssiFromString(rs.getString(1));
	Integer targetPinLifetime = TSRMFootprintUtil.getValue(rs.getInt(2), -1);
	
	return new SrmPrepareToGetRequest((String)(v.get("authId")), null, (String)(v.get("desc")), strSSI,
					  (TFileStorageType)(v.get("targetFST")),					  
					  (Integer)(v.get("totalReqTime")), targetPinLifetime,
					  (String)(v.get("targetSpaceToken")), 
					  (TRetentionPolicyInfo)(v.get("targetRPI")), null); 

    } finally {	
	rs.close();
    }
}


private SrmCopyRequest getReqLevelInputCopy(String rid) throws Exception {
    ResultSet rs = null;

    try {	
	/*
	String[] attrs = {"authID", "userRequestDesc", "spaceToken", "targetFileRetentionPolicy", 
			  "targetFileAccessLatency", "fileType", "desiredTotalRequestTime"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableRequest, attrs, "rid", rid);
	
	rs = stmt.executeQuery(sql);

	if (!rs.next()) {
	    return null;
	}
	String authId = rs.getString(1);
	String desc = rs.getString(2);
	String targetSpaceToken = rs.getString(3);
	TRetentionPolicyInfo targetRPI = TSRMFootprintUtil.rpiFromString(rs.getString(4), rs.getString(5));
	TFileStorageType targetFST = TSRMFootprintUtil.fstFromString(rs.getString(6));
	Integer desiredTotalReqTime = TSRMFootprintUtil.getValue(rs.getInt(7), -1);
	*/
	HashMap v = getReqLevelInputCommon(rid);
	if (v == null) {
	    return null;
	}
	String attrList[] = {"sourceStorageSystemInfo", "targetStorageSystemInfo", "overWriteMode", "desiredSURLLifeTime"};
	String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableReqInputCopy, attrList, "rid", rid);
	Statement stmt = _conn.createStatement();
	rs = stmt.executeQuery(sql);
	if (!rs.next()) {
	    return null;
	}
	TOverwriteMode overwriteMode = TSRMFootprintUtil.owFromString(rs.getString(3));
	ArrayOfTExtraInfo srcSSI = TSRMFootprintUtil.ssiFromString(rs.getString(1));
	ArrayOfTExtraInfo tgtSSI = TSRMFootprintUtil.ssiFromString(rs.getString(2));
	Integer desiredTargetSURLLifetime = TSRMFootprintUtil.getValue(rs.getInt(4), -1);
	
	/*return new SrmCopyRequest(authId, null, desc, overwriteMode, desiredTotalReqTime,
	                            desiredTargetSURLLifetime, targetFST, targetSpaceToken, 
				  targetRPI, srcSSI, tgtSSI);
	*/
	return new SrmCopyRequest((String)(v.get("authId")), null, (String)(v.get("desc")), overwriteMode, 
				  (Integer)(v.get("totalReqTime")), desiredTargetSURLLifetime, 
				  (TFileStorageType)(v.get("targetFST")), (String)(v.get("targetSpaceToken")), 
				  (TRetentionPolicyInfo)(v.get("targetRPI")), srcSSI, tgtSSI);
    } finally {	
	rs.close();
    }
}
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getAttributesFromRequestExec
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public Hashtable getAttributesFromRequestExec (String rid, int pos) throws Exception
{
    String evtName="event=getAttrFromReqExec";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "rid="+rid+" pos="+pos);    
    TSRMFootprintUtil.noTrivialString("rid=", rid, evtName);
    
    Hashtable ht = new Hashtable();
    
    Statement s2 = _conn.createStatement();
    String[] listAttr = {"isSourceADirectory","allLevelRecursive", "numOfLevels"};
    String sql  = TSRMFootprintUtil.generateSqlSelect(_DefTableOpDir, listAttr, "rid", rid);
    sql += " and pos="+pos;
    ResultSet rs = s2.executeQuery(sql);

    if(rs.next()) {
	ht.put("issourceadirectory", new Boolean(rs.getBoolean(1)));
	ht.put("alllevelrecursive", new Boolean(rs.getBoolean(2)));
	ht.put("numoflevels", new Integer(rs.getInt(3)));
    }
    rs.close();
    
    String[] attrs = {"expectedFileSize", "TURL", "pinLifeTime", "statusTime", "status", "explanation"};
    sql = TSRMFootprintUtil.generateSqlSelect(_DefTableFileRequest, attrs, "rid="+"\'"+rid+"\'"+ " and pos="+pos);
    rs = s2.executeQuery(sql);			 
    
    if(rs.next()) {
	ht.put("expectedfilesize", new Long(rs.getLong(1)));
	if(rs.getString(2) != null) {
	    ht.put("turl", rs.getString(2));
	}
	if (rs.getString(3) != null) {
	    ht.put("pinlifetime", new Integer(rs.getInt(3)));
	}
	if(rs.getTimestamp(4) != null) {
	    ht.put("statustime", rs.getTimestamp(4));
	}
	if(rs.getString(5) != null) {
	    ht.put("status", rs.getString(5));
	}
	if(rs.getString(6) != null) {
	    ht.put("explanation", rs.getString(6));
	}
    }
    rs.close();
    s2.close();
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", null);
    
    return ht;
}

/* not sure what this is for
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getAttributesFromRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

//public Hashtable getAttributesFromRequest (String rid) throws Exception {

    String evtName="event=getAttributesFromRequest";
    
    TSRMLog.info(TSRMFootprintDB.class,"begin", evtName, "rid="+rid);
    TSRMFootprintUtil.noTrivialString("rid", rid, evtName);
    
    ResultSet rs = _statement.executeQuery("SELECT " +
					   "desiredTotalRequestTime,createdTime,"+
					   "totalNumFilesInRequest,"+
					   "bandwidthStatus,transferParametersClientNetworks,"+
					   "transferParametersAccessPattern, userRequestDesc, "+
					   "authID, targetFileRetentionPolicy, spaceToken, "+
					   "transferParametersTransferProtocols, status, "+
					   "transferParametersConnectionType, numOfCompletedFiles,"+
					   "explanation, targetFileAccessLatency, requestType,"+
					   "fileType, numOfWaitingFiles, numOfFailedFiles,statusTime"  +
					   " from Request where rid = " + "\'"+rid+"\'");
    
    Hashtable ht = new Hashtable();
    if (rs.next()) {
	ht.put("desiredtotalrequesttime", new Integer(rs.getInt(1)));
	if (rs.getTimestamp(2) != null) 
	    ht.put("createdtime", rs.getTimestamp(2));
	ht.put("totalnumfilesinrequest", new Integer(rs.getInt(3)));
	if (rs.getString(4) != null)
	    ht.put("bandwidthstatus", rs.getString(4));
	if (rs.getString(5) != null)
	    ht.put("transferparametersclientnetworks", rs.getString(5));
	if(rs.getString(6) != null)
	    ht.put("transferparametersaccesspattern", rs.getString(6));
	if(rs.getString(7) != null)
	    ht.put("userrequestdesc", rs.getString(7));
	if(rs.getString(8) != null)
	    ht.put("authid", rs.getString(8));
	if(rs.getString(9) != null)
	    ht.put("targetfileretentionpolicy", rs.getString(9));
	if(rs.getString(10) != null)
	    ht.put("spacetoken", rs.getString(10));
	if(rs.getString(11) != null)
	    ht.put("transferparameterstransferprotocols", rs.getString(11));
	if(rs.getString(12) != null)
	    ht.put("status", rs.getString(12));
	if(rs.getString(13) != null)
	    ht.put("transferparametersconnectiontype", rs.getString(13));
	ht.put("numofcompletedfiles", new Integer(rs.getInt(14)));
	if(rs.getString(15) != null)
	    ht.put("explanation", rs.getString(15));
	if(rs.getString(16) != null)
	    ht.put("targetfileaccesslatency", rs.getString(16));
	if(rs.getString(17) != null)
	    ht.put("requesttype", rs.getString(17));
	if(rs.getString(18) != null)
	    ht.put("filetype", rs.getString(18));
	ht.put("numofwaitingfiles", new Integer(rs.getInt(19)));
	ht.put("numoffailedfiles", new Integer(rs.getInt(20)));
	if(rs.getTimestamp(21) != null)
	    ht.put("statustime", rs.getTimestamp(21));
    } else {    
	String err = "RID does not exist in the Request table. rid=" +rid;
	TSRMLog.info(TSRMFootprintDB.class,"error",evtName, err);
	throw new TSRMFootprintDBException(err);
    }
    rs.close();
    
    TSRMLog.info(TSRMFootprintDB.class,"end",evtName, null);
    
    return ht;
}
*/

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//findUsersFromDatabase
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public int[] findUsersFromDatabase() throws Exception 
{   
    String evtName = "event=findUsersFromDatabase";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", null);
    
    Vector vec = new Vector ();
    
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableUser, "u_id", null);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
    
    while(rs.next()) {
	int u_id = rs.getInt(1);
	vec.addElement(new Integer(u_id));
    }
    rs.close();
    int[] ids = new int[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
	Integer ii = (Integer)vec.elementAt(i);
	ids[i] = ii.intValue();
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "resultSize="+vec.size());
    
    if(vec.size() == 0) return null;
    return ids;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//findNextFileInRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public Vector findNextFileInRequest(String rid, int limit) throws Exception 
{
    Vector<Integer> pos = new Vector<Integer> ();
    
    String evtName = "event=findNextFileInRequest";
    String[] strArray = {"rid=", rid, " limit="+limit};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    ResultSet rs = _statement.executeQuery("SELECT pos FROM RequestExecution " + 
					   " where status=" + "\'SRM_REQUEST_QUEUED\'" + 
					   " and rid="+"\'"+rid+"\'"+ " LIMIT "+limit);
    while(rs.next()) {			
	int curr = rs.getInt(1);
	pos.add(curr);
    }
    rs.close();

    if (pos.size() == 0) {
	updateReqStatus(rid);
	return null;
    }

    Vector result = new Vector(pos.size());

    for(int i = 0; i < pos.size(); i++) {
	int curr = pos.get(i);
	
	long tt = System.currentTimeMillis();
	java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(tt);
	
	String str = "update RequestExecution set status=?, statusTime=? where rid=? and pos=?";
	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "query="+str);
	
	PreparedStatement psUpdate = _conn.prepareStatement(str);
	psUpdate.setString(1, "SRM_REQUEST_INPROGRESS");
	psUpdate.setTimestamp(2, queuedTimeStamp);
	psUpdate.setString(3, rid);
	psUpdate.setInt(4, curr);
	
	psUpdate.executeUpdate();
	_conn.commit();

	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status=committed");
	//result.add(getAttributesFromRequestExec(rid, curr)); 
	result.add(getFileInput(rid, curr));
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return result;
}

private void updateReqStatus(String rid) throws Exception 
{
    String evtName = "updateReqStats";

    ResultSet rs = _statement.executeQuery("SELECT status FROM RequestExecution where rid= \'"+rid+"\'");
    
    int count = 0;
    int nQueued = 0;
    int nFinished = 0;
    int nFailed = 0;
    int nProgress = 0;

    TSRMRequestStatusReturn r = new TSRMRequestStatusReturn();
    while (rs.next()) {
	String curr = rs.getString(1);
	TStatusCode code = TStatusCode.fromValue(curr);	
	TSRMLog.info(TSRMFootprintDB.class, null, evtName, "status="+code);
	if (code == TStatusCode.SRM_REQUEST_SUSPENDED) {
	    return;
	}
	r.add(code);
	count ++;
	if (TSRMRequest.isQueued(code)) {
	    nQueued++;
	} else if (TSRMRequest.isFinished(code)) {
	    nFinished++;
	    if (TSRMRequest.isFailed(code)) { 
		nFailed++;
	    }
	} else {
	    nProgress++;
	}	
    }

    TStatusCode result = r.getStatusCode();

    if (count == 0) {
	return ; // no need to update.
    }

    long tt = System.currentTimeMillis();
    java.sql.Timestamp queuedTimeStamp = new java.sql.Timestamp(tt);
    String sql = "update Request set status=?,statusTime=?, numOfWaitingFiles=?, numOfFailedFiles=?, numOfCompletedFiles=?, totalNumFilesInRequest=? where rid=?";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName, "query="+sql+" sts="+result);
	
    PreparedStatement psUpdate = _conn.prepareStatement(sql);
    psUpdate.setString(1,result.toString());
    psUpdate.setTimestamp(2,queuedTimeStamp);
    psUpdate.setInt(3, nQueued+nProgress);
    psUpdate.setInt(4, nFailed);
    psUpdate.setInt(5, nFinished);
    psUpdate.setInt(6, count);
    psUpdate.setString(7,rid);
    
    psUpdate.executeUpdate();
    _conn.commit();
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//findQueuedRequestIds
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String[] findQueuedRequestIds(int u_id, int limit)   throws Exception 
{
    String evtName = "event=findQueuedRequestIds";
    String[] strArray = {"u_id="+u_id, " limit="+limit};
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", TSRMFootprintUtil.appendString(strArray));
    
    String sqlstr = TSRMFootprintUtil.generateSqlSelect(_DefTableRequest, "rid, sortId", "status", "SRM_REQUEST_QUEUED");
    sqlstr += " order by sortId asc LIMIT "+limit;
    ResultSet rs = _statement.executeQuery(sqlstr);    

    Vector vec = new Vector ();
    //Vector sid = new Vector ();
    
    while(rs.next()) {
	String rid = rs.getString(1);
	//int sortId = rs.getInt(2);
	vec.addElement(rid);
	//sid.addElement(new Integer(sortId));
    }
    rs.close();
    
    if (vec.size() == 0) {
	_aux.checkUserReqEntries(_statement);
    }
    String[] rids = new String[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
	rids[i] = (String)vec.elementAt(i);
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return rids;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getMappedIdsForDN
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public String[] getMappedIdsForDN(String DN) throws Exception 
{    
    String evtName = "event=getMappedIdsForDN";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", "DN="+DN);
    
    Vector vec = new Vector ();
    String sql = TSRMFootprintUtil.generateSqlSelect(_DefTableUser, "mappedId", "DN", DN);
    ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);

    while(rs.next()) {
	String str = rs.getString(1);
	vec.addElement(str);
    }
    rs.close();
    String[] ids = new String[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
	ids[i] = (String)vec.elementAt(i);
    }
    
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", null);
    
    return ids;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//checkSchemaExistence
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public boolean checkSchemaExistence () throws Exception 
{ 
    Vector vec = new Vector (); 
    vec.addElement(_DefTableFileAttr);
    vec.addElement(_DefTableFileSP);
    vec.addElement(_DefTableFileRequest);
    vec.addElement(_DefTableRequest);
    vec.addElement(_DefTableDir);
    vec.addElement(_DefTableFile);
    vec.addElement(_DefTableOpDir);
    vec.addElement(_DefTableUser);
    vec.addElement("RequestLevelInputForLs");
    vec.addElement(_DefTableReqInputCopy);
    vec.addElement(_DefTableReqInputGet);
    vec.addElement(_DefTableReqInputPut);
    vec.addElement("RequestLevelInputForBringOnline");
    vec.addElement("RequestLevelInputForReserveSpace");
    vec.addElement("RequestLevelInputForUpdateSpace");
    vec.addElement("RequestLevelInputForChangeSpace");
    vec.addElement(_DefTableComp);
    vec.addElement(_DefTableTxfParameter);
    vec.addElement(_DefTableSpace);
    vec.addElement(_DefTableBandwidth);
    
    String evtName = "event=checkSchemaExistence";
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Starts", null);
    
    int size = vec.size();
    int counter = 0;
    DatabaseMetaData metadata = null;
    metadata = _conn.getMetaData();
    
    ResultSet rs = _statement.executeQuery("SELECT TABLE_NAME from INFORMATION_SCHEMA.TABLES ");  
    
    while(rs.next()) {
	//String tableSchema = rs.getString(1);
	String tableName = rs.getString(1);

	for(int i = 0; i < size; i++) {
	    String str = (String) vec.elementAt(i);   
	    if(str.equalsIgnoreCase(tableName)) {
		counter++;		
		break;
	    }
	}
    }
    rs.close();
    
    boolean result= (counter == size);
    TSRMLog.info(TSRMFootprintDB.class, null, evtName+"Ends", "size="+size+" counter="+counter);
    return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//createTables
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public void createTables() {
     //
     //
    String str = 
	" (u_id " + " bigint not null generated always as identity constraint " + 	  	
	" u_id_PK primary key, uid varchar(2000) not null unique)";
    
    createTable(str, _DefTableUser);
    
     //
     //
      str = 
	  " (fileAttributesId bigint not null generated always as identity constraint " +
	  " fileAttributesId_PK primary key, " +
	  " path varchar(4000) not null unique, hashValue long, size long, " +
	  " createdAtTime timestamp, lastModificationTime timestamp, " +
	  " fileStorageType varchar(255), retentionPolicy varchar(255),"  +
	  " accessLatency varchar(255), " + 
	  " fileLocality varchar(255), type varchar(255), lifeTimeAssigned int, "+
	  " lifeTimeEndsAt long, ownerPermission varchar(255),userID varchar(500)," +
	  " groupPermission varchar(255), groupID varchar(500),"+
	  " otherPermission varchar(255), " + 
	  " isReady boolean, " + 
	  " physicalPath varchar(255), " +
	  " checkSumType varchar(255), checkSumValue varchar(255))";

      createTable(str, _DefTableFileAttr);

     //
     //
      str = 
	  " (fileAttributesIdSP " +
	  " bigint not null generated always as identity constraint " + 
	  " fileAttributesIdSP_PK primary key, " +
	  " spaceToken varchar(2000), hashValue long," +
	  " fileAttributesId bigint not null, " +
	  " constraint sp_fk1 foreign key (fileAttributesId) " +
	  " references "+_DefTableFileAttr+" (fileAttributesId))";
      
      createTable(str, _DefTableFileSP);

     //
     //      
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " fullDetailedList boolean, allLevelRecursive boolean, " +
	  " numOfLevels int, offset int, count int, " +
	  " storageSystemInfo varchar(4000))";
      
      createTable(str,"RequestLevelInputForLs");
      
      //
      //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), "+
	  " accessPattern varchar(1000), " +
	  " connectionType varchar(1000), " +
	  " clientNetworks varchar(1000), " +
	  " transferProtocols varchar(1000))";

      createTable(str, _DefTableTxfParameter);


      //
      //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " desiredPinLifeTime int, " +
	  " storageSystemInfo varchar(4000))";
      
      createTable(str,_DefTableReqInputGet);
      
     //
     //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " desiredPinLifeTime int, desiredFileLifeTime int," +
	  " overWriteMode varchar(255), storageSystemInfo varchar(4000))";
      
      createTable(str,_DefTableReqInputPut);

     //
     //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " desiredPinLifeTime int, deferredStartTime int, " +	  
	  " storageSystemInfo varchar(4000))";
      
      createTable(str,"RequestLevelInputForBringOnline");
      
     //
     //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " desiredSURLLifeTime int, " +
	  " overWriteMode varchar(255), sourceStorageSystemInfo varchar(4000),"+
	  " targetStorageSystemInfo varchar(4000))";
      createTable(str,_DefTableReqInputCopy);

     //
     //
      str = 
	  " (rid varchar(255) unique not null, primary key(rid), " +
	  " userSpaceTokenDesc varchar(4000), " +
	  " desiredSizeOfTotalSpace long, " +
	  " desiredSizeOfGuaranteedSpace long, " +
	  " desiredLifeTimeOfReservedSpace int, " +
	  " expectedFileSizesGiven boolean,"+
	  " storageSystemInfo varchar(4000))";

     createTable(str,"RequestLevelInputForReserveSpace");

     //
     //
     str = 
	 " (rid varchar(255) unique not null, primary key(rid), " +
	 " newLifeTime int, " +
	 " newSizeOfTotalSpaceDesired long, " +
	 " newSizeOfGuaranteedSpaceDesired long, " +
	 " storageSystemInfo varchar(4000))";

     createTable(str,"RequestLevelInputForUpdateSpace");
     
     //
     //
     str = 
	 " (rid varchar(255) unique not null, primary key(rid), " +
	 " storageSystemInfo varchar(4000))";
     
     createTable(str,"RequestLevelInputForChangeSpace");

     //
     //
     str = 
	 " (sortId bigint not null " + " generated always as identity constraint "+ " sortId_PK primary key, " +	 
	 " rId varchar(255) unique not null, " +
	 " userRequestDesc varchar(4000), authID varchar(255), " + 
	 " requestLevelInputId varchar(255), status varchar(500), " +
	 " explanation varchar(4000), bandwidthStatus varchar(500), " +
	 " requestType varchar(255), fileType varchar(255), " +
	 " spaceToken varchar(255), targetFileRetentionPolicy varchar(1000),"+
	 " targetFileAccessLatency varchar(1000),"+
	 " desiredTotalRequestTime int, " +

	 " totalNumFilesInRequest int, numOfCompletedFiles int, " +
	 " numOfWaitingFiles int, numOfFailedFiles int, createdTime timestamp,"+
	 " statusTime timestamp, userId bigint not null, " +
	 " constraint request_fk1 foreign key (userId) " +
	 " references "+_DefTableUser+" (u_id))";
     
     createTable(str, _DefTableRequest);

     //
     //
     str = 
	 " (rid varchar(255) not null, pos int, "+
	 " primary key (rId, pos), " +
	 " isSourceADirectory boolean, allLevelRecursive boolean, " +
	 " numOfLevels int)";

     createTable(str, _DefTableOpDir);

     //
     //
     str = 
	 " (rid varchar(255) not null, "+
	 " pos int,"+
	 " fromSURL varchar(4000) not null, "+
	 " toSURL varchar(4000) not null, "+
	 //"primary key (rId, localSURL, remoteSURL), " +
	 " primary key (rid, pos), " +
	 //" sortId bigint unique not null generated always,"+
	 " expectedFileSize long, "+
	 " actualFileSize long, "+
	 " TURL varchar(4000), " +
	 " pinLifeTime int, statusTime timestamp," +
	 " status varchar(255), " + " explanation varchar(4000))";

     createTable(str, _DefTableFileRequest);

     //
     //     
     str = 
	 " (dirId bigint not null generated always as identity constraint " + 
	 " dirId_PK primary key, parentId bigint not null, " +
	 " dirEntry varchar(4000), " +
	 " hashValue long, fileAttributesId bigint not null, " +
	 " constraint directory_fk1 foreign key (fileAttributesId) " +
	 " references "+_DefTableFileAttr+" (fileAttributesId))"; 

     createTable(str, _DefTableDir);

     //
     //     
     str = 
	 " (fileId bigint not null generated always as identity constraint " + 
	 " fileId_PK primary key, dirId bigint not null, constraint" +
	 " files_fk1 foreign key (dirId) references "+_DefTableDir+" (dirId), " +
	 " isSurl boolean, " +
	 " fileName varchar(4000), fileAttributesId bigint not null, " +
	 " constraint files_fk2 foreign key (fileAttributesId) " +
	 " references "+_DefTableFileAttr+" (fileAttributesId))"; 
     
     createTable(str, _DefTableFile);

     //
     //
     str = 
	 " (componentId bigint not null generated " +
	 " always as identity constraint " + 
	 " componentId_PK primary key, "+
	 " componentName varchar(4000) unique not null, " +
	 " componentType varchar(500), " +
	 " totalSize bigint, description varchar(4000), " +
	 " mountingPoint varchar(4000))";
    
     createTable(str, _DefTableComp);

     //
     //    
     str = 
	" (spaceToken varchar(500) unique not null, "+
	" primary key (spaceToken), " + 
	" totalSize long, guaranteedSize long, unUsedSize long, " +
	" lifeTimeAssigned long, lifeTimeStart timestamp, " +
	" userSpaceTokenDesc varchar(4000)," +
	" userId bigint not null, constraint" +
	" user_fk1 foreign key (userId) references "+_DefTableUser+" (u_id), " +
	" componentId bigint not null, " +
	" constraint component_fk2 foreign key (componentId) " +
	" references "+_DefTableComp+" (componentId))"; 

    createTable(str, _DefTableSpace);

     //
     //
    /*
    str = 
	" (bandwidthId bigint not null generated always as identity constraint " + 
	" bandwidthId_PK primary key, " +
	" timestart timestamp, timeend timestamp, bandwidthwindow long," +
	" sortId bigint not null, " +
	" constraint sortId_fk1 foreign key (sortId) " +
	" references "+_DefTableRequest+" (sortId))";
    */
    str = 
	" (bandwidthRId varchar(255) not null, "+
	" pos int, "+
	" primary key (bandwidthRId, pos), " +
	" timeStart timestamp, timeEnd timestamp, bandwidthReservedkbps long)";

    createTable(str, _DefTableBandwidth);
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//createTable
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void createTable(String str, String tableName) {  
    try {	
	_statement.execute("create table "+tableName+str);
	_conn.commit();
	
	TSRMLog.info(TSRMFootprintDB.class,null,"event=createtable", "table="+tableName);
    } catch (Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, "createtable:"+tableName, e);       
	System.err.println("\nDB:...Table "+tableName+" could not be created." +e.getMessage());	    	     
	e.printStackTrace();
	throw new RuntimeException("BeStMan DB Error: Unable to create table:"+tableName);
    }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//loadDriver
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private boolean loadDriver() {
    String evtName = "loadDriver";
    try {
	Class.forName(_driver.getName()).newInstance();
	System.out.println("\nDB:...Loaded the appropriate driver");
    } catch (ClassNotFoundException cnfe) {
	System.err.println("\nUnable to load the JDBC driver " + _driver.getName());
	System.err.println("Please check your CLASSPATH.");
	TSRMLog.exception(TSRMFootprintDB.class, evtName ,cnfe);
	cnfe.printStackTrace(System.err);
	return false;
    } catch (InstantiationException ie) {
	System.err.println("\nUnable to instantiate the JDBC driver " + _driver.getName());			   
	TSRMLog.exception(TSRMFootprintDB.class, evtName, ie);
	ie.printStackTrace(System.err);
	return false;
    } catch (IllegalAccessException iae) {
	System.err.println("\nNot allowed to access the JDBC driver " + _driver.getName());			   
	TSRMLog.exception(TSRMFootprintDB.class, evtName, iae);
	iae.printStackTrace(System.err);
	return false;
    } catch (Exception e) {
	TSRMLog.exception(TSRMFootprintDB.class, evtName, e);
	e.printStackTrace();
	return false;
    }
    return true;
}


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Prints details of an SQLException chain to <code>System.err</code>.
// Details included are SQL State, Error code, Exception message.
// @param e the SQLException from which to print details.
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public static void printSQLException(SQLException e)
{
  // Unwraps the entire exception chain to unveil the real cause of the
  // Exception.
  while (e != null)
  {
     System.err.println("\n----- SQLException -----");
     System.err.println("  SQL State:  " + e.getSQLState());
     System.err.println("  Error Code: " + e.getErrorCode());
     System.err.println("  Message:    " + e.getMessage());
     // for stack traces, refer to derby.log or uncomment this:
     //e.printStackTrace(System.err);
     e = e.getNextException();
  }
}
}