/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.db;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.*;

import java.util.*;

import gov.lbl.srm.StorageResourceManager.*;
import org.apache.axis.types.URI;
import gov.lbl.srm.util.*;

public class TSRMFootprintDBDoubleChecker {
    Statement _statement;

    public TSRMFootprintDBDoubleChecker(Statement statement) {
	_statement = statement;
    }

    public void listDevices() {
	ResultSet rs = null;
	try {
	    String sql="SELECT componentId, componentName, componentType, totalSize, description, mountingPoint from Component";  
	    rs = _statement.executeQuery(sql);
	    System.out.println("==listing device==>");
	    while(rs.next()) {
		int cId = rs.getInt(1);
		String cname = rs.getString(2);
		String ctype = rs.getString(3);
		int tSize = rs.getInt(4);
		String description = rs.getString(5);
		String mp = rs.getString(6);
		System.out.println("==listing device==>\n\nDB:cid="+cId);
		System.out.println("DB:cname="+cname);
		System.out.println("DB:ctype="+ctype);
		System.out.println("DB:tsize="+tSize);
		System.out.println("DB:description="+description);
		System.out.println("DB:mountingpoint="+mp);		
	    }
	    System.out.println("<==listed device==");
	    rs.close();
	} catch (java.sql.SQLException e) {
	    TSRMLog.exception(this.getClass(), "listDevices", e);
	    throw new TSRMException("failed to call db for listDevices()", false);
	}
    }    

    public void checkUserReqEntries(Statement s2) {
	try {
	    System.out.println("....Checking rid entries.....");
	    String sqlstr = TSRMFootprintUtil.generateSqlSelect(TSRMFootprintDB._DefTableRequest, "rid, status", null);
	    ResultSet rs = s2.executeQuery(sqlstr);    
	    while (rs.next()) {
		System.out.println("\t[rid]="+rs.getString(1)+"  => "+rs.getString(2));
	    }
	}  catch (Exception e) {
	    e.printStackTrace();
	} 	
    }

    public void checkFileReqEntries(Statement s2) {
	try {
	    String[] attrs = {"status", "explanation", "TURL", "statusTime", "fromSURL", "toSURL", "rid", "actualFileSize"};
	    
	    String sql0 = TSRMFootprintUtil.generateSqlSelect(TSRMFootprintDB._DefTableFileRequest, attrs, null);
	    ResultSet rs0 = s2.executeQuery(sql0);
	    System.out.println(".................checking file req entries in database ==>> ");
	    while (rs0.next()) {
		System.out.println("\t[rid]="+rs0.getString(7));
		System.out.println("\t  [From]="+rs0.getString(5)+"\n\t  [to]="+rs0.getString(6)+"    [status]="+rs0.getString(1));		 
	    }
	    System.out.println(".................checking entries <<== ");
	} catch (Exception e) {
	    e.printStackTrace();
	}   
    }

    public void listUsers() {
	ResultSet rs = null;
	try {
	  System.out.println(" ==> listing users:");
	  rs = _statement.executeQuery("SELECT uid from User");
	  while(rs.next()) {
	      String curr = rs.getString(1);
	      System.out.println("userId="+curr);
	  }
	  rs.close();
	  System.out.println("<== listed users");
      } catch (java.sql.SQLException e) {
	  TSRMLog.exception(this.getClass(), "listUsers", e);
	  throw new TSRMException("failed to call db for listUsers()", false);
      }
  }

  public void listTokenAssignment() {
      String sql = TSRMFootprintUtil.generateSqlSelect(TSRMFootprintDB._DefTableFileSP, "spaceToken, fileAttributesId", null);
      System.out.println("==listing token assignements==>");
      try {
	  ResultSet rs = TSRMFootprintUtil.execSql(_statement, sql);
	  while (rs.next()) {
	      String tid = rs.getString(1);
	      int fid = rs.getInt(2);
	      System.out.println("\tfileAttrId="+fid+"  assigns to token: "+tid);
	  }
	  System.out.println("<==listed token assignements==");
      } catch (java.sql.SQLException e) {
	  TSRMLog.exception(this.getClass(), "listTokenAssignemnts", e);
	  throw new TSRMException("failed to call db for listing token assignments", false);
      }
  }
	
  public void listToken(String spaceToken) {
      String sql = "SELECT totalSize,guaranteedSize, unUsedSize, componentId, userId, lifeTimeAssigned, lifeTimeStart, userSpaceTokenDesc";
      
      if (spaceToken != null) {
	  sql += " from SpaceAssignment";
	  sql += " where spaceToken="+ "\'"+spaceToken+"\'";
      } else {
	  sql += ", spaceToken from SpaceAssignment";
      }
      System.out.println("======> list token,  token="+spaceToken);
      try {
	  ResultSet rs = _statement.executeQuery(sql); 
	  while (rs.next()) {	      
	      long tSize = rs.getLong(1);
	      long gSize = rs.getLong(2);
	      long uSize = rs.getLong(3);
	      int cid = rs.getInt(4);
	      int u_id = rs.getInt(5);
	      long lifeTimeAssigned = rs.getLong(6);
	      Timestamp lifeTimeStart = rs.getTimestamp(7);
	      String userDesc = rs.getString(8);

	      if (spaceToken != null) {
		  System.out.println("=>  tid="+spaceToken);
	      } else {
		  System.out.println("=>  tid="+rs.getString(9));
	      }
	      System.out.println("\ttotalSize,guaranteedSize, unUsedSize, componentId, userId, lifeTimeAssigned, lifeTimeStart, userSpaceTokenDesc ");
	      System.out.print("\t");
	      System.out.println(tSize + " " + gSize + " " + uSize +" " + cid + " " + u_id + " " + lifeTimeAssigned + " " +lifeTimeStart + " " + userDesc);
	  } 
	  //System.out.println("no token found with id="+spaceToken);
	  System.out.print("<=== listed token");
	  rs.close();
      } catch (Exception e) {
	  e.printStackTrace();
      }
  }
  
  public void listRequestLs(String rid) {   
      System.out.println(".......listing ls req");  
      String[] attrList={"rid", "fromSURL", "status", "explanation"};
      listRequest(rid, attrList, null);
  }

  public void listRequestGet(String rid) {     
      System.out.println(".......listing get req");
      String[] attrList={"rid", "fromSURL", "status", "explanation", "dirOption", "spaceToken"};
      listRequest(rid, attrList, null);
  }
  public void listRequestPut(String rid) {     
      System.out.println(".......listing put req");
      String[] strList={"rid", "fromSURL", "toSURL", "status", "explanation"};
      String[] longList = {"expectedFileSize"};
      listRequest(rid, strList, longList);
  }

  public void listDir() {
      String sql="SELECT dirId,dirEntry,fileAttributesId from Directory";

      System.out.println("==>listDir()..... sql="+sql);
      try {
	  ResultSet rs = _statement.executeQuery(sql);
	  while (rs.next()) {
	      System.out.println("dirEntry="+rs.getString(2)+" id="+rs.getString(1)+" fileattrId="+rs.getString(3));
	  }
	  System.out.println("<==");
	  rs.close();
      } catch (Exception e) {
	  e.printStackTrace();
      }	  
  }

  public void listFiles() {
      String sql="SELECT fileName, dirId, fileAttributesId from Files";

      System.out.println("==>listFiles()..... sql="+sql);
      try {
	  ResultSet rs = _statement.executeQuery(sql);
	  while (rs.next()) {
	      System.out.println("   [fileEntry]="+rs.getString(1)+" dirId="+rs.getInt(2)+" fileattrid="+rs.getInt(3));
	  }
	  System.out.println("<==");
	  rs.close();
      } catch (Exception e) {
	  e.printStackTrace();
      }	  
  }

  public void listFileAttrs(long hashValue) {
      String sql = 
	  "SELECT fileAttributesId, size, createdAtTime, lastModificationTime,"+	  
	  " fileStorageType, retentionPolicy, accessLatency,fileLocality, type,"+
	  " lifeTimeAssigned, lifeTimeEndsAt, ownerPermission, userID,"+
	  " groupPermission, groupID," +
	  " otherPermission, checkSumType, checkSumValue, isReady " +
	  " from FileAttributes";
      if (hashValue != -1) {
	  sql += " where hashValue="+hashValue;
      }	  
      
      System.out.println("==> listing file attrs for hashvalue=>"+hashValue);

      try {
	  ResultSet rs = _statement.executeQuery(sql);

	  if (hashValue == -1) {
	      while (rs.next()) {
		  System.out.println("  .........fileAttrId="+rs.getInt(1)+" size="+rs.getLong(2)+" isReady="+rs.getBoolean(19));
	      }
	      System.out.println("<== ");
	  }

	  if (rs.next()) {
	      int fileAttributesId = rs.getInt(1);    
	      long size = rs.getLong(2);    
	      Timestamp createdAtTime = rs.getTimestamp(3);    
	      Timestamp lastModificationTime = rs.getTimestamp(4);    
	      String fileStorageType = rs.getString(5);    
	      String retentionPolicy = rs.getString(6);    
	      String accessLatency = rs.getString(7);    
	      String fileLocality = rs.getString(8);    
	      String type = rs.getString(9);    
	      int lifeTimeAssigned = rs.getInt(10);    
	      long lifeTimeEndsAt = rs.getLong(11);    
	      String ownerPermission = rs.getString(12);    
	      String userID = rs.getString(13);    
	      String groupPermission = rs.getString(14);    
	      String groupID = rs.getString(15);    
	      String otherPermission = rs.getString(16);    
	      String checkSumType = rs.getString(17);    
	      String checkSumValue = rs.getString(18);    
	      
	      System.out.println("\nDB:...........................");
	      System.out.println("#fid="+fileAttributesId + " isReady="+rs.getBoolean(19)+"\n size=" + size + 
				 "\n createdAt=" + createdAtTime + " lastModified=" + lastModificationTime + 
				 "\n fileStorageType=" + fileStorageType+
				 "\n retentionPolicy=" + retentionPolicy + " accesslatency=" + accessLatency + " fileLocality=" + fileLocality + 
				 "\n type=" + type + 
				 "\n lifetime=" + lifeTimeAssigned + " endsAt=" + lifeTimeEndsAt + 
				 "\n ownerPermission=" + ownerPermission + " uid=" + userID + 				 
				 "\n groupPermission=" + groupPermission + " groupID=" + groupID + "  otherPermission" +otherPermission + 
				 "\n checksumType=" + checkSumType+ "  checksumValue=" + checkSumValue);      
				 
	      rs.close();    
	  }
	  System.out.println("<==== listed file attrs");
      } catch (Exception e) {
	  e.printStackTrace();
      }
  }

  private void listRequest(String rid,  String[] strAttrs, String[] longAttrs) {     
      try {
	  if ((strAttrs == null) || (strAttrs.length == 0)) {
	      return;
	  }
	  
	  String attrList = TSRMFootprintUtil.appendString(strAttrs, ", ");
	  String longAttrList = TSRMFootprintUtil.appendString(longAttrs, ", ");

	  if (longAttrList != null) {
	      attrList = attrList+", "+longAttrList;
	  }

	  String queryStr = "SELECT "+attrList+ " from RequestExecution where rid="+ "\'"+rid+"\'";
	  System.out.println("==> listing request: "+queryStr);
	  ResultSet rs = _statement.executeQuery(queryStr);
						 
	  while(rs.next()) {
	      for (int i=0; i<strAttrs.length; i++) {
		  System.out.println(strAttrs[i]+"="+rs.getString(i+1));
	      }
	      if (longAttrs == null) {
		  break;
	      }
	      for (int j=0; j<longAttrs.length; j++) {
		  int k = j+strAttrs.length+1;
		  System.out.println(longAttrs[j]+"="+rs.getLong(j+strAttrs.length+1));
	      }
	  }
	  rs.close();
	  System.out.println("<== listed ");
      } catch (Exception e) {
	  TSRMLog.exception(this.getClass(), "listRid"+rid, e);
	  throw new TSRMException("failed to list request info for "+rid, false);
      }  
  }
}