/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.db;

import gov.lbl.srm.util.*;
import gov.lbl.srm.StorageResourceManager.*;

import java.sql.*;
import java.util.*;

public class TSRMFootprintUtil {
    public static final String _DefSSISeperator = ",";
    public static final String _DefSSIKeyValSeperator = ":";

  public TSRMFootprintUtil() {}

  public static  String appendString(String[] strArray) {
      return appendString(strArray, null);
  }

  public static String appendString(String[] strArray, String seperator) {
    if (strArray == null) {
      return null;
    }
    if (strArray.length == 0) {
      return "";
    }
    StringBuffer msgBuf = new StringBuffer(strArray[0]);
    for (int i=1; i<strArray.length; i++) {
	if (seperator != null) {
	    msgBuf.append(seperator);
	}
	msgBuf.append(strArray[i]);
    }
    return msgBuf.toString();
  }


  public static void noTrivialString(String name, String val, String evtName) {
    if ((val == null) || val.equals("")) {
      TSRMLog.error(TSRMFootprintDB.class, null, evtName, "error=nullOrEmpty:"+name);  
      throw new TSRMException(name+" cannot be null or empty",false);
    }
  }


  public static void noTrivialArray(String name, Object[] val, String evtName) {
    if ((val == null) || (val.length == 0)) {
      TSRMLog.error(TSRMFootprintDB.class, null, evtName, "error=nullOrEmptyArray:"+name);  
      throw new TSRMException(name+" cannot be null or empty",false);
    }
  }

    public static void noTrivialHashtable(String name, Hashtable ht, String evtName) {
	if ((ht == null) || (ht.size() == 0)) {
	    TSRMLog.error(TSRMFootprintDB.class, null, evtName, "error=nullOrEmptyHashtable:"+name);
	    throw new TSRMException(name+" cannt be null or empty", false);	   
	}
    }

  public static String getStorageSystemInfoString(ArrayOfTExtraInfo arrayOfTExtraInfo) {
    String key = "";
    String value = "";
    
    if (arrayOfTExtraInfo != null) {
        TExtraInfo[] extraInfo = arrayOfTExtraInfo.getExtraInfoArray();
	if (extraInfo != null) {	  
	    for(int i = 0; i < extraInfo.length; i++) {
	      if(i == 0) {
		 key = extraInfo[i].getKey();
		 value = extraInfo[i].getValue();
	      } else {      
		 key = key+","+extraInfo[i].getKey();
		 value = value+","+extraInfo[i].getValue();
	      }
	    }
	}    
    }
    return key+":"+value;
  }


  public static java.util.Calendar createCalendar(java.util.Date d) {
    if (d == null) {
      return null;
    }
     GregorianCalendar gCal = new GregorianCalendar();
     gCal.setGregorianChange(d);
     return gCal;
    
  }

  public static TRetentionPolicyInfo getRetentionPolicyInfo(String policy, String latency) {    
    if ((policy == null) && (latency == null)) {
      return null;
    }
    
    TRetentionPolicyInfo retentionPolicyInfo = new TRetentionPolicyInfo();    
    if(policy != null) {
      retentionPolicyInfo.setRetentionPolicy(TRetentionPolicy.fromValue(policy));       
   }
   
    if(latency != null) {
      retentionPolicyInfo.setAccessLatency(TAccessLatency.fromValue(latency));       
    }
    return retentionPolicyInfo;
  }

  public static TUserPermission getOwnerPermission(String umode, String uid) {    
    if (umode == null) {
      return null;
    }
    TUserPermission ownerPermission = new TUserPermission();   
    TPermissionMode uMode = TPermissionMode.fromValue(umode);
    ownerPermission.setMode(uMode);
   
    if(uid != null) {
      ownerPermission.setUserID(uid);
    }
     return ownerPermission;
  }

  public static  TGroupPermission getGroupPermission(String gmode, String gid) {       
    if (gmode == null) {
      return null;
    }

    TGroupPermission groupPermission = new TGroupPermission();
    
    TPermissionMode gMode = TPermissionMode.fromValue(gmode);
    groupPermission.setMode(gMode);
    
    if(gid != null) {
      groupPermission.setGroupID(gid);
    }
    return groupPermission;
  }


    public static boolean getBoolean(Boolean b, boolean defaultValue) {
	if (b != null) {
	    return b.booleanValue();
	}
	return defaultValue;
    }

    public static int getInt(Integer i, int defaultValue) {
	if (i != null) {
	    return i.intValue();
	}
	return defaultValue;	
    }

    public static long getLong(Long v, long defaultValue) {
	if (v != null) {
	    return v.longValue();
	}
	return defaultValue;	
    }

    public static String getString(String v, String defaultValue) {
	if (v != null) {
	    return v;
	}
	return defaultValue;
    }

    public static String getString(TOverwriteMode v) {
	if (v != null) {
	    return v.toString();
	}
	return "NEVER";
    }

    public static Integer getValue(int v, int treatAsNull) {
	if (v == treatAsNull) {
	    return null;
	}
	return new Integer(v);
    }


    public static org.apache.axis.types.UnsignedLong getValue(long v, long treatAsNull) {
	if (v == treatAsNull) {
	    return null;
	}
	return new org.apache.axis.types.UnsignedLong(v);
    }

    public static String[] getStringArray(String str, String seperator) {
	if ((str == null) || (str.length() == 0)) {
	    return null;
	}
	return str.split(seperator);
    }

    public static TFileStorageType fstFromString(String str) {
	if (str == null) {
	    return null;
	}
	return TFileStorageType.fromString(str);	
    }

    public static TOverwriteMode owFromString(String str) {
	if (str == null) {
	    return null;
	}
	return TOverwriteMode.fromValue(str);
    }

    public static ArrayOfTExtraInfo ssiFromString(String ssiStr) {
	if ((ssiStr == null) || (ssiStr.length() == 0)) {
	    return null;
	}

	int pos = ssiStr.indexOf(_DefSSIKeyValSeperator);
	String keyStr = ssiStr.substring(0, pos);
	String valStr = ssiStr.substring(pos+1);

	String[] keyList = getStringArray(keyStr, _DefSSISeperator);
	String[] valList = getStringArray(valStr, _DefSSISeperator);

	if ((keyList == null) || (valList == null)) {
	    TSRMLog.info(TSRMFootprintUtil.class, null, "event=unableToExtract", "keyStr="+keyStr+" valStr="+valStr);
	    return null;
	}

	if (keyList.length != valList.length) {
	    TSRMLog.info(TSRMFootprintUtil.class, null, "event=unableToExtract", "keyListSize="+keyList.length+" val="+valList.length);
	    return null;
	}

	TExtraInfo[] ssiArray = new TExtraInfo[keyList.length];

	for (int i=0; i<keyList.length; i++) {
	    String key = keyList[i];
	    String val = valList[i];

	    ssiArray[i] = new TExtraInfo(key, val);
	}

	return new ArrayOfTExtraInfo(ssiArray);
    }

    public static org.apache.axis.types.URI uriFromString(String uriStr) throws Exception {	
	if ((uriStr ==  null) || (uriStr.length() == 0)) {
	    return null;
	}
	return new org.apache.axis.types.URI(uriStr);
    }
    
    public static TRetentionPolicyInfo rpiFromString(String retentionStr, String latencyStr) {
	TAccessLatency latency = null;
	if ((latencyStr!= null) && (latencyStr.length() >= 0)) {
	    latency = TAccessLatency.fromValue(latencyStr);
	}
	TRetentionPolicy retention = null;
	if ((retentionStr!=null) && (retentionStr.length() >= 0)) {
	    retention = TRetentionPolicy.fromValue(retentionStr);
	}

	if ((latency == null) && (retention == null)) {
	    return null;
	} 
	return new TRetentionPolicyInfo(retention, latency);
    }

    public static String generateSqlSelect(String tableName, String displayAttr, String condition) {
	StringBuffer buf = new StringBuffer("SELECT ");
	buf.append(displayAttr);
	buf.append(" from ");
	buf.append(tableName);
	if (condition != null) {
	    buf.append(" where ");
	    buf.append(condition);
	}

	return buf.toString();
    }

    public static String generateSqlSelect(String tableName, String displayAttr, String attrName, String value) {
	StringBuffer buf = new StringBuffer("SELECT ");
	buf.append(displayAttr);
	buf.append(" from ");
	buf.append(tableName);
	if (attrName != null) {
	    buf.append(" where ");
	    buf.append(attrName);
	    buf.append("=\'");
	    buf.append(value);
	    buf.append("\'");
	}

	return buf.toString();
    }

    public static  String generateSqlSelect(String tableName, String[] attrList, String attrName, String value) {     
	String attrStr = appendString(attrList, ", ");
	
	String sql = generateSqlSelect(tableName, attrStr, attrName, value);
	return sql;
    }

    public static  String generateSqlSelect(String tableName, String[] attrList, String cond) {     
	String attrStr = appendString(attrList, ", ");
	
	String sql = generateSqlSelect(tableName, attrStr, cond);
	return sql;
    }

    // example 	 "DELETE from FileAttributesSP where hashValue=? and fileAttributesId=?";
    public static String generateSqlDelete(String tableName, String[] attrList) {
	if ((attrList == null) || (attrList.length == 0)) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("DELETE from ");
	buf.append(tableName);
	buf.append(" where " );
	for (int i=0; i<attrList.length; i++) {
	    if (i > 0) {
		buf.append(" and ");
	    }
	    buf.append(attrList[i]+"=?");
	}
	return buf.toString();			        
    }

    public static String generateSqlDelete(String tableName, String attr) {
	if (attr == null) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("DELETE from ");
	buf.append(tableName);
	buf.append(" where " );
	buf.append(attr+"=?");
	
	System.out.println("deletSql="+buf.toString());
	return buf.toString();			        
    }

    public static String generateSqlDelete(String tableName, String attr, String val) {
	if ((attr == null) || (val == null)) {
	    return null;
	}
	
	StringBuffer buf = new StringBuffer("DELETE from ");
	buf.append(tableName);
	buf.append(" where " );
	buf.append(attr+"="+val);
	
	System.out.println("deletSql="+buf.toString());
	return buf.toString();			        
    }

    public static String generateSqlUpdate(String tableName, String attr, String condAttr) {
	if ((attr == null) || (condAttr == null)) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("update ");
	buf.append(tableName);
	buf.append(" set ");
	buf.append(attr+"=? where ");
	buf.append(condAttr+"=?");

	System.out.println("......sql="+buf.toString());
	return buf.toString();
    }

    public static String generateSqlUpdate(String tableName, String[] attr, String condAttr) {
	if ((attr == null) || (condAttr == null)) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("update ");
	buf.append(tableName);
	buf.append(" set ");
	for (int i=0; i<attr.length; i++) {
	    if (i > 0) {
		buf.append(", ");
	    }
	    buf.append(attr[i]);
	    buf.append("=?");	    		
	}

	buf.append(" where "+condAttr+"?");
	System.out.println("......sql="+buf.toString());
	return buf.toString();
    }

    public static String generateSqlUpdate(String tableName, String[] attr, String[] condAttr) {
	if ((attr == null) || (condAttr == null)) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("update ");
	buf.append(tableName);
	buf.append(" set ");
	for (int i=0; i<attr.length; i++) {
	    if (i > 0) {
		buf.append(", ");
	    }
	    buf.append(attr[i]);
	    buf.append("=?");	    		
	}

	buf.append(" where ");
	for (int i=0; i<condAttr.length; i++) {
	    if (i > 0) {
		buf.append(" and ");
	    }
	    buf.append(condAttr[i]);
	    buf.append("=?");
	}
	System.out.println("......sql="+buf.toString());
	return buf.toString();
    }

    public static  String generateSqlInsert(String tableName, String[] attrList) {
	if ((attrList == null) || (attrList.length == 0)) {
	    return null;
	}

	StringBuffer buf = new StringBuffer("insert into ");
	buf.append(tableName);
	buf.append(" (");
	buf.append(appendString(attrList, ","));
	buf.append(") values (");

	if (attrList.length >= 1) {
	    for (int i=0; i<attrList.length-1; i++) {
		buf.append("?,");
	    }
	}   
	buf.append("?)");    
	System.out.println("......sql="+buf.toString());
	return buf.toString();
    }

    public static ResultSet execSql(Statement stmt, String sql) {
	try {
	    TSRMLog.info(TSRMFootprintUtil.class, null, "event=execSql", "sql="+sql);
	    return stmt.executeQuery(sql);
	} catch (Exception e) {
	    TSRMLog.exception(TSRMFootprintUtil.class, "sql="+sql, e);
	    throw new TSRMException("failed sql", false);
	}
    }


    public static  void execSql(Connection conn, String evt, String sql, String[] strV, boolean[] booleanV, int[] intV, Timestamp[] tsV, long[] longV) 
    {				
	TSRMLog.info(TSRMFootprintUtil.class, null, "event=execSql", "sql="+sql);
	try {
	    PreparedStatement ps = conn.prepareStatement(sql);

	    int count=1;
	    if ((strV != null) && (strV.length > 0)) {
		for (int i=0; i<strV.length; i++) {
		    ps.setString(count, strV[i]);
		    count++;
		}
	    }
	    if ((booleanV != null) && (booleanV.length > 0)) {
		for (int i=0; i<booleanV.length; i++) {
		    ps.setBoolean(count, booleanV[i]);
		    count++;
		}
	    }
	    if ((intV != null) && (intV.length > 0)) {
		for (int i=0; i<intV.length; i++) {
		    ps.setInt(count, intV[i]);
		    count++;
		}
	    }
	    	    
	    if ((tsV != null) && (tsV.length > 0)) {
		for (int i=0; i<tsV.length; i++) {
		    ps.setTimestamp(count, tsV[i]);
		    count++;
		}
	    }
	    if ((longV != null) && (longV.length > 0)) {
		for (int i=0; i<longV.length; i++) {
		    ps.setLong(count, longV[i]);
		    count ++;
		}
	    }

	    ps.executeUpdate();	    
	    conn.commit();

	    TSRMLog.info(TSRMFootprintDB.class, null, evt, "action=commited value="+sql);       
	} catch(Exception e) {
	    TSRMLog.exception(TSRMFootprintDB.class,evt,e);      
	    throw new TSRMException(e.getMessage(), false);
	}    
    }
}