/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.terapath;

import gov.lbl.srm.server.*;
import gov.lbl.srm.util.*;
import gov.lbl.srm.StorNet.*; // will use the bandwidth window defined here
import java.util.*;

import EDU.oswego.cs.dl.util.concurrent.Mutex;

public class TBandwidthMgr {
    Vector _values = new Vector();
    Mutex _accessGuard = new Mutex();
    private static final java.text.SimpleDateFormat _dateFormatter = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z' ");

    public TBandwidthMgr(long initBandwidth) 
    {
	long curr = System.currentTimeMillis();
	_values.add(new TBAGInfo(initBandwidth,  -1, curr));

	//test();
	//System.exit(1);
	_dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    }

    public void reserved(TBandwidthResponseParameters[] committedArray) 
    {
	if (committedArray == null) {
	    return;
	}

	if (!TSRMUtil.acquireSync(_accessGuard)) {
	    return;
	}

	try {	    
	    for (int k=0; k<committedArray.length; k++) {
		TBandwidthResponseParameters curr = committedArray[k];
		// terapath returns with bps instead of kbps, so adjusting it.
		adjustWithReservation(curr.getBeginTime(), curr.getEndTime(), curr.getAvailableBandwidth()/(long)1000);
	    }
	} finally {
	    TSRMUtil.releaseSync(_accessGuard);
	}
    }


    public void upToDate() {
	if (!TSRMUtil.acquireSync(_accessGuard)) {
	    return;
	}
	TSRMLog.info(this.getClass(), null, "event=upToDate", null);
	try {
	    if (_values.size() <=1) {
		return; // nothing needs to be done
	    }
	    long currTime = System.currentTimeMillis();
	    for (int i=_values.size()-1; i>=0; i--) {
		TBAGInfo curr = (TBAGInfo)(_values.get(i));
		//System.out.println(" .... "+i+" "+(curr.getSegmentStartTime() - currTime));
		if (curr.getSegmentEndTime() == -1) {
		    if (curr.getSegmentStartTime() < currTime) {
			curr.setSegmentStartTime(currTime);			
			removeRange(0, i);
			break;
		    }
		} else if (curr.getSegmentEndTime() <= currTime) {
		    removeRange(0, i+1);
		    break;
		} else if (curr.getSegmentStartTime() < currTime) {
		    curr.setSegmentStartTime(currTime);
		    removeRange(0, i);		    
		    break;
		}
		// continue 
	    }
	    consolidate(_values);
	} finally {
	    TSRMUtil.releaseSync(_accessGuard);
	}
    }

    // mutex locked
    private static void consolidate (Vector v) {
	for (int i=v.size()-1; i>=1; i--) {
	    TBAGInfo curr = (TBAGInfo)(v.get(i));
	    TBAGInfo prev = (TBAGInfo)(v.get(i-1));
	    if (curr.getSegmentBandwidth() == prev.getSegmentBandwidth()) {
		prev.setSegmentEndTime(curr.getSegmentEndTime());
		v.remove(i);
	    }
	}
    }		    
    
    // mutex locked
    private void removeRange(int start, int end) {
	if (end <= start) {
	    return;
	}
	TSRMLog.info(this.getClass(), null, "event=removeRange", "start="+start+" end="+end);

	int i = end-1;
	while (i < end && i >= start) {
	    _values.remove(i);
	    i--;
	}
    }
     
    private int getStartIdx(long begin) {
	int idxStart = -1;
	for (int i=0; i<_values.size(); i++) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(i));
	    if (curr.getSegmentStartTime() <= begin) {
		if (curr.getSegmentEndTime() > begin) {
		    idxStart = i;
		    break;
		} else if (curr.getSegmentEndTime() == -1) {
		    idxStart = i;
		    break;
		}
	    }
	}	
	return idxStart;
    }
    
    private int getEndIdx(long end) {
	int idxEnd = -1;
	for (int i=0; i<_values.size(); i++) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(i));
	    if (curr.getSegmentEndTime() == -1) {
		idxEnd = i;
		break;
	    } else if (curr.getSegmentEndTime() >= end) {
		if (curr.getSegmentStartTime() < end) {
		    idxEnd = i;
		    break;
		}
	    }
	}
	return idxEnd;
    }

    // mutex Locked
    private void adjustWithReservation(long begin, long end, long bandwidth) {
	String evt = "event=reservedBdwth";
	TSRMLog.info(this.getClass(), null, evt, "begin="+begin+" end="+end+" bdwth="+bandwidth);

	int idxStart = getStartIdx(begin);
	int idxEnd = getEndIdx(end);
	
	TSRMLog.info(this.getClass(), null, evt, "idxStart="+idxStart+" idxEnd="+idxEnd);

	if ((idxStart == -1) || (idxEnd == -1)) {
	    throw new RuntimeException("Unable to adjust to the reservation: "+idxStart+" "+idxEnd);
	}
	
	if (idxStart == idxEnd) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(idxStart));       
	    _values.insertElementAt(new TBAGInfo(curr.getSegmentBandwidth()-bandwidth, end, begin), idxStart+1);
	    _values.insertElementAt(new TBAGInfo(curr.getSegmentBandwidth(), curr.getSegmentEndTime(), end), idxStart+2);   
	    curr.setSegmentEndTime(begin);
	} else {
	    for (int i=idxStart+1; i<idxEnd; i++) {
		TBAGInfo curr = (TBAGInfo)(_values.get(i));
		curr.setSegmentBandwidth(curr.getSegmentBandwidth()-bandwidth);
	    }
	    TBAGInfo endWindow = (TBAGInfo)(_values.get(idxEnd));
	    _values.insertElementAt(new TBAGInfo(endWindow.getSegmentBandwidth()-bandwidth, end, endWindow.getSegmentStartTime()), idxEnd);
	    endWindow.setSegmentStartTime(end);
	    
	    TBAGInfo startWindow = (TBAGInfo)(_values.get(idxStart));
	    _values.insertElementAt(new TBAGInfo(startWindow.getSegmentBandwidth()-bandwidth, startWindow.getSegmentEndTime(), begin), idxStart+1);
	    startWindow.setSegmentEndTime(begin);
	}
    }

    // mutex Locked
    private void adjustWithRelease(long begin, long end, long bandwidth) {
	String evt = "event=releaseBdwth";
	TSRMLog.info(this.getClass(), null, evt, "begin="+begin+" end="+end+" bdwth="+bandwidth);

	int idxStart = getStartIdx(begin);
	int idxEnd = getEndIdx(end);
	
	TSRMLog.info(this.getClass(), null, evt, "idxStart="+idxStart+" idxEnd="+idxEnd);
	
	if (idxEnd == -1) {
	    return; // nothing needs to be done
	}
	if (idxStart == -1) {
	    idxStart = 0; // start time has passed	   
	}
	
	if (idxStart == idxEnd) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(idxStart));
	    _values.insertElementAt(new TBAGInfo(curr.getSegmentBandwidth()+bandwidth, end, begin), idxStart+1);
	    if (curr.getSegmentEndTime() >  end) {
		_values.insertElementAt(new TBAGInfo(curr.getSegmentBandwidth(), curr.getSegmentEndTime(), end), idxStart+2);   
	    }
	    curr.setSegmentEndTime(begin);
	    if (begin == curr.getSegmentStartTime()) {
		_values.remove(idxStart);
	    }
	} else {
	    for (int i=idxStart+1; i<idxEnd; i++) {
		TBAGInfo curr = (TBAGInfo)(_values.get(i));
		curr.setSegmentBandwidth(curr.getSegmentBandwidth()+bandwidth);
	    }
	    TBAGInfo endWindow = (TBAGInfo)(_values.get(idxEnd));
	    if (endWindow.getSegmentEndTime() > end) {		    
		_values.insertElementAt(new TBAGInfo(endWindow.getSegmentBandwidth()+bandwidth, end, endWindow.getSegmentStartTime()), idxEnd);
		endWindow.setSegmentStartTime(end);
	    } else {
		endWindow.setSegmentBandwidth(endWindow.getSegmentBandwidth()+bandwidth);
	    }
	    
	    TBAGInfo startWindow = (TBAGInfo)(_values.get(idxStart));
	    if (begin > startWindow.getSegmentStartTime()) {
		_values.insertElementAt(new TBAGInfo(startWindow.getSegmentBandwidth()+bandwidth, startWindow.getSegmentEndTime(), begin), idxStart+1);
		startWindow.setSegmentEndTime(begin);
	    } else {
		startWindow.setSegmentBandwidth(startWindow.getSegmentBandwidth()+bandwidth);
	    }
	}

	consolidate(_values);
	/*
	// if intervals have same bandwidth, then need to consolidate
	int i=0;
	while (true) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(i));
	    if (i == _values.size()-1) {
		break;
	    }
	    TBAGInfo next = (TBAGInfo)(_values.get(i+1));
	    if (curr.getSegmentBandwidth() == next.getSegmentBandwidth()) {
		curr.setSegmentEndTime(next.getSegmentEndTime());
		_values.remove(i+1);
	    } else {
		i++;
	    }
	}
	*/
    }
    
    public void released(TBandwidthResponseParameters[] committedArray) 
    {
	if (committedArray == null) {
	    return;
	}
	if (!TSRMUtil.acquireSync(_accessGuard)) {
	    return;
	}

	try {
	    for (int k=0; k<committedArray.length; k++) {
		TBandwidthResponseParameters curr = committedArray[k];
		adjustWithRelease(curr.getBeginTime(), curr.getEndTime(), curr.getAvailableBandwidth());
	    }
	} finally {
	    TSRMUtil.releaseSync(_accessGuard);
	}

    }

    public Vector getAvailabilityGraph() 
    {
	upToDate();

	if (!TSRMUtil.acquireSync(_accessGuard)) {
	    return _values;
	}

	try {
	    return _values;
    	} finally {
	    TSRMUtil.releaseSync(_accessGuard);
	}
    }

    private static void addInterval(Vector result, long timeStamp) {
	if (timeStamp < 0) {
	    return;
	}
	if (!result.contains(timeStamp)) {
	    result.add(timeStamp);
	}
    }

    private Object[] sortTimeline(TBAGInfo[] otherSite) {
	Vector result = new Vector();
	
	for (int i=0; i<_values.size(); i++) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(i));
	    if (i == 0) {
		result.add(curr.getSegmentStartTime());
	    }
	    addInterval(result, curr.getSegmentEndTime());
	}

	for (int i=0; i < otherSite.length; i++) {
	    TBAGInfo curr = otherSite[i];	   
	    
	    addInterval(result, curr.getSegmentStartTime());
	    addInterval(result, curr.getSegmentEndTime());	
	}
    
	/*
	for (int i=0; i < result.size(); i++) {
	    System.out.println(i+"th = "+result.get(i));
	}
	*/

	Object[] arrayResult = result.toArray();
	java.util.Arrays.sort(arrayResult);

	return arrayResult;
    }

    public TBAGInfo[] adjustAvailability(TBAGInfo desired) {
	TBAGInfo[] padded = new TBAGInfo[3];
	padded[0] = new TBAGInfo(0, desired.getSegmentStartTime(), 0);
	padded[1] = desired;
	padded[2] = new TBAGInfo(0, -1, desired.getSegmentEndTime());

	Vector resultVec = tangle(padded);
	
	System.out.println("...............==> double checking");
	System.out.println("\t\t input start="+_dateFormatter.format(desired.getSegmentStartTime()));
	System.out.println("\t\t       end  ="+_dateFormatter.format(desired.getSegmentEndTime()));
	System.out.println("\t\t       bdwth="+desired.getSegmentBandwidth());
	printMe(_values,0, "curr");
	printMe(resultVec, 0, "adjusted");
	System.out.println("...............<== double checking");
	
	TBAGInfo[] result = new TBAGInfo[resultVec.size()];
	resultVec.toArray(result);

	return result;
    }

    private static Object[] sortTimeline(TBAGInfo[] baseGraph, TBAGInfo[] otherSite) {
	Vector result = new Vector();
	
	for (int i=0; i<baseGraph.length; i++) {
	    TBAGInfo curr = (TBAGInfo)(baseGraph[i]);
	    if (i == 0) {
		result.add(curr.getSegmentStartTime());
	    }
	    addInterval(result, curr.getSegmentEndTime());
	}

	for (int i=0; i < otherSite.length; i++) {
	    TBAGInfo curr = otherSite[i];	   
	    
	    addInterval(result, curr.getSegmentStartTime());
	    addInterval(result, curr.getSegmentEndTime());	
	}
    
	/*
	for (int i=0; i < result.size(); i++) {
	    System.out.println(i+"th = "+result.get(i));
	}
	*/

	Object[] arrayResult = result.toArray();
	java.util.Arrays.sort(arrayResult);

	return arrayResult;
    }

    //
    // assumed to be not null for both inputs
    //
    public static TBAGInfo[] tangle(TBAGInfo[] graphOne, TBAGInfo[] graphTwo) 
    {
	System.out.println(".............tangle:: "+graphOne.length+"  "+graphTwo.length);

	printMe(graphOne, "one");
	printMe(graphTwo, "two");

	Vector result = new Vector();

	Object[] arrayResult = sortTimeline(graphOne, graphTwo);

	TBAGInfo[] overlapped = new TBAGInfo[arrayResult.length];
	for (int i=0; i < overlapped.length; i++) {
	    long start = ((Long)arrayResult[i]).longValue();
	    long end   = -1;
	    if (i < overlapped.length -1) {
		end = ((Long)arrayResult[i+1]).longValue();
	    } else {
		end = -1;
	    }

	    long hostBandwidth = getBandwidth(graphOne, start);
	    long guestBandwidth = getBandwidth(graphTwo, start);

	    System.out.println("...........["+_dateFormatter.format(start)+" "+_dateFormatter.format(end)+" ]");
	    System.out.println("........... "+hostBandwidth+" "+guestBandwidth);
	    if ((hostBandwidth < guestBandwidth) && (hostBandwidth > 0)) {
		result.add(new TBAGInfo(hostBandwidth, end, start));
	    } else if ((guestBandwidth < hostBandwidth) && (guestBandwidth > 0)) {
		result.add(new TBAGInfo(guestBandwidth, end, start));
	    }
	}

	consolidate(result);

	TBAGInfo[] resultArray = new TBAGInfo[result.size()];
	result.toArray(resultArray);

	return resultArray;
    }

    public Vector  tangle(TBAGInfo[] otherSite) {
	Vector result = new Vector();

	if ((otherSite == null) || (otherSite.length == 0)) {
	    return result;
	}
	Object[] arrayResult = sortTimeline(otherSite);

	TBAGInfo[] overlapped = new TBAGInfo[arrayResult.length];
	for (int i=0; i < overlapped.length; i++) {
	    long start = ((Long)arrayResult[i]).longValue();
	    long end   = -1;
	    if (i < overlapped.length -1) {
		end = ((Long)arrayResult[i+1]).longValue();
	    } else {
		end = -1;
	    }

	    long hostBandwidth = getBandwidth(start);
	    long guestBandwidth = getBandwidth(otherSite, start);
	    
	    //System.out.print("hostBandwidth="+hostBandwidth+"  guest"+guestBandwidth);
	    //System.out.print("   start="+_dateFormatter.format(start));
	    //System.out.println(" end="+_dateFormatter.format(end));

	    if ((hostBandwidth < guestBandwidth) && (hostBandwidth > 0)) {
		result.add(new TBAGInfo(hostBandwidth, end, start));
	    } else if ((guestBandwidth < hostBandwidth) && (guestBandwidth > 0)) {
		result.add(new TBAGInfo(guestBandwidth, end, start));
	    }
	}

	consolidate(result);
	return result;
    }

    private long getBandwidth(long start) {
	for (int i=0; i<_values.size(); i++) {
	    TBAGInfo curr = (TBAGInfo)(_values.get(i));
	    if (curr.getSegmentStartTime() <= start) {
		if (curr.getSegmentEndTime() > start) {
		    return curr.getSegmentBandwidth();
		} else if (curr.getSegmentEndTime() < 0) {
		    return curr.getSegmentBandwidth();
		}
	    }
	}
	return 0;
    }

    private static long getBandwidth(TBAGInfo[] otherSite, long start) {
	for (int i=0; i<otherSite.length; i++) {
	    TBAGInfo curr = otherSite[i];
	    if (start >= curr.getSegmentStartTime()) {
		if (start < curr.getSegmentEndTime()) {
		    return curr.getSegmentBandwidth();
		} else if (curr.getSegmentEndTime() < 0) {
		    return curr.getSegmentBandwidth();
		}
	    }
	}
	return 0;
    }


    public void print(String tag) {
	long curr = System.currentTimeMillis();
	printMe(0, tag);
    }

    public void printMe(long relavent, String tag) {
	printMe(_values, relavent, tag);
    }

    private void printMe(long relavent) {
	printMe(relavent, null);
    }

    private static void printMe(TBAGInfo[] input, String msg) {
	System.out.println("   !start "+msg);
	for (int i=0; i<input.length; i++) {
	    TBAGInfo curr = input[i];
	    System.out.println(curr.getSegmentBandwidth()+
			       "kbps  ["+_dateFormatter.format(curr.getSegmentStartTime())+" "+
			       _dateFormatter.format(curr.getSegmentEndTime())+"] ");
	}

	System.out.println("   !end");
    }

    private void printMe(Vector v, long relavent, String tag) {
	if (tag == null) {
	    System.out.println("====> reporting");
	} else {
	    System.out.println("====> reporting "+tag);
	}
	for (int i=0; i<v.size(); i++) {
	    System.out.print("==> "+i+"th: ");
	    TBAGInfo curr = (TBAGInfo)(v.get(i));
	    System.out.println(curr.getSegmentBandwidth()+
			       "kbps  ["+_dateFormatter.format(curr.getSegmentStartTime()-relavent)+" "+
			       _dateFormatter.format(curr.getSegmentEndTime()-relavent)+"] ");
	}
	System.out.println("<===  end");
    }

    private void test() {	       
	System.out.println("........testing starts........");
	long curr = System.currentTimeMillis();
	printMe(curr);
	upToDate();

	System.out.println("adjustWithReservation(curr+30000, curr+100000, 5000);");
	adjustWithReservation(curr+30000, curr+100000, 5000);
	printMe(curr);
	upToDate();
	printMe(curr);

	System.out.println("adjustWithReservation(curr+10000, curr+20000, 3000);");
	adjustWithReservation(curr+10000, curr+20000, 3000);
	printMe(curr);
	upToDate();
	printMe(curr);

	System.out.println("adjustWithReservation(curr+15000, curr+200000, 4000);");
	adjustWithReservation(curr+15000, curr+200000, 4000);
	printMe(curr);
	upToDate();
	printMe(curr);

	System.out.println("adjustWithRelease(curr+10000, curr+35000, 3000);");
	adjustWithRelease(curr+10000, curr+35000, 3000);
	printMe(curr);

	System.out.println("adjustWithRelease(curr+15000, curr+55000, 4000);");
	adjustWithRelease(curr+15000, curr+55000, 4000);
	printMe(curr);
	upToDate();
	printMe(curr);
	
	System.out.println("sleep 25 sec");
	TSRMUtil.sleep(25000);
	upToDate();
	printMe(curr);
	
	TBAGInfo[] others = new TBAGInfo[2];
	others[0] = new TBAGInfo(496666, curr+40000, curr+34000);
	others[1] = new TBAGInfo(494444, curr+60000, curr+53000);
	System.out.println("tangle with 2 windows:");
	printMe(tangle(others), curr, null);

	System.out.println("........testing done........");

    }
}


