/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/


package gov.lbl.srm.terapath;

import gov.lbl.srm.StorNet.*;
import gov.lbl.srm.util.*;
import gov.lbl.srm.server.*;

public class TSRMCallTeraPathAPI {
    StorNetAPISEI _storNetAPISEIPort = null;
    static TSRMCallTeraPathAPI _caller = null;
    TBandwidthMgr _mgr = new TBandwidthMgr(1000000);
    java.text.SimpleDateFormat _df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z' ");         
  
  public TSRMCallTeraPathAPI() {    
      _df.setTimeZone(java.util.TimeZone.getTimeZone("UTC"));  
    //url used in test is "https://198.124.220.9:48583/StorNetAPI/StorNetAPI";
    init(TeraPathServer._terapathEndpoint);    
  }
  
  public static TSRMCallTeraPathAPI getInstance() {
    if (_caller == null) {
        _caller = new TSRMCallTeraPathAPI();
    }
    return _caller;
  }

  public TBandwidthMgr getBandwidthMgr() {
      return _mgr;
  }

  public boolean isValid() {
    if (_storNetAPISEIPort == null) {
      return false;
    }
    return true;
  }

  private void init(String url) {
    TSRMLog.info(TSRMCallTeraPathAPI.class, null, "event=init", "url="+url);
    try {
      String keyStoreDir = System.getProperty("user.dir") + System.getProperty("file.separator");
      System.out.println("keyStore/trustStore directory:" + keyStoreDir);
      System.setProperty("javax.net.ssl.keyStoreType", "JKS");
      System.setProperty("javax.net.ssl.keyStore", keyStoreDir + "testKeystore.jks");
      System.setProperty("javax.net.ssl.keyStorePassword", "helloingrid");
      System.setProperty("javax.net.ssl.trustStoreType", "JKS");
      System.setProperty("javax.net.ssl.trustStore", keyStoreDir + "testTruststore.jks");
      System.setProperty("javax.net.ssl.trustStorePassword", "password2");
    } catch (Exception e) {
      TSRMLog.info(this.getClass(), null, "event=failedInitTeraPathAPI", null);
      TSRMLog.exception(this.getClass(), null, e);
      return;
    }

    try {
      _storNetAPISEIPort  = (new StorNetAPILocator()).getStorNetAPISEIPort();
      ((javax.xml.rpc.Stub) _storNetAPISEIPort)._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, url);
    } catch (Exception e) {
      TSRMLog.info(this.getClass(), null, "event=failedToGetTerapathPort", null);
      TSRMLog.exception(this.getClass(), null, e);
      _storNetAPISEIPort = null;
    }
            
  }

  public TResponse commit(String rid) {
	try {
	    TSRMLog.info(this.getClass(), null, "event=commit", "rid="+rid);
	    TRequestReference treq = new TRequestReference(); 
	    treq.setRequestId(rid);
	    TResponse resp = _storNetAPISEIPort.commitRequest(treq);
	    printMe(resp);
	    return resp;
	} catch (Exception e) {
	    TSRMLog.info(this.getClass(), null, "event=failedToReserveAtTP", null);
	    TSRMLog.exception(this.getClass(), null, e);
	    return null;
	}
  }
  
  private long getDefaultBandwidth() { // unit kbps
    long srmDefault=300000;
    String defaultBandwidth = System.getProperty("DefaultBandwidthKbs");
    if (defaultBandwidth == null) {
	return srmDefault;
    } 

    try {
	long bwd = Long.parseLong(defaultBandwidth);      
	return bwd;
    } catch (Exception e) {
	return srmDefault;
    }
  }

    private TBAGInfo[] createAllBagInfo(long currBandwidth, long volumnMB, long durMillis, long startInMinutes, long endInMinutes) {        
	TBAGInfo[] tbinfo = new TBAGInfo[1];
	tbinfo[0] = new TBAGInfo();
	tbinfo[0].setSegmentBandwidth(currBandwidth);
	
	TSRMLog.debug(this.getClass(), null, "event=bagInfoDurInput", "value="+durMillis);
	
	if (durMillis <= 0) {
	    durMillis = 1000L*volumnMB*1024L*8L/currBandwidth;
	    if (durMillis == 0) {
		durMillis = 20000L;
	    }
	}
	TSRMLog.debug(this.getClass(), null, "event=bagInfoDurFinal", "value="+durMillis);
	TSRMLog.debug(this.getClass(), null, "event=bagInfoStartInMinutes", "value="+startInMinutes);
	TSRMLog.debug(this.getClass(), null, "event=bagInfoEndInMinutes", "value="+endInMinutes);
	
	long now = System.currentTimeMillis();    
	long startTime = now+60000L;
	if (startInMinutes > 0) {
	    startTime = now + 60000*startInMinutes;
	}   
	long endTime = startTime+durMillis;
	if ((endInMinutes > 0) && (endInMinutes > startInMinutes)) {
	    endTime = now + 60000*endInMinutes;
	}
	
	//java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z' ");         
	TSRMLog.debug(this.getClass(), null, "event=bagInfoStart", "value="+startTime+" UTC="+_df.format(startTime));
	TSRMLog.debug(this.getClass(), null, "event=bagInfoEnd",   "value="+endTime+" UTC="+_df.format(endTime));
	tbinfo[0].setSegmentStartTime(startTime);
	tbinfo[0].setSegmentEndTime(endTime);

	TBAGInfo[] i = _mgr.adjustAvailability(tbinfo[0]);
	TSRMLog.debug(this.getClass(), null, "event=baginfoSize", "value="+i.length);
	return i;
	//return tbinfo;
  }

  private TFlowInfo createFlowInfo(String key, String val) {
    TFlowInfo r = new TFlowInfo();
    r.setKey(key);
    r.setValue(val);
    return r;
  }

  private TFlowInfo[] createAllFlowInfo(TSupportedURL src, TSupportedURL tgt) {
    //String srcIP = src.getSourceURL().getHost(); //"198.124.220.134";
    //String tgtIP = tgt.getSourceURL().getHost(); //"192.84.86.28"; 
    String tgtIP;
    String srcIP;
    try {
	srcIP = java.net.InetAddress.getByName(src.getHost()).getHostAddress();
	tgtIP = java.net.InetAddress.getByName(tgt.getHost()).getHostAddress();
    } catch (Exception e)  {
	TSRMLog.exception(this.getClass(), null, e);
	throw new TSRMException("Invalid host", false);
    }
    TSRMLog.info(this.getClass(), null, "event=srcIP", "value="+srcIP);
    if (srcIP.equals("198.124.220.137")) {
	srcIP = "198.124.220.197";
    } else if (srcIP.equals("198.124.220.136")) {
	srcIP = "198.124.220.196";
    }
    TSRMLog.info(this.getClass(), null, "event=srcIP", "value="+srcIP);
    
    TSRMLog.info(this.getClass(), null, "event=tgtIP", "value="+tgtIP);
    String ports = "1000-65535";
    
    TFlowInfo[] tfinfo = new TFlowInfo[4];
    tfinfo[0] = createFlowInfo("SRC", srcIP);
    tfinfo[1] = createFlowInfo("DST", tgtIP);
    tfinfo[2] = createFlowInfo("SRC_PORTS", ports);
    tfinfo[3] = createFlowInfo("DST_PORTS", ports);

    TSRMLog.info(this.getClass(), null, "event=tpInputPORTS", "value="+ports);

    return tfinfo;
  }

  private TReserveRequest createReserveReq(TUserRequestCopy r, long volumnMB, TSupportedURL src, TSupportedURL tgt) {
      TReserveRequest treq = new TReserveRequest();
      treq.setUserId(r.getOwner().getID());
      treq.setSchedulePreference(ESchedulePreference.ANY);
      
      TBandwidthRequestParameters tbreq = new TBandwidthRequestParameters();                                                                                     
      //volumnMB = volumnMB*2; // double the size for good padding as the terapath gives only min time 
      //TSRMLog.info(this.getClass(), null, "event=tpInputVolumnMB", "status=doublePadded");
      TSRMLog.info(this.getClass(), null, "event=tpInputVolumnMB", "value="+volumnMB);
      tbreq.setVolume(volumnMB);
      
      String desc = r.getDesc();
      long bandwidthKbps = getDefaultBandwidth();
      
      if (desc != null) {
	  //long v = Long.parseLong(desc);
	  long v = parseDesc(desc, "bandwidth");
	  if (v > 0) { 
	      bandwidthKbps = v;
	  } else if (v < 0) {
	      System.out.println("No reservation for invalid input:"+desc);
	      return null;
	  }
      }

      TSRMLog.info(this.getClass(), null, "event=tpInputBandwidthkbps", "value="+bandwidthKbps);

      //System.out.println("...............bandwidth asked: "+bandwidthKbps);
      TBAGInfo[] bagInfo = createAllBagInfo(bandwidthKbps, volumnMB, r.getRetryPeriod(), parseDesc(desc, "start"), parseDesc(desc, "end"));    
      tgt.useCredential(r.getGSSCredential());
      TBAGInfo[] remoteSiteBagInfo = tgt.getBandwidthGraph();
      System.out.println("remote: "+remoteSiteBagInfo+"  local:"+bagInfo);
      if (remoteSiteBagInfo == null) {      
	  tbreq.setBAGInfo(bagInfo);
      } else {
	  tbreq.setBAGInfo(TBandwidthMgr.tangle(bagInfo, remoteSiteBagInfo));
      }
      //tbreq.setBAGInfo(bagInfo);

      tbreq.setFlowInfo(createAllFlowInfo(src, tgt)); 
      treq.setDesiredValues(tbreq);  
      
      return treq;
  }

  public TResponse reserve(TUserRequestCopy copyReq, long volumnBytes, TSupportedURL src, TSupportedURL tgt) {
      long volumnMB = volumnBytes/1048576L;
      if (volumnMB == 0) {
	  volumnMB=1;
      }
      TReserveRequest treq = createReserveReq(copyReq, volumnMB, src, tgt);  
      if (treq == null) {
	  return null;
      }
      //return reserve(treq);
      TResponse r = reserve(treq);
      // cancel(copyReq.getOwner().getID(), r.getRequestId());
      return r;      
  }

  private TResponse reserve(TReserveRequest treq)  {
    TSRMLog.info(this.getClass(), null, "event=reserveBandwidth", null);
    try {
	TResponse resp = _storNetAPISEIPort.reserveRequest(treq);
	if (isValid(resp, "reserve")) {
	    if (resp.getStatus().getCode() == EStatusCode.TEMPORARY) {
		TResponse commitResponse = commit(resp.getRequestId());
		if (commitResponse.getStatus().getCode() == EStatusCode.RESERVED) {		    
		    
		    TSRMServer.getFootprintDB().insertIntoBandwidthCommited(resp);
		    _mgr.reserved(resp.getArrayOfReservationData());
		    _mgr.print(" after reservation");
		    waitTillReservationStarts(resp);		    
		}
		return commitResponse;
	    } else {
		return resp;
	    }
	}
	return null;
    } catch (Exception e) {
	TSRMLog.info(this.getClass(), null, "event=failedToReserveAtTP", null);
	TSRMLog.exception(this.getClass(), null, e); 
	return null;
    }
  }

  private boolean isValid(TResponse resp, String method) {
    if ((resp == null) || (resp.getStatus() == null)) {
      TSRMLog.info(this.getClass(), null, "event=nullResponse val="+resp, "method="+method);
      return false;
    }
	printMe(resp);
    return true;
  }

    private void waitForReservation(long resStartTime) {
	long curr=System.currentTimeMillis();
	long diff = resStartTime-curr;
	System.out.print("....................diff millis: "+diff);
	_mgr.print(" snapshot while waiting");
	if (diff -5 > 0) {
	    TSRMUtil.sleep(diff-5); // getting gsiftp ready takes time	
	}
    }

    private void waitTillReservationStarts(TResponse r) {
	TBandwidthResponseParameters curr = r.getArrayOfReservationData()[0];
	waitForReservation(curr.getBeginTime());
    }

  private void printMe(TResponse r) {
	TSRMLog.info(this.getClass(), null, "event=tpRespDisplayStarts", "rid="+r.getRequestId());
	TSRMLog.info(this.getClass(), null, "event=tpRespDisplay", "expT="+r.getRequestExpirationTime()+"("+_df.format(r.getRequestExpirationTime())+")");
	TSRMLog.info(this.getClass(), null, "event=tpRespDisplay", "statusCode="+r.getStatus().getCode());
	TSRMLog.info(this.getClass(), null, "event=tpRespDisplay", "statusExp="+r.getStatus().getExplanation());
	if (r.getArrayOfReservationData() == null) {
	    TSRMLog.info(this.getClass(), null, "event=tpRespDisplay", "reservationData=null");
	  return;
	}

	//df.setTimeZone(java.util.TimeZone.getTimeZone("UTC"));  

	int size = r.getArrayOfReservationData().length;
	for (int i=0; i<size; i++) {
	    TBandwidthResponseParameters curr = r.getArrayOfReservationData()[i];
	    String startTimeStr=curr.getBeginTime()+"("+_df.format(curr.getBeginTime())+")";
	    String endTimeStr=curr.getEndTime()+"("+_df.format(curr.getEndTime())+")";
	    String resOutput="id="+curr.getReservationId()+" status="+curr.getReservationStatus()+" "+startTimeStr+"=>"+endTimeStr+" bandwidth="+curr.getAvailableBandwidth();
	    TSRMLog.info(this.getClass(), null, "event=tpRespDisplay("+i+"/"+size+")", resOutput);
	  
	    if (curr.getFlowInfo() == null) {
		TSRMLog.info(this.getClass(), null, "event=tpRespDisplay("+i+"/"+size+")", "flow=null");
		break;
	    }
	    String flowOutput="flow:";	  
	    int fsize=curr.getFlowInfo().length;
	    for (int j=0; j<fsize; j++) {
		String key=curr.getFlowInfo()[j].getKey();
		String val=curr.getFlowInfo()[j].getValue();
		
		flowOutput+="["+key+"="+val+"]";
	    }
	    TSRMLog.info(this.getClass(), null, "event=tpResDisplay("+i+"/"+size+")", flowOutput);
	}
	TSRMLog.info(this.getClass(), null, "event=tpRespDisplayEnds", null);
  }

  public TResponse cancel(String uid, String rid)
  {
      TSRMLog.info(this.getClass(), null, "event=cancel", "rid="+rid);
      try {
	  TRequestReference treq = new TRequestReference(); 
	  treq.setRequestId(rid);
	  TResponse result = _storNetAPISEIPort.cancelRequest(treq);
	  printMe(result);
	  if (result.getStatus().getCode() == EStatusCode.CANCELLED) {
	      //_mgr.released(result.getArrayOfReservationData());
	      _mgr.released(TSRMServer.getFootprintDB().getAttributesFromBandwidthCommited(rid));
	      _mgr.print(" after cancel");
	  } else if (TSRMServer.getFootprintDB().isBandwidthReservationExpired(rid)) {
	      _mgr.released(TSRMServer.getFootprintDB().getAttributesFromBandwidthCommited(rid));
	      _mgr.print(" reservation expired");
	  }
	  return result;
      } catch (Exception e) {
	  TSRMLog.info(this.getClass(), null, "event=failedToCancelAtTP", null);
	  TSRMLog.exception(this.getClass(), null, e);
	  return null;
      }
  }

  private long parseDesc(String desc, String keyword) {
      String splitter = keyword+"=";
      String[] tokens = desc.split(splitter);
      try {
	  if (tokens.length == 1) {
	      return Long.parseLong(tokens[0]);
	  }
	  return Long.parseLong(tokens[1].split(";")[0]);
      } catch (Exception e) {
	  TSRMLog.exception(this.getClass(), null, e);
	  return -1;
      }
  }
}
