/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2011, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/



package gov.lbl.srm.server;

import gov.lbl.srm.util.*;
//import srm.common.StorageResourceManager.*;
import gov.lbl.srm.StorageResourceManager.*;
import gov.lbl.srm.storage.TSRMStorage;
import java.util.*;
import org.apache.axis.types.URI;

public class TUserRequestCopy extends TUserRequest {
    HashMap _collection = new HashMap();
    int _size = 0;
    TOverwriteMode   	_overwriteMode = null;
    TFileStorageType    _storType = null;
    String _terapathRid;

    TSupportedURL _srcCurr = null;
    TSupportedURL _tgtCurr = null;
    long _totalBytesToTxf = 0;
    boolean _preprocessed = false;

    public TUserRequestCopy(String userRequestDescription, TAccount owner) {
	super(userRequestDescription, owner);
	decorateID("COPY");
    }

    public TSRMRequest resurrection(Object o) {
	return TSRMRequestCopy.create((TSRMFileInputCopy)o, this);
    }

    private long getSourceBytes() {
	return _totalBytesToTxf;
    }

    private void addSrcBytes(TSupportedURL src) {	    
	src.isDir();
	_totalBytesToTxf += src.getTrustedSize(0);
	TSRMLog.info(this.getClass(), null, "event=addSrcBytes", "total="+_totalBytesToTxf);
    }

    public boolean  isCompatible(TSupportedURL srcExpected, TSupportedURL tgtExpected) 				 
    {        
	if (_srcCurr == null) {
	    _srcCurr = srcExpected;
	    if (!_srcCurr.checkExistence()) {
		setReturnStatus(TSRMUtil.createReturnStatusFailed("No such file: "+srcExpected.getURLString()));
		return false;
	    }
	    _tgtCurr = tgtExpected;
	    _srcCurr.useCredential(getGSSCredential());
	    addSrcBytes(_srcCurr);
	    return true;
	}

	if (!tgtExpected.getHost().equalsIgnoreCase(_tgtCurr.getHost())) {
	    setReturnStatus(TSRMUtil.createReturnStatusFailed("Target hosts mismatch. expected: "+tgtExpected.getHost()+" actual:"+_tgtCurr.getHost()));
	    return false;
	}
	
	if (!srcExpected.getHost().equalsIgnoreCase(_srcCurr.getHost())) {
	    setReturnStatus(TSRMUtil.createReturnStatusFailed("Source hosts mismatch. expected: "+srcExpected.getHost()+" actual:"+_srcCurr.getHost()));
	    return false;
	}
	addSrcBytes(srcExpected);
	return true;
    }
    
    public void run() {
	runMe();
    }
    
    public void runMe() {
	TSRMLog.info(this.getClass(), null, description(), "event=schedule");
	//TSRMServer.getFootprintDB().suspend(getID());
	try {
	    // add up source file sizes
	    //Vector srmSpaceUsageCollection = getSourceBytes();	    
	    long volumn = getSourceBytes();
	    TSRMLog.debug(this.getClass(), null, "volumn="+volumn, null);

	    if (volumn == 0) {
		return;
	    } 
	    
	    if (_tgtCurr.isLocal()) {
		System.out.println(".....go to token:"+_token);				
		if (_token == null) {
		    TSRMStorage.iSRMSpace spaceManager = TSRMStorage.getSpaceManager(TSRMStorage._DefDefaultUserSpaceType);		   
		    _token = spaceManager.locateDefaultToken(TSRMStorage._DefDefaultUserFileStorageType);
		}

		TReturnStatus tokenStatus = _token.fitAllFiles(volumn);
		if (tokenStatus != null) {
		    setReturnStatus(tokenStatus);
		    return;
		}
	    }
	    
	    if (!reserveBandwidth(volumn, _srcCurr, _tgtCurr)) {
		// setReturnStatus(TSRMUtil.createReturnStatus(TStatusCode.SRM_FAILURE, "failed to reserve bandwidth"));
		// return;
		TSRMLog.info(this.getClass(), null, "event=resumeWithoutFesevation", null);
		RuntimeException hi = new RuntimeException(getID()+"bandwidth reservation failed, resume without it!!!!!!!!!");
		hi.printStackTrace();
		TSRMLog.exception(this.getClass(), null, hi);
	    }		
	    TSRMServer.getFootprintDB().activate(getID());
	} catch (Exception e) {
	    e.printStackTrace();	
	} finally {
	    TSRMLog.info(this.getClass(), null, description(), "event=end");
	    _preprocessed = true;
	}
    }
    
    public void done(TSRMRequestCopy c) {
	if (_terapathRid != null) {
	    gov.lbl.srm.terapath.TSRMCallTeraPathAPI.getInstance().cancel(this.getOwner().getID(), _terapathRid);
	}
    }
    
    /*public TSRMRequestCopy getHeadElement() {
	HashSet reqSet = new HashSet(_collection.values());
	Iterator iter = reqSet.iterator();
	HashMap subset = (HashMap)(iter.next());
	HashSet temp = new HashSet(subset.values());
	Iterator tempIter = temp.iterator();  
	TSRMRequestCopy r = (TSRMRequestCopy)(tempIter.next());          
	return r;
    }*/

    private boolean reserveBandwidth(long volumn, TSupportedURL src, TSupportedURL tgt) {
	TSRMLog.info(this.getClass(), null, "event=reserveBandwidth", "src="+src.getHost()+" tgt="+tgt.getHost());
	
	gov.lbl.srm.StorNet.TResponse resp = 
	    gov.lbl.srm.terapath.TSRMCallTeraPathAPI.getInstance().reserve(this, volumn, src, tgt);
	
	if (resp == null) {
	    return false;
	}
	
	if (resp.getStatus() == null) {
	    return false;
	}
	
	TSRMLog.info(this.getClass(), null, "event=reserveResponse "+"rid="+resp.getRequestId(), "val="+resp.getStatus().getCode()+" exp="+resp.getStatus().getExplanation());
	if (resp.getRequestId() != null) {
	    _terapathRid = resp.getRequestId();
	} else {
	    _terapathRid = null;
	}
	if (resp.getStatus().getCode() == gov.lbl.srm.StorNet.EStatusCode.RESERVED) {
	    return true;     
	} else {
	    return false;
	}
    }

    public void setTargetFileStorageType(TFileStorageType f) {
	_storType = f;
    }

    public TFileStorageType getTargetFileStorageType() {
	return _storType;
    }

    public void setOverwriteMode(TOverwriteMode m) {
	if (m != null) {
	    _overwriteMode = m;
	} else {
	    _overwriteMode = TOverwriteMode.NEVER;
	}
    }
    
    public TOverwriteMode getOverwriteMode() {
	return _overwriteMode;
    }	

    public Vector getStatusOfRequest(URI[] fromSurlArray, URI[] toSurlArray) {
	if ((fromSurlArray == null) || (toSurlArray == null)) {
	    return TSRMServer.getFootprintDB().getFileStatusesOfRequest(getID(), null, null);	    	    
	}

	if ((fromSurlArray.length == 0) || (toSurlArray.length==0)) {
	    return null;
	}

	if (fromSurlArray.length != toSurlArray.length) {
	    throw new TSRMException("from/to surl arrays are not matching", false);
	}       

	if (!_preprocessed) {
	    Vector result = new Vector();
	    for (int i=0; i<fromSurlArray.length; i++) {
	    	TCopyRequestFileStatus s = new TCopyRequestFileStatus();
		s.setStatus(TSRMUtil.createReturnStatusQueued(null));
		s.setSourceSURL(fromSurlArray[i]);
		s.setTargetSURL(toSurlArray[i]);
		result.add(s);
	    }

	    return result;
	}

	return TSRMServer.getFootprintDB().getFileStatusesOfRequest(getID(), fromSurlArray, toSurlArray);
    }

    
    TSRMRequestCopy getRequest(URI from, URI to) {
	HashMap existing = (HashMap)(_collection.get(from.toString()));
	if (existing == null) {
	    return null;
	}
	return (TSRMRequestCopy)(existing.get(to.toString()));
    }
    
    public TRequestType getType() {
	return TRequestType.COPY;
    }
    
    public int getSize() {
	return _size;
    }

    /*    public TRequestSummary getSummary() {
	try {
	    String[] ids = {getID()};
	    TRequestSummary[] result = TSRMServer.getFootprintDB().srmGetRequestSummary(ids, getOwner().getID());
	    return result[0];
	} catch (Exception e) {
	    TSRMLog.exception(this.getClass(), "event=getSummary", e);
	    return null;
	}
    }
    */
    private Vector getAllStatuses() {
	Vector result = new Vector(_size);

	if (TSRMUtil.acquireSync(_requestCollectionMutex)) {
	    //HashSet reqSet = new HashSet(_collection.values());
	    Set keys = _collection.keySet();

	    Iterator iter = keys.iterator();
	    while (iter.hasNext()) {
		try {
		    URI from = new URI((String)(iter.next()));
		    
		    Set temp = ((HashMap)(_collection.get(from.toString()))).keySet();
		    Iterator tempIter = temp.iterator();
		    while (tempIter.hasNext()) {
			URI to = new URI((String)(tempIter.next()));
			TCopyRequestFileStatus s = getStatus(from, to);
			if (s != null) {
			    result.add(s);
			} else {
			    result.add(getStatusNotFound(from, to));
			}
		    }
		} catch (Exception e) {		    
		    TSRMLog.exception(TUserRequestCopy.class, "details", e);
		}
	    }
	    TSRMUtil.releaseSync(_requestCollectionMutex);				  
	}       
	
	return result;
    }

    private TCopyRequestFileStatus getStatusNotFound(URI from, URI to) {
	TCopyRequestFileStatus s = new TCopyRequestFileStatus();
	s.setSourceSURL(from);
	s.setTargetSURL(to);
	s.setStatus(TSRMUtil.createReturnStatusFailed("No such request."));
	s.setFileSize(TSRMUtil.createTSizeInBytes(0));

	return s;
    }

    private TCopyRequestFileStatus getStatus(URI from, URI to) {
	TSRMRequestCopy r = getRequest(from, to);

	if (r != null) {
	    TCopyRequestFileStatus s = new TCopyRequestFileStatus();
	    s.setStatus(r.getReturnStatusBulk());
	    
	    s.setSourceSURL(from);
	    s.setTargetSURL(to);
	    //s.setTransferURL(r.getTransferURL());
	    if (showSizeInStatus(r.getReturnStatus())) {
		s.setFileSize(TSRMUtil.createTSizeInBytes(r.getFileSize()));
	    } else {
		s.setFileSize(TSRMUtil.createTSizeInBytes(0));
	    }
	    //s.setEstimatedProcessingTime(r.getEstProcessingTimeInSeconds());
	    s.setEstimatedWaitTime(TSRMUtil.createTLifeTimeInSeconds(r.getEstWaitTimeInSeconds(), false));
	    s.setRemainingFileLifetime(r.getRemainingPinTime());
	    return s;
	}  

	return null;
    }
}

