/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMDirClient
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

class SRMDirClient {
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private static java.util.logging.Logger _theLogger;
  private static Vector inputVec = new Vector ();
  private String userDesc = ""; 
  private String authId = "";
  private String  protocolsList ;
  private String spaceToken;
  private int totalRequestTime;
  private int pinLifeTime;
  private int fileLifeTime;
  private String storageInfo;
  private boolean overwrite;
  //private TFileStorageType fileStorageType = TFileStorageType.VOLATILE;
  private TFileStorageType fileStorageType;
  private TRetentionPolicyInfo retentionPolicyInfo;
  private int bufferSize;
  private boolean dcau;
  private int parallelism;
  private boolean fullDetailedList=false;
  private int offset;
  private int count;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMDirClient --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SRMDirClient (ISRM srm, GSSCredential credential,
	boolean debug, boolean silent, 
	java.util.logging.Logger theLogger, int bufferSize, int parallelism,
	boolean dcau) throws Exception 
{
   _srm = srm;
   _credential = credential;
   _debug = debug;
   this.silent = silent;
   _theLogger = theLogger;
   this.bufferSize = bufferSize;
   this.parallelism = parallelism;
   this.dcau = dcau;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRetentionPolicyInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setRetentionPolicyInfo(TRetentionPolicyInfo retentionPolicy) {
   retentionPolicyInfo = retentionPolicy;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFileStorageInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileStorageType(TFileStorageType fileStorageType) {
    this.fileStorageType= fileStorageType;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setUserDesc
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setUserDesc(String userDesc) {
  this.userDesc = userDesc;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAuthID
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setAuthID(String authId) {
  this.authId = authId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetSpaceToken(String spaceToken) {
  this.spaceToken = spaceToken;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setProtocolsList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setProtocolsList(String protocolsList) {
  this.protocolsList = protocolsList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setStorageSystemInfo(String storageInfo) {
   this.storageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOverwrite
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setOverwrite(boolean overwrite) {
   this.overwrite = overwrite;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTotalRequestTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTotalRequestTime(int totalRequestTime) {
   this.totalRequestTime = totalRequestTime;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setPinTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setPinLifeTime(int pinLifeTime) {
  this.pinLifeTime = pinLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileLifeTime(int fileLifeTime) {
  this.fileLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFullDetailedList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFullDetailedList(boolean fullDetailedList) {
  this.fullDetailedList = fullDetailedList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getFullDetailedList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

boolean getFullDetailedList() {
  return fullDetailedList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOffSet
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setOffSet(int offset) {
   this.offset = offset;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getOffSet
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

int getOffSet() {
   return offset;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setCount
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setCount(int count) {
   this.count = count;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getCount
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

int getCount() {
   return count;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doStatus
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmStatusOfLsRequestResponse doStatus(String rToken) 
	throws SRMClientException, Exception {

  SrmStatusOfLsRequestResponse lsStatusResponse = null;
  try {
   if(_debug) {
     util.printMessage("\nSRM-DIR: ..........................",silent);
   }

   util.printMessage("\nSRM-DIR: " + new Date() +
      " Calling srmLsStatus ",silent);
   util.printMessage(
        "SRM-DIR: ....Input parameters for SrmStatusOfLsRequest...", silent);

   inputVec.addElement("AuthorizationID="+authId);
   inputVec.addElement("Offset="+offset);
   inputVec.addElement("count="+count);
   if(_debug) {
     util.printMessage("SRM-DIR: AuthorizationID="+authId,silent);
     util.printMessage("SRM-DIR: FileStorageType="+fileStorageType,silent);
   }

   if(offset != 0 && _debug) {
      util.printMessage("SRM-DIR: Offset="+offset,silent);
   }
   if(count != 0 && _debug) {
      util.printMessage("SRM-DIR: Count="+count,silent);
   }


   util.printEventLog(_theLogger,"SrmStatusOfLsRequestRequest", inputVec,silent);

   if(rToken == null || rToken.equals("")) {
      util.printMessage ( "\nSRM-DIR: Expecting requestToken for this " +
		"status code for lsRequest from SRM", silent);
      inputVec.addElement("Expecting requestToken for this status code " +
            "for lsRequest from SRM");
      util.printEventLog(_theLogger,"SrmStatusOfLsRequestResponse", inputVec,silent);
      return null;
   }

   SrmStatusOfLsRequestRequest lsStatusRequest =
        new SrmStatusOfLsRequestRequest();
   if(!rToken.equals("")) {
     lsStatusRequest.setRequestToken(rToken);
   }
   if(!authId.equals("")) {
     lsStatusRequest.setAuthorizationID(authId);
   }
   if(count != 0) {
      lsStatusRequest.setCount(new Integer(count));
   }
   if(offset != 0) {
      lsStatusRequest.setOffset(new Integer(offset));
   }

   lsStatusResponse = _srm.srmStatusOfLsRequest(lsStatusRequest);

   inputVec = new Vector ();
   util.printMessage("SRM-DIR: .......................",silent);

   if(lsStatusResponse == null) {
      util.printMessage(
       "\nSRM-DIR: Null response from srmStatusOfLsRequest",silent);
	  inputVec.addElement("Null response from srmStatusOfLsRequest");
      util.printEventLog
	    (_theLogger,"SrmStatusOfLsRequestResponse", inputVec,silent);
      return lsStatusResponse;
   }

   if(lsStatusResponse.getReturnStatus() == null) {
      inputVec.addElement(
        "Null return status from srmStatusOfLsRequest");
      util.printEventLog(_theLogger,
        "SrmStatusOfLsRequestResponse", inputVec,silent);
      util.printMessage(
        "\nSRM-DIR: Null return status from srmStatusOfLsRequest",silent);
      return lsStatusResponse;
   }

   TReturnStatus rStatus = lsStatusResponse.getReturnStatus();
   TStatusCode sCode = rStatus.getStatusCode();
   util.printMessage("SRM-DIR: Status Code for lsStatusRequest=" +
     sCode.toString(), silent);
   util.printMessage("SRM-DIR: explanation=" +
     rStatus.getExplanation(), silent);
   inputVec.addElement("Status="+sCode.toString());
   inputVec.addElement("Explanation="+rStatus.getExplanation());
   TMetaDataPathDetail[] pathDetail =
      lsStatusResponse.getDetails().getPathDetailArray();
   util.printEventLog(_theLogger,
     "SrmStatusOfLsRequestResponse", inputVec,silent);
   if(_debug) {
     for(int i = 0; i < pathDetail.length; i++) {
       inputVec = new Vector ();
       printMetaData("\t\t",pathDetail[i]);
     }
   }
   return lsStatusResponse;
  }
  catch(Exception e) {
   inputVec = new Vector ();
   inputVec.addElement("Exception="+e.getMessage());
   util.printEventLog(_theLogger,"SrmStatusOfLsRequest", inputVec,silent);
   String msg = e.getMessage();
   int idx = msg.indexOf("Connection refused");
   util.printMessage("\nSRM-DIR: Exception " + e.getMessage(),silent);
   if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
      idx != -1) {
     util.printMessage("\nException : "+msg,silent);
   }
   else {
     throw e;
   }
  }
  return null;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// srmLs
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmLsResponse srmLs(String[] surls, boolean allLevelsRecursive, 
   int numLevels) throws SRMClientException, Exception {
    
 SrmLsResponse result = null;
 try {
   ExtendedGSSCredential cred = (ExtendedGSSCredential) _credential;
   byte[] bb = cred.export(ExtendedGSSCredential.IMPEXP_OPAQUE);
   String proxyString = new String(bb);

   util.printMessage("\nSRM-DIR: " + new Date() +
        " Calling srmLsRequest", silent);

   if(_debug) {
     util.printMessage("\nSRM-DIR: ..........................",silent);       
     util.printMessage
		("SRM-DIR: .....Input parameters for SrmLsRequest ....", silent);
   }
  
   inputVec = new Vector ();
   inputVec.addElement("AuthorizationID="+authId);
   inputVec.addElement("FullDetailedList="+fullDetailedList);
   inputVec.addElement("Recursive="+allLevelsRecursive);
   inputVec.addElement("NumOfLevels="+numLevels);
   inputVec.addElement("Offset="+offset);
   inputVec.addElement("Count="+count);


   if(_debug) {
     util.printMessage("SRM-DIR: AuthorizationID="+authId,silent);
     util.printMessage("SRM-DIR: FullDetailedList="+fullDetailedList, silent);
     util.printMessage("SRM-DIR: Recursive="+allLevelsRecursive,silent);
     if(numLevels != -1) {
       util.printMessage("SRM-DIR: NumOfLevels="+numLevels,silent);
     }
     if(offset != 0) {
       util.printMessage("SRM-DIR: Offset="+offset,silent);
     }
     if(count != 0) {
       util.printMessage("SRM-DIR: Count="+count,silent);
     }
   } 

   SrmLsRequest req = new SrmLsRequest();
   int size = surls.length;
   URI[] path = new URI[size];

   for(int i = 0; i < size; i++) {
     URI uri = new URI(surls[i]);
     if(_debug) {
      util.printMessage("SRM-DIR: SURL="+ uri.toString(),silent);
     }
     inputVec.addElement("SURL="+uri.toString());

    if(storageInfo.equals("")) {
      int idx = surls[i].indexOf("mss://");
      if(idx != -1) {
        storageInfo = ""+true;
      }
    }

    path[i] = uri;
   }

   if(_debug) {
    util.printMessage("SRM-DIR: UseStorgaeSystemInfo="+ storageInfo,silent);
   }

   inputVec.addElement("UseStorgaeSystemInfo="+storageInfo);

   req.setArrayOfSURLs(SRMUtilClient.convertToArrayOfAnyURI(path));

   if(allLevelsRecursive) {
     req.setAllLevelRecursive(new Boolean(allLevelsRecursive));
   }
   if(fullDetailedList) {
     req.setFullDetailedList(new Boolean(fullDetailedList));
   }
   if(!authId.equals("")) {
     req.setAuthorizationID(authId);
   }
   if(numLevels != -1) {
     req.setNumOfLevels(new Integer(numLevels));
   }
   if(count != 0) {
     req.setCount(new Integer(count));
   }
   if(offset != 0) {
     req.setOffset(new Integer(offset));
   }

   ArrayOfTExtraInfo storageInfoArray = new ArrayOfTExtraInfo();
   TExtraInfo tExtra = new TExtraInfo();
   Vector vec = new Vector ();
   if(storageInfo.equals("true")) {

     tExtra.setKey("uid");
     tExtra.setValue("ls");
     vec.addElement(tExtra);

     tExtra = new TExtraInfo();
     tExtra.setKey("pwd");
     tExtra.setValue(proxyString);
     vec.addElement(tExtra);
   }
   else if(storageInfo.startsWith("for")) {
     StringTokenizer stk = new StringTokenizer(storageInfo,",");
     while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           tExtra.setKey(key);
           tExtra.setValue(value);
           vec.addElement(tExtra);
         }
         else {
           String msg = "Given storage info is not in the correct format " +
            "Please use the format for:source,login:uid,passwd:pwd" +
            " Given storage info is " + storageInfo;
           throw new Exception(msg);
         }
      }
   }


    TExtraInfo[] a_tExtra = new TExtraInfo[vec.size()];
    for(int i = 0; i < vec.size(); i++) {
       a_tExtra[i] = (TExtraInfo)vec.elementAt(i);
    }
    storageInfoArray.setExtraInfoArray(a_tExtra);
    req.setStorageSystemInfo(storageInfoArray);

    if(fileStorageType != null) {
      req.setFileStorageType(fileStorageType);
      if(_debug) {
        util.printMessage("SRM-DIR: FileStorageType="+fileStorageType.getValue(),silent);
      }
      inputVec.addElement("FileStorageType="+fileStorageType.getValue());
    }

    util.printEventLog(_theLogger,"srmLsRequest", inputVec,silent);
 
    result = _srm.srmLs(req);

    inputVec = new Vector ();

    util.printMessage("\nSRM-DIR: ..........................",silent);
    if(_debug) {
      util.printMessage("SRM-DIR: #### Output from SRM ####", silent);
    }

    if(result == null) {
      util.printMessage("\nNull result from srmLs", silent);
      inputVec.addElement("Null result from srmLs");
      util.printEventLog(_theLogger,"DoSrmLsResponse", inputVec,silent);
      return null;
    }

    if(result.getReturnStatus () == null) {
      util.printMessage("Null ReturnStatus from srmLs", silent);
      inputVec.addElement("Null ReturnStatus from srmLs");
      util.printEventLog(_theLogger,"DoSrmLsResponse", inputVec,silent);
      return null;
    }

    TReturnStatus rStatus = result.getReturnStatus();
    TStatusCode sCode = rStatus.getStatusCode();

    util.printMessage("\tStatus    : " + sCode,silent);
    util.printMessage("\tExplanation : " +
       result.getReturnStatus().getExplanation(), silent);

    inputVec.addElement("Status=" + sCode);
    inputVec.addElement("Explanation=" +
            result.getReturnStatus().getExplanation());

    String rToken = result.getRequestToken();
    util.printMessage("\tRequest token=" + rToken, silent);
    inputVec.addElement("Request token=" + rToken);

    TMetaDataPathDetail[] pathDetail = null;

    if(result.getDetails() != null) {
      pathDetail = result.getDetails().getPathDetailArray();
    }
    else {
      util.printMessage("\nSRM-DIR: SRM returned null getDetails for srmLs",
             silent);
      return null; 
    }

    if(pathDetail == null) {
      util.printMessage("\nSRM-DIR: SRM returned Null pathDetail for srmLs",
        silent);
      inputVec.addElement("SRM returned Null result for srmLs");
      util.printEventLog(_theLogger,"SrmStatusOfLsResponse",inputVec,silent);
      return null;
    }

    if(_debug) {
      for(int i = 0; i < pathDetail.length; i++) {
        printMetaData("\t\t",pathDetail[i]);
      }
    }

    return result;
  } 
  catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement("Exception="+e.getMessage());
    util.printEventLog(_theLogger,"PrintingTextReport",inputVec,silent);
    String msg = e.getMessage();
    int idx = msg.indexOf("Connection refused");
    util.printMessage("SRM-DIR: Exception " + msg, silent);
    if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
       idx != -1) {
      util.printMessage("\nException : "+msg,silent);
    }
    else {
      throw e;
    }
  }
  return null;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printMetaData
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void printMetaData(String prefix, 
	TMetaDataPathDetail pDetail) {

  inputVec = new Vector ();

  if(pDetail.getPath() != null) {
    inputVec.addElement("SURL="+pDetail.getPath());
    util.printMessage(prefix+"SURL="+ pDetail.getPath(), silent);
  }

  if(pDetail.getSize() != null) {
    inputVec.addElement("Bytes="+pDetail.getSize());
    util.printMessage(prefix+"Bytes=" + pDetail.getSize(), silent);
  }

  if(pDetail.getType() != null) {
    inputVec.addElement("FileType="+pDetail.getType().getValue());
    util.printMessage(prefix+"FileType=" + pDetail.getType().getValue(),
            silent);
  }

  if(pDetail.getFileStorageType() != null) {
     inputVec.addElement("StorageType="+pDetail.getFileStorageType());
     util.printMessage(prefix+"StorageType=" + pDetail.getFileStorageType(),
        silent);
  }

  if(pDetail.getStatus() != null && pDetail.getStatus().getStatusCode() != null) {
    inputVec.addElement("Status="+pDetail.getStatus().getStatusCode());
    util.printMessage(prefix+"Status=" + pDetail.getStatus().getStatusCode(),
        silent);
  }

  if(pDetail.getStatus().getExplanation() != null) {
    inputVec.addElement("Explanation="+pDetail.getStatus().getExplanation());
    util.printMessage(prefix+"Explanation=" +
        pDetail.getStatus().getExplanation(), silent);
  }

  if(pDetail.getOwnerPermission() != null) {
    inputVec.addElement("OwnerPermission="+
        pDetail.getOwnerPermission().getUserID());
    util.printMessage(prefix+"OwnerPermission=" +
            pDetail.getOwnerPermission().getUserID(),silent);
  }

  if(pDetail.getLifetimeLeft() != null) {
    inputVec.addElement("LifetimeLeft="+pDetail.getLifetimeLeft().intValue());
    util.printMessage(prefix+"LifetimeLeft=" +
            pDetail.getLifetimeLeft().intValue(),silent);
  }

  if(pDetail.getLifetimeAssigned() != null) {
    inputVec.addElement("LifetimeAssigned="+
        pDetail.getLifetimeAssigned().intValue());
    util.printMessage(prefix+"LifetimeAssigned=" +
            pDetail.getLifetimeAssigned().intValue(),silent);
  }

  if(pDetail.getCheckSumType() != null) {
    inputVec.addElement("CheckSumType="+pDetail.getCheckSumType());
    util.printMessage(prefix+"CheckSumType=" +
            pDetail.getCheckSumType(),silent);
  }

  if(pDetail.getCheckSumValue() != null) {
    inputVec.addElement("CheckSumValue="+pDetail.getCheckSumValue());
    util.printMessage(prefix+"CheckSumValue=" +
            pDetail.getCheckSumValue(),silent);
  }

  if(pDetail.getFileLocality() != null) {
    inputVec.addElement("FileLocality="+pDetail.getFileLocality().getValue());
    util.printMessage(prefix+"FileLocality=" +
            pDetail.getFileLocality().getValue(),silent);
  }

  if(pDetail.getOwnerPermission() != null) {
    TUserPermission uPermission = pDetail.getOwnerPermission();
    inputVec.addElement("OwnerPermission.userId="+uPermission.getUserID());
    util.printMessage(prefix+"OwnerPermission.userId=" +
            uPermission.getUserID(),silent);
    inputVec.addElement("OwnerPermission.mode="+uPermission.getMode());
    util.printMessage(prefix+"OwnerPermission.mode=" +
            uPermission.getMode().getValue(),silent);
  }

  if(pDetail.getGroupPermission() != null) {
    TGroupPermission gPermission = pDetail.getGroupPermission();
    util.printMessage(prefix+"GroupPermission.groupId=" +
            gPermission.getGroupID(),silent);
    inputVec.addElement("GroupPermission.groupId="+gPermission.getGroupID());
    util.printMessage(prefix+"GroupPermission.mode=" +
            gPermission.getMode().getValue(),silent);
    inputVec.addElement("GroupPermission.mode="+gPermission.getMode());
  }

  if(pDetail.getOtherPermission() != null) {
    util.printMessage(prefix+"OtherPermission=" +
            pDetail.getOtherPermission().getValue(),silent);
    inputVec.addElement("OtherPermission="+
        pDetail.getOtherPermission().getValue());
  }

  if(pDetail.getArrayOfSpaceTokens() != null) {
     ArrayOfString spaceTokens = pDetail.getArrayOfSpaceTokens();
     String[] tokens = spaceTokens.getStringArray();
     for(int k = 0; k < tokens.length; k++) {
       util.printMessage(prefix+"\t"+"Tokens("+k+")="+tokens[k], silent);
       inputVec.addElement("Tokens("+k+")="+tokens[k]);
     }
  }

  if(pDetail.getRetentionPolicyInfo() != null) {
     TRetentionPolicyInfo retentionPolicyInfo =
           pDetail.getRetentionPolicyInfo();
     TRetentionPolicy retentionPolicy = retentionPolicyInfo.getRetentionPolicy();
     TAccessLatency accessLatency = retentionPolicyInfo.getAccessLatency();
     if(retentionPolicy != null) {
        util.printMessage(prefix+"RetentionPolicy="+
           retentionPolicy.getValue(), silent);
        inputVec.addElement("RetentionPolicy="+retentionPolicy.getValue());
     }
     if(accessLatency != null) {
        util.printMessage(prefix+"AccessLatency="+accessLatency.getValue(),
           silent);
        inputVec.addElement("AccessLatency="+accessLatency.getValue());
     }
  }

  if (pDetail.getLastModificationTime() != null) {
    Calendar gcal = pDetail.getLastModificationTime();
    Date dd = gcal.getTime();
    util.printMessage(prefix+"Last accessed="+dd.toString(),silent);
    /*
    int year = dd.getYear()+1900;
    int month = dd.getMonth();
    int day = dd.getDate();
    int hour = dd.getHours();
    int minute = dd.getMinutes();
    int second = dd.getSeconds();
    util.printMessage(prefix+"     Year   : " + year,silent);
    util.printMessage(prefix+"     Month  : " + (month+1),silent);
    util.printMessage(prefix+"     Day    : " + day,silent);
    util.printMessage(prefix+"     Hour   : " + hour,silent);
    util.printMessage(prefix+"     Minute : " + minute,silent);
    util.printMessage(prefix+"     Second : " + second,silent);
    inputVec.addElement("LastAccessed="+"year:"+year+":month:"+(month+1)+
       ":day:"+day+":hour:"+hour+":minute:"+minute+":second:"+second);
    */
  }

  if (pDetail.getCreatedAtTime() != null) {
    Calendar gcal = pDetail.getCreatedAtTime();
    Date dd = gcal.getTime();
    util.printMessage(prefix+"CreatedAtTime="+dd.toString(),silent);
    /*
    int year = dd.getYear()+1900;
    int month = dd.getMonth();
    int day = dd.getDate();
    int hour = dd.getHours();
    int minute = dd.getMinutes();
    int second = dd.getSeconds();
    util.printMessage(prefix+"     Year   : " + year,silent);
    util.printMessage(prefix+"     Month  : " + (month+1),silent);
    util.printMessage(prefix+"     Day    : " + day,silent);
    util.printMessage(prefix+"     Hour   : " + hour,silent);
    util.printMessage(prefix+"     Minute : " + minute,silent);
    util.printMessage(prefix+"     Second : " + second,silent);
    inputVec.addElement("CreatedAtTime="+"year:"+year+":month:"+(month+1)+
       ":day:"+day+":hour:"+hour+":minute:"+minute+":second:"+second);
    */
  }

  util.printEventLog(_theLogger,"PrintMetaData",inputVec,silent);
  if(pDetail.getArrayOfSubPaths() != null) {
    inputVec = new Vector ();
    TMetaDataPathDetail[] subPDetail =
            pDetail.getArrayOfSubPaths().getPathDetailArray();
    for(int i = 0; i<subPDetail.length;i++) {
       printMetaData(prefix+"\t\t\t",subPDetail[i]);
    }
  }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// rmFile
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
SrmRmResponse rmFile (String[] surls) 
	throws SRMClientException, Exception {
 
  ExtendedGSSCredential cred = (ExtendedGSSCredential) _credential;
  byte[] bb = cred.export(ExtendedGSSCredential.IMPEXP_OPAQUE);
  String proxyString = new String(bb);
 
  inputVec = new Vector ();
 
  util.printMessage("\nSRM-DIR: " + new Date() +
        " Calling SrmRmFile",silent);

  if(_debug) {
     util.printMessage("SRM-DIR: .... Input Parameters for SrmRmRequest ....", silent);
  }

  if(_debug) {
     util.printMessage("SRM-DIR: AuthorizationID="+authId,silent);
  }

  inputVec.addElement("AuthorizationID="+authId);
 
  URI[] uris = new URI[surls.length];

   for(int i = 0; i < surls.length; i++) {
     String surl = surls[i];
     if(_debug) {
       util.printMessage("SRM-DIR: SURL[="+i+"]=" + surl, silent);
     }
     inputVec.addElement("SURL["+i+"]=" + surl);
     URI uri = new URI(surl);
     uris[i] = uri;
 
     if(storageInfo.equals("")) {
       int idx = surl.indexOf("mss://");
       if(idx != -1) {
         storageInfo = ""+true;
       }
     }
  }

  if(_debug) {
     util.printMessage("SRM-DIR: UseStorageSystemInfo="+ storageInfo,silent);
  }

  inputVec.addElement("UseStorageSystemInfo="+storageInfo);

  ArrayOfAnyURI aURI = SRMUtilClient.convertToArrayOfAnyURI(uris);

  try {
   if(uris.length == 0) {
      util.printMessage("\nSRM-DIR: No files to remove",silent);
      inputVec.addElement("No files to remove");
      util.printEventLog(_theLogger,"SrmRmFile",inputVec,silent);
      return null;
   }

   util.printMessage("SRM-DIR: Total files to remove: "+
        uris.length,silent);
   inputVec.addElement("Total files to remove="+ uris.length);
 
 
  SrmRmRequest r = new SrmRmRequest();
  if(!authId.equals("")) {
    r.setAuthorizationID(authId);
  }
  r.setArrayOfSURLs(aURI);
 
  ArrayOfTExtraInfo storageInfoArray = new ArrayOfTExtraInfo();
  TExtraInfo tExtra = new TExtraInfo();
  Vector vec = new Vector ();
  if(storageInfo.equals("true")) {
       tExtra.setKey("uid");
       tExtra.setValue("rm");
       vec.addElement(tExtra);
 
       tExtra = new TExtraInfo();
       tExtra.setKey("pwd");
       tExtra.setValue(proxyString);
       vec.addElement(tExtra);
  }
  else if(storageInfo.startsWith("for")) {
      StringTokenizer stk = new StringTokenizer(storageInfo,",");
      while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           if(key.equals("source") ||
              key.equals("for") ||
              key.equals("login") ||
              key.equals("passwd") ||
              key.equals("projectid") ||
              key.equals("readpasswd") ||
              key.equals("writepasswd")) {
             tExtra.setKey(key);
             tExtra.setValue(value);
             vec.addElement(tExtra);
           }
           else {
             throw new Exception(
              "Given storage info is not in the correct format " + storageInfo);
           }
         }
         else {
           throw new Exception(
            "Given storage info is not in the correct format " + storageInfo);
         }
      }
  }

  TExtraInfo[] a_tExtra = new TExtraInfo[vec.size()];
 
  for(int i = 0; i < vec.size(); i++) {
    a_tExtra[i] = (TExtraInfo) vec.elementAt(i);
  }
 
  storageInfoArray.setExtraInfoArray(a_tExtra);
  r.setStorageSystemInfo(storageInfoArray);
 
  SrmRmResponse result = _srm.srmRm(r);
 
  if(_debug) {
    util.printMessage("\nSRM-DIR: #### Output from SRM ####",silent);
  }

                                             
  inputVec = new Vector ();
 
  if(result == null) {
    inputVec.addElement("Null result from SrmRm");
    util.printEventLog(_theLogger,"SrmRmFileResponse",inputVec,silent);
    util.printMessage("SRM-DIR: Null result from SrmRm", silent);
    return result;
  }
  
   if(result.getReturnStatus() == null) {
    util.printMessage("\nSRM-DIR: Null return status form SrmRm", silent);
    inputVec.addElement("Null return status form SrmRm");
    util.printEventLog(_theLogger,"SrmRmFileResponse",inputVec,silent);
    return result;
  }

  TStatusCode sCode = result.getReturnStatus().getStatusCode();
  util.printMessage("\tstatus=" +
     result.getReturnStatus().getStatusCode(),silent);
  util.printMessage("\texplanation=" +
     result.getReturnStatus().getExplanation(),silent);
 
  inputVec.addElement("status="+result.getReturnStatus().getStatusCode());
  inputVec.addElement("explanation="+result.getReturnStatus().getExplanation());
 
  ArrayOfTSURLReturnStatus arrayFileStatuses =
    result.getArrayOfFileStatuses();
 
  if(arrayFileStatuses == null) {
    util.printMessage("SRM-DIR: Null file statuses from SrmRm", silent);
    inputVec.addElement("Null file statuses from SrmRm");
    util.printEventLog(_theLogger,"SrmRmFileResponse",inputVec,silent);
    return result;
  }
 
  util.printEventLog(_theLogger,"SrmRmFileResponse",inputVec,silent);
 
  TSURLReturnStatus[] arrayFileStatus = arrayFileStatuses.getStatusArray();
 
  for(int i = 0; i < arrayFileStatus.length; i++) {
     inputVec = new Vector ();
     inputVec.addElement("SURL="+
        arrayFileStatus[i].getSurl().toString());
     util.printMessage("\tsurl="+
        arrayFileStatus[i].getSurl().toString(), silent);
     if(arrayFileStatus[i].getStatus() == null) {
        util.printMessage("SRM-DIR: Null return file status", silent);
        inputVec.addElement("Null return file status");
     }
     else {
       inputVec.addElement("status="+
          arrayFileStatus[i].getStatus().getStatusCode());
       inputVec.addElement("explanation="+
          arrayFileStatus[i].getStatus().getExplanation());
       util.printMessage("\tstatus="+
          arrayFileStatus[i].getStatus().getStatusCode(), silent);
       util.printMessage("\texplanation="+
          arrayFileStatus[i].getStatus().getExplanation(), silent);
     }
     util.printEventLog(_theLogger,"SrmRmFileResponse",inputVec,silent);
  }
  return result;
 }
 catch(Exception e) {
   inputVec = new Vector ();
   inputVec.addElement("Exception="+e.getMessage());
   util.printEventLog(_theLogger,"SrmRmFile",inputVec,silent);
   String msg = e.getMessage();
   int idx = msg.indexOf("Connection refused");
   util.printMessage("SRM-DIR: Exception " + msg, silent);
   if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
      idx != -1) {
      util.printMessage("\nException : "+msg,silent);
   }
   else {
     throw e;
   }
 }
 return null;
}
}
