/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMPutClient
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

class SRMPutClient {
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private static java.util.logging.Logger _theLogger;
  private static Vector inputVec = new Vector ();
  private String userDesc = ""; 
  private String authId = "";
  private String  protocolsList ;
  private String spaceToken;
  private int totalRequestTime;
  private int pinLifeTime;
  private int fileLifeTime;
  private String storageInfo;
  private boolean overwrite;
  private TFileStorageType fileStorageType;
  private TRetentionPolicyInfo retentionPolicyInfo;
  private int bufferSize;
  private boolean dcau;
  private int parallelism;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMPutClient --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SRMPutClient (ISRM srm, GSSCredential credential,
	boolean debug, boolean silent, 
	java.util.logging.Logger theLogger, int bufferSize, int parallelism,
	boolean dcau) throws Exception 
{
   _srm = srm;
   _credential = credential;
   _debug = debug;
   this.silent = silent;
   _theLogger = theLogger;
   this.bufferSize = bufferSize;
   this.parallelism = parallelism;
   this.dcau = dcau;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doPrepareToPut
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmPrepareToPutResponse doPrepareToPut(String[] turl, long[] fileSize) 
	throws SRMClientException, Exception {
   
  try {
   String[] turls = new String[turl.length];
   long[] fileSizes = new long[turl.length];
   for(int i = 0; i < turl.length; i++) {
     turls[i] = turl[i];
     if(fileSize[i] > 0) {
       fileSizes[0] = fileSize[i]; 
     }
     else {
       fileSizes[0] = 100000000; 
     }
   }
   inputVec = new Vector();
   validateInput (turls);
   inputVec.addElement("Validated input");
   util.printEventLog(_theLogger,"doPrepareToPut",inputVec,silent);
   SrmPrepareToPutResponse response = 
		prepareToPutMultiFiles (turls,fileSizes,spaceToken);
   if(_debug) {
     SRMPutClient.printPutResult(response);
   }
   return response;
  }catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement("Exception="+e.getMessage());
    util.printEventLog(_theLogger,"doPrepareToPut",inputVec,silent);
    util.printStackTrace(e,_theLogger);
    throw new SRMClientException(e.getMessage());
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// validateInput
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void validateInput(String[] turl) throws Exception { 
  for(int i = 0; i < turl.length; i++) {
    if(turl[i].startsWith("srm:")) {; }
    else {
      throw new Exception("Given Input is not valid, " +
			"turl must start with srm for put request");
    }
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRetentionPolicyInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setRetentionPolicyInfo(TRetentionPolicyInfo retentionPolicy) {
   retentionPolicyInfo = retentionPolicy;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFileStorageInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileStorageType(TFileStorageType fileStorageType) {
    this.fileStorageType= fileStorageType;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setUserDesc
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setUserDesc(String userDesc) {
  this.userDesc = userDesc;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAuthID
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setAuthID(String authId) {
  this.authId = authId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetSpaceToken(String spaceToken) {
  this.spaceToken = spaceToken;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setProtocolsList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setProtocolsList(String protocolsList) {
  this.protocolsList = protocolsList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setStorageSystemInfo(String storageInfo) {
   this.storageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOverwrite
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setOverwrite(boolean overwrite) {
   this.overwrite = overwrite;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTotalRequestTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTotalRequestTime(int totalRequestTime) {
   this.totalRequestTime = totalRequestTime;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setPinTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setPinLifeTime(int pinLifeTime) {
  this.pinLifeTime = pinLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileLifeTime(int fileLifeTime) {
  this.fileLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// prepareToPutMultiFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	
private SrmPrepareToPutResponse prepareToPutMultiFiles(String[] turls,
	long[] fileSizes, String token)  throws Exception {


  /*
  if(token != null) {
      if(_debug)  {
        getSpaceTokenMeta(token);
      }
   }
   */

   ExtendedGSSCredential cred = (ExtendedGSSCredential) _credential;
   byte[] bb = cred.export(ExtendedGSSCredential.IMPEXP_OPAQUE);
   String proxyString = new String(bb);

   SrmPrepareToPutRequest r = new SrmPrepareToPutRequest();

   inputVec = new Vector();
   inputVec.addElement("UserDescription="+userDesc);
   if(!userDesc.equals("")) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: UserDescription="+userDesc, silent);
     }
     r.setUserRequestDescription(userDesc);
   }

   inputVec.addElement("\tAuthorizationID="+authId);
   if(_debug) {
     util.printMessage("SRM-CLIENT: AuthorizationID="+authId, silent);
   }
   if(!authId.equals("")) {
     r.setAuthorizationID(authId);
   }

   if(overwrite) {
      r.setOverwriteOption(TOverwriteMode.ALWAYS);
   }
   else {
      r.setOverwriteOption(TOverwriteMode.NEVER);
   }

   inputVec.addElement("\tOverwrite="+overwrite);
   if(_debug) {
     util.printMessage("SRM-CLIENT: Overwrite="+overwrite, silent);
   }

   TTransferParameters transferParameters = new TTransferParameters();
   ArrayOfString protocolsArray = new ArrayOfString();
   Vector protocolVec = new Vector ();

   if(protocolsList.equals("")) {
     protocolVec.addElement("gsiftp");
     protocolVec.addElement("http");
     protocolVec.addElement("https");
     protocolVec.addElement("ftp");
   }
   else {
     StringTokenizer st = new StringTokenizer(protocolsList,",");
     while (st.hasMoreTokens()) {
       String vv = st.nextToken(); 
       protocolVec.addElement(vv);
     }
   }


   String [] protocols = new String[protocolVec.size()];
   for(int i = 0; i < protocolVec.size(); i++) {
      protocols[i] = (String) protocolVec.elementAt(i);
   }

   String tt = "";
   for(int i = 0; i < protocols.length;i++) {
     tt = tt+protocols[i]+",";
   }

   inputVec.addElement("Protocols="+tt);
   if(_debug) {
     util.printMessage("SRM-CLIENT: Protocols="+tt, silent);
   }
   protocolsArray.setStringArray(protocols);
   transferParameters.setArrayOfTransferProtocols(protocolsArray);
   r.setTransferParameters(transferParameters);

   if(retentionPolicyInfo != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: RetentionPolicy="+
			retentionPolicyInfo.getRetentionPolicy(),silent);
     } 
     inputVec.addElement("RetentionPolicy="+retentionPolicyInfo.getRetentionPolicy());
     if(_debug) {
       util.printMessage("SRM-CLIENT: ACCESSINFO="+
			retentionPolicyInfo.getAccessLatency(),silent);
     }
     inputVec.addElement("ACCESSINFO="+retentionPolicyInfo.getAccessLatency());
     r.setTargetFileRetentionPolicyInfo(retentionPolicyInfo);
   }

   if(token != null && !token.equals("")) {
     if(_debug) {
        util.printMessage("SRM-CLIENT: SpaceToken="+token, silent);
     }
     inputVec.addElement("SpaceToken="+token);
     r.setTargetSpaceToken(token);
   }
   if(fileStorageType != null) {
     if(_debug) {
        util.printMessage("SRM-CLIENT: FileStorageType="+fileStorageType, silent);
     }
     inputVec.addElement("FileStorageType="+fileStorageType);
     r.setDesiredFileStorageType(fileStorageType);
   }
   if(totalRequestTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: TotalRequestTime="+totalRequestTime, silent);
    }
    inputVec.addElement("TotalRequestTime="+totalRequestTime);
    r.setDesiredTotalRequestTime(new Integer(totalRequestTime));
   }
   if(pinLifeTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: PinTime="+pinLifeTime, silent);
    }
    inputVec.addElement("PinTime="+pinLifeTime);
    r.setDesiredPinLifeTime(new Integer(pinLifeTime));
   }
   if(fileLifeTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: FileLifeTime="+fileLifeTime, silent);
    }
    inputVec.addElement("FileLifeTime="+fileLifeTime);
    r.setDesiredFileLifeTime(new Integer(fileLifeTime));
   }

   int total = turls.length;

   TPutFileRequest[] fileReqList = new TPutFileRequest[total];

   for (int i=0; i<total; i++) {
     TPutFileRequest curr = new TPutFileRequest();
     if(turls[i] == null || turls[i].equals("")) {
       curr.setTargetSURL(null);
     }
     else {
       if(_debug) {
          inputVec.addElement("SURL["+i+"]="+turls[i]);
          util.printMessage("SRM-CLIENT: SURL["+i+"]="+ turls[i],silent);
       }
       curr.setTargetSURL(new URI(turls[i]));

       if(fileSizes[i] > 0) {
         UnsignedLong ulong = new UnsignedLong(fileSizes[i]);
//         ulong.setValue(fileSizes[i]);
         curr.setExpectedFileSize(ulong);
       }
       else {
         curr.setExpectedFileSize(null);
       }

       if(storageInfo.equals("")) {
        int idx = turls[i].indexOf("mss://");
        if(idx != -1) {
          storageInfo = ""+true;
        }
       }
     }

      fileReqList[i] = curr;
   }


   ArrayOfTExtraInfo storageInfoArray = new ArrayOfTExtraInfo();
   Vector vec = new Vector();

   TExtraInfo tExtra = new TExtraInfo();
   tExtra.setKey("gsiftpStream");
   tExtra.setValue(""+parallelism);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: gsiftpStream="+parallelism, silent);
   } 
   inputVec.addElement("gsiftpStream="+parallelism);

   tExtra = new TExtraInfo();
   tExtra.setKey("bufferSize");
   tExtra.setValue(""+bufferSize);
   vec.addElement(tExtra);
   if(_debug) {
      util.printMessage("SRM-CLIENT: BufferSize="+bufferSize, silent);
   }
   inputVec.addElement("BufferSize="+bufferSize);

   tExtra = new TExtraInfo();
   tExtra.setKey("dcau");
   tExtra.setValue(""+dcau);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: DCAU="+dcau, silent);
     util.printMessage("SRM-CLIENT: StorageInfo="+storageInfo, silent);
   }
   inputVec.addElement("DCAU="+dcau);
   inputVec.addElement("StorageInfo="+storageInfo);

   if(storageInfo.equals("true")) {

     tExtra = new TExtraInfo();
     tExtra.setKey("uid");
     tExtra.setValue("put");
     vec.addElement(tExtra);

     tExtra = new TExtraInfo();
     tExtra.setKey("pwd");
     tExtra.setValue(proxyString);
     vec.addElement(tExtra);
  }
  else if(storageInfo.startsWith("for")) {
     StringTokenizer stk = new StringTokenizer(storageInfo,",");
     while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           tExtra.setKey(key);
           tExtra.setValue(value);
           vec.addElement(tExtra);
         }
         else {
           String msg = "Given storage info is not in the correct format " +
            "Please use the format for:source,login:uid,passwd:pwd" +
            " Given storage info is " + storageInfo;
           throw new Exception(msg);
         }
      }
  }

  TExtraInfo[] a_tExtra = new TExtraInfo[vec.size()];
  for(int i = 0; i < vec.size(); i++) {
    a_tExtra[i] = (TExtraInfo) vec.elementAt(i);
  }
  storageInfoArray.setExtraInfoArray(a_tExtra);
  r.setStorageSystemInfo(storageInfoArray);

  ArrayOfTPutFileRequest arrayOfFileRequest = new ArrayOfTPutFileRequest();
  arrayOfFileRequest.setRequestArray(fileReqList);
  r.setArrayOfFileRequests(arrayOfFileRequest);

  util.printEventLog(_theLogger,"Input parameters for SrmPrepareToPutRequest",inputVec,silent);
  util.printMessage("\nSRM-CLIENT: " + new Date() + 
		" Calling SrmPrepareToPutRequest now ...", silent);

  inputVec = new Vector();
  inputVec.addElement("TimeStamp="+new Date());
  util.printEventLog(_theLogger,"Sending SrmPrepareToPutRequest now",inputVec,silent);

  try {
     SrmPrepareToPutResponse result = _srm.srmPrepareToPut(r);
     util.printMessage("\nSRM-CLIENT: " + new Date() + 
		" Returning result from the srm server ...", silent);
     return result;
  }catch(Exception e) {
      inputVec = new Vector();
      inputVec.addElement(e.getMessage());
      util.printEventLog(_theLogger,"Exception",inputVec,silent);
      util.printMessage("SRM-CLIENT: " + e.getMessage(),silent);
      util.printStackTrace(e,_theLogger);
      throw e;
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmStatusOfPutRequestResponse doStatus (String requestId, String[] surl) 
	throws Exception {

   util.printMessage("SRM-CLIENT: "+ 
        " Calling Status at " + new Date(),silent);

    inputVec = new Vector();
    inputVec.addElement("RID="+requestId);
    util.printEventLog(_theLogger,"Input parameters for SrmStatusOfPutRequest",inputVec,silent);
   if(_debug) {
     util.printMessage("SRM-CLIENT: ::::::::::::: Input parameters for SrmStatusOfPutRequest ::::", silent);
     util.printMessage("SRM-CLIENT: RID="+requestId,silent);
   }
 
   if(requestId == null || requestId.equals("")) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null request token");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null request token", silent);
     util.printEventLog(_theLogger,"SrmStatusOfPutResponse",inputVec,silent);
     return null;
   }
   
   int size = surl.length;
   URI[] tsurl = new URI[size];

   for(int i = 0; i < size; i++) {
     URI temp = new URI(surl[i]);
     tsurl[i] = temp;
   }

   SrmStatusOfPutRequestRequest r = new SrmStatusOfPutRequestRequest();

   r.setRequestToken(requestId);
   if(size > 0) {
     r.setArrayOfTargetSURLs(SRMUtilClient.convertToArrayOfAnyURI(tsurl));
   }
   inputVec = new Vector ();
   inputVec.addElement("Status Calling at " + new Date());
   util.printEventLog(_theLogger,"SrmStatusOfPutResquest",inputVec,silent);


   SrmStatusOfPutRequestResponse result = _srm.srmStatusOfPutRequest(r);

   if(result == null) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null result");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null result", silent);
     util.printEventLog(_theLogger,"SrmStatusOfPutResponse",inputVec,silent);
     return result;
   }

   if(result.getReturnStatus() == null) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null return status");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null return status", silent);
     util.printEventLog(_theLogger,"SrmStatusOfPutResponse",inputVec,silent);
     return result;
   }

   String explanation = result.getReturnStatus().getExplanation();

   if(_debug) {
     util.printMessage("\nSRM-CLIENT: ### Output from SRM ###",silent);
   }

   inputVec = new Vector();
   inputVec.addElement("Result Status from SRM (srmStatusOfPutRequest)=" +
        result.getReturnStatus().getStatusCode());
   if(_debug) {
     util.printMessage("Status=" + result.getReturnStatus().getStatusCode(), silent);
   } 

   if(explanation != null) {
       if(_debug) {
         util.printMessage("\tSRM-CLIENT: " +
            "Explanation=" + explanation, silent);
       }
       inputVec.addElement("\tCheck Status Explanation=" + explanation);
   }
   util.printEventLog(_theLogger,"SrmStatusOfPutResponse",inputVec,silent);

   TPutRequestFileStatus[] fileStatus = null;

   if(result.getArrayOfFileStatuses() != null) {
       fileStatus = result.getArrayOfFileStatuses().getStatusArray();
       if(fileStatus != null) {
         for(int i = 0; i < fileStatus.length; i++) {
           TPutRequestFileStatus status = (TPutRequestFileStatus) fileStatus[i];
           if(_debug) {
            if(status.getSURL() != null) {
              util.printMessage("\tSURL="+status.getSURL().toString(),silent);
            } 
            if(status.getTransferURL() != null) {
              util.printMessage("\tTransferURL="+status.getTransferURL().toString(),silent);
            }
            if(status.getStatus() != null) {
              util.printMessage("\tStatus="+status.getStatus().getStatusCode(),silent);
              util.printMessage("\tExplanation="+
				status.getStatus().getExplanation(),silent);
            }
           }
          }
       }
   }

   return result;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// putDone
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmPutDoneResponse putDone(String[] siteUrl, String rid) throws Exception {

  util.printMessage("SRM-CLIENT:  " + new Date() + " Calling putDone ", silent);
  SrmPutDoneResponse result = null;
  inputVec = new Vector();
  if(siteUrl != null) {
   for(int i = 0; i < siteUrl.length; i++) {
     inputVec.addElement("surl["+i+"]="+siteUrl[i]);
   }
  }
  inputVec.addElement("token="+rid);
  inputVec.addElement("uid="+authId);
  util.printEventLog(_theLogger,"Input parameters for srmPutDone", inputVec,silent);
  if(_debug) {
    util.printMessage("SRM-CLIENT: ::::::::::::: Input parameters for srmPutDone :::::",silent);
    if(siteUrl != null) {
     for(int i = 0; i < siteUrl.length; i++) {
       util.printMessage("SRM-CLIENT: surl["+i+"]="+siteUrl[i],silent);
     }
    }
    util.printMessage("SRM-CLIENT: token="+rid,silent);
    util.printMessage("SRM-CLIENT: uid="+authId,silent);
  }

  org.apache.axis.types.URI[] uri = null;

  if(siteUrl != null) {
     uri = new org.apache.axis.types.URI[siteUrl.length];

   for(int i = 0; i < siteUrl.length; i++) {
     uri[i] = new org.apache.axis.types.URI (siteUrl[i]);
   }
  }

  SrmPutDoneRequest req = new SrmPutDoneRequest();
  if(uri != null && uri.length > 0) {
    req.setArrayOfSURLs(SRMUtilClient.convertToArrayOfAnyURI(uri));
  }
  req.setRequestToken(rid);
  if(!authId.equals("")) {
    req.setAuthorizationID(authId);
  }

  inputVec = new Vector();
  try {
    result = _srm.srmPutDone(req);
    if(result == null) {
      inputVec.addElement("SRM returned null result for the srmPutDone request");
      util.printEventLog(_theLogger,"Response from srmPutDone", inputVec,silent);
      util.printMessage("\nSRM-CLIENT: " + new Date() +
            " SRM returned null result for the srmPutDone request", silent);
      return result;
    }

    if(result.getArrayOfFileStatuses() == null) {
       inputVec = new Vector();
       util.printMessage("\nSRM-CLIENT: " + new Date() +
            " SRM returned null getArrayOfFileStatus for srmPutDone request",
          silent);
       inputVec.addElement("SRM returned null getArrayOfFileStatus for srmPutDone request");
       util.printEventLog(_theLogger,"Response from srmPutDone", inputVec,silent);
       return result;
    }

    TSURLReturnStatus[] status = result.getArrayOfFileStatuses().getStatusArray();

    if(_debug) {
      util.printMessage("SRM-CLIENT: " + "### Output of SRM ### ", silent);
    }

    if(status != null) {
      for(int i = 0; i < status.length; i++) {
        TStatusCode temp =null;
        if(status[i] != null & status[i].getStatus() != null) {
           temp = status[i].getStatus().getStatusCode();
        }
        inputVec = new Vector ();
        if(status[i] != null && status[i].getSurl() != null) {
          inputVec.addElement("SURL="+status[i].getSurl().toString());
        }
        inputVec.addElement("status=["+i+"]="+temp);
        if(status[i] != null && status[i].getStatus() != null) {
          inputVec.addElement("explanation="+status[i].getStatus().getExplanation());
        } 
        util.printEventLog(_theLogger,"SrmPutDoneResponse",inputVec,silent);
        if(_debug) {
          if(status[i] != null && status[i].getSurl() != null) {
            util.printMessage("\tSURL="+status[i].getSurl().toString(),silent);
          }
          util.printMessage("\tstatus="+temp,silent);
          if(status[i] != null && status[i].getStatus() != null) {
            util.printMessage("\texplanation="+status[i].getStatus().getExplanation(),silent);
          }
        }
      }
    }

  }catch(Exception e) {
     util.printMessage("SRM-CLIENT: " + e.getMessage(),silent);
     util.printStackTrace(e,_theLogger);
  }
  return result;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// abortFiles
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
SrmAbortFilesResponse abortFiles (String[] surl, String rid) throws Exception {
 
  SrmAbortFilesResponse result = null;

  util.printMessage("\nSRM-CLIENT: " + new Date() +
        " Calling abortFiles", silent);
  inputVec = new Vector();
  inputVec.addElement("TimeStamp="+new Date());
  inputVec.addElement("Token="+rid);
  inputVec.addElement("Uid="+authId);
  for(int i = 0; i < surl.length; i++) {
    inputVec.addElement("surl["+i+"]="+surl[i]);
  }
  if(_debug) {
    util.printMessage("SRM-CLIENT: ::::::::::::: Input parameters for SrmAbortFiles :::::", silent);
    for(int i = 0; i < surl.length; i++) {
      util.printMessage("SRM-CLIENT: surl["+i+"]="+surl[i],silent);
    }
    util.printMessage("SRM-CLIENT: token="+rid,silent);
    util.printMessage("SRM-CLIENT: uid="+authId,silent);
  }

  util.printEventLog(_theLogger,"Input parameters for SrmAbortFiles",inputVec,silent);

  org.apache.axis.types.URI[] uri = new org.apache.axis.types.URI[surl.length];

  for(int i = 0; i < surl.length; i++) {
    uri[i] = new org.apache.axis.types.URI (surl[i]);
  }

 
  SrmAbortFilesRequest req = new SrmAbortFilesRequest();
  if(uri.length > 0) {
    req.setArrayOfSURLs(SRMUtilClient.convertToArrayOfAnyURI(uri));
  }
  req.setRequestToken(rid);
  if(!authId.equals("")) {
    req.setAuthorizationID(authId);
  }

  try {
 
    result = _srm.srmAbortFiles(req);
 
    util.printMessage("\nSRM-CLIENT: ### Output of SRM ###",silent);
    if(result == null) {
      inputVec = new Vector();
      inputVec.addElement("Null result from SrmAbortFiles");
      util.printEventLog(_theLogger,"Response from SrmAbortFiles",inputVec,silent);
      util.printMessage("\nSRM-CLIENT: " + new Date() +
            " Null result from SrmAbortFiles", silent);
      return result;
    }
 
    if(result.getArrayOfFileStatuses() == null) {
      inputVec = new Vector();
      inputVec.addElement("Null ArrayOfFileStatuses from SrmAbortFiles");
      util.printMessage("\nSRM-CLIENT: " + new Date() +
        " Null ArrayOfFileStatuses from SrmAbortFiles", silent);
      util.printEventLog(_theLogger,"Response from SrmAbortFiles",inputVec,silent);
      return result;
    }
  
    TSURLReturnStatus[] status = result.getArrayOfFileStatuses().getStatusArray();
 
    if(status == null) {
     inputVec = new Vector();
     inputVec.addElement("Null TSURLReturnStatus from SrmAbortFiles");
     util.printMessage("\nSRM-CLIENT: " + new Date() +
        " Null TSURLReturnStatus from SrmAbortFiles", silent);
     util.printEventLog(_theLogger,"Response from SrmAbortFiles",inputVec,silent);
     return result;
    }

    for(int i = 0; i < status.length; i++) {
      if(_debug) {
        if(status[i] != null && status[i].getSurl() != null) {
          util.printMessage("\tSURL="+status[i].getSurl().toString(),silent);
        } 
        if(status[i] != null && status[i].getStatus() != null) {
          util.printMessage("\tstatus="+status[i].getStatus().getStatusCode(), silent);
          util.printMessage("\texplanation="+status[i].getStatus().getExplanation(), silent);
        }
      }
      inputVec = new Vector();
      if(status[i] != null && status[i].getSurl() != null) {
         inputVec.addElement("SURL="+status[i].getSurl().toString());
      }
      if(status[i] != null && status[i].getStatus() != null) {
         inputVec.addElement("status="+status[i].getStatus().getStatusCode());
         inputVec.addElement("explanation="+status[i].getStatus().getExplanation());
      }
      util.printEventLog(_theLogger,"Response from SrmAbortFiles",inputVec,silent);
    }
  }catch(Exception e) {
     util.printMessage("SRM-CLIENT: " + e.getMessage(),silent);
     util.printStackTrace(e,_theLogger);
  }
  return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// extendFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmExtendFileLifeTimeResponse extendFileLifeTime(String requestId,
	String[] surls, int fileLifeTime) 
	   throws SRMClientException, Exception {

  inputVec = new Vector ();
  try {
    SrmExtendFileLifeTimeRequest r = new SrmExtendFileLifeTimeRequest();
    if(!authId.equals("")) {
      r.setAuthorizationID(authId);
    }
    if(requestId.equals("")) {
      r.setNewFileLifeTime(new Integer(fileLifeTime));
    }
    else {
      r.setRequestToken(requestId);
      r.setNewPinLifeTime(new Integer(fileLifeTime));
    }

    if(requestId.equals("")) {
      org.apache.axis.types.URI[] uri = new org.apache.axis.types.URI[surls.length];

      for(int i = 0; i < surls.length; i++) {
        uri[i] = new org.apache.axis.types.URI (surls[i]);
        inputVec.addElement("SRM-REQUEST: SURL["+i+"]="+surls[i]);
        if(_debug) {
           util.printMessage("SRM-REQUEST: SURL["+i+"]="+surls[i], silent);
        }
      }

      if(uri.length > 0)  {
        r.setArrayOfSURLs(SRMUtilClient.convertToArrayOfAnyURI(uri));
      }
    }

    inputVec.addElement("SRM-REQUEST: AuthId="+authId);
    inputVec.addElement("SRM-REQUEST: RequestToken="+requestId);
    if(requestId.equals("")) {
      inputVec.addElement("SRM-REQUEST: NewFileLifeTime="+fileLifeTime);
    }
    else {
      inputVec.addElement("SRM-REQUEST: NewPinLifeTime="+fileLifeTime);
    }

    if(_debug) {
       util.printMessage("SRM-REQUEST: AuthId="+authId,silent);
       util.printMessage("SRM-REQUEST: RequestToken="+requestId,silent);
       if(requestId.equals("")) {
         util.printMessage("SRM-REQUEST: NewFileLifeTime="+fileLifeTime,silent);
       }
       else {
         util.printMessage("SRM-REQUEST: NewPinLifeTime="+pinLifeTime,silent);
       }
    }
 
    SrmExtendFileLifeTimeResponse response = _srm.srmExtendFileLifeTime(r);
 
    if(_debug) {
     util.printMessage("\nSRM-REQUEST: #### Output from SRM ####",silent);
    }
 
    if(response == null) {
      util.printMessage(
       "\nSRM-REQUEST: Null response from srmExtendFileLifeTime", silent);
      inputVec.addElement("Null response from srmExtendFileLifeTime");
      util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
      return response;
    }

    if(response.getReturnStatus() == null) {
       util.printMessage(
		"\nSRM-REQ-EXTENDFIELIFETIME: Null returnStatus from srmExtendFileLifeTime",silent);
       inputVec.addElement("Null returnStatus from srmExtendFileLifeTime");
       util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
       return response;
    }

    util.printMessage("status="+
      response.getReturnStatus().getStatusCode(),silent);
    util.printMessage("explanation="+
      response.getReturnStatus().getExplanation(),silent);
    inputVec.addElement("Status="+response.getReturnStatus().getStatusCode());
    inputVec.addElement("Explanation="+response.getReturnStatus().getExplanation());
    ArrayOfTSURLLifetimeReturnStatus fileStatuses =
        response.getArrayOfFileStatuses();
    if(fileStatuses == null) {
      inputVec.addElement("Null FileStatuses from SrmExtendFileLifeTimeRequest");
      util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
      util.printMessage(
        "\nSRM-REQUEST: Null FileStatuses from SrmExtendFileLifeTimeRequest", silent);
      return response;
    }

  TSURLLifetimeReturnStatus[] lReturnStatuses = fileStatuses.getStatusArray();
  if(lReturnStatuses == null) {
    util.printMessage
     ("\nSRM-REQUEST: Null fileStatuses.getStatusArray from SrmExtendFileLifeTimeRequest", silent);
    inputVec.addElement("Null fileStatuses.getStatusArray " +
        "from SrmExtendFileLifeTimeRequest");
    util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
    return response;
  }

  util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
  for(int i = 0; i < lReturnStatuses.length; i++) {
    inputVec = new Vector ();
    TSURLLifetimeReturnStatus lReturnStatus = lReturnStatuses[i];
    URI surl = lReturnStatus.getSurl();
    util.printMessage("\tSURL="+surl,silent);
    inputVec.addElement("SURL="+surl);
    TReturnStatus status = lReturnStatus.getStatus();
    if(status == null) {
      util.printMessage("\n\tSRM-REQUEST: Null return status from LifeTimeReturnStatus", silent);
      inputVec.addElement("\nNull return status from LifeTimeReturnStatus");
    }
    else {
      inputVec.addElement("status="+status.getStatusCode());
      inputVec.addElement("explanation="+status.getExplanation());
      util.printMessage("\tstatus="+status.getStatusCode(), silent);
      util.printMessage("\texplanation="+status.getExplanation(), silent);
    }
    util.printMessage("\tFileLifeTime="+lReturnStatus.getFileLifetime(), silent);
    inputVec.addElement("FileLifeTime="+lReturnStatus.getFileLifetime());
    util.printMessage("\tPinLifeTime="+lReturnStatus.getPinLifetime(), silent);
    inputVec.addElement("PinLifeTime="+lReturnStatus.getPinLifetime());
    util.printEventLog(_theLogger,"SrmExtendLifeTimeResponse",inputVec,silent);
  }
  return response;
 }
 catch(Exception e) {
   inputVec = new Vector ();
   inputVec.addElement("Exception="+e.getMessage());
   util.printEventLog(_theLogger,"SrmExtendLifeTime",inputVec,silent);
   String msg = e.getMessage();
   int idx = msg.indexOf("Connection refused");
   util.printMessage("SRM-REQUEST: Exception " + msg, silent);
   if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
      idx != -1) {
        util.printMessage("\nException : "+msg,silent);
   }
   else {
     throw e;
   }
 }
 return null;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// abortRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmAbortRequestResponse abortRequest(String requestId) 
  throws SRMClientException, Exception {

 try {
   util.printMessage("\nSRM-REQUEST: " + new Date() +
        " Calling SrmAbortRequest", silent);
   if(_debug) {
     util.printMessage(
       "\nSRM-REQUEST: ...Input parameters for SrmAbortRequest ....", silent);
   }
   SrmAbortRequestRequest r = new SrmAbortRequestRequest();
   r.setRequestToken(requestId);
   inputVec = new Vector ();
   inputVec.addElement("RequestToken="+requestId);
   if(!authId.equals("")) {
     r.setAuthorizationID(authId);
     inputVec.addElement("AuthorizationID="+authId);
   }
 
   SrmAbortRequestResponse result = _srm.srmAbortRequest(r);
 
   if(_debug) {
     util.printMessage("SRM-REQUEST: #### Output from SRM ####",silent);
   }
 
   inputVec = new Vector ();
 
   if(result == null) {
     inputVec.addElement("Null response from SrmAbortRequest");
     util.printEventLog(_theLogger,"SrmAbortResponse", inputVec,silent);
     util.printMessage("SRM-REQUEST: Null response from SrmAbortRequest", silent);
     return result;
   }

   if(result.getReturnStatus() == null) {
     inputVec.addElement("Null return status from SrmAbortRequest");
     util.printEventLog(_theLogger,"SrmAbortResponse", inputVec,silent);
     util.printMessage("SRM-REQUEST: Null return status from SrmAbortRequest",
        silent);
     return result;
   }

   inputVec.addElement("status="+ result.getReturnStatus().getStatusCode());
   inputVec.addElement("explanation="+ result.getReturnStatus().getExplanation());
   util.printMessage("\tstatus="+
     result.getReturnStatus().getStatusCode(),silent);
   util.printMessage("\texplanation="+
     result.getReturnStatus().getExplanation(),silent);
   util.printEventLog(_theLogger,"SrmAbortResponse", inputVec,silent);
   return result;
  }
  catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement("Exception="+e.getMessage());
    util.printEventLog(_theLogger,"SrmAbort", inputVec,silent);
    String msg = e.getMessage();
    int idx = msg.indexOf("Connection refused");
    util.printMessage("SRM-REQUEST: Exception " + msg, silent);
    if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
       idx != -1) {
      util.printMessage("\nException : "+msg,silent);
    }
    else {
      throw e;
    }
  }
  return null;
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printPutResult
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void printPutResult (SrmPrepareToPutResponse response) throws Exception {

    if(_debug) {
      util.printMessage("\nSRM-CLIENT: ####### Output from SRM #####",silent);
    }

    if (response == null) {
      inputVec = new Vector();
      inputVec.addElement("printPutResult:SRM returned null response");
      util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);
      util.printMessage("SRM-CLIENT: " +
            "printPutResult:SRM returned null response", silent);
    }

    if(response.getRequestToken() != null) {
      inputVec = new Vector();
      inputVec.addElement("request.token="+response.getRequestToken());
      util.printMessage("request.token="+ response.getRequestToken(),silent);
    }
    else {
      inputVec = new Vector();
      inputVec.addElement("request.token=null");
      util.printMessage("request.token=null",silent);
    }

    if(response.getReturnStatus() == null) {
       inputVec = new Vector();
       inputVec.addElement("printPutResult:SRM returned null return status for the response");
       util.printMessage(
		"SRM-CLIENT: printPutResult:SRM returned null return status for the response", silent);
       util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);
       return;
    }

    inputVec.addElement("response.status="+response.getReturnStatus().getStatusCode());
    inputVec.addElement("response.explanation="+response.getReturnStatus().getExplanation());

    util.printMessage("status="+
      response.getReturnStatus().getStatusCode(),silent);
    util.printMessage("explanation="+
      response.getReturnStatus().getExplanation(),silent);

    if(response.getRemainingTotalRequestTime() != null) {
        inputVec.addElement("request.remainingTotalRequestTime="+
            response.getRemainingTotalRequestTime());
        if(_debug) {
          util.printMessage("request.remainingTotalRequestTime=" +
             response.getRemainingTotalRequestTime(),silent);
        }
    }
    else {
        inputVec.addElement("request.remainingTotalRequestTime=null");
        if(_debug) {
          util.printMessage("request.remainingTotalRequestTime=" +
             null,silent);
        }
    }

    if(response.getArrayOfFileStatuses() == null) {
       inputVec.addElement("printPutResult:SRM returns null getArrayOfFileStatuses");
       util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);
       util.printMessage("SRM-CLIENT: " + 
			"printPutResult:SRM returns null getArrayOfFileStatuses ", silent);
       return;
    }

    if(response.getArrayOfFileStatuses().getStatusArray() == null) {
       inputVec.addElement("printPutResult:SRM returns null getStatusArray");
       util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);
       util.printMessage("SRM-CLIENT: " +
            "printPutResult:SRM returns null getStatusArray", silent);
       return;
    }

    int size = response.getArrayOfFileStatuses().getStatusArray().length;

    util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);

     for (int i=0;  i < size; i++) {
       TPutRequestFileStatus fileStatus =
         response.getArrayOfFileStatuses().getStatusArray(i);
       if(fileStatus != null) {
         inputVec = new Vector();
         if(fileStatus.getStatus() != null) {
           inputVec.addElement("Status="+fileStatus.getStatus().getStatusCode());
           inputVec.addElement("Explanation="+fileStatus.getStatus().getExplanation());
           util.printMessage("\tStatus="+
              fileStatus.getStatus().getStatusCode(),silent);
           util.printMessage("\tExplanation="+
              fileStatus.getStatus().getExplanation(),silent);
         }
         else {
           inputVec.addElement("Status=null");
           inputVec.addElement("Explanation=null");
           util.printMessage("\tStatus=null",silent);
           util.printMessage("\tExplanation=null",silent);
         }
         if(fileStatus.getSURL() != null) {
           util.printMessage("\tSURL="+
              fileStatus.getSURL().toString(),silent);
           inputVec.addElement("SURL="+fileStatus.getSURL().toString());
         }
         util.printEventLog(_theLogger,"SrmPrepareToPutResponse",inputVec,silent);
       }
   }

   if(_debug) {
     util.printMessage("...........................................",silent);
   }
}


}

