/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;
import gov.lbl.srm.client.util.*;
import java.util.*;
import java.util.logging.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** The SRMRequest object holds all outstanding request information and status.
*/

public class SRMRequest {
  private Stub _stub;
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private boolean submitCalled=false;
  private String log4jlocation="";
  private java.util.logging.Logger _theLogger;
  public SRMServer _srmServer;

  private Vector inputVec = new Vector ();

  private HashMap map = new HashMap();
  private int requestNo=0;
  private String requestId="";
  private String srmRequestId="";
  private String requestType="GET";
  private SRMRequestStatus srmRequestResponse = new SRMRequestStatus();
  private boolean fakeRequest;

  //input parameters list
  private TRetentionPolicyInfo retentionPolicyInfo = new TRetentionPolicyInfo ();
  //private TFileStorageType fileStorageType = TFileStorageType.VOLATILE;
  private TFileStorageType fileStorageType; 

  private String userDesc = "";
  private String authId = "";
  private String protocolsList = ""; 
  private String spaceToken="";
  private int totalRequestTime;
  private int pinLifeTime;
  private int fileLifeTime ;
  private int targetSURLLifeTime ;
  private int deferredStartTime;
  private String storageInfo ="";
  private String targetStorageInfo ="";
  private boolean dcau =true;
  private int parallelism = 1;
  private int bufferSize = 1048576;
  private boolean pushMode = false;
  private boolean overwrite=true;
  private int offset;
  private int count;
  private int numLevels;
  private boolean fullDetailedList=false;
  private boolean allLevelsRecursive;

  private Vector surlsVec = new Vector();
  private Vector fileSizeVec = new Vector();
  private Vector copyTargetSurlsVec = new Vector (); 
  private SRMRequestSummary requestSummary;

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMRequest --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** constructor
*/

public SRMRequest () throws Exception 
{
   //retentionPolicyInfo.setRetentionPolicy(TRetentionPolicy.REPLICA);
   //retentionPolicyInfo.setAccessLatency(TAccessLatency.ONLINE);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setSRMServer
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** This ties up SRMRequest to one SRM server that all request
    and status will go to.
    @param srmServer where SRMServer object is created and connected successfully to one SRM server
    @see    SRMServer  
*/

public void setSRMServer (SRMServer srmServer) throws Exception 
{
  _srmServer = srmServer;
  _stub = srmServer.getStub();
  _srm = srmServer.getISRM();
  _credential = srmServer.getCredential();
  _theLogger = srmServer.getLogger();
  _debug = srmServer.getDebug();
  silent = srmServer.getSilent();
  log4jlocation = srmServer.getLog4jLocation(); 
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getSRMServer
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** This will return the SRMServer object that SRMRequest object is 
    tied with.
    @return SRMServer
    @see SRMServer
*/

public SRMServer getSRMServer() {
  return _srmServer;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setUserDesc
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the user description for this request
    @param userDesc  user description input parameter
    @see String
*/

public void setUserDesc(String userDesc) {
  this.userDesc = userDesc;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAuthID
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the user authorization id for the request
   @param authId  authorizationId input parameter 
   @see String
*/

public void setAuthID(String authId) {
  this.authId = authId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the pre-reserved space token for the request
   @param spaceToken -- valid space token returned by the SRM server 
   @see String
*/

public void setTargetSpaceToken(String spaceToken) {
  this.spaceToken = spaceToken;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setProtocolsList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the preferred transfer protocol list for the request
   @param protocolsList --- preferred protocol list
   @see String
*/

public void setProtocolList(String protocolsList) {
  this.protocolsList = protocolsList; 
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFullDetailedList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the full detailed list flag for the request. Used only in LS. requests
*   @param fullDetailedList -- true or false is set
*/

public void setFullDetailedList(boolean fullDetailedList) {
  this.fullDetailedList = fullDetailedList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getFullDetailedList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** returns the full detailed list flag for the request. Used only in LS. requests
*   @return boolean -- true or false is returned
*/


public boolean getFullDetailedList() {
  return fullDetailedList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOffSet
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Sets the offset number for the LS request. Used in case the returned
    list is too large.
    @param offset -- valid integer
*/
public void setOffSet(int offset) {
   this.offset = offset;
}
 
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getOffSet
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** returns the offset set for the LS request.
*/

public int getOffSet() {
   return offset;
}
 
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setCount
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** sets the count number for the LS request. Used in case the returned list result is too large.
    @param count --- valid integer.
*/
public void setCount(int count) {
   this.count = count;
}
 
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getCount
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** returns the count number for the LS request.
    @return int
*/

public int getCount() {
   return count;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAllLevelsRecursive
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** sets the recursive flag for the LS requests.
    @param allLevelsRecursive -- true or false is set.
*/

public void setAllLevelsRecursive(boolean allLevelsRecursive) {
  this.allLevelsRecursive = allLevelsRecursive;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getAllLevelsRecursive
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** gets the recursive flag set for the LS requests.
*/
 
public boolean getAllLevelsRecursive() {
  return allLevelsRecursive;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setNumLevels
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** sets the recursive levels for the LS requests.
    @param numLevels --- num levels is set.
*/

public void setNumberLevels(int numLevels) {
   this.numLevels = numLevels;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getNumLevels
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
/** gets the number levels set for the LS requests.
    @return int
*/

public int getNumberLevels() {
   return numLevels;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the additional storage system info for the source SURLs in the request.
    @param storageInfo -- storage system info
*/

public void setStorageSystemInfo(String storageInfo) {
   this.storageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the additional storage system info for the target SURLs in the request.
    Used only for COPY request between two storages.
    @param storageInfo -- storage system info
*/

public void setTargetStorageSystemInfo(String storageInfo) {
   this.targetStorageInfo = storageInfo;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setOverwrite
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the overwrite flag for the request. It makes the target file to be
    overwritten when exists already, and used for put or copy requests.
    @param overwrite --- true or false is set.
*/

public void setOverwrite(boolean overwrite) {
   this.overwrite = overwrite;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTotalRequestTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the desired total request time for the request.
    @param totalRequestTime --- valid integer is set.
*/
public void setTotalRequestTime(int totalRequestTime) {
   this.totalRequestTime = totalRequestTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setPinLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the desired pin lifetime for the request.
    @param pinLifeTime --- valid integer is set.
*/

public void setPinLifeTime(int pinLifeTime) {
  this.pinLifeTime = pinLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the desired file lifetime for the request.
    @param fileLifeTime --- valid integer is set.
*/

public void setFileLifeTime(int fileLifeTime) {
  this.fileLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setTargetSURLLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the desired target file lifetime for the request. Used only for the COPY request.
    @param fileLifeTime --- valid integer is set.
*/

public void setTargetSURLLifeTime(int fileLifeTime) {
  this.targetSURLLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setDeferredStartTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the desired deferred start time for the request
   @param deferredStartTime --- valid integer is set.
*/

public void setDeferredStartTime(int deferredStartTime) {
  this.deferredStartTime = deferredStartTime;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setPushModd
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the push mode flag for the request. Used only in COPY request
    from the SRM server's point of view.
    @param pushMode --- true or false is set.
*/
public void setPushMode (boolean pushMode) {
   this.pushMode = pushMode;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setDCAU
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the DCAU option for the file transfer
    @param dcau --- true or false is set.
*/

public void setDCAU (boolean dcau) {
   this.dcau = dcau;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setParallelism
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the parallelism option for the file transfer
    @param parallelism --- true or false is set.
*/

public void setParallelism (int parallelism) {
   this.parallelism = parallelism;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setBufferSize
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the buffersize option for the file transfer
    @param bufferSize --- valid integer is set.
*/

public void setBufferSize (int bufferSize) {
   this.bufferSize = bufferSize;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFileStorageType
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the file storage type for the request: 
    VOLATILE|DURABLE|PERMANENT
    @param tokenType --- durable, volatile or permanent is set.
*/

public void setFileStorageType(String tokenType) {
  if(tokenType.toUpperCase().equals("DURABLE")) {
      fileStorageType = TFileStorageType.DURABLE;
  }
  else if(tokenType.toUpperCase().equals("PERMANENT")) {
      fileStorageType = TFileStorageType.PERMANENT;
  }
  else if(tokenType.toUpperCase().equals("VOLATILE")) {
      fileStorageType = TFileStorageType.VOLATILE;
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRetentionPolicy
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the retention policy for the request
    @param retentionPolicy --- replica or output or custodial is set.
*/

public void setRetentionPolicy(String retentionPolicy) {
  if(retentionPolicy.equalsIgnoreCase("REPLICA")) {
    retentionPolicyInfo.setRetentionPolicy(TRetentionPolicy.REPLICA);
  }
  else if(retentionPolicy.equalsIgnoreCase("OUTPUT")) {
    retentionPolicyInfo.setRetentionPolicy(TRetentionPolicy.OUTPUT);
  }
  else if(retentionPolicy.equalsIgnoreCase("CUSTODIAL")) {
    retentionPolicyInfo.setRetentionPolicy(TRetentionPolicy.CUSTODIAL);
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAccessLatency
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets the access latency for the request.
    @param accessLatency --- nearline or online is set.
*/
public void setAccessLatency(String accessLatency) {

   if(accessLatency.equalsIgnoreCase("NEARLINE")) {
      retentionPolicyInfo.setAccessLatency(TAccessLatency.NEARLINE);
   }
   else if(accessLatency.equalsIgnoreCase("ONLINE")) {
      retentionPolicyInfo.setAccessLatency(TAccessLatency.ONLINE);
   }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRequestType
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


/** sets requesttype for the request
    @param requestType --- put, get, copy, bringonline or ls is set.
*/

public void setRequestType(String requestType) throws SRMClientException {
  if(requestType.toUpperCase().equals("PUT") ||
     requestType.toUpperCase().equals("GET") ||
     requestType.toUpperCase().equals("COPY") ||
     requestType.toUpperCase().equals("BRINGONLINE") ||
     requestType.toUpperCase().equals("LS")) {
    this.requestType = requestType.toUpperCase();
  }
  else {
    throw new SRMClientException("Request type not supported");
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getRequestType
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** gets requesttype for the request
    @return String
    @see String
*/

public String getRequestType() {
  return requestType;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRequestToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** sets requesttoken for this request, used while reconstructing the request.
    @param reqId --- valid request token returned by the SRM server.
*/
public void setRequestToken(String reqId) throws SRMClientException {
  srmRequestId = reqId;
  fakeRequest = true;
  System.out.println("requesttype="+requestType);
  if(requestType.equals("PUT")) {
    requestId = "put:"+requestNo;
    requestNo++;
  }
  else if(requestType.equals("GET")) {
    requestId = "get:"+requestNo;
    requestNo++;
  }
  else if(requestType.equals("BRINGONLINE")) {
    requestId = "bringonline:"+requestNo;
    requestNo++;
  }
  else if(requestType.equals("COPY")) {
    requestId = "copy:"+requestNo;
    requestNo++;
  }
  else if(requestType.equals("LS")) {
    requestId = "ls:"+requestNo;
    requestNo++;
  }
  else
    throw new SRMClientException("Please set the requestType for this request");
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getRequestToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** returns request token after successful request submission.
    @return String -- valid request token for this request.
*/

public String getRequestToken () {
  return srmRequestId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** returns SURL in the request.
    @return String
    @see String
*/
public String[] getFiles() {
  int size = surlsVec.size();
  String[] result = new String[size];
  for(int i = 0; i < size; i++) {
    result[i] = (String)surlsVec.elementAt(i);
  }
  return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// addFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** adds SURLs for the request.
    @param surls --- source surls used for get, put, copy and bringonline requests. 
    @param copyTargetSurls -- target surls for the copy requests.
    @param fileSize ---- fileSizes
*/

public boolean addFiles(String[] surls, String[] copyTargetSurls, long[] fileSize) 
	throws SRMClientException {
   if(submitCalled) { 
	 throw new SRMClientException ("Cannot addFiles after calling submit for this request");
   }
  //for put surls and fileSize and needed
  //for get only surls is needed
  //for copy surls and copyTargetSurls are needed.
  boolean result = true;
  if(surls == null || surls.length == 0) {
    throw new SRMClientException ("Please provide the valid surls to add");
  }
  if(copyTargetSurls != null && copyTargetSurls.length > 0) {
    for(int i = 0; i < surls.length; i++) {
       if(surlsVec.contains(surls[i])) {
          result = false;
       }
       else {
          surlsVec.addElement(surls[i]);
          copyTargetSurlsVec.addElement(copyTargetSurls[i]);
       } 
    }
  }
  else if(fileSize != null && fileSize.length > 0) {
    for(int i = 0; i < surls.length; i++) {
       if(surlsVec.contains(surls[i])) {
          result = false;
       }
       else {
         surlsVec.addElement(surls[i]);
         fileSizeVec.addElement(new Long(fileSize[i]));
       }
    }
  }
  else { 
   for(int i = 0; i < surls.length; i++) {
       if(surlsVec.contains(surls[i])) {
          result = false;
       }
       else {
         surlsVec.addElement(surls[i]);
       }
   }
  }
  return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// removeFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** removes SURLs for the request before the request submission.
    @param surls --- surls to be removed from the request.
    @return boolean 
*/

public boolean removeFiles(String[] surls) throws SRMClientException {
   if(submitCalled) { 
	 throw new SRMClientException ("Cannot removeFiles after calling submit for this request");
   }
   boolean result = true;
   for(int i = 0; i < surls.length; i++) {
      if(surlsVec.contains(surls[i])) {
        surlsVec.remove(surls[i]);
      }
      else {
        result = false;
      }
   }
   return result;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// submit
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes a request on files that a user selected to the SRM server,
    and submission status and the request token get returned and stored 
    in the object.
    submits the prepared request. Depending on the request type, 
    srmPrepareToGet, srmPrepareToPut, srmBringOnline, or srmCopy
    is used internally. The returned status and request token are 
    stored in the object with a time stamp of the request submission
    time for the client retrieval.
    @return boolean
*/

public boolean submit() throws Exception {
    if(fakeRequest) {
 	  throw new SRMClientException 
		("Cannot call submit for this user reconstructed request");
    }
    if(submitCalled) { 
 	  throw new SRMClientException 
		("Cannot call submit again after calling submit for this request");
    }
    submitCalled=true;
    srmRequestResponse.setTimeStamp(new Date());
    if(requestType.equals("PUT")) {
      requestId = "put:"+requestNo;
      requestNo++;
      SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize, parallelism, dcau);
      putClient.setUserDesc(userDesc); 
      putClient.setAuthID(authId); 
      putClient.setProtocolsList(protocolsList); 
      putClient.setTargetSpaceToken(spaceToken);
      putClient.setTotalRequestTime(totalRequestTime);
      putClient.setPinLifeTime(pinLifeTime);
      putClient.setFileLifeTime(fileLifeTime);
      putClient.setStorageSystemInfo(storageInfo);
      putClient.setRetentionPolicyInfo(retentionPolicyInfo);
      putClient.setFileStorageType(fileStorageType);
      putClient.setOverwrite(overwrite);
      int length = surlsVec.size();
      String[] surls = new String[length];
      long[] fileSize = new long[length];
      for(int i = 0; i < length; i++) {
         surls[i] = (String) surlsVec.elementAt(i);
         fileSize[i] = ((Long) fileSizeVec.elementAt(i)).longValue();
      }
      SrmPrepareToPutResponse response = putClient.doPrepareToPut(surls,fileSize); 
      if(response == null) return false;

      if(response.getRequestToken() != null) {
        srmRequestId = response.getRequestToken();
      } 
      mapResponseToSrmRequestResponse((Object)response);
    }
    else if(requestType.equals("GET")) {
      requestId = "get:"+requestNo;
      requestNo++;
      SRMGetClient getClient = new SRMGetClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize, parallelism, dcau);
      getClient.setUserDesc(userDesc); 
      getClient.setAuthID(authId); 
      getClient.setProtocolsList(protocolsList); 
      getClient.setTargetSpaceToken(spaceToken);
      getClient.setTotalRequestTime(totalRequestTime);
      getClient.setPinLifeTime(pinLifeTime);
      getClient.setStorageSystemInfo(storageInfo);
      getClient.setRetentionPolicyInfo(retentionPolicyInfo);
      getClient.setFileStorageType(fileStorageType);
      int length = surlsVec.size();
      String[] surls = new String[length];
      for(int i = 0; i < length; i++) {
         surls[i] = (String) surlsVec.elementAt(i);
      }
      SrmPrepareToGetResponse response = getClient.doPrepareToGet(surls); 
      if(response == null) return false;

      if(response.getRequestToken() != null) {
        srmRequestId = response.getRequestToken();
      } 
      mapResponseToSrmRequestResponse((Object)response);
    }
    else if(requestType.equals("BRINGONLINE")) {
      requestId = "bringonline:"+requestNo;
      requestNo++;
      SRMBringOnlineClient bringOnlineClient = 
		new SRMBringOnlineClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize, parallelism, dcau);
      bringOnlineClient.setUserDesc(userDesc); 
      bringOnlineClient.setAuthID(authId); 
      bringOnlineClient.setProtocolsList(protocolsList); 
      bringOnlineClient.setTargetSpaceToken(spaceToken);
      bringOnlineClient.setTotalRequestTime(totalRequestTime);
      bringOnlineClient.setFileLifeTime(fileLifeTime);
      bringOnlineClient.setDeferredStartTime(deferredStartTime);
      bringOnlineClient.setStorageSystemInfo(storageInfo);
      bringOnlineClient.setRetentionPolicyInfo(retentionPolicyInfo);
      bringOnlineClient.setFileStorageType(fileStorageType);
      int length = surlsVec.size();
      String[] surls = new String[length];
      for(int i = 0; i < length; i++) {
         surls[i] = (String) surlsVec.elementAt(i);
      }
      SrmBringOnlineResponse response = bringOnlineClient.doBringOnline(surls); 
      if(response == null) return false;
      if(response.getRequestToken() != null) {
        srmRequestId = response.getRequestToken();
      } 
      mapResponseToSrmRequestResponse((Object)response);
    }
    else if(requestType.equals("COPY")) {
      requestId = "copy:"+requestNo;
      requestNo++;
      SRMCopyClient copyClient = 
		new SRMCopyClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize, parallelism, dcau);
      copyClient.setUserDesc(userDesc); 
      copyClient.setAuthID(authId); 
      copyClient.setTargetSpaceToken(spaceToken);
      copyClient.setTotalRequestTime(totalRequestTime);
      copyClient.setTargetSURLLifeTime(targetSURLLifeTime);
      copyClient.setStorageSystemInfo(storageInfo);
      copyClient.setTargetStorageSystemInfo(targetStorageInfo);
      copyClient.setRetentionPolicyInfo(retentionPolicyInfo);
      copyClient.setFileStorageType(fileStorageType);
      copyClient.setOverwrite(overwrite);
      copyClient.setPushMode(pushMode);
      int length = surlsVec.size();
      String[] surls = new String[length];
      String[] turls = new String[length];
      for(int i = 0; i < length; i++) {
         surls[i] = (String) surlsVec.elementAt(i);
         turls[i] = (String) copyTargetSurlsVec.elementAt(i);
      }
      SrmCopyResponse response = copyClient.doCopy(surls,turls); 
      if(response == null) return false;
      if(response.getRequestToken() != null) {
        srmRequestId = response.getRequestToken();
      } 
      mapResponseToSrmRequestResponse((Object)response);
    }
    return true;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mapResponseToSrmRequestResponse
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void mapResponseToSrmRequestResponse(Object response) 
 throws SRMClientException, Exception {

  if(response instanceof SrmPrepareToPutResponse) {
    SrmPrepareToPutResponse putResponse = (SrmPrepareToPutResponse) response;
    if(putResponse != null) {
       srmRequestResponse.setReturnStatus(putResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(putResponse.getRequestToken());
       srmRequestResponse.setRemainingTotalRequestTime
			(putResponse.getRemainingTotalRequestTime());
      
       ArrayOfTPutRequestFileStatus arrayOfFileStatus = 
				putResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TPutRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = new HashMap();
               for(int i = 0; i < fileStatuses.length; i++) {
                   FileStatus srmFileStatus = new FileStatus();
                   TPutRequestFileStatus fileStatus = fileStatuses[i];
                   if(fileStatus.getSURL() != null) {
                     srmFileStatus.setSURL(fileStatus.getSURL());
                     map.put(fileStatus.getSURL().toString(), srmFileStatus);
                   }
                   if(fileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(fileStatus.getStatus());
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       TStatusCode code = fileStatus.getStatus().getStatusCode();
                       SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                       srmRequestReturnStatus.setStatus(code.toString());
                       srmRequestReturnStatus.setExplanation
							(fileStatus.getStatus().getExplanation());
                       srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     srmFileStatus.setFileSize(fileStatus.getFileSize());
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                   }
                   if(fileStatus.getRemainingPinLifetime() != null) {
                     srmFileStatus.setRemainingPinLifeTime(
						fileStatus.getRemainingPinLifetime());
                   }
                   if(fileStatus.getRemainingFileLifetime() != null) {
                     srmFileStatus.setRemainingFileLifeTime(
						fileStatus.getRemainingFileLifetime());
                   }
                   if(fileStatus.getTransferURL() != null) {
                     srmFileStatus.setTransferSURL(
						fileStatus.getTransferURL());
                   }
                   if(fileStatus.getTransferProtocolInfo() != null) {
                     srmFileStatus.setTransferProtocolInfo(
						fileStatus.getTransferProtocolInfo());
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmPrepareToPutResponse)
  else if(response instanceof SrmStatusOfPutRequestResponse) {
    SrmStatusOfPutRequestResponse putStatusResponse = 
			(SrmStatusOfPutRequestResponse) response;
    if(putStatusResponse != null) {
       srmRequestResponse.setReturnStatus(putStatusResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(srmRequestId);
       srmRequestResponse.setRemainingTotalRequestTime
			(putStatusResponse.getRemainingTotalRequestTime());
      
       ArrayOfTPutRequestFileStatus arrayOfFileStatus = 
				putStatusResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TPutRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = srmRequestResponse.getFileStatuses();
               for(int i = 0; i < fileStatuses.length; i++) {
                   TPutRequestFileStatus fileStatus = fileStatuses[i];
                   FileStatus srmFileStatus = null;
                   if(fileStatus.getSURL() != null) {
                     Object obj = map.get(fileStatus.getSURL().toString());
                     if(obj != null) {
                        srmFileStatus = (FileStatus) obj;
                        srmFileStatus.setSURL(fileStatus.getSURL());
                     }
                     else {
                       if(fakeRequest) {
                         srmFileStatus = new FileStatus();
                         srmFileStatus.setSURL(fileStatus.getSURL());
                         map.put(fileStatus.getSURL().toString(), srmFileStatus);
                         surlsVec.addElement(fileStatus.getSURL().toString());
                       }
                     }
                   }
                   if(fileStatus.getStatus() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setStatus(fileStatus.getStatus());
                     }
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       if(srmFileStatus != null) {
                         TStatusCode code = fileStatus.getStatus().getStatusCode();
                         SrmRequestReturnStatus srmRequestReturnStatus = 
								srmFileStatus.getSrmRequestReturnStatus();
                         if(srmRequestReturnStatus == null) {
                           srmRequestReturnStatus = new SrmRequestReturnStatus();
                         }
                         srmRequestReturnStatus.setStatus(code.toString());
                         srmRequestReturnStatus.setExplanation
							(fileStatus.getStatus().getExplanation());
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       } 
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     if(srmFileStatus != null) {
                        srmFileStatus.setFileSize(fileStatus.getFileSize());
                     }
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                     }
                   }
                   if(fileStatus.getRemainingPinLifetime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setRemainingPinLifeTime(
					   	  fileStatus.getRemainingPinLifetime());
                     }
                   }
                   if(fileStatus.getRemainingFileLifetime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setRemainingFileLifeTime(
					   	   fileStatus.getRemainingFileLifetime());
                     }
                   }
                   if(fileStatus.getTransferURL() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setTransferSURL(
						  fileStatus.getTransferURL());
                     } 
                   }
                   if(fileStatus.getTransferProtocolInfo() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setTransferProtocolInfo(
						  fileStatus.getTransferProtocolInfo());
                     }
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmStatusofPutRequestResponse)
  else if(response instanceof SrmPrepareToGetResponse) {
    SrmPrepareToGetResponse getResponse = (SrmPrepareToGetResponse) response;
    if(getResponse != null) {
       srmRequestResponse.setReturnStatus(getResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(getResponse.getRequestToken());
       srmRequestResponse.setRemainingTotalRequestTime
			(getResponse.getRemainingTotalRequestTime());
      
       ArrayOfTGetRequestFileStatus arrayOfFileStatus = 
				getResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TGetRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = new HashMap();
               for(int i = 0; i < fileStatuses.length; i++) {
                   FileStatus srmFileStatus = new FileStatus();
                   TGetRequestFileStatus fileStatus = fileStatuses[i];
                   if(fileStatus.getSourceSURL() != null) {
                     srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                   }
                   if(fileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(fileStatus.getStatus());
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       TStatusCode code = fileStatus.getStatus().getStatusCode();
                       SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                       srmRequestReturnStatus.setStatus(code.toString());
                       srmRequestReturnStatus.setExplanation(
						fileStatus.getStatus().getExplanation());
                       srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     srmFileStatus.setFileSize(fileStatus.getFileSize());
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                   }
                   if(fileStatus.getRemainingPinTime() != null) {
                     srmFileStatus.setRemainingPinLifeTime(
						fileStatus.getRemainingPinTime());
                   }
                   if(fileStatus.getTransferURL() != null) {
                     srmFileStatus.setTransferSURL(
						fileStatus.getTransferURL());
                   }
                   if(fileStatus.getTransferProtocolInfo() != null) {
                     srmFileStatus.setTransferProtocolInfo(
						fileStatus.getTransferProtocolInfo());
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmPrepareToGetResponse)
  else if(response instanceof SrmStatusOfGetRequestResponse) {
    SrmStatusOfGetRequestResponse getStatusResponse = 
			(SrmStatusOfGetRequestResponse) response;
    if(getStatusResponse != null) {
       srmRequestResponse.setReturnStatus(getStatusResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(srmRequestId);
       srmRequestResponse.setRemainingTotalRequestTime
			(getStatusResponse.getRemainingTotalRequestTime());
      
       ArrayOfTGetRequestFileStatus arrayOfFileStatus = 
				getStatusResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TGetRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = srmRequestResponse.getFileStatuses();
               for(int i = 0; i < fileStatuses.length; i++) {
                   TGetRequestFileStatus fileStatus = fileStatuses[i];
                   FileStatus srmFileStatus = null;
                   if(fileStatus.getSourceSURL() != null) {
                     Object obj = map.get(fileStatus.getSourceSURL().toString());
                     if(obj != null) {
                        srmFileStatus = (FileStatus) obj;
                        srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     }
                     else {
                       if(fakeRequest) {
                         srmFileStatus = new FileStatus();
                         srmFileStatus.setSURL(fileStatus.getSourceSURL());
                         map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                         surlsVec.addElement(fileStatus.getSourceSURL().toString());
                       }
                     }
                   }
                   if(fileStatus.getStatus() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setStatus(fileStatus.getStatus());
                     }
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       if(srmFileStatus != null) {
                         TStatusCode code = fileStatus.getStatus().getStatusCode();
                         SrmRequestReturnStatus srmRequestReturnStatus = 
								srmFileStatus.getSrmRequestReturnStatus();
                         if(srmRequestReturnStatus == null) {
                           srmRequestReturnStatus = new SrmRequestReturnStatus();
                         }
                         srmRequestReturnStatus.setStatus(code.toString());
                         srmRequestReturnStatus.setExplanation(
								fileStatus.getStatus().getExplanation());
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       } 
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     if(srmFileStatus != null) {
                        srmFileStatus.setFileSize(fileStatus.getFileSize());
                     }
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                     }
                   }
                   if(fileStatus.getRemainingPinTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setRemainingPinLifeTime(
					   	  fileStatus.getRemainingPinTime());
                     }
                   }
                   if(fileStatus.getTransferURL() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setTransferSURL(
						  fileStatus.getTransferURL());
                     } 
                   }
                   if(fileStatus.getTransferProtocolInfo() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setTransferProtocolInfo(
						  fileStatus.getTransferProtocolInfo());
                     }
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmStatusofGetRequestResponse)
  else if(response instanceof SrmBringOnlineResponse) {
    SrmBringOnlineResponse bringOnlineResponse = (SrmBringOnlineResponse) response;
    if(bringOnlineResponse != null) {
       srmRequestResponse.setReturnStatus(bringOnlineResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(bringOnlineResponse.getRequestToken());
       srmRequestResponse.setRemainingTotalRequestTime
			(bringOnlineResponse.getRemainingTotalRequestTime());
       srmRequestResponse.setRemainingDeferredStartTime
			(bringOnlineResponse.getRemainingDeferredStartTime());
      
       ArrayOfTBringOnlineRequestFileStatus arrayOfFileStatus = 
				bringOnlineResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TBringOnlineRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = new HashMap();
               for(int i = 0; i < fileStatuses.length; i++) {
                   FileStatus srmFileStatus = new FileStatus();
                   TBringOnlineRequestFileStatus fileStatus = fileStatuses[i];
                   if(fileStatus.getSourceSURL() != null) {
                     srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                   }
                   if(fileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(fileStatus.getStatus());
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       TStatusCode code = fileStatus.getStatus().getStatusCode();
                       SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                       srmRequestReturnStatus.setStatus(code.toString());
                       srmRequestReturnStatus.setExplanation
							(fileStatus.getStatus().getExplanation());
                       srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     srmFileStatus.setFileSize(fileStatus.getFileSize());
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                   }
                   if(fileStatus.getRemainingPinTime() != null) {
                     srmFileStatus.setRemainingPinLifeTime(
						fileStatus.getRemainingPinTime());
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
         }
      }
   }
  else if(response instanceof SrmStatusOfBringOnlineRequestResponse) {
    SrmStatusOfBringOnlineRequestResponse bringOnlineStatusResponse = 
			(SrmStatusOfBringOnlineRequestResponse) response;
    if(bringOnlineStatusResponse != null) {
       srmRequestResponse.setReturnStatus(bringOnlineStatusResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(srmRequestId);
       srmRequestResponse.setRemainingTotalRequestTime
			(bringOnlineStatusResponse.getRemainingTotalRequestTime());
       srmRequestResponse.setRemainingDeferredStartTime
			(bringOnlineStatusResponse.getRemainingDeferredStartTime());
      
       ArrayOfTBringOnlineRequestFileStatus arrayOfFileStatus = 
				bringOnlineStatusResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TBringOnlineRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = srmRequestResponse.getFileStatuses();
               for(int i = 0; i < fileStatuses.length; i++) {
                   TBringOnlineRequestFileStatus fileStatus = fileStatuses[i];
                   FileStatus srmFileStatus = null;
                   if(fileStatus.getSourceSURL() != null) {
                     Object obj = map.get(fileStatus.getSourceSURL().toString());
                     if(obj != null) {
                        srmFileStatus = (FileStatus) obj;
                        srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     }
                     else {
                       if(fakeRequest) {
                         srmFileStatus = new FileStatus();
                         srmFileStatus.setSURL(fileStatus.getSourceSURL());
                         map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                         surlsVec.addElement(fileStatus.getSourceSURL().toString());
                       }
                     }
                   }
                   if(fileStatus.getStatus() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setStatus(fileStatus.getStatus());
                     }
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       if(srmFileStatus != null) {
                         TStatusCode code = fileStatus.getStatus().getStatusCode();
                         SrmRequestReturnStatus srmRequestReturnStatus = 
								srmFileStatus.getSrmRequestReturnStatus();
                         if(srmRequestReturnStatus == null) {
                           srmRequestReturnStatus = new SrmRequestReturnStatus();
                         }
                         srmRequestReturnStatus.setStatus(code.toString());
                         srmRequestReturnStatus.setExplanation
								(fileStatus.getStatus().getExplanation());
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       } 
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     if(srmFileStatus != null) {
                        srmFileStatus.setFileSize(fileStatus.getFileSize());
                     }
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                     }
                   }
                   if(fileStatus.getRemainingPinTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setRemainingPinLifeTime(
					   	  fileStatus.getRemainingPinTime());
                     }
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmStatusofBringOnlineRequestResponse)
  else if(response instanceof SrmCopyResponse) {
    SrmCopyResponse copyResponse = (SrmCopyResponse) response;
    if(copyResponse != null) {
       srmRequestResponse.setReturnStatus(copyResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(copyResponse.getRequestToken());
       srmRequestResponse.setRemainingTotalRequestTime
			(copyResponse.getRemainingTotalRequestTime());
      
       ArrayOfTCopyRequestFileStatus arrayOfFileStatus = 
				copyResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TCopyRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = new HashMap();
               for(int i = 0; i < fileStatuses.length; i++) {
                   FileStatus srmFileStatus = new FileStatus();
                   TCopyRequestFileStatus fileStatus = fileStatuses[i];
                   if(fileStatus.getSourceSURL() != null) {
                     srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                   }
                   if(fileStatus.getTargetSURL() != null) {
                     srmFileStatus.setTURL(fileStatus.getTargetSURL());
                   }
                   if(fileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(fileStatus.getStatus());
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       TStatusCode code = fileStatus.getStatus().getStatusCode();
                       SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                       srmRequestReturnStatus.setStatus(code.toString());
                       srmRequestReturnStatus.setExplanation
							(fileStatus.getStatus().getExplanation());
                       srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     srmFileStatus.setFileSize(fileStatus.getFileSize());
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                   }
                   if(fileStatus.getRemainingFileLifetime() != null) {
                     srmFileStatus.setRemainingFileLifeTime(
						fileStatus.getRemainingFileLifetime());
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
         }
      }
   }//end if(SrmCopyResponse)
   else if(response instanceof SrmStatusOfCopyRequestResponse) {
    SrmStatusOfCopyRequestResponse copyStatusResponse = 
			(SrmStatusOfCopyRequestResponse) response;
    if(copyStatusResponse != null) {
       srmRequestResponse.setReturnStatus(copyStatusResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(srmRequestId);
       srmRequestResponse.setRemainingTotalRequestTime
			(copyStatusResponse.getRemainingTotalRequestTime());
      
       ArrayOfTCopyRequestFileStatus arrayOfFileStatus = 
				copyStatusResponse.getArrayOfFileStatuses();
       if(arrayOfFileStatus != null) {
            TCopyRequestFileStatus[] fileStatuses = arrayOfFileStatus.getStatusArray();
            if(fileStatuses != null) {
               HashMap map = srmRequestResponse.getFileStatuses();
               for(int i = 0; i < fileStatuses.length; i++) {
                   TCopyRequestFileStatus fileStatus = fileStatuses[i];
                   FileStatus srmFileStatus = null;
                   if(fileStatus.getSourceSURL() != null) {
                     Object obj = map.get(fileStatus.getSourceSURL().toString());
                     if(obj != null) {
                        srmFileStatus = (FileStatus) obj;
                        srmFileStatus.setSURL(fileStatus.getSourceSURL());
                     }
                     else {
                       if(fakeRequest) {
                         srmFileStatus = new FileStatus();
                         srmFileStatus.setSURL(fileStatus.getSourceSURL());
                         map.put(fileStatus.getSourceSURL().toString(), srmFileStatus);
                         surlsVec.addElement(fileStatus.getSourceSURL().toString());
                       }
                     }
                   }
                   if(fileStatus.getTargetSURL() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setTURL(fileStatus.getTargetSURL());
                     }
                   }
                   if(fileStatus.getStatus() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setStatus(fileStatus.getStatus());
                     }
                     if(fileStatus.getStatus().getStatusCode() != null) {
                       if(srmFileStatus != null) {
                         TStatusCode code = fileStatus.getStatus().getStatusCode();
                         SrmRequestReturnStatus srmRequestReturnStatus = 
								srmFileStatus.getSrmRequestReturnStatus();
                         if(srmRequestReturnStatus == null) {
                           srmRequestReturnStatus = new SrmRequestReturnStatus();
                         }
                         srmRequestReturnStatus.setStatus(code.toString());
                         srmRequestReturnStatus.setExplanation
							(fileStatus.getStatus().getExplanation());
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       } 
                     }
                   }
                   if(fileStatus.getFileSize() != null) {
                     if(srmFileStatus != null) {
                        srmFileStatus.setFileSize(fileStatus.getFileSize());
                     }
                   }
                   if(fileStatus.getEstimatedWaitTime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
                     }
                   }
                   if(fileStatus.getRemainingFileLifetime() != null) {
                     if(srmFileStatus != null) {
                       srmFileStatus.setRemainingFileLifeTime(
					   	   fileStatus.getRemainingFileLifetime());
                     }
                   }
               }
               srmRequestResponse.setFileStatuses(map);
            }
       }
    }
  }//end if (instanceof SrmStatusofCopyRequestResponse)
  else if(response instanceof SrmLsResponse) {
    SrmLsResponse lsResponse = (SrmLsResponse) response;
    if(lsResponse != null) {
       srmRequestResponse.setReturnStatus(lsResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(lsResponse.getRequestToken());
       if(lsResponse.getDetails() != null) {
         ArrayOfTMetaDataPathDetail arrayOfPathDetails = lsResponse.getDetails();
         HashMap map = new HashMap();   
         srmRequestResponse.setPathDetails(map);
         if(arrayOfPathDetails.getPathDetailArray() != null) {
           TMetaDataPathDetail[] pathDetails = arrayOfPathDetails.getPathDetailArray();
           for(int i = 0; i < pathDetails.length; i++) {
             TMetaDataPathDetail pathDetail = pathDetails[i];
             PathDetail pDetail = mapMetaDataPathDetailToPathDetail(pathDetail,map);
             if(pathDetail.getArrayOfSubPaths() != null) {
                if(pDetail != null) {
                  ArrayOfTMetaDataPathDetail details = pathDetail.getArrayOfSubPaths(); 
                  if(details != null) {
                    TMetaDataPathDetail[] subPaths = details.getPathDetailArray();
                    PathDetail[] pSubPaths = new PathDetail[subPaths.length];
                    for(int j = 0; j < subPaths.length; j++) { 
                       pSubPaths[j] = mapMetaDataPathDetailToPathDetail(subPaths[j],null);
                    }
                    pDetail.setSubPath(pSubPaths);
                  }//end if details!= null
                }//end if pDetail != null
              }//end if pathDetail.getArrayOfSubPaths() != null
             }//end for
         }//if(getPathDetailsArray != null)
       }//if(lsResponse.getDetails() != null)
    }//if(lsResponse != null)
  }//end if(instanceof SrmLsResponse)
  else if(response instanceof SrmStatusOfLsRequestResponse) {
    SrmStatusOfLsRequestResponse lsResponse = (SrmStatusOfLsRequestResponse) response;
    if(lsResponse != null) {
       srmRequestResponse.setReturnStatus(lsResponse.getReturnStatus()); 
       srmRequestResponse.setRequestToken(srmRequestId);
       if(lsResponse.getDetails() != null) {
         ArrayOfTMetaDataPathDetail arrayOfPathDetails = lsResponse.getDetails();
         HashMap map = srmRequestResponse.getPathDetails();
         if(arrayOfPathDetails.getPathDetailArray() != null) {
           TMetaDataPathDetail[] pathDetails = arrayOfPathDetails.getPathDetailArray();
           for(int i = 0; i < pathDetails.length; i++) {
             TMetaDataPathDetail pathDetail = pathDetails[i];
             PathDetail pDetail = null;
             if(pathDetail.getPath() != null) {   
               Object obj = map.get(pathDetail.getPath());
               if(obj != null) {
                  pDetail = (PathDetail) obj;
               }
               else {
                 if(fakeRequest) {
                    pDetail = new PathDetail();
                    pDetail.setPath(pathDetail.getPath());
                    map.put(pathDetail.getPath(),pDetail);
                    surlsVec.addElement(pathDetail.getPath());
                 }
               }
             }
             if(pDetail != null) {
               if(pathDetail.getStatus() != null) {
                  pDetail.setStatus(pathDetail.getStatus());
               } 
             }
             if(pDetail != null) {
               if(pathDetail.getSize() != null) {
                 pDetail.setSize(pathDetail.getSize());
               }
             }
             if(pDetail != null) {
               if(pathDetail.getCreatedAtTime() != null) {
                 pDetail.setCreatedAtTime(pathDetail.getCreatedAtTime());
               } 
             }
             if(pDetail != null) {
               if(pathDetail.getLastModificationTime() != null) {
                 pDetail.setLastModificationTime(pathDetail.getLastModificationTime());
               }
             } 
             if(pDetail != null) {
               if(pathDetail.getFileStorageType() != null) {
                  pDetail.setFileStorageType(pathDetail.getFileStorageType());
               }
             }
             if(pDetail != null) {
               if(pathDetail.getRetentionPolicyInfo() != null) {
                  pDetail.setRetentionPolicyInfo(pathDetail.getRetentionPolicyInfo());
               }
             }
             if(pDetail != null) {
               if(pathDetail.getFileLocality() != null) {
                  pDetail.setFileLocality(pathDetail.getFileLocality());
               }
             }
             if(pDetail != null) {
               if(pathDetail.getType() != null) {
                  pDetail.setFileType(pathDetail.getType());
               }
             }
             if(pDetail != null) {
               if(pathDetail.getLifetimeAssigned() != null) {
                  pDetail.setLifeTimeAssigned(pathDetail.getLifetimeAssigned());
               }
             }
             if(pDetail != null) { 
              if(pathDetail.getLifetimeLeft() != null) {
                 pDetail.setLifeTimeLeft(pathDetail.getLifetimeLeft());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getCheckSumType() != null) {
                 pDetail.setCheckSumType(pathDetail.getCheckSumType());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getCheckSumValue() != null) {
                 pDetail.setCheckSumValue(pathDetail.getCheckSumValue());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getArrayOfSpaceTokens() != null) {
                 pDetail.setArrayOfSpaceTokens(pathDetail.getArrayOfSpaceTokens());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getOwnerPermission() != null) {
                 pDetail.setOwnerPermission(pathDetail.getOwnerPermission());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getGroupPermission() != null) {
                 pDetail.setGroupPermission(pathDetail.getGroupPermission());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getOtherPermission() != null) {
                 pDetail.setOtherPermission(pathDetail.getOtherPermission());
              }
             }
             if(pDetail != null) {
              if(pathDetail.getArrayOfSubPaths() != null) {
                 ArrayOfTMetaDataPathDetail details  = pathDetail.getArrayOfSubPaths();
                 if(details != null) {
                   TMetaDataPathDetail[] pp = details.getPathDetailArray();
                   PathDetail[] pSubPaths = new PathDetail[pp.length];
                   for(int j = 0; j < pp.length; j++) {
                     pSubPaths[j] = mapMetaDataPathDetailToPathDetail(pp[j],null);
                   }
                   pDetail.setSubPath(pSubPaths);
                 }
              }
             }
           }//end for
         }//if(getPathDetailsArray != null)
       }//end if (lsResponse.getDetails != null)
     }//end if (lsResponse)
   }//end if (instanace of SrmStatusOfLsRequestResponse)
}
       
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mapMetaDataPathDetailToPathDetail
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private PathDetail mapMetaDataPathDetailToPathDetail
	(TMetaDataPathDetail pathDetail, HashMap map) throws Exception
{

   PathDetail pDetail = new PathDetail();
   if(pathDetail.getPath() != null) {   
      pDetail.setPath(pathDetail.getPath());
      if(map != null) {
        map.put(pathDetail.getPath(),pDetail);
      }
   }
   if(pathDetail.getStatus() != null) {
      pDetail.setStatus(pathDetail.getStatus());
   }
   if(pathDetail.getSize() != null) {
      pDetail.setSize(pathDetail.getSize());
   }
   if(pathDetail.getCreatedAtTime() != null) {
      pDetail.setCreatedAtTime(pathDetail.getCreatedAtTime());
   }
   if(pathDetail.getLastModificationTime() != null) {
      pDetail.setLastModificationTime(pathDetail.getLastModificationTime());
   }
   if(pathDetail.getFileStorageType() != null) {
      pDetail.setFileStorageType(pathDetail.getFileStorageType());
   }
   if(pathDetail.getRetentionPolicyInfo() != null) {
      pDetail.setRetentionPolicyInfo(pathDetail.getRetentionPolicyInfo());
   }
   if(pathDetail.getFileLocality() != null) {
      pDetail.setFileLocality(pathDetail.getFileLocality());
   }
   if(pathDetail.getType() != null) {
      pDetail.setFileType(pathDetail.getType());
   }
   if(pathDetail.getLifetimeAssigned() != null) {
      pDetail.setLifeTimeAssigned(pathDetail.getLifetimeAssigned());
   }
   if(pathDetail.getLifetimeLeft() != null) {
      pDetail.setLifeTimeLeft(pathDetail.getLifetimeLeft());
   }
   if(pathDetail.getCheckSumType() != null) {
      pDetail.setCheckSumType(pathDetail.getCheckSumType());
   }
   if(pathDetail.getCheckSumValue() != null) {
      pDetail.setCheckSumValue(pathDetail.getCheckSumValue());
   }
   if(pathDetail.getArrayOfSpaceTokens() != null) {
      pDetail.setArrayOfSpaceTokens(pathDetail.getArrayOfSpaceTokens());
   }
   if(pathDetail.getOwnerPermission() != null) {
      pDetail.setOwnerPermission(pathDetail.getOwnerPermission());
   }
   if(pathDetail.getGroupPermission() != null) {
      pDetail.setGroupPermission(pathDetail.getGroupPermission());
   }
   if(pathDetail.getOtherPermission() != null) {
      pDetail.setOtherPermission(pathDetail.getOtherPermission());
   }
   return pDetail;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mapToSrmRequestResponseAbortRequest
// used for abortRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void mapToSrmRequestResponseAbortRequest(TReturnStatus rStatus)
  throws SRMClientException, Exception {

  if(srmRequestResponse != null) {
    srmRequestResponse.setReturnStatus(rStatus);
    HashMap map = srmRequestResponse.getFileStatuses();
    Collection c = map.values();
    Object[] obj = c.toArray();
    for(int i = 0; i < obj.length; i++) {
       FileStatus srmFileStatus = (FileStatus) obj[i];
       SrmRequestReturnStatus srmRequestReturnStatus = 
		  new SrmRequestReturnStatus();
       if(rStatus != null) {
          TStatusCode code = rStatus.getStatusCode();
          if(code != null) {
             if(code == TStatusCode.SRM_SUCCESS) {
                srmRequestReturnStatus.setStatus("SRM_ABORT_REQUEST_DONE");
                srmRequestReturnStatus.setExplanation("Abort Request is successful");
             }
             else {
                srmRequestReturnStatus.setStatus("SRM_ABORT_REQUEST_FAILED");
                srmRequestReturnStatus.setExplanation("Abort Request is not successful");
             }
             srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
          }//if(code)
       }//if(rStatus)
    }//end for
  }//if(srmRequestResponse != null)
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mapToSrmRequestResponse
// used for extendfilelifetime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void mapToSrmRequestResponse(TSURLLifetimeReturnStatus[] returnFileStatuses)
  throws SRMClientException, Exception {

  if(srmRequestResponse != null) {
    HashMap map = srmRequestResponse.getFileStatuses();
    for(int i = 0; i < returnFileStatuses.length; i++) {
       TSURLLifetimeReturnStatus returnFileStatus = returnFileStatuses[i];
       if(returnFileStatus != null) {
           if(returnFileStatus.getSurl() != null) {
              String surl = returnFileStatus.getSurl().toString();
              Object obj = map.get(surl);
              if(obj != null) {
                FileStatus srmFileStatus = (FileStatus) obj;
                if(srmFileStatus != null) { 
                    if(returnFileStatus.getPinLifetime() != null) {
                      srmFileStatus.setPinLifeTime(returnFileStatus.getPinLifetime());
                    }
                    if(returnFileStatus.getFileLifetime() != null) {
                      srmFileStatus.setFileLifeTime(returnFileStatus.getFileLifetime());
                    }
                }
                if(returnFileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(returnFileStatus.getStatus());
                     SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                     TReturnStatus  rStatus = returnFileStatus.getStatus(); 
                     if(rStatus != null) {
                       TStatusCode code = rStatus.getStatusCode();
                       if(code != null) {
                         if(code == TStatusCode.SRM_SUCCESS) {
                           srmRequestReturnStatus.setStatus("SRM_EXTENDED_FILE_TIME");
                           srmRequestReturnStatus.setExplanation
								("Extend File Time is successful");
                         }
                         else {
                           srmRequestReturnStatus.setStatus("SRM_EXTENDED_FILE_TIME_FAILED");
                           srmRequestReturnStatus.setExplanation
								("Extend File Time is not successful");
                         }
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       }//if(code)
                     }//if(rStatus)
                }//end if
              }//enf if(obj)
           }//if(returnFileStatus.getSurl()); 
       }//if(returnFileStatus)
    }//end for
  }//if(srmRequestResponse != null)
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// mapToSrmRequestResponse
// used for putdone and abortfiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void mapToSrmRequestResponse(TSURLReturnStatus[] returnFileStatuses,
	int type) throws SRMClientException, Exception {

  //type == 0, putdone
  //type == 1, abortFiles
  //type == 2, releaseFiles
  //type == 3, rmFile
  if(srmRequestResponse != null) {
    HashMap map = srmRequestResponse.getFileStatuses();
    for(int i = 0; i < returnFileStatuses.length; i++) {
       TSURLReturnStatus returnFileStatus = returnFileStatuses[i];
       if(returnFileStatus != null) {
           if(returnFileStatus.getSurl() != null) {
              String surl = returnFileStatus.getSurl().toString();
              Object obj = map.get(surl);
              if(obj != null) {
                FileStatus srmFileStatus = (FileStatus) obj;
                if(returnFileStatus.getStatus() != null) {
                     srmFileStatus.setStatus(returnFileStatus.getStatus());
                     SrmRequestReturnStatus srmRequestReturnStatus = 
							new SrmRequestReturnStatus();
                     TReturnStatus  rStatus = returnFileStatus.getStatus(); 
                     if(rStatus != null) {
                       TStatusCode code = rStatus.getStatusCode();
                       if(code != null) {
                         if(code == TStatusCode.SRM_SUCCESS) {
                            if(type == 0) {
                              srmRequestReturnStatus.setStatus("SRM_PUT_DONE");
                              srmRequestReturnStatus.setExplanation("PUT Done is successful");
                            }
                            else if(type == 1) {
                              srmRequestReturnStatus.setStatus("SRM_ABORT_FILES");
                              srmRequestReturnStatus.setExplanation
									("Abort Files is successful");
                            }
                            else if(type == 2) {
                              srmRequestReturnStatus.setStatus("SRM_FILE_RELEASED");
                              srmRequestReturnStatus.setExplanation
									("Release Files is successful");
                            }
                            else if(type == 3) {
                              srmRequestReturnStatus.setStatus("SRM_FILE_REMOVED");
                              srmRequestReturnStatus.setExplanation
									("Remove Files is successful");
                            }
                         }
                         else {
                            if(type == 0) {
                              srmRequestReturnStatus.setStatus("SRM_PUT_DONE_FAILED");
                              srmRequestReturnStatus.setExplanation
									("PUT Done is not successful");
                            }
                            else if(type == 1) {
                              srmRequestReturnStatus.setStatus("SRM_ABORT_FILES_FAILED");
                              srmRequestReturnStatus.setExplanation
									("Abort Files is not successful");
                            }
                            else if(type == 2) {
                              srmRequestReturnStatus.setStatus("SRM_FILE_RELEASE_FAILED");
                              srmRequestReturnStatus.setExplanation
									("Release Files is not successful");
                            }
                            else if(type == 3) {
                              srmRequestReturnStatus.setStatus("SRM_FILE_REMOVE_FAILED");
                              srmRequestReturnStatus.setExplanation
									("Remove Files is not successful");
                            }
                         }
                         srmFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
                       }//if(code)
                     }//if(rStatus)
                }//end if
              }//enf if(obj)
           }//if(returnFileStatus.getSurl()); 
       }//if(returnFileStatus)
    }//end for
  }//if(srmRequestResponse != null)
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// extendFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes an extension request on the files.
    submits the lifetime extension request to SRM server.
    When SURLs are set in the SRMRequest, all SURLs in the
    request will have a new file lifetime. When TURLs exist in 
    SRMRequest, all TURLs wil have a new pin lifetime. It uses
    srmExtendFileLifeTime inernally to extend lifetime of files.
    @param fileLifetime --- valid integer.
    @return boolean --- returns true or false.
*/

public boolean extendFileLifeTime(int fileLifetime) throws Exception
{
   if(srmRequestResponse.getTimeStamp() != null) {
     srmRequestResponse.setTimeStamp(new Date());
   }

   SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   putClient.setUserDesc(userDesc); 
   putClient.setAuthID(authId); 
   putClient.setProtocolsList(protocolsList); 
   putClient.setTargetSpaceToken(spaceToken);
   putClient.setTotalRequestTime(totalRequestTime);
   putClient.setPinLifeTime(pinLifeTime);
   putClient.setFileLifeTime(fileLifeTime);
   putClient.setStorageSystemInfo(storageInfo);
   putClient.setRetentionPolicyInfo(retentionPolicyInfo);
   putClient.setFileStorageType(fileStorageType);
   putClient.setOverwrite(overwrite);

   boolean putContents = false;
   HashMap map = new HashMap();
   if(srmRequestResponse.getFileStatuses().size() == 0) {
      putContents=true;
   }

   int size = surlsVec.size();

   String[] surls = new String[surlsVec.size()];
   for(int i = 0; i < surlsVec.size(); i++) {
     String str = (String) surlsVec.elementAt(i);
     FileStatus srmFileStatus = new FileStatus();
     srmFileStatus.setSURL(new URI(str));
     surls[i] = str;
     map.put(str,srmFileStatus);
   }

   if(putContents) {
     srmRequestResponse.setFileStatuses(map);
   }

   SrmExtendFileLifeTimeResponse response = 
		//putClient.extendFileLifeTime(srmRequestId,surls,fileLifeTime); 
		putClient.extendFileLifeTime("",surls,fileLifeTime); 

   if(response == null) return false;

   TReturnStatus rStatus = response.getReturnStatus();
    
   if(srmRequestResponse != null) {
       srmRequestResponse.setReturnStatus(rStatus);
   }
   if(response.getArrayOfFileStatuses() != null) {
       TSURLLifetimeReturnStatus[] lifeTimeReturnStatus =
	     response.getArrayOfFileStatuses().getStatusArray();
       if(lifeTimeReturnStatus != null) {
         mapToSrmRequestResponse(lifeTimeReturnStatus);
       }
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// abortRequest
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes an abort request on the files or the entire request
    that a user previously requested. Upon the sussessful abort,
    the occupied space by the file gets released and would be available
    for more files.
    submits the abort request to SRM server. It uses srmAbortRequest
    internally with the stored request token to abort the request.
    The status of each file is stored afterwards in the object
    for client retrieval.
    @return boolean
*/

public boolean abortRequest() throws Exception
{
   if(srmRequestResponse.getTimeStamp() != null) {
     srmRequestResponse.setTimeStamp(new Date());
   }
   SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   putClient.setUserDesc(userDesc); 
   putClient.setAuthID(authId); 
   putClient.setProtocolsList(protocolsList); 
   putClient.setTargetSpaceToken(spaceToken);
   putClient.setTotalRequestTime(totalRequestTime);
   putClient.setPinLifeTime(pinLifeTime);
   putClient.setFileLifeTime(fileLifeTime);
   putClient.setStorageSystemInfo(storageInfo);
   putClient.setRetentionPolicyInfo(retentionPolicyInfo);
   putClient.setFileStorageType(fileStorageType);
   putClient.setOverwrite(overwrite);

   boolean putContents = false;
   HashMap map = new HashMap();
   if(srmRequestResponse.getFileStatuses().size() == 0) {
      putContents=true;
   }

   int size = surlsVec.size();

   for(int i = 0; i < size; i++) {
       String str = (String) surlsVec.elementAt(i);
       FileStatus srmFileStatus = new FileStatus();
       srmFileStatus.setSURL(new URI(str));
       map.put(str,srmFileStatus);
   }

   if(putContents) {
     srmRequestResponse.setFileStatuses(map);
   }

   SrmAbortRequestResponse response = putClient.abortRequest(srmRequestId); 
   if(response == null) return false;
   TReturnStatus rStatus = response.getReturnStatus();
   if (rStatus != null) {
         mapToSrmRequestResponseAbortRequest(rStatus);
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// srmLs
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client can query a file property to find out the size,
    online availability, creation time etc.
    submits the file browsing request to SRM server. It uses
    srmLs internally to get files information. When request token
    is returned, the return status will reflect the status and
    additional status checking needs to be done through checkStatus().
    @return boolean
*/

public boolean srmLs() throws SRMClientException, Exception
{
   srmRequestResponse.setTimeStamp(new Date());
   requestId = "ls:"+requestNo;
   requestNo++;
   SRMDirClient dirClient = new SRMDirClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   dirClient.setAuthID(authId); 
   dirClient.setStorageSystemInfo(storageInfo);
   dirClient.setFileStorageType(fileStorageType);
   dirClient.setOffSet(offset);
   dirClient.setCount(count);

   String[] siteUrl = new String[surlsVec.size()]; 
   for(int i = 0; i < surlsVec.size(); i++) {
     siteUrl[i] = (String) surlsVec.elementAt(i);
   }
  
   SrmLsResponse response = dirClient.srmLs(siteUrl, allLevelsRecursive, numLevels); 

   if(response == null) return false;

   if(response.getRequestToken() != null) {
      srmRequestId = response.getRequestToken();
   } 
   mapResponseToSrmRequestResponse((Object)response);
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// srmRm
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** submits the file removal request to SRM server. It uses srmRm
    internally to remove files from the SRM managed space.
    @return boolean
*/

public boolean srmRm() throws SRMClientException, Exception
{
   srmRequestResponse.setTimeStamp(new Date());
   SRMDirClient dirClient = new SRMDirClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   dirClient.setAuthID(authId); 
   dirClient.setStorageSystemInfo(storageInfo);

   HashMap map = new HashMap();
   String[] siteUrl = new String[surlsVec.size()];
   for(int i = 0; i < surlsVec.size(); i++) {
     FileStatus srmFileStatus = new FileStatus();
     siteUrl[i] = (String) surlsVec.elementAt(i);
     srmFileStatus.setSURL(new URI(siteUrl[i]));
     map.put(siteUrl[i], srmFileStatus);
   }
  
   srmRequestResponse.setFileStatuses(map);

   SrmRmResponse response = dirClient.rmFile(siteUrl); 
   if(response == null) return false;

   if(response.getReturnStatus() != null) {
     srmRequestResponse.setReturnStatus(response.getReturnStatus());
   }

   if(response.getArrayOfFileStatuses() != null) {
     ArrayOfTSURLReturnStatus returnStatus = response.getArrayOfFileStatuses();
     TSURLReturnStatus[] returnFileStatus = returnStatus.getStatusArray();
     if(returnFileStatus != null) {
       mapToSrmRequestResponse(returnFileStatus,3);
     }
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// putDone
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** submits srmPutDone request to SRM serer. It is for PUT request
    only, and SURLs are optional. When surls are not provided,
    all SURLs in the request will be submitted. It uses srmPutDone
    internally, and the returned status of each file is stored
    in the object for retrieval.
    @param siteUrl --- source urls to be requested for putDone
    @return boolean
**/

public boolean putDone(String[] siteUrl) throws Exception
{
   if(srmRequestResponse.getTimeStamp() != null) {
     srmRequestResponse.setTimeStamp(new Date());
   }
   SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   putClient.setUserDesc(userDesc); 
   putClient.setAuthID(authId); 
   putClient.setProtocolsList(protocolsList); 
   putClient.setTargetSpaceToken(spaceToken);
   putClient.setTotalRequestTime(totalRequestTime);
   putClient.setPinLifeTime(pinLifeTime);
   putClient.setFileLifeTime(fileLifeTime);
   putClient.setStorageSystemInfo(storageInfo);
   putClient.setRetentionPolicyInfo(retentionPolicyInfo);
   putClient.setFileStorageType(fileStorageType);
   putClient.setOverwrite(overwrite);

   boolean putContents = false;
   HashMap map = new HashMap();
   if(srmRequestResponse.getFileStatuses().size() == 0) {
      putContents=true;
   }
   if(siteUrl == null || siteUrl.length == 0) { 
     siteUrl = new String[surlsVec.size()];
     int size = surlsVec.size(); 
     for(int i = 0; i < size; i++) {
       FileStatus srmFileStatus = new FileStatus();
       siteUrl[i] = (String) surlsVec.elementAt(i);
       srmFileStatus.setSURL(new URI(siteUrl[i]));
       map.put(siteUrl[i],srmFileStatus);
     }
   }
   if(putContents) {
     srmRequestResponse.setFileStatuses(map);
   }
   SrmPutDoneResponse response = putClient.putDone(siteUrl, srmRequestId); 
   if(response == null) return false;

   if(response.getReturnStatus() != null) {
       srmRequestResponse.setReturnStatus(response.getReturnStatus());
     }
     if(response.getArrayOfFileStatuses() != null) {
       ArrayOfTSURLReturnStatus returnStatus = response.getArrayOfFileStatuses();
       TSURLReturnStatus[] returnFileStatus = returnStatus.getStatusArray();
       if(returnFileStatus != null) {
         mapToSrmRequestResponse(returnFileStatus,0);
       }
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// abortFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** submits the file abort request to SRM server. SURLs are optional,
    and when they are not provided, all SURLs in the request will
    be submitted for abort. It uses srmAbortFiles internally with the
    stored request token to abort files in the request. The status
    of each file is stored afterwards in the object for retrieval.
    @param surl -- source urls to be called for abort files.
    @return boolean
**/

public boolean abortFiles(String[] surl) throws SRMClientException,Exception
{
   if(srmRequestResponse.getTimeStamp() != null) {
     srmRequestResponse.setTimeStamp(new Date());
   }
   SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   putClient.setUserDesc(userDesc); 
   putClient.setAuthID(authId); 
   putClient.setProtocolsList(protocolsList); 
   putClient.setTargetSpaceToken(spaceToken);
   putClient.setTotalRequestTime(totalRequestTime);
   putClient.setPinLifeTime(pinLifeTime);
   putClient.setFileLifeTime(fileLifeTime);
   putClient.setStorageSystemInfo(storageInfo);
   putClient.setRetentionPolicyInfo(retentionPolicyInfo);
   putClient.setFileStorageType(fileStorageType);
   putClient.setOverwrite(overwrite);
   
   boolean putContents = false;
   HashMap map = new HashMap();
   if(srmRequestResponse.getFileStatuses().size() == 0) {
      putContents=true;
   }
   if(surl == null || surl.length == 0) {
     int size = surlsVec.size();
     surl = new String[size];   
     for(int i = 0; i < size; i++) {
       FileStatus srmFileStatus = new FileStatus();
       surl[i] = (String) surlsVec.elementAt(i);
       srmFileStatus.setSURL(new URI(surl[i]));
       map.put(surl[i],srmFileStatus);
     }
   }
   if(putContents) {
     srmRequestResponse.setFileStatuses(map);
   }
   SrmAbortFilesResponse response = putClient.abortFiles(surl, srmRequestId); 

   if(response == null) return false;

   if(response.getReturnStatus() != null) {
       srmRequestResponse.setReturnStatus(response.getReturnStatus());
   }
   if(response.getArrayOfFileStatuses() != null) {
       ArrayOfTSURLReturnStatus returnStatus = response.getArrayOfFileStatuses();
       TSURLReturnStatus[] returnFileStatus = returnStatus.getStatusArray();
       if(returnFileStatus != null) {
         mapToSrmRequestResponse(returnFileStatus,1);
       }
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// releaseFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes a release request on the files or on the
    request that previously requested. Upon the successful release,
    the occupied space by the file gets released and would be
    available for more files.
    @param siteUrl --- surls to be released
    @return boolean
**/

public boolean releaseFiles(String[] siteUrl) 
	throws SRMClientException, Exception
{
   if(srmRequestResponse.getTimeStamp() != null) {
     srmRequestResponse.setTimeStamp(new Date());
   }
   boolean keepSpace = false;
   SRMGetClient getClient = new SRMGetClient(_srm,_credential,_debug,silent,_theLogger,
     bufferSize,parallelism,dcau);
   getClient.setUserDesc(userDesc); 
   getClient.setAuthID(authId); 
   getClient.setProtocolsList(protocolsList); 
   getClient.setTargetSpaceToken(spaceToken);
   getClient.setTotalRequestTime(totalRequestTime);
   getClient.setPinLifeTime(pinLifeTime);
   getClient.setStorageSystemInfo(storageInfo);
   getClient.setRetentionPolicyInfo(retentionPolicyInfo);
   getClient.setFileStorageType(fileStorageType);

   boolean putContents = false;
   HashMap map = new HashMap();
   if(srmRequestResponse.getFileStatuses().size() == 0) {
      putContents=true;
   }
   if(siteUrl != null) {
     int size = siteUrl.length;
     for(int i = 0; i < size; i++) {
       FileStatus srmFileStatus = new FileStatus();
       srmFileStatus.setSURL(new URI(siteUrl[i]));
       map.put(siteUrl[i],srmFileStatus);
     }
   }
   if(putContents) {
     srmRequestResponse.setFileStatuses(map);
   }

   SrmReleaseFilesResponse response = getClient.releaseFile(siteUrl, srmRequestId, keepSpace); 

   if(response == null) return false;

   if(response.getArrayOfFileStatuses() != null) {
     ArrayOfTSURLReturnStatus returnStatus = response.getArrayOfFileStatuses();
     TSURLReturnStatus[] returnFileStatus = returnStatus.getStatusArray();
     if(returnFileStatus != null) {
       mapToSrmRequestResponse(returnFileStatus,2);
     }
   }
   return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes a status of previously submitted request on files
    that a user selected, and the status of files get returned.
    Status of selective files or all files in the request can be
    queried.
    getStatus --- retrieves the request status
    @return SRMRequestStatus
    @see SRMRequestStatus
**/

public SRMRequestStatus getStatus() {
  return srmRequestResponse;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//getStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes a status of previously submitted request on files
    that a user selected, and the status of files get returned.
    Status of selective files or all files in the request can be
    queried.
    getStatus --- retrieves the request status
    @param surl --- surl to be retrieved for status
    @return SRMRequestStatus
    @see SRMRequestStatus
**/

public SRMRequestStatus getStatus(String[] surl) {
  try {
     return getSubStatus(surl);
  }catch(Exception e) {
     System.out.println("Exception="+e.getMessage());
     return null;
  }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// srmGetRequestSummary
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/*** returns the request summary for the given request id
     @return SRMRequestSummary
     @see SRMRequestSummary
*/

public SRMRequestSummary getRequestSummary() {

  try {
    if(_debug) {
      System.out.println("........Input parameters ....");
    }
    SrmGetRequestSummaryRequest req = new SrmGetRequestSummaryRequest();

    if(!authId.equals("")) {
      if(_debug) {
         System.out.println("AuthId="+authId);
      }
      req.setAuthorizationID(authId); 
    }
    ArrayOfString aString = new ArrayOfString();
    String[] aa = new String[1];
    aa[0] = srmRequestId;
    aString.setStringArray(aa);
    req.setArrayOfRequestTokens(aString);
    if(_debug) {
         System.out.println("RequestId="+srmRequestId);
    }
    SrmGetRequestSummaryResponse response = _srm.srmGetRequestSummary(req);
    if(response == null) {
      System.out.println("SrmGetRequestSummary response is null");
      return null;
    }
    requestSummary = new SRMRequestSummary();
    requestSummary.status = response.getReturnStatus();
    ArrayOfTRequestSummary aSummary = response.getArrayOfRequestSummaries();
    if(aSummary == null) {
      return requestSummary;
    }
    TRequestSummary[] summaries = aSummary.getSummaryArray(); 

    for(int i = 0; i < summaries.length; i++) {
      TRequestSummary ss = summaries[i];
      if(ss.getRequestToken().equals(srmRequestId)) {
        requestSummary.requestId = srmRequestId;
        requestSummary.setSummaryArray(ss);
        if(ss.getStatus() != null) {
          requestSummary.status = ss.getStatus(); 
        }
      }
    }
  }catch(Exception e) {
    System.out.println("Exception="+e.getMessage());
    return null;
  }
  return requestSummary;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printSummary
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** prints the request summary for this request id.
*/

public void printSummary() {
   
  if(requestSummary == null) {
    System.out.println("SrmGetRequestSummary response is null");
    return;
  }
  TReturnStatus rStatus = requestSummary.status;
  if(rStatus == null) {
    System.out.println("SrmGetRequestSummary return status  is null");
    return;
  }
  System.out.println("\n...........RequestSummary ...........");
  System.out.println("StatusCode="+rStatus.getStatusCode()); 
  System.out.println("Explanation="+rStatus.getExplanation()); 
  TRequestSummary ss = requestSummary.getSummaryArray();

    System.out.println("RequestToken="+ss.getRequestToken());
    TReturnStatus rsStatus = ss.getStatus();
    if(rsStatus == null) {
       System.out.println("Return status is null");
    }
    else {
      System.out.println("StatusCode="+rsStatus.getStatusCode());
      System.out.println("Explanation="+rsStatus.getExplanation());
    }
    if(ss.getRequestType() != null) {
     System.out.println("RequestType="+ss.getRequestType().toString());
    }
    if(ss.getTotalNumFilesInRequest() != null) {
     System.out.println("TotalNumFilesInRequest="+
		ss.getTotalNumFilesInRequest().intValue());
    }
    if(ss.getNumOfCompletedFiles() != null) {
     System.out.println("GetNumOfCompletedFiles="+
		ss.getNumOfCompletedFiles().intValue());
    }
    if(ss.getNumOfWaitingFiles() != null) {
     System.out.println("GetNumOfWaitingFiles="+
		ss.getNumOfWaitingFiles().intValue());
    }
    if(ss.getNumOfFailedFiles() != null) {
     System.out.println("GetNumOfFailedFiles="+
		ss.getNumOfFailedFiles().intValue());
    }
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//checkStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** Client makes a status of previously submitted request on files
    that a user selected, and the status of files gets returned.
    Status of selective files or all files in the request can be queried.
    checkStatus -- checks the request status. Depending on the request type,
    srmStatusOfGetRequest, srmStatusOfPutRequest, srmStatusOfCopyRequest
    or srmStatusOfBringOnlineRequest will be used internally with its 
    stored request token from the previous submission. The returned status
    result is stored in the object with a time stamp of the status check
    time for client retrieval, and should be retrieved with getStatus().
    @param surl --- source surls for get, put, copy, bringonline requests.
    @param copyTargetSurl --- target surls for copy requests.
    @return boolean
**/

public boolean checkStatus(String[] surl, String[] copyTargetSurl) 
	throws SRMClientException, Exception {
  if(surl == null) {
    surl = new String[0];
  }
  if(copyTargetSurl == null) {
    copyTargetSurl = new String[0];
  }
  return doStatus(surl,copyTargetSurl);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//checkStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** checkStatus for all files in this request
**/

public boolean checkStatus() throws SRMClientException, Exception {
   String[] surl = new String[0];
   String[] turl = new String[0];
   return doStatus(surl,turl);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//doStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private boolean doStatus (String[] surl, String[] turl) {

   srmRequestResponse.setTimeStamp(new Date());
   try {
      if(requestId.startsWith("put")) {
        SRMPutClient putClient = new SRMPutClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize,parallelism,dcau);
        SrmStatusOfPutRequestResponse putResponse = putClient.doStatus(srmRequestId,surl); 
        if(putResponse == null) return false;
        mapResponseToSrmRequestResponse((Object)putResponse);
      }
      else if(requestId.startsWith("get")) {
        SRMGetClient getClient = new SRMGetClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize,parallelism,dcau);
        SrmStatusOfGetRequestResponse getResponse = getClient.doStatus(srmRequestId,surl); 
        if(getResponse == null) return false;
        mapResponseToSrmRequestResponse((Object)getResponse);
      }
      else if(requestId.startsWith("bringonline")) {
        SRMBringOnlineClient bringOnlineClient = 
		  new SRMBringOnlineClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize,parallelism,dcau);
        SrmStatusOfBringOnlineRequestResponse bringOnlineResponse = 
				bringOnlineClient.doStatus(srmRequestId,surl); 
        if(bringOnlineResponse == null) return false;
        mapResponseToSrmRequestResponse((Object)bringOnlineResponse);
      }
      else if(requestId.startsWith("copy")) {
        SRMCopyClient copyClient = 
		  new SRMCopyClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize,parallelism,dcau);
        SrmStatusOfCopyRequestResponse copyResponse = 
				copyClient.doStatus(srmRequestId,surl,turl); 
        if(copyResponse == null) return false;
        mapResponseToSrmRequestResponse((Object)copyResponse);
      }
      else if(requestId.startsWith("ls")) {
        SRMDirClient lsClient = 
		  new SRMDirClient(_srm,_credential,_debug,silent,_theLogger,
			bufferSize,parallelism,dcau);
        SrmStatusOfLsRequestResponse lsResponse = 
				lsClient.doStatus(srmRequestId); 
        if(lsResponse == null) return false;
        mapResponseToSrmRequestResponse((Object)lsResponse);
       }
     }catch(Exception e) { 
        System.out.println("Exception="+e.getMessage());
        util.printStackTrace(e,_theLogger);
        return false;
     }
     return true;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printMetaDataDetails
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void printMetaDataDetails(String prefix, PathDetail pDetails) 
	throws Exception {

   if(pDetails.getPath() != null) {
     util.printMessage(prefix+"SURL="+ pDetails.getPath(), silent); 
   }
   if(pDetails.getSize() != null) {
     util.printMessage(prefix+"Bytes="+ pDetails.getSize(), silent); 
   }
   if(pDetails.getFileType()  != null) {
     util.printMessage(prefix+"FileType="+pDetails.getFileType(),silent); 
   }
   if(pDetails.getFileStorageType() != null) {
     util.printMessage(prefix+"StorageType="+ pDetails.getFileStorageType(), silent); 
   }
   if(pDetails.getStatus() != null) {
     TReturnStatus rStatus = pDetails.getStatus();
     TStatusCode code = rStatus.getStatusCode(); 
     util.printMessage(prefix+"Status="+ code, silent); 
     if(rStatus.getExplanation () != null) {
       util.printMessage(prefix+"Explanation="+ rStatus.getExplanation(), silent); 
     }
   }
   if(pDetails.getCreatedAtTime() != null) {
     util.printMessage(prefix+"CreatedAtTime",silent);
     Calendar cal = pDetails.getCreatedAtTime();
     Date dd = cal.getTime();
     int year = dd.getYear()+1900;
     int month = dd.getMonth();
     int day = dd.getDate();
     int hour = dd.getHours();
     int minute = dd.getMinutes();
     int second = dd.getSeconds();
     util.printMessage(prefix+"\tYear="+ year, silent); 
     util.printMessage(prefix+"\tMonth="+ month, silent); 
     util.printMessage(prefix+"\tDay="+ day, silent); 
     util.printMessage(prefix+"\tHour="+ hour, silent); 
     util.printMessage(prefix+"\tMinute="+ minute, silent); 
     util.printMessage(prefix+"\tSecond="+ second, silent); 
   }
   if(pDetails.getLastModificationTime() != null) {
     util.printMessage(prefix+"LastModificationTime",silent);
     Calendar cal = pDetails.getLastModificationTime();
     Date dd = cal.getTime();
     int year = dd.getYear()+1900;
     int month = dd.getMonth();
     int day = dd.getDate();
     int hour = dd.getHours();
     int minute = dd.getMinutes();
     int second = dd.getSeconds();
     util.printMessage(prefix+"\tYear="+ year, silent); 
     util.printMessage(prefix+"\tMonth="+ month, silent); 
     util.printMessage(prefix+"\tDay="+ day, silent); 
     util.printMessage(prefix+"\tHour="+ hour, silent); 
     util.printMessage(prefix+"\tMinute="+ minute, silent); 
     util.printMessage(prefix+"\tSecond="+ second, silent); 
   }
   if(pDetails.getRetentionPolicyInfo() != null) {
     TRetentionPolicy retentionPolicy = 
       pDetails.getRetentionPolicyInfo().getRetentionPolicy();
     TAccessLatency accessLatency = 
	   pDetails.getRetentionPolicyInfo().getAccessLatency();
     if(retentionPolicy != null && retentionPolicy.getValue() != null) {
       util.printMessage(prefix+"RetentionPolicy="+ retentionPolicy.getValue(),silent); 
     }
     if(accessLatency != null && accessLatency.getValue() != null) {
       util.printMessage(prefix+"AccessLatency="+ accessLatency.getValue(),silent); 
     }
   }
   if(pDetails.getFileLocality() != null) {
     util.printMessage(prefix+"FileLocality="+ 
	   pDetails.getFileLocality().getValue(),silent); 
   }
   if(pDetails.getArrayOfSpaceTokens() != null) {
     ArrayOfString arrayOfString = pDetails.getArrayOfSpaceTokens();
     String[] sss = arrayOfString.getStringArray();
     for(int j = 0; j < sss.length; j++) {
       util.printMessage(prefix+"SpaceTokens["+j+"]="+sss[j],silent); 
     }
   }
   if(pDetails.getLifeTimeAssigned()  != null) {
     Integer ii = pDetails.getLifeTimeAssigned();
     util.printMessage(prefix+"LifeTimeAssigned="+ii.intValue(),silent); 
   }
  if(pDetails.getLifeTimeLeft()  != null) {
     Integer ii = pDetails.getLifeTimeLeft();
     util.printMessage(prefix+"LifeTimeLeft="+ii.intValue(),silent); 
  }
  if(pDetails.getCheckSumType()  != null) {
     util.printMessage(prefix+"CheckSumType="+pDetails.getCheckSumType(),silent); 
  } 
  if(pDetails.getCheckSumValue()  != null) {
     util.printMessage(prefix+"CheckSumValue="+pDetails.getCheckSumValue(),silent); 
  } 
  if(pDetails.getOwnerPermission()  != null) {
     TUserPermission perm = pDetails.getOwnerPermission();
     util.printMessage
		(prefix+"OwnerPermission.getUserID="+perm.getUserID(),silent); 
     TPermissionMode mode = perm.getMode();
     if(mode != null) {
      util.printMessage
		(prefix+"OwnerPermission.getMode="+mode.toString(),silent); 
     }
  } 
  if(pDetails.getGroupPermission()  != null) {
     TGroupPermission perm = pDetails.getGroupPermission();
     util.printMessage
		(prefix+"GroupPermission.getUserID="+perm.getGroupID(),silent); 
     TPermissionMode mode = perm.getMode();
     if(mode != null) {
      util.printMessage
		(prefix+"GroupPermission.getMode="+mode.toString(),silent); 
     }
  } 
  if(pDetails.getOtherPermission()  != null) {
     TPermissionMode perm = pDetails.getOtherPermission();
     if(perm != null) {
      util.printMessage
		(prefix+"OtherPermission.getMode="+perm.toString(),silent); 
     }
  } 
  if(pDetails.getSubPath() != null) {
     PathDetail[] mp = pDetails.getSubPath();
     for(int i = 0; i < mp.length; i++) {
       printMetaDataDetails(prefix+"\t\t\t",mp[i]);
     }
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// getSubStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private SRMRequestStatus getSubStatus(String[] surl) 
  throws Exception {
  SRMRequestStatus subStatus = new SRMRequestStatus();
  if(srmRequestResponse.getTimeStamp() != null) {
    subStatus.setTimeStamp(srmRequestResponse.getTimeStamp());
  }
  if(srmRequestResponse.getRequestToken() != null) {
    subStatus.setRequestToken(srmRequestResponse.getRequestToken());
  }
  TReturnStatus rStatus = srmRequestResponse.getReturnStatus();
  if(rStatus != null) {
     subStatus.setReturnStatus(rStatus);
  }
  if(srmRequestResponse.getRemainingTotalRequestTime() != null) {
     subStatus.setRemainingTotalRequestTime(srmRequestResponse.getRemainingTotalRequestTime());
  }
  if(srmRequestResponse.getRemainingDeferredStartTime() != null) {
	 subStatus.setRemainingDeferredStartTime(srmRequestResponse.getRemainingDeferredStartTime()); 
  }
  HashMap pathDetails = srmRequestResponse.getPathDetails();
  subStatus.setPathDetails(pathDetails);

  HashMap map = srmRequestResponse.getFileStatuses();
  HashMap subMap = new HashMap();
  for(int i = 0; i < surl.length; i++) {
    Object obj = map.get(surl[i]);
    if(obj != null) {
      FileStatus fileStatus = (FileStatus)obj;
      FileStatus newFileStatus = new FileStatus();
      if(fileStatus.getSURL() != null) {
        newFileStatus.setSURL(fileStatus.getSURL());
      }
      if(fileStatus.getTURL() != null) {
        newFileStatus.setTURL(fileStatus.getTURL());
      }
      if(fileStatus.getTransferSURL() != null) {
        newFileStatus.setTransferSURL(fileStatus.getTransferSURL());
      }
      if(fileStatus.getFileSize() != null) {
        newFileStatus.setFileSize(fileStatus.getFileSize());
      }
      if(fileStatus.getStatus() != null) {
        newFileStatus.setStatus(fileStatus.getStatus());
      }
      if(fileStatus.getSrmRequestReturnStatus() != null) {
        SrmRequestReturnStatus srmRequestReturnStatus = 
			  fileStatus.getSrmRequestReturnStatus();
        newFileStatus.setSrmRequestReturnStatus(srmRequestReturnStatus);
      }
      if(fileStatus.getEstimatedWaitTime() != null) {
        newFileStatus.setEstimatedWaitTime(fileStatus.getEstimatedWaitTime());
      }
      if(fileStatus.getRemainingPinLifeTime() != null) {
        newFileStatus.setRemainingPinLifeTime(fileStatus.getRemainingPinLifeTime());
      }
      if(fileStatus.getTransferProtocolInfo() != null) {
         ArrayOfTExtraInfo info = fileStatus.getTransferProtocolInfo();
         newFileStatus.setTransferProtocolInfo(info);
      }
      subMap.put(newFileStatus.getSURL(),newFileStatus);
    }//end if (obj != null)
  }//end for
  subStatus.setFileStatuses(subMap);
  return subStatus;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printResults
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** prints results for this request
*/

public void printResults() throws Exception {
  print(srmRequestResponse);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printResults
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

/** prints the given SRMRequestStatus object.
    @param subStatus
    @see SRMRequestStatus
*/

public void printResults(SRMRequestStatus subStatus) throws Exception {
  print(subStatus);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// print
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void print(SRMRequestStatus srmRequestResponse) throws Exception {

   if(srmRequestResponse != null) {
     util.printMessage("SRM-CLIENT: ..........................................",silent);
     util.printMessage("SRM-CLIENT: ....Printing Results.......... ",silent);
     util.printMessage("TimeStamp="+srmRequestResponse.getTimeStamp(), silent); 
     util.printMessage("RequestToken="+srmRequestResponse.getRequestToken(), silent); 
     TReturnStatus rStatus = srmRequestResponse.getReturnStatus();
     if(rStatus != null) {
       TStatusCode code = rStatus.getStatusCode();
       util.printMessage("Status="+code, silent); 
       util.printMessage("Explanation="+rStatus.getExplanation(), silent); 
     }
     if(srmRequestResponse.getRemainingTotalRequestTime() != null) {
       util.printMessage("RemainingTotalRequestTime="+
			srmRequestResponse.getRemainingTotalRequestTime(), silent); 
     }
     if(srmRequestResponse.getRemainingDeferredStartTime() != null) {
       util.printMessage("RemainingDeferredStartTime="+
			srmRequestResponse.getRemainingDeferredStartTime(), silent); 
     }
     HashMap pathDetails = srmRequestResponse.getPathDetails();
     Collection pathDetailsCollection = pathDetails.values();
     Object[] obj = pathDetailsCollection.toArray();
     if(obj.length > 0) {
       util.printMessage("LS output",silent);
     }
     for(int i = 0; i < obj.length; i++) {
       PathDetail pDetails = (PathDetail) obj[i];
       printMetaDataDetails("\t\t",pDetails);
     }//end for(pathDetails.size() > 0)

     HashMap map = srmRequestResponse.getFileStatuses();
     Collection c = map.values();
     obj = c.toArray();
     for(int i = 0; i < obj.length; i++) {
         if(i > 0) {
            util.printMessage("\t.............................................................",silent);
         }
         FileStatus fileStatus = (FileStatus) obj[i];
         if(fileStatus.getSURL() != null) {
           util.printMessage("\tSURL="+ fileStatus.getSURL(), silent); 
         }
         if(fileStatus.getTURL() != null) {
           util.printMessage("\tTURL="+ fileStatus.getTURL(), silent); 
         }
         if(fileStatus.getTransferSURL() != null) {
           util.printMessage("\tTransferSURL="+ fileStatus.getTransferSURL(), silent); 
         }
         if(fileStatus.getFileSize() != null) {
           util.printMessage("\tFileSize="+ fileStatus.getFileSize(), silent); 
         }
         if(fileStatus.getStatus() != null) {
           rStatus = fileStatus.getStatus();
           TStatusCode code = rStatus.getStatusCode(); 
           util.printMessage("\tStatus="+ code, silent); 
           if(rStatus.getExplanation () != null) {
             util.printMessage("\tExplanation="+ rStatus.getExplanation(), silent); 
           }
         }
         if(fileStatus.getSrmRequestReturnStatus() != null) {
           SrmRequestReturnStatus srmRequestReturnStatus = fileStatus.getSrmRequestReturnStatus();
           String code = srmRequestReturnStatus.getStatus(); 
           util.printMessage("\tStatus(2)="+ code, silent); 
           if(srmRequestReturnStatus.getExplanation () != null) {
             util.printMessage("\tExplanation(2)="+ 
					srmRequestReturnStatus.getExplanation(), silent); 
           }
         }
         if(fileStatus.getEstimatedWaitTime() != null) {
             util.printMessage("\tEstimatedWaitTime="+ 
					fileStatus.getEstimatedWaitTime(), silent); 
         }
         if(fileStatus.getRemainingPinLifeTime() != null) {
             util.printMessage("\tPinLifeTime="+ 
					fileStatus.getRemainingPinLifeTime(), silent); 
         }
         if(fileStatus.getRemainingFileLifeTime() != null) {
             util.printMessage("\tRemainingFileLifeTime="+ 
					fileStatus.getRemainingFileLifeTime(), silent); 
         }
         if(fileStatus.getTransferProtocolInfo() != null) {
             ArrayOfTExtraInfo info = fileStatus.getTransferProtocolInfo();
             TExtraInfo[] extraInfo = info.getExtraInfoArray();
             for(int j = 0; j < extraInfo.length; j++) {
               util.printMessage("\tKey="+ extraInfo[j].getKey(), silent); 
               util.printMessage("\tValue="+ extraInfo[j].getValue(), silent); 
             }
              
         }
     }//end for
   }//end if(srmRequestResponse != null)
}

}
