/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import gov.lbl.srm.client.exception.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMUtilClient
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

class SRMUtilClient {
  private ISRM _srm;
  private static boolean _debug;
  private static boolean silent;
  private static java.util.logging.Logger _theLogger;
  private static Vector inputVec = new Vector ();

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMUtilClient --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SRMUtilClient (ISRM srm,  java.util.logging.Logger theLogger,
	  boolean debug,boolean silent) throws Exception 
{
    this._srm = srm;
    this._theLogger = theLogger;
    this._debug = debug;
    this.silent = silent;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doPing
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmPingResponse doPing(String uid) 
	throws SRMClientException, Exception {

 inputVec = new Vector ();
 try {
  SrmPingRequest request = new SrmPingRequest();
  util.printMessage("\nSRM-PING: " + new Date() + " Calling SrmPing Request..." , silent);
  if(!uid.equals("")) {
    inputVec.addElement("UID="+uid);
    if(_debug) {
    util.printMessage("\nSRM-PING: .... Input parameters ....", silent);
    util.printMessage("SRM-PING : UID="+uid,silent);
    }
    request.setAuthorizationID(uid);
  }

  if(_srm == null) {
    inputVec.addElement("Cannot continue, connection failed with null srm");
    util.printEventLog(_theLogger,"doPing",inputVec,silent);
    util.printMessage("\nSRM-PING: Cannot continue, connection failed with null _srm", silent);
    return null;
  }

  inputVec.addElement("TimeStamp="+new Date());
  util.printEventLog(_theLogger,"SendingSrmPingRequest",inputVec,silent);

  SrmPingResponse response = _srm.srmPing(request);
  if(_debug) {
    SRMUtilClient.printPingResults(response);
  }
  return response;
  }catch(Exception e) {
   throw e;
  }
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printPingResults
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void printPingResults(SrmPingResponse response) throws Exception{
 try {
  if(response != null) {
    inputVec = new Vector ();
    
    String version = response.getVersionInfo();
    inputVec.addElement("PingVersionInfo="+version);
    util.printMessage("Ping versionInfo=" + version+"\n", silent);
    ArrayOfTExtraInfo aExtraInfo = response.getOtherInfo();
    TExtraInfo[] info = null;
    if(aExtraInfo != null) {
      info = aExtraInfo.getExtraInfoArray();
      if(info != null && info.length > 0)  {
        inputVec.addElement("ExtraInformation");
        util.printMessage("Extra information", silent);
       for(int i = 0; i < info.length; i++) {
         inputVec.addElement("Key="+info[i].getKey());
         inputVec.addElement("Value="+info[i].getValue());
         util.printMessage("\tKey="+info[i].getKey(),silent);
         util.printMessage("\tValue="+info[i].getValue(),silent);
        }
      }
      else {
        inputVec.addElement("Ping did not return any extra information");
        util.printMessage("\nPing did not return any extra information", silent);
      }
      util.printEventLog(_theLogger,"SrmPingResponse",inputVec,silent);
    }
   }
  }
  catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement(e.getMessage());
    util.printEventLog(_theLogger,"SrmPingResponse",inputVec,silent);
    String msg = e.getMessage();
    int idx = msg.indexOf("Connection refused");
    util.printMessage("SRM-PING : Exception " + e.getMessage(),silent);
    if(msg.startsWith("CGSI-gSOAP: Could not find mapping") ||
       idx != -1) {
      util.printMessage("\nException : "+msg,silent);
      if(idx != -1) {
        inputVec = new Vector ();
        inputVec.addElement("ExitStatus="+900);
        util.printEventLog(_theLogger,"ExitCodeStatus",inputVec,silent);
        System.exit(900);
      }
      else {
        inputVec = new Vector ();
        inputVec.addElement("ExitStatus="+901);
        util.printEventLog(_theLogger,"ExitCodeStatus",inputVec,silent);
        System.exit(901);
      }
     }
     else {
       throw new Exception(e.getMessage());
     }
    }
  }



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// converToArrayOfAnyURI
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static ArrayOfAnyURI convertToArrayOfAnyURI
        (org.apache.axis.types.URI[] uris) {
  ArrayOfAnyURI aURI = new ArrayOfAnyURI();
  aURI.setUrlArray(uris);
  return aURI;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// formArrayOfAnyURI
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static ArrayOfAnyURI formArrayOfAnyURI(URI surl) {
    org.apache.axis.types.URI[] urlArray = new org.apache.axis.types.URI[1];
    urlArray[0] = surl;
    return SRMUtilClient.convertToArrayOfAnyURI(urlArray);
}


}

